/**
  @file /ai/Diplomacy/diplomacy.cpp
  @brief Zdrojovy kod modulu diplomacie.
  
  Zdrojovy kod modulu diplomacie.
  
  @author PZ
  @version 0.1
*/

#include "ai/Diplomacy/diplomacy.h"

using namespace std;
namespace ai_ns
{
namespace diplomacy_ns
{

CDiplomacyEngine::CDiplomacyEngine()
{
  int i;
  for (i=0;i<MAX_PLAYERS;i++)
  {
    DiplomatRecords[i].world_player_id=-1;
    DiplomatRecords[i].slotused=false;
  }
  diplomats_number=0;
  loadCoeficients();
}

CDiplomacyEngine::~CDiplomacyEngine()
{
  clear();
}

void CDiplomacyEngine::initDiplomacy()
{
  clear();
}

void CDiplomacyEngine::clear()
{
  int i;
  for(i=0;i<MAX_PLAYERS;i++)
  {
    if (DiplomatRecords[i].slotused)
    {
      DiplomatRecords[i].world_player_id=-1;
      DiplomatRecords[i].slotused=false;
    }
  }
  diplomats_number=0;
}

void CDiplomacyEngine::loadCoeficients()
{
  ai_ns::CAIScriptCaller* sc=new ai_ns::CAIScriptCaller();
  ai_ns::CTCLAIVar* varsout=sc->countScript(ID_SCRIPT_DIPLOMACY_COEFICIENTS,0);
  
  Coeficients.coefActionAttack=varsout->getReal("coefActionAttack");
  Coeficients.coefActionAttackOnAlly=varsout->getReal("coefActionAttackOnAlly");
  Coeficients.coefActionAttackOnTruce=varsout->getReal("coefActionAttackOnTruce");
  Coeficients.coefActionAttackOnEnemy=varsout->getReal("coefActionAttackOnEnemy");
  Coeficients.coefActionAttackOnWar=varsout->getReal("coefActionAttackOnWar");
  Coeficients.coefActionConquerBuilding=varsout->getReal("coefActionConquerBuilding");                    
  Coeficients.coefActionConquerBuildingOfAlly=varsout->getReal("coefActionConquerBuildingOfAlly");               
  Coeficients.coefActionConquerBuildingOfTruce=varsout->getReal("coefActionConquerBuildingOfTruce");
  Coeficients.coefActionConquerBuildingOfEnemy=varsout->getReal("coefActionConquerBuildingOfEnemy");              
  Coeficients.coefActionConquerBuildingOfWar=varsout->getReal("coefActionConquerBuildingOfWar");
  Coeficients.coefActionConquerCity=varsout->getReal("coefActionConquerCity");                    
  Coeficients.coefActionConquerCityOfAlly=varsout->getReal("coefActionConquerCityOfAlly");               
  Coeficients.coefActionConquerCityOfTruce=varsout->getReal("coefActionConquerCityOfTruce");
  Coeficients.coefActionConquerCityOfEnemy=varsout->getReal("coefActionConquerCityOfEnemy");              
  Coeficients.coefActionConquerCityOfWar=varsout->getReal("coefActionConquerCityOfWar");
  Coeficients.coefBetrayalAllyToTruce=varsout->getInt("coefBetrayalAllyToTruce");
  Coeficients.coefBetrayalAllyToEnemy=varsout->getInt("coefBetrayalAllyToEnemy");
  Coeficients.coefBetrayalAllyToWar=varsout->getInt("coefBetrayalAllyToWar");
  Coeficients.coefBetrayalTruceToEnemy=varsout->getInt("coefBetrayalTruceToEnemy");
  Coeficients.coefBetrayalTruceToWar=varsout->getInt("coefBetrayalTruceToWar");
  Coeficients.coefBetrayalEnemyToWar=varsout->getInt("coefBetrayalEnemyToWar");

  delete varsout;
  delete sc;  
}

int CDiplomacyEngine::getFirstFreeDiplomatSlot()
{
  int i=0;
  while ((i<MAX_PLAYERS) && (DiplomatRecords[i].slotused)) i++;
  return i;
}

void CDiplomacyEngine::addDiplomat(int world_player_ID,TDiplomatType dt) throw(E_8K_AI_Diplomacy_MaximumNumberOfDiplomats,E_8K_AI_Diplomacy_DiplomatWorldIDsNotUnique)
{
  int i,j,fs;
  bool dnf=true;
  
  if ((fs=getFirstFreeDiplomatSlot())==MAX_PLAYERS)
  {
    THROW(E_8K_AI_Diplomacy_MaximumNumberOfDiplomats,"Neni misto pro noveho diplomata.")
  }

  i=0;
  while ((dnf) && (i<MAX_PLAYERS))
  {
    if ((DiplomatRecords[i].slotused) && (DiplomatRecords[i].world_player_id==world_player_ID))
    {
      dnf=false;
    }
    i++;
  }

  if (!dnf)
  {
    THROW(E_8K_AI_Diplomacy_DiplomatWorldIDsNotUnique,"ID pridavaneho diplomata je stejne jako nektereho jiz existujiciho.")
  }

  DiplomatRecords[fs].slotused=true;
  DiplomatRecords[fs].dt=dt;
  DiplomatRecords[fs].world_player_id=world_player_ID;

  for (j=0;j<MAX_PLAYERS;j++)
  {
    if ((fs!=j) && (DiplomatRecords[j].slotused))
    {
      setNeutralRelation(DiplomatRelations[fs][j]);
      setNeutralRelation(DiplomatRelations[j][fs]);
    }
  }
  diplomats_number++;
}

void CDiplomacyEngine::removeDiplomat(int world_player_ID) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  int dipid=findDiplomatIDByWorldID(world_player_ID);
  DiplomatRecords[dipid].slotused=false;
  DiplomatRecords[dipid].world_player_id=-1;
  diplomats_number--;
}

int CDiplomacyEngine::getDiplomatsNumber()
{
  return diplomats_number;
}

int CDiplomacyEngine::findDiplomatIDByWorldID(int world_player_ID) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  bool dnf=true;
  int i=0;
  
  while ((dnf) && (i<MAX_PLAYERS))
  {
    if ((DiplomatRecords[i].slotused) && (DiplomatRecords[i].world_player_id==world_player_ID))
    {
      dnf=false;
    }
    i++;
  }

  if (!dnf)
  {
    return (i-1);
  }
  else
  {
    THROW(E_8K_AI_Diplomacy_DiplomatNotFound,"Diplomat was not found.")
  }
}

void CDiplomacyEngine::setNeutralRelation(TDipRelation & nr)
{
  nr.rs=RELATIONSHIP_TRUCE;
  nr.offeredrs=RELATIONSHIP_TRUCE;
  nr.relationship_properties.belief=DEFAULT_BELIEF_FOR_TRUCE;
  nr.relationship_properties.guess_belief=DEFAULT_BELIEF_FOR_TRUCE;
  nr.relationship_properties.offer_seen_by_enemy=false;
}

TRelationship CDiplomacyEngine::getRelationshipByIDs(int dip1,int dip2)
{
  return DiplomatRelations[dip1][dip2].rs;
}

TRelationship CDiplomacyEngine::getRelationshipByWorldIDs(int dip1,int dip2) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  return getRelationshipByIDs(findDiplomatIDByWorldID(dip1),findDiplomatIDByWorldID(dip2));
}

void CDiplomacyEngine::setRelationshipByIDs(TRelationship relshp,int dip1,int dip2)
{
  DiplomatRelations[dip1][dip2].rs=relshp;
}

void CDiplomacyEngine::setRelationshipByWorldIDs(TRelationship relshp,int dip1,int dip2) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  setRelationshipByIDs(relshp,findDiplomatIDByWorldID(dip1),findDiplomatIDByWorldID(dip2));
}

TRelationship CDiplomacyEngine::getOfferRelationshipByIDs(int dip1,int dip2)
{
  return DiplomatRelations[dip1][dip2].offeredrs;
}

TRelationship CDiplomacyEngine::getOfferRelationshipByWorldIDs(int dip1,int dip2) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  return getOfferRelationshipByIDs(findDiplomatIDByWorldID(dip1),findDiplomatIDByWorldID(dip2));
}

void CDiplomacyEngine::setOfferRelationshipByIDs(TRelationship relshp,int dip1id,int dip2id)
{
  if (DiplomatRelations[dip1id][dip2id].offeredrs!=relshp)
  {
    DiplomatRelations[dip1id][dip2id].offeredrs=relshp;
    DiplomatRelations[dip1id][dip2id].relationship_properties.offer_seen_by_enemy=false;
  }
}

TPacket_DIPLOMACY_CHANGE* CDiplomacyEngine::setOfferRelationshipByWorldIDs(TRelationship relshp,int dip1,int dip2) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  TPacket_DIPLOMACY_CHANGE* rv=new TPacket_DIPLOMACY_CHANGE();
  TDipChange tdch;
  tdch.source_diplomat_world_id=dip1;
  tdch.target_diplomat_world_id=dip2;
  tdch.source_diplomat_penalty=0;
  tdch.target_diplomat_penalty=0;
  tdch.old_relationship=DiplomatRelations[findDiplomatIDByWorldID(dip1)][findDiplomatIDByWorldID(dip2)];

  setOfferRelationshipByIDs(relshp,findDiplomatIDByWorldID(dip1),findDiplomatIDByWorldID(dip2));

  tdch.new_relationship=DiplomatRelations[findDiplomatIDByWorldID(dip1)][findDiplomatIDByWorldID(dip2)];
  (rv->changes).push_back(tdch);
  return rv;
}

TRelationshipProperties CDiplomacyEngine::getOtherRelationshipPropertiesByIDs(int dip1,int dip2)
{
  return (DiplomatRelations[dip1][dip2].relationship_properties);
}

TRelationshipProperties CDiplomacyEngine::getOtherRelationshipPropertiesByWorldIDs(int dip1,int dip2) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  return getOtherRelationshipPropertiesByIDs(findDiplomatIDByWorldID(dip1),findDiplomatIDByWorldID(dip2));
}

void CDiplomacyEngine::setOtherRelationshipPropertiesByIDs(TRelationshipProperties & relshp_prop,int dip1,int dip2)
{
  DiplomatRelations[dip1][dip2].relationship_properties.offer_seen_by_enemy=relshp_prop.offer_seen_by_enemy;
  if (relshp_prop.belief>MAXIMAL_BELIEF)
    relshp_prop.belief=MAXIMAL_BELIEF;
  DiplomatRelations[dip1][dip2].relationship_properties.belief=relshp_prop.belief;
  if (relshp_prop.guess_belief>MAXIMAL_BELIEF)
    relshp_prop.guess_belief=MAXIMAL_BELIEF;
  DiplomatRelations[dip1][dip2].relationship_properties.guess_belief=relshp_prop.guess_belief;
}

void CDiplomacyEngine::setOtherRelationshipPropertiesByWorldIDs(TRelationshipProperties & relshp_prop,int dip1,int dip2) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  setOtherRelationshipPropertiesByIDs(relshp_prop,findDiplomatIDByWorldID(dip1),findDiplomatIDByWorldID(dip2));
}

bool CDiplomacyEngine::diplomatDefined(int id)
{
  return (DiplomatRecords[id].slotused);
}

int CDiplomacyEngine::getDiplomatWorldID(int id)
{
  return (DiplomatRecords[id].world_player_id);
}


void CDiplomacyEngine::calculateEvent(TEvent & new_event) throw(E_8K_AI_Diplomacy_RelationError,E_8K_AI_Diplomacy_UnknownEvent,E_8K_AI_Diplomacy_InvalidEventWeight)
{
  int dip_source_id,dip_target_id;
  
  if (new_event.event_weight==0) return; // pokud je vaha udalosti nulova nebo zaporna (nemelo by nastat)
  if (new_event.event_weight<0) THROW(E_8K_AI_Diplomacy_InvalidEventWeight,"Zaporna hodnota vahy diplomaticke udalosti neni povolena.");

  for(dip_source_id=0;dip_source_id<MAX_PLAYERS;dip_source_id++)
  {
    for(dip_target_id=0;dip_target_id<MAX_PLAYERS;dip_target_id++)
    {
      if ((dip_source_id!=dip_target_id) && (DiplomatRecords[dip_source_id].slotused) && (DiplomatRecords[dip_target_id].slotused))
      {
        switch (new_event.done_event)
        {
          case ACTION_TYPE_ATTACK:
            if (dip_source_id==new_event.akter_id) // S utocil
            {
              if (dip_target_id==new_event.receiver_id) // S utocil na T
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(DEFAULT_BELIEF_FOR_TRUCE,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttack));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_TRUCE) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_TRUCE))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_TRUCE)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_TRUCE;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_TRUCE)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_TRUCE;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  dch.target_diplomat_penalty=0;
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // S neutocil na T
              {
                if (IS_ALLY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl spojenec T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnAlly));
                }
                else
                if (IS_TRUCE(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v primeri s T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnTruce));
                }
                else
                if (IS_ENEMY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v nepratelskem vztahu T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnEnemy));
                }
                else
                if (IS_WAR(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl ve valce T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
            }
            else
            if (dip_source_id==new_event.receiver_id) // na S zautocili
            {
              if (dip_target_id==new_event.akter_id) // T utocil na S
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(DEFAULT_BELIEF_FOR_TRUCE,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttack));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_TRUCE) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_TRUCE))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_TRUCE)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_TRUCE;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_TRUCE)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_TRUCE;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=0;
                  dch.target_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // T neutocil na S - v tomto pripade se vztah S vuci T nemeni
              {
              }
            }
            else // S se utoku nijak neucastnil
            {
              if (dip_target_id==new_event.akter_id) // T utocil
              {
                if (IS_ALLY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl spojenec S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnAlly));
                }
                else
                if (IS_TRUCE(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v primeri s S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnTruce));
                }
                else
                if (IS_ENEMY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v nepratelskem vztahu se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnEnemy));
                }
                else
                if (IS_WAR(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl ve valce se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
              else // ani S ani T nebyli zapleteni do utoku
              {
              }
            }
            break;

          case ACTION_TYPE_ATTACK_BUILDING:
            if (dip_source_id==new_event.akter_id) // S utocil
            {
              if (dip_target_id==new_event.receiver_id) // S utocil na T
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(DEFAULT_BELIEF_FOR_ENEMY,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttack));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_ENEMY) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_ENEMY))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_ENEMY)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_ENEMY;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_ENEMY)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_ENEMY;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  dch.target_diplomat_penalty=0;
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // S neutocil na T
              {
                if (IS_ALLY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl spojenec T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnAlly));
                }
                else
                if (IS_TRUCE(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v primeri s T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnTruce));
                }
                else
                if (IS_ENEMY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v nepratelskem vztahu T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnEnemy));
                }
                else
                if (IS_WAR(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl ve valce T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
            }
            else
            if (dip_source_id==new_event.receiver_id) // na S zautocili
            {
              if (dip_target_id==new_event.akter_id) // T utocil na S
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(DEFAULT_BELIEF_FOR_ENEMY,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttack));
                if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_ENEMY)
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_ENEMY;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=0;
                  dch.target_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // T neutocil na S - v tomto pripade se vztah S vuci T nemeni
              {
              }
            }
            else // S se utoku nijak neucastnil
            {
              if (dip_target_id==new_event.akter_id) // T utocil
              {
                if (IS_ALLY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl spojenec S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnAlly));
                }
                else
                if (IS_TRUCE(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v primeri s S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnTruce));
                }
                else
                if (IS_ENEMY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v nepratelskem vztahu se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnEnemy));
                }
                else
                if (IS_WAR(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl ve valce se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
              else // ani S ani T nebyli zapleteni do utoku
              {
              }
            }
            break;

          case ACTION_TYPE_ATTACK_CITY:
            if (dip_source_id==new_event.akter_id) // S utocil
            {
              if (dip_target_id==new_event.receiver_id) // S utocil na T
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(DEFAULT_BELIEF_FOR_WAR,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttack));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_WAR;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_WAR;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  dch.target_diplomat_penalty=0;
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // S neutocil na T
              {
                if (IS_ALLY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl spojenec T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnAlly));
                }
                else
                if (IS_TRUCE(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v primeri s T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnTruce));
                }
                else
                if (IS_ENEMY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v nepratelskem vztahu T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnEnemy));
                }
                else
                if (IS_WAR(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl ve valce T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
            }
            else
            if (dip_source_id==new_event.receiver_id) // na S zautocili
            {
              if (dip_target_id==new_event.akter_id) // T utocil na S
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(DEFAULT_BELIEF_FOR_WAR,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttack));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_WAR;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_WAR;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=0;
                  dch.target_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }                
              }
              else // T neutocil na S - v tomto pripade se vztah S vuci T nemeni
              {
              }
            }
            else // S se utoku nijak neucastnil
            {
              if (dip_target_id==new_event.akter_id) // T utocil
              {
                if (IS_ALLY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl spojenec S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnAlly));
                }
                else
                if (IS_TRUCE(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v primeri s S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnTruce));
                }
                else
                if (IS_ENEMY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v nepratelskem vztahu se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnEnemy));
                }
                else
                if (IS_WAR(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl ve valce se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionAttackOnWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
              else // ani S ani T nebyli zapleteni do utoku
              {
              }
            }
            break;

          case ACTION_TYPE_CONQUER_BUILDING:
            if (dip_source_id==new_event.akter_id) // S utocil
            {
              if (dip_target_id==new_event.receiver_id) // S utocil na T
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(DEFAULT_BELIEF_FOR_ENEMY,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuilding));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_ENEMY) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_ENEMY))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_ENEMY)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_ENEMY;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_ENEMY)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_ENEMY;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  dch.target_diplomat_penalty=0;
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // S neutocil na T
              {
                if (IS_ALLY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl spojenec T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfAlly));
                }
                else
                if (IS_TRUCE(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v primeri s T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfTruce));
                }
                else
                if (IS_ENEMY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v nepratelskem vztahu T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfEnemy));
                }
                else
                if (IS_WAR(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl ve valce T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
            }
            else
            if (dip_source_id==new_event.receiver_id) // na S zautocili
            {
              if (dip_target_id==new_event.akter_id) // T utocil na S
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(DEFAULT_BELIEF_FOR_ENEMY,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuilding));
                if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_ENEMY)
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_ENEMY;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=0;
                  dch.target_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // T neutocil na S - v tomto pripade se vztah S vuci T nemeni
              {
              }
            }
            else // S se utoku nijak neucastnil
            {
              if (dip_target_id==new_event.akter_id) // T utocil
              {
                if (IS_ALLY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl spojenec S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfAlly));
                }
                else
                if (IS_TRUCE(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v primeri s S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfTruce));
                }
                else
                if (IS_ENEMY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v nepratelskem vztahu se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfEnemy));
                }
                else
                if (IS_WAR(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl ve valce se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerBuildingOfWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
              else // ani S ani T nebyli zapleteni do utoku
              {
              }
            }
            break;

          case ACTION_TYPE_CONQUER_CITY:
            if (dip_source_id==new_event.akter_id) // S utocil
            {
              if (dip_target_id==new_event.receiver_id) // S utocil na T
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(DEFAULT_BELIEF_FOR_WAR,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCity));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_WAR;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_WAR;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  dch.target_diplomat_penalty=0;
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }
              }
              else // S neutocil na T
              {
                if (IS_ALLY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl spojenec T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfAlly));
                }
                else
                if (IS_TRUCE(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v primeri s T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfTruce));
                }
                else
                if (IS_ENEMY(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl v nepratelskem vztahu T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfEnemy));
                }
                else
                if (IS_WAR(dip_target_id,new_event.receiver_id)) // napadeny diplomat byl ve valce T
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.guess_belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
            }
            else
            if (dip_source_id==new_event.receiver_id) // na S zautocili
            {
              if (dip_target_id==new_event.akter_id) // T utocil na S
              {
                DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(DEFAULT_BELIEF_FOR_WAR,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCity));
                if (((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR) || ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR))
                {
                  TPacket_DIPLOMACY_CHANGE tpdch;
                  TDipChange dch;
                  dch.old_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  if ((DiplomatRelations[dip_source_id][dip_target_id].rs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].rs=RELATIONSHIP_WAR;
                  if ((DiplomatRelations[dip_source_id][dip_target_id].offeredrs)>RELATIONSHIP_WAR)
                    DiplomatRelations[dip_source_id][dip_target_id].offeredrs=RELATIONSHIP_WAR;
                  dch.new_relationship=DiplomatRelations[dip_source_id][dip_target_id];
                  dch.source_diplomat_world_id=DiplomatRecords[dip_source_id].world_player_id;
                  dch.target_diplomat_world_id=DiplomatRecords[dip_target_id].world_player_id;
                  dch.source_diplomat_penalty=0;
                  dch.target_diplomat_penalty=countPenalty(dch.old_relationship.rs,dch.new_relationship.rs);
                  tpdch.changes.push_back(dch);
                  KSendGlobalMessage(MSG_DIPLOMACY_CHANGE,MOD_STRATEGY,MOD_WORLD_SERVER,&tpdch);
                }                
              }
              else // T neutocil na S - v tomto pripade se vztah S vuci T nemeni
              {
              }
            }
            else // S se utoku nijak neucastnil
            {
              if (dip_target_id==new_event.akter_id) // T utocil
              {
                if (IS_ALLY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl spojenec S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfAlly));
                }
                else
                if (IS_TRUCE(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v primeri s S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfTruce));
                }
                else
                if (IS_ENEMY(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl v nepratelskem vztahu se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfEnemy));
                }
                else
                if (IS_WAR(dip_source_id,new_event.receiver_id)) // diplomat napadeny T byl ve valce se S
                {
                  DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief=MIN(MAXIMAL_BELIEF,DiplomatRelations[dip_source_id][dip_target_id].relationship_properties.belief+roundReal(new_event.event_weight*Coeficients.coefActionConquerCityOfWar));
                }
                else
                {
                  THROW(E_8K_AI_Diplomacy_RelationError,"Invalidni diplomaticky vztah.")
                }
              }
              else // ani S ani T nebyli zapleteni do utoku
              {
              }
            }
            break;
                   
          default:THROW(E_8K_AI_Diplomacy_UnknownEvent,"Invalidni typ akce.");
        }

      }
    }
  }

}

TRelationship CDiplomacyEngine::getBeliefRelationship(int bel) throw(E_8K_AI_Diplomacy_InvalidBelief)
{
  if (BELIEF_ALLY(bel)) return RELATIONSHIP_ALLY;
  if (BELIEF_TRUCE(bel)) return RELATIONSHIP_TRUCE;
  if (BELIEF_ENEMY(bel)) return RELATIONSHIP_ENEMY;
  if (BELIEF_WAR(bel)) return RELATIONSHIP_WAR;
  THROW(E_8K_AI_Diplomacy_InvalidBelief,"Invalidni hodnota duvery.");
}

void CDiplomacyEngine::changeOfficialRelations(int dipid1,int dipid2)
{
  if ((DiplomatRelations[dipid1][dipid2].rs!=DiplomatRelations[dipid1][dipid2].offeredrs) && (DiplomatRelations[dipid2][dipid1].rs!=DiplomatRelations[dipid2][dipid1].offeredrs)) // oba diplomate neco navrhuji
  {
    if ((DiplomatRelations[dipid1][dipid2].rs>DiplomatRelations[dipid1][dipid2].offeredrs) || (DiplomatRelations[dipid2][dipid1].rs>DiplomatRelations[dipid2][dipid1].offeredrs)) // alespon jeden z diplomatu navrhuje horsi vztah
    {
      if (DiplomatRelations[dipid1][dipid2].offeredrs<DiplomatRelations[dipid2][dipid1].offeredrs)
      {
        if (DiplomatRelations[dipid1][dipid2].relationship_properties.offer_seen_by_enemy)
        {
          DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
          DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
        }
      }
      else
      if (DiplomatRelations[dipid1][dipid2].offeredrs>DiplomatRelations[dipid2][dipid1].offeredrs)
      {
        if (DiplomatRelations[dipid2][dipid1].relationship_properties.offer_seen_by_enemy)
        {
          DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
          DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
        }
      }
      else // oba diplomate navrhuji zhorseni na stejny stav
      {
        if ((DiplomatRelations[dipid1][dipid2].relationship_properties.offer_seen_by_enemy) || (DiplomatRelations[dipid2][dipid1].relationship_properties.offer_seen_by_enemy))
        {
          DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
          DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
        }
      }      
    }
    else // oba diplomate navrhuji lepsi vztah
    {
      if (DiplomatRelations[dipid1][dipid2].offeredrs<DiplomatRelations[dipid2][dipid1].offeredrs)
      {
//        if (DiplomatRelations[dipid1][dipid2].relationship_properties.offer_seen_by_enemy)
//        {
          DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
          DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
//        }
      }
      else
      if (DiplomatRelations[dipid1][dipid2].offeredrs>DiplomatRelations[dipid2][dipid1].offeredrs)
      {
//        if (DiplomatRelations[dipid2][dipid1].relationship_properties.offer_seen_by_enemy)
//        {
          DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
          DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
//        }
      }
      else // oba diplomate navrhuji zlepseni na stejny stav
      {
//        if ((DiplomatRelations[dipid1][dipid2].relationship_properties.offer_seen_by_enemy) && (DiplomatRelations[dipid2][dipid1].relationship_properties.offer_seen_by_enemy))
//        {
          DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
          DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
//        }
      }
    }
  }
  else
  if (DiplomatRelations[dipid1][dipid2].rs>DiplomatRelations[dipid1][dipid2].offeredrs) // diplomat 1 navrhuje diplomatovi 2 horsi vztah
  {
    if (DiplomatRelations[dipid1][dipid2].relationship_properties.offer_seen_by_enemy)
    {
      DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
      DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid1][dipid2].offeredrs;
      DiplomatRelations[dipid2][dipid1].offeredrs=DiplomatRelations[dipid1][dipid2].offeredrs;
    }
  }
  else
  if (DiplomatRelations[dipid2][dipid1].rs>DiplomatRelations[dipid2][dipid1].offeredrs) // diplomat 2 navrhuje diplomatovi 1 horsi vztah
  {
    if (DiplomatRelations[dipid2][dipid1].relationship_properties.offer_seen_by_enemy)
    {
      DiplomatRelations[dipid2][dipid1].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
      DiplomatRelations[dipid1][dipid2].rs=DiplomatRelations[dipid2][dipid1].offeredrs;
      DiplomatRelations[dipid1][dipid2].offeredrs=DiplomatRelations[dipid2][dipid1].offeredrs;
    }
  }
}


TPacket_DIPLOMACY_CHANGE* CDiplomacyEngine::playerTurnBegin(int world_player_ID) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  int dipid=findDiplomatIDByWorldID(world_player_ID);
  
  int i;
  for(i=0;i<MAX_PLAYERS;i++)
  {
    if ((i!=dipid) && (DiplomatRecords[i].slotused))
    {
      DiplomatRelations[i][dipid].relationship_properties.offer_seen_by_enemy=true;
    }    
  }

  // synchronizace
  TPacket_DIPLOMACY_CHANGE* sync_changes=0;
  int u,v;
  for(u=0;u<MAX_PLAYERS;u++)
  {
    for(v=0;v<MAX_PLAYERS;v++)
    {
      if ((u!=v) && ((DiplomatRecords[u].slotused) && (DiplomatRecords[v].slotused)))
      {
        if ((DiplomatRelations[u][v].rs!=DiplomatRecords[dipid].SyncedRelations[u][v].rs) || (DiplomatRelations[u][v].offeredrs!=DiplomatRecords[dipid].SyncedRelations[u][v].offeredrs))
        {
          if (!sync_changes) sync_changes=new TPacket_DIPLOMACY_CHANGE();
          TDipChange tdch;
          tdch.source_diplomat_world_id=DiplomatRecords[u].world_player_id;
          tdch.target_diplomat_world_id=DiplomatRecords[v].world_player_id;
          tdch.old_relationship=DiplomatRecords[dipid].SyncedRelations[u][v];
          tdch.new_relationship=DiplomatRelations[u][v];
          tdch.source_diplomat_penalty=0;
          tdch.target_diplomat_penalty=0;
          DiplomatRecords[dipid].SyncedRelations[u][v]=DiplomatRelations[u][v];
          (sync_changes->changes).push_back(tdch);
        }
      }
    }
  }
  return sync_changes;
}

void CDiplomacyEngine::playerTurnEnd(int world_player_ID) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  int dipid=findDiplomatIDByWorldID(world_player_ID);

  if (DiplomatRecords[dipid].dt==DT_AI) // AI hrac
  {
    int i;
    for(i=0;i<MAX_PLAYERS;i++)
    {
      if ((i!=dipid) && (DiplomatRecords[i].slotused))
      {
        // FOR SOME REASON belief can grow over MAXIMAL_BELIEF (tho it should not) - btw. the highest interval <ALLY_TRUCE_BORDER,MAXIMAL_BELIEF> represents alliance
        if (DiplomatRelations[dipid][i].relationship_properties.guess_belief>MAXIMAL_BELIEF)
          DiplomatRelations[dipid][i].relationship_properties.guess_belief=MAXIMAL_BELIEF;
        if (DiplomatRelations[dipid][i].relationship_properties.belief>MAXIMAL_BELIEF)
          DiplomatRelations[dipid][i].relationship_properties.belief=MAXIMAL_BELIEF;
        TRelationship believedrs=getBeliefRelationship(DiplomatRelations[dipid][i].relationship_properties.belief);
        if (DiplomatRelations[dipid][i].rs!=believedrs)
        {
          setOfferRelationshipByIDs(believedrs,dipid,i);
        }
      }    
    }
  }
  
  // zmeny oficialnich vztahu dle navrzenych vztahu
  int i;
  for(i=0;i<MAX_PLAYERS;i++)
  {
    if ((i!=dipid) && (DiplomatRecords[i].slotused))
    {
      changeOfficialRelations(dipid,i);
    }    
  }

  // synchronizace
  int u,v;
  for(u=0;u<MAX_PLAYERS;u++)
  {
    for(v=0;v<MAX_PLAYERS;v++)
    {
      if ((u!=v) && ((DiplomatRecords[u].slotused) && (DiplomatRecords[v].slotused)))
      {
        if ((DiplomatRelations[u][v].rs!=DiplomatRecords[dipid].SyncedRelations[u][v].rs) || (DiplomatRelations[u][v].offeredrs!=DiplomatRecords[dipid].SyncedRelations[u][v].offeredrs))
        {
          DiplomatRecords[dipid].SyncedRelations[u][v]=DiplomatRelations[u][v];
        }
      }
    }
  }

}

void CDiplomacyEngine::syncDiplomats()
{
  for (int k=0;k<MAX_PLAYERS;k++)
  {
    if (DiplomatRecords[k].slotused)
    {
      for (int i=0;i<MAX_PLAYERS;i++)
      {
        for (int j=0;j<MAX_PLAYERS;j++)
        {
          DiplomatRecords[k].SyncedRelations[i][j]=DiplomatRelations[i][j];
        }
      }
    }
  }
}

int CDiplomacyEngine::findWorstEnemyByWorldIDs(int world_player_ID,std::list<int>* excluded_diplomats) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  srand((unsigned) (time(NULL)));
  int dipid=findDiplomatIDByWorldID(world_player_ID);
  int bid=-1;
  int worst_belief=MAXIMAL_BELIEF;
  
  for (int k=0;k<MAX_PLAYERS;k++)
  {
    if ((k!=dipid) && (DiplomatRecords[k].slotused))
    {
      if (excluded_diplomats)
      {
        std::list<int>::iterator dwIDIt;
        bool not_in_excluded=true;
        for(dwIDIt=excluded_diplomats->begin();((not_in_excluded) && (dwIDIt!=excluded_diplomats->end()));dwIDIt++)
        {
          if ((*dwIDIt)==DiplomatRecords[k].world_player_id) not_in_excluded=false;
        }
        if (not_in_excluded)
        {
          if (DiplomatRelations[dipid][k].relationship_properties.belief==worst_belief)
          {
            // pokud je belief stejny jako nejhorsi, je na nahode, zdali bude bran tento diplomat jako nejhorsi
            if ((rand())%2)
            {
              worst_belief=DiplomatRelations[dipid][k].relationship_properties.belief;
              bid=DiplomatRecords[k].world_player_id;
            }
          }
          else
          if (DiplomatRelations[dipid][k].relationship_properties.belief<worst_belief)
          {
            worst_belief=DiplomatRelations[dipid][k].relationship_properties.belief;
            bid=DiplomatRecords[k].world_player_id;
          }
        }
      }
    }
  }
  return bid;
}

int CDiplomacyEngine::countPenalty(TRelationship old_relationship,TRelationship new_relationship)
{
  if (old_relationship<=new_relationship)
    return 0;

  switch (old_relationship)
  {
    case RELATIONSHIP_ALLY:
      {
        switch (new_relationship)
        {
          case RELATIONSHIP_TRUCE:return (Coeficients.coefBetrayalAllyToTruce);
          case RELATIONSHIP_ENEMY:return (Coeficients.coefBetrayalAllyToEnemy);
          case RELATIONSHIP_WAR:return (Coeficients.coefBetrayalAllyToWar);
          default:return 0;
        }
      }
      break;
    case RELATIONSHIP_TRUCE:
      {
        switch (new_relationship)
        {
          case RELATIONSHIP_ENEMY:return (Coeficients.coefBetrayalTruceToEnemy);
          case RELATIONSHIP_WAR:return (Coeficients.coefBetrayalTruceToWar);
          default:return 0;
        }
      }
      break;
    case RELATIONSHIP_ENEMY:
      {
        if (new_relationship==RELATIONSHIP_WAR)
          return (Coeficients.coefBetrayalEnemyToWar);
        else
          return 0;
      }
      break;
    default:return 0;   
  }
  return 0;
}

void CDiplomacyEngine::AIWarDeclaration(int ai_diplomat_world_id,int enemy_diplomat_of_ai_world_id) throw(E_8K_AI_Diplomacy_NotAIDiplomat,E_8K_AI_Diplomacy_DiplomatNotFound)
{
  int ai_dipid=findDiplomatIDByWorldID(ai_diplomat_world_id);
  int enemy_dipid=findDiplomatIDByWorldID(enemy_diplomat_of_ai_world_id);

  if (DiplomatRecords[ai_dipid].dt!=DT_AI) THROW(E_8K_AI_Diplomacy_NotAIDiplomat,"Diplomat neni pozadovany umely hrac.");

  DiplomatRelations[ai_dipid][enemy_dipid].relationship_properties.belief=MIN(DEFAULT_BELIEF_FOR_WAR,DiplomatRelations[ai_dipid][enemy_dipid].relationship_properties.belief);
  DiplomatRelations[enemy_dipid][ai_dipid].relationship_properties.guess_belief=MIN(DEFAULT_BELIEF_FOR_WAR,DiplomatRelations[enemy_dipid][ai_dipid].relationship_properties.guess_belief);
}

void CDiplomacyEngine::changeDiplomatType(int diplomat_world_id,TDiplomatType new_diplomat_type) throw(E_8K_AI_Diplomacy_DiplomatNotFound)
{
  int dipid=findDiplomatIDByWorldID(diplomat_world_id);
  DiplomatRecords[dipid].dt=new_diplomat_type;
}

}
}
