/**
  @file /ai/Strategizer/strategytransciever.cpp
  @brief Zdrojovy kod s deklaraci transcieveru pro Diplomacii.
  
  Zdrojovy kod s deklaraci transcieveru pro Diplomacii.
  
  @author PZ
  @version 0.1
*/

#include "ai/Diplomacy/diplomacytransciever.h"
namespace ai_ns
{
namespace diplomacy_ns
{

CDiplomacyTransciever* DipEngine;

CDiplomacyTransciever::CDiplomacyTransciever()
{
  DipEngine=this;
  // inicializace transcieveru
  initTransciever();
}

void CDiplomacyTransciever::initTransciever()
{
  diplomacy_transceiver.fnc=DIPLOMACY_MSG_HANDLER;
  diplomacy_transceiver.iActived=true;
  diplomacy_transceiver.iPriority=5;
  diplomacy_transceiver.PrimaryID=MOD_DIPLOMACY;
  KRegisterGlobalTransceiver(&diplomacy_transceiver, MOD_DIPLOMACY);
}

RVAL DIPLOMACY_MSG_HANDLER(MESSAGE_ID mid,SENDER s,PARAM p)
{
//  GLOBALLOG("Diplomacy: Message Id: %d; Sender %d; Param %p\n", mid, s, &p);

  switch (mid)
  {
    case MSG_DIPLOMACY_INIT:DipEngine->initDiplomacy();
                            break;

    case MSG_DIPLOMACY_ADD_DIPLOMAT:DipEngine->addDiplomat(((TPacket_Diplomacy_AddDiplomat*) p)->diplomat_world_id,((TPacket_Diplomacy_AddDiplomat*) p)->diplomat_type);
                                    break;

    case MSG_DIPLOMACY_REMOVE_DIPLOMAT:DipEngine->removeDiplomat(((TPacket_Diplomacy_RemoveDiplomat*) p)->diplomat_world_id);
                                       break;

    case MSG_DIPLOMACY_SYNC:DipEngine->syncDiplomats();
                            break;

    case MSG_DIPLOMACY_GET_DIPLOMAT_COUNT:{
                                            int* rv=(int*) KMemAlloc (sizeof(int));
                                            *rv=DipEngine->getDiplomatsNumber();
                                            return rv;
                                          }
                                          break;

    case MSG_DIPLOMACY_CONQUER_BUILDING:{
                                          ai_ns::CAIScriptCaller* sc=new ai_ns::CAIScriptCaller();
                                          ai_ns::CTCLAIVar* varsin=new ai_ns::CTCLAIVar();

                                          varsin->addInt("DIPLOMATIC_RELATION",((int) (DipEngine->getRelationshipByWorldIDs(((TPacket_Diplomacy_ConquerBuilding*) p)->attacking_diplomat_world_id,((TPacket_Diplomacy_ConquerBuilding*) p)->attacked_diplomat_world_id)))); // 0 - War,1 - Enemy,2 - Truce, 3 - Ally
                                          varsin->addInt("BUILDING_TYPE",((TPacket_Diplomacy_ConquerBuilding*) p)->building_type);
                                          varsin->addInt("BUILDING_COST",((TPacket_Diplomacy_ConquerBuilding*) p)->building_cost);
                                          ai_ns::CTCLAIVar* varsout=sc->countScript(ID_SCRIPT_DIPLOMACY_COUNQUER_BUILDING_WEIGHT,varsin);
                                          
                                          TEvent event;
                                          try
                                          {
                                            event.akter_id=DipEngine->findDiplomatIDByWorldID(((TPacket_Diplomacy_ConquerBuilding*) p)->attacking_diplomat_world_id);
                                            event.receiver_id=DipEngine->findDiplomatIDByWorldID(((TPacket_Diplomacy_ConquerBuilding*) p)->attacked_diplomat_world_id);
                                          }
                                          catch (E_8K_AI_Diplomacy_DiplomatNotFound e)
                                          {
                                            GLOBALLOGID(PRIORITY_FATAL, "Diplomacy Transciever: zpracovani MSG_DIPLOMACY_CONQUER_BUILDING - nenalezen diplomat.");
                                            THROW(E_8K_AI,"Fatalni chyba v 8K AI modulu. Viz zapis v logovacim souboru.");
                                          }
                                          event.done_event=ACTION_TYPE_CONQUER_BUILDING;
                                          event.event_weight=varsout->getInt("RESULT_WEIGHT");

                                          delete varsin;
                                          delete varsout;
                                          delete sc;
                                          DipEngine->calculateEvent(event);
                                        }
                                        break;

    case MSG_DIPLOMACY_CONQUER_CITY:{
                                      ai_ns::CAIScriptCaller* sc=new ai_ns::CAIScriptCaller();
                                      ai_ns::CTCLAIVar* varsin=new ai_ns::CTCLAIVar();

                                      varsin->addInt("DIPLOMATIC_RELATION",((int) (DipEngine->getRelationshipByWorldIDs(((TPacket_Diplomacy_ConquerCity*) p)->attacking_diplomat_world_id,((TPacket_Diplomacy_ConquerCity*) p)->attacked_diplomat_world_id)))); // 0 - War,1 - Enemy,2 - Truce, 3 - Ally
                                      varsin->addInt("CITY_PAYMENT",((TPacket_Diplomacy_ConquerCity*) p)->payment);

                                      ai_ns::CTCLAIVar* varsout=sc->countScript(ID_SCRIPT_DIPLOMACY_COUNQUER_CITY_WEIGHT,varsin);
                                      
                                      TEvent event;
                                      try
                                      {
                                        event.akter_id=DipEngine->findDiplomatIDByWorldID(((TPacket_Diplomacy_ConquerCity*) p)->attacking_diplomat_world_id);
                                        event.receiver_id=DipEngine->findDiplomatIDByWorldID(((TPacket_Diplomacy_ConquerCity*) p)->attacked_diplomat_world_id);
                                      }
                                      catch (E_8K_AI_Diplomacy_DiplomatNotFound e)
                                      {
                                        GLOBALLOGID(PRIORITY_FATAL, "Diplomacy Transciever: zpracovani MSG_DIPLOMACY_CONQUER_CITY - nenalezen diplomat.");
                                        THROW(E_8K_AI,"Fatalni chyba v 8K AI modulu. Viz zapis v logovacim souboru.");
                                      }
                                      event.done_event=ACTION_TYPE_CONQUER_CITY;
                                      event.event_weight=varsout->getInt("RESULT_WEIGHT");

                                      delete varsin;
                                      delete varsout;
                                      delete sc;
                                      DipEngine->calculateEvent(event);
                                    }
                                    break;

    case MSG_DIPLOMACY_ATTACK_INFO:{
                                    ai_ns::CAIScriptCaller* sc=new ai_ns::CAIScriptCaller();
                                    ai_ns::CTCLAIVar* varsin=new ai_ns::CTCLAIVar();

                                    varsin->addInt("DIPLOMATIC_RELATION",((int) (DipEngine->getRelationshipByWorldIDs(((TPacket_Diplomacy_AttackInfo*) p)->attacking_diplomat_world_id,((TPacket_Diplomacy_AttackInfo*) p)->attacked_diplomat_world_id)))); // 0 - War,1 - Enemy,2 - Truce, 3 - Ally
                                    varsin->addInt("ATTACKING_UNIT_TYPE",((TPacket_Diplomacy_AttackInfo*) p)->attacking_unit_type);
                                    varsin->addInt("ATTACKING_UNIT_LEVEL",((TPacket_Diplomacy_AttackInfo*) p)->attacking_unit_level);
                                    varsin->addInt("ATTACKING_UNIT_MEN_NUMBER_FULL",((TPacket_Diplomacy_AttackInfo*) p)->attacking_unit_men_number_full);
                                    varsin->addInt("ATTACKING_UNIT_MEN_NUMBER_BEFORE",((TPacket_Diplomacy_AttackInfo*) p)->attacking_unit_men_number_before);
                                    varsin->addInt("ATTACKING_UNIT_MEN_NUMBER_AFTER",((TPacket_Diplomacy_AttackInfo*) p)->attacking_unit_men_number_after);
                                    varsin->addInt("ATTACKED_UNIT_TYPE",((TPacket_Diplomacy_AttackInfo*) p)->attacked_unit_type);
                                    varsin->addInt("ATTACKED_UNIT_LEVEL",((TPacket_Diplomacy_AttackInfo*) p)->attacked_unit_level);
                                    varsin->addInt("ATTACKED_UNIT_MEN_NUMBER_FULL",((TPacket_Diplomacy_AttackInfo*) p)->attacked_unit_men_number_full);
                                    varsin->addInt("ATTACKED_UNIT_MEN_NUMBER_BEFORE",((TPacket_Diplomacy_AttackInfo*) p)->attacked_unit_men_number_before);
                                    varsin->addInt("ATTACKED_UNIT_MEN_NUMBER_AFTER",((TPacket_Diplomacy_AttackInfo*) p)->attacked_unit_men_number_after);
                                    varsin->addInt("ATTACKED_BUILDING_TYPE",((TPacket_Diplomacy_AttackInfo*) p)->attacked_building_type);
                                    varsin->addInt("ATTACKED_BUILDING_LIVES_FULL",((TPacket_Diplomacy_AttackInfo*) p)->attacked_building_lives_full);
                                    varsin->addInt("ATTACKED_BUILDING_LIVES_BEFORE",((TPacket_Diplomacy_AttackInfo*) p)->attacked_building_lives_before);
                                    varsin->addInt("ATTACKED_BUILDING_LIVES_AFTER",((TPacket_Diplomacy_AttackInfo*) p)->attacked_building_lives_after);
                                    varsin->addInt("ATTACKING_UNIT_COST",((TPacket_Diplomacy_AttackInfo*) p)->attacking_unit_cost);
                                    varsin->addInt("ATTACKED_UNIT_COST",((TPacket_Diplomacy_AttackInfo*) p)->attacked_unit_cost);
                                    varsin->addInt("ATTACKED_BUILDING_COST",((TPacket_Diplomacy_AttackInfo*) p)->attacked_building_cost);
                                    if (((TPacket_Diplomacy_AttackInfo*) p)->attack_on_city)
                                      varsin->addInt("ATTACK_ON_CITY",1);
                                    else
                                      varsin->addInt("ATTACK_ON_CITY",0);

                                    ai_ns::CTCLAIVar* varsout=sc->countScript(ID_SCRIPT_DIPLOMACY_ATTACK_WEIGHT,varsin);
                                    
                                    TEvent event;
                                    try
                                    {
                                      event.akter_id=DipEngine->findDiplomatIDByWorldID(((TPacket_Diplomacy_AttackInfo*) p)->attacking_diplomat_world_id);
                                      event.receiver_id=DipEngine->findDiplomatIDByWorldID(((TPacket_Diplomacy_AttackInfo*) p)->attacked_diplomat_world_id);
                                    }
                                    catch (E_8K_AI_Diplomacy_DiplomatNotFound e)
                                    {
                                      GLOBALLOGID(PRIORITY_FATAL, "Diplomacy Transciever: zpracovani MSG_DIPLOMACY_ATTACK_INFO - nenalezen diplomat.");
                                      THROW(E_8K_AI,"Fatalni chyba v 8K AI modulu. Viz zapis v logovacim souboru.");
                                    }
                                    
                                    if (((TPacket_Diplomacy_AttackInfo*) p)->attacked_building_type==-1)
                                    {
                                      if (((TPacket_Diplomacy_AttackInfo*) p)->attack_on_city)
                                        event.done_event=ACTION_TYPE_ATTACK_CITY;
                                      else
                                        event.done_event=ACTION_TYPE_ATTACK;
                                    }
                                    else
                                    {
                                      event.done_event=ACTION_TYPE_ATTACK_BUILDING;
                                    }
                                    event.event_weight=varsout->getInt("RESULT_WEIGHT");

                                    delete varsin;
                                    delete varsout;
                                    delete sc;
                                    DipEngine->calculateEvent(event);
                                   }
                                   break;

    case MSG_DIPLOMACY_GET_RELATION:{
                                      TRelationship* ret_rel=(TRelationship*) KMemAlloc(sizeof(TRelationship));
                                      // osetrit neutralniho hrace
                                      if ((((TPacket_Diplomacy_GetRelation*) p)->source_diplomat_world_id==0) || (((TPacket_Diplomacy_GetRelation*) p)->target_diplomat_world_id==0))
                                      {
                                        *ret_rel=RELATIONSHIP_WAR;
                                      }
                                      else
                                      {
                                        *ret_rel=DipEngine->getRelationshipByWorldIDs(((TPacket_Diplomacy_GetRelation*) p)->source_diplomat_world_id,((TPacket_Diplomacy_GetRelation*) p)->target_diplomat_world_id);
                                      }
                                      return ret_rel;
                                    }
                                    break;

    case MSG_DIPLOMACY_SET_RELATION:DipEngine->setRelationshipByWorldIDs(((TPacket_Diplomacy_SetRelation*) p)->set_relationship,((TPacket_Diplomacy_SetRelation*) p)->source_diplomat_world_id,((TPacket_Diplomacy_SetRelation*) p)->target_diplomat_world_id);
                                    break;

    case MSG_DIPLOMACY_GET_OFFER_RELATION:{
                                            TRelationship* ret_rel=(TRelationship*) KMemAlloc(sizeof(TRelationship));
                                            *ret_rel=DipEngine->getOfferRelationshipByWorldIDs(((TPacket_Diplomacy_GetOfferRelation*) p)->source_diplomat_world_id,((TPacket_Diplomacy_GetOfferRelation*) p)->target_diplomat_world_id);
                                            return ret_rel;
                                          }
                                          break;

    case MSG_DIPLOMACY_SET_OFFER_RELATION:{
                                            TPacket_DIPLOMACY_CHANGE* sync_offered_relation=DipEngine->setOfferRelationshipByWorldIDs(((TPacket_Diplomacy_SetOfferRelation*) p)->offered_relationship,((TPacket_Diplomacy_SetOfferRelation*) p)->source_diplomat_world_id,((TPacket_Diplomacy_SetOfferRelation*) p)->target_diplomat_world_id);
                                            return sync_offered_relation;
                                          }
                                          break;

    case MSG_DIPLOMACY_GET_OTHER_RELATION_PROPERTIES:{
                                                       TRelationshipProperties* ret_rel_prop=(TRelationshipProperties*) KMemAlloc(sizeof(TRelationshipProperties));
                                                       *ret_rel_prop=DipEngine->getOtherRelationshipPropertiesByWorldIDs(((TPacket_Diplomacy_GetOtherRelationProperties*) p)->source_diplomat_world_id,((TPacket_Diplomacy_GetOtherRelationProperties*) p)->target_diplomat_world_id);
                                                       return ret_rel_prop;
                                                     }
                                                     break;

    case MSG_DIPLOMACY_SET_OTHER_RELATION_PROPERTIES:DipEngine->setOtherRelationshipPropertiesByWorldIDs(((TPacket_Diplomacy_SetOtherRelationProperties*) p)->relationship_properties,((TPacket_Diplomacy_SetOtherRelationProperties*) p)->source_diplomat_world_id,((TPacket_Diplomacy_SetOtherRelationProperties*) p)->target_diplomat_world_id);
                                                     break;

    case MSG_DIPLOMACY_FIND_WORST_ENEMY:{
                                          int worst_enemy=DipEngine->findWorstEnemyByWorldIDs(((TPacket_Diplomacy_FindWorstEnemy*) p)->diplomat_world_id,((TPacket_Diplomacy_FindWorstEnemy*) p)->excluded_diplomats);
                                          if (worst_enemy!=-1)
                                          {
                                            int* rv=(int*) KMemAlloc(sizeof(int));
                                            *rv=worst_enemy;
                                            return rv;
                                          }
                                          else
                                            return ((int*) 0);
                                        }
                                        break;

    case MSG_DIPLOMACY_AI_WAR_DECLARATION:DipEngine->AIWarDeclaration(((TPacket_Diplomacy_AIWarDeclaration*) p)->ai_diplomat_world_id,((TPacket_Diplomacy_AIWarDeclaration*) p)->enemy_diplomat_of_ai_world_id);
                                          break;

    case MSG_DIPLOMACY_CHANGE_DIPLOMAT_TYPE:DipEngine->changeDiplomatType(((TPacket_Diplomacy_ChangeDiplomatType*) p)->diplomat_world_id,((TPacket_Diplomacy_ChangeDiplomatType*) p)->new_diplomat_type);
                                            break;

    case MSG_DIPLOMACY_DIPLOMAT_TURN_BEGIN:{
                                             TPacket_DIPLOMACY_CHANGE* sync_changes=DipEngine->playerTurnBegin(((TPacket_Diplomacy_DiplomatTurnBegin*) p)->diplomat_world_id);
                                             return sync_changes;
                                           }
                                           break;

    case MSG_DIPLOMACY_DIPLOMAT_TURN_END:DipEngine->playerTurnEnd(((TPacket_Diplomacy_DiplomatTurnEnd*) p)->diplomat_world_id);
                                         break;

    default:;//break;
  }
  return 0;
}

}
}
