/**
  @file /ai/Diplomacy/diplomacytransciever.h
  @brief Header s deklaraci transcieveru pro Diplomacii.
  
  Header s deklaraci transcieveru pro Diplomacii.
  
  @author PZ
  @version 0.1
*/

#ifndef __PETR_ZITA_DIPLOMACY_TRANSCIEVER__
#define __PETR_ZITA_DIPLOMACY_TRANSCIEVER__

#include "common/Msg.h"
#include "common/Log.h"
#include "common/Interface.h"
#include "world/world_messages.h"
#include "ai/aiscriptids.h"
#include "ai/aiscriptcaller.h"
#include "ai/Diplomacy/diptypes.h"

namespace ai_ns {
namespace diplomacy_ns {

/** Forward deklarace message handleru. */
RVAL DIPLOMACY_MSG_HANDLER(MESSAGE_ID mid,SENDER s,PARAM p);

/**
  Trida CDiplomacyTransciever obsahuje oddelene cast pro komunikaci s vnejsim svetem.
  Je ciste virtualni, tudiz nemuze byt instanciovana.
*/
class CDiplomacyTransciever
{
public:
  /** Konstruktor. */
  CDiplomacyTransciever();

  /** Metoda inicializujici ci resetujici diplomaticky engine. */
  virtual void initDiplomacy()=0;

  /** Prida diplomata dle jeho typu a World ID do vnitrnich struktur modulu Diplomacie. */
  virtual void addDiplomat(int world_player_ID,TDiplomatType dt)=0;

  /** Odebere diplomata dle jeho World ID ze vnitrnich struktur modulu Diplomacie. */
  virtual void removeDiplomat(int world_player_ID)=0;

  /** Metoda synchonizujici vsechny zaznamy diplomatu s aktualnim stavem. */
  virtual void syncDiplomats()=0;

  /** Metoda zapocita danou udalost do diplomatickych vztahu. */
  virtual void calculateEvent(TEvent & new_event)=0;

  /** Metoda vrati pocet diplomatu, jez Diplomacie prave zpracovava. */
  virtual int getDiplomatsNumber()=0;

  /** Metoda vraci aktualni diplomaticky vztah mezi dvema diplomaty dle jejich World ID. */
  virtual TRelationship getRelationshipByWorldIDs(int dip1,int dip2)=0;

  /** Metoda nastavuje aktualni diplomaticky vztah mezi dvema diplomaty dle jejich World ID. */
  virtual void setRelationshipByWorldIDs(TRelationship relshp,int dip1,int dip2)=0;

  /** Metoda vraci aktualni navrhovany diplomaticky vztah jednim diplomatem druhemu dle jejich World ID. */
  virtual TRelationship getOfferRelationshipByWorldIDs(int dip1,int dip2)=0;

  /**
    Metoda nastavuje aktualni navrhovany diplomaticky vztah jednim diplomatem druhemu dle jejich World ID.
    Zaroven vraci synchronizacni informace.
    @return TPacket_DIPLOMACY_CHANGE*  paket se synchronizacnimi informacemi
  */
  virtual TPacket_DIPLOMACY_CHANGE* setOfferRelationshipByWorldIDs(TRelationship relshp,int dip1,int dip2)=0;

  /** Metoda vraci aktualni vlastnosti diplomatickeho vztahu jednoho diplomata k druhemu dle jejich World ID. */
  virtual TRelationshipProperties getOtherRelationshipPropertiesByWorldIDs(int dip1,int dip2)=0;

  /** Metoda nastavuje aktualni vlastnosti diplomatickeho vztahu jednoho diplomata k druhemu dle jejich World ID. */
  virtual void setOtherRelationshipPropertiesByWorldIDs(TRelationshipProperties & relshp_prop,int dip1,int dip2)=0;

  /** Metoda nalezne ID diplomata v modulu Diplomacie dle jeho World ID. */
  virtual int findDiplomatIDByWorldID(int world_player_ID)=0;

  /** Metoda zpusobi pro AI hrace vyhlaseni valky nejhorsimu nepriteli. */
  virtual void AIWarDeclaration(int ai_diplomat_world_id,int enemy_diplomat_of_ai_world_id)=0;

  /** Metoda zmeni typ diplomata (AI/clovek). */
  virtual void changeDiplomatType(int diplomat_world_id,TDiplomatType new_diplomat_type)=0;

  /**
    Metoda nalezne nejhorsiho nepritele pro daneho diplomata. 
    @param world_player_ID        World ID diplomata
    @param excluded_diplomats     seznam World ID diplomatu, kteri maji byt z hledani vynechani
    @return int                   World ID nejvetsiho nepritele
  */
  virtual int findWorstEnemyByWorldIDs(int world_player_ID,std::list<int>* excluded_diplomats)=0;

  /**
    Metoda provadi synchronizaci na zacatku kola pro daneho diplomata.
    Vraci synchronizacni informace.
    @return TPacket_DIPLOMACY_CHANGE*  paket se synchronizacnimi informacemi
  */
  virtual TPacket_DIPLOMACY_CHANGE* playerTurnBegin(int world_player_ID)=0;

  /** Metoda provadi diplomaticke akce na konci kola pro daneho diplomata. */
  virtual void playerTurnEnd(int world_player_ID)=0;

protected:
  /** Definice transcieveru pro komunikaci. */
  TMessageTransceiver diplomacy_transceiver;
  /** Metoda pro defaultni inicializaci a registraci tranceiveru v globalni fronte. */
  void initTransciever();

private:
  /** Zpratelena funkce transceiveru obsluhujici prichozi zpravy. */
  friend RVAL DIPLOMACY_MSG_HANDLER(MESSAGE_ID mid,SENDER s,PARAM p);
};

/** Konkretni instance tridy transceiveru pristupna pro funkci transceiveru. */
extern CDiplomacyTransciever* DipEngine;

} // end of namespace "diplomacy_ns"
} // end of namespace "ai_ns"

#endif
