/**
  @file /ai/Diplomacy/diptypes.h
  @brief Header modulu s definicemi typu pro diplomacii.
  
  Header modulu s definicemi typu pro diplomacii.
  
  @author PZ
  @version 0.1
*/

#ifndef __PETR_ZITA_DIPLOMACY_TYPES__
#define __PETR_ZITA_DIPLOMACY_TYPES__

namespace ai_ns {
namespace diplomacy_ns {

/** Typ diplomaticke akce. */
typedef enum 
{
  ACTION_TYPE_ATTACK,              ///< utok na jednotku
  ACTION_TYPE_ATTACK_BUILDING,     ///< utok na budovu
  ACTION_TYPE_ATTACK_CITY,         ///< utok na mesto
  ACTION_TYPE_CONQUER_BUILDING,    ///< dobyti budovy
  ACTION_TYPE_CONQUER_CITY         ///< dobyti mesta
} TDipAction;

/** Typ diplomata (AI nebo clovek). */
typedef enum
{
  DT_HUMAN,
  DT_AI
} TDiplomatType;

/** Typ diplomatickeho vztahu mezi hraci */
typedef enum type_relationship
{    
  RELATIONSHIP_WAR=0,   ///< valka
  RELATIONSHIP_ENEMY,   ///< nepratelstvi
  RELATIONSHIP_TRUCE,   ///< primeri
  RELATIONSHIP_ALLY,    ///< spojenectvi
  RELATIONSHIPS_NUMBER  ///< nevyuzivana a zakazana hodnota - slouzi k urceni poctu vztahu
} TRelationship;

/** Dalsi parametry diplomatickeho vztahu. */
typedef struct type_player_relationship_properties
{
  bool offer_seen_by_enemy; ///< byl jiz navrhovany vztah zobrazen druhe strane
  int belief;               ///< duvera druheho diplomata vuci prvnimu diplomatovi (vyuziva AI)
  int guess_belief;         ///< predpokladana duvera prvniho diplomata vuci druhemu diplomatovi (vyuziva AI)
} TRelationshipProperties;

/** Kompletni jednosmerny diplomaticky vztah mezi diplomaty. */
typedef struct type_player_relationship
{
  TRelationship rs;                                 ///< aktualni vztah mezi diplomaty
  TRelationship offeredrs;                          ///< navrhovany vztah prviho diplomata druhym diplomatem
  TRelationshipProperties relationship_properties;  ///< dalsi parametry diplomatickeho vztahu
} TDipRelation;

/** Udaj o zmene vztahu mezi dvema diplomaty. */
typedef struct
{    
  int source_diplomat_world_id;   ///< world ID prvniho diplomata
  int target_diplomat_world_id;   ///< world ID druheho diplomata
  TDipRelation old_relationship;  ///< stary vztah
  TDipRelation new_relationship;  ///< novy vztah
  int source_diplomat_penalty;    ///< financni penale, ktere je strhnuto prvnimu diplomatovi
  int target_diplomat_penalty;    ///< financni penale, ktere je strhnuto druhemu diplomatovi
} TDipChange;

/** Datovy typ o evidenci udalosti relevantni pro modul Diplomacie. */
typedef struct
{
  int akter_id;            ///< diplomaticke ID prvniho diplomata
  int receiver_id;         ///< diplomaticke ID druheho diplomata
  TDipAction done_event;   ///< typ diplomaticke udalosti
  int event_weight;        ///< vaha diplomaticke udalosti
} TEvent;

/** Datovy typ s informacemi o diplomatovi. */
typedef struct
{
  int world_player_id;                                      ///< world ID diplomata
  TDiplomatType dt;                                         ///< typ diplomata
  bool slotused;                                            ///< priznak aktivity diplomata
  TDipRelation SyncedRelations[MAX_PLAYERS][MAX_PLAYERS];   ///< diplomaticke relace, ktere diplomat videl pri ukonceni posledniho kola
} TDipInfo;

/** Datovy typ pro diplomaticke koeficienty nacitane z TCL skriptu. */
typedef struct
{
  float coefActionAttack;
  float coefActionAttackOnAlly;
  float coefActionAttackOnTruce;
  float coefActionAttackOnEnemy;
  float coefActionAttackOnWar;
  float coefActionConquerBuilding;                    
  float coefActionConquerBuildingOfAlly;               
  float coefActionConquerBuildingOfTruce;
  float coefActionConquerBuildingOfEnemy;              
  float coefActionConquerBuildingOfWar;
  float coefActionConquerCity;                    
  float coefActionConquerCityOfAlly;               
  float coefActionConquerCityOfTruce;
  float coefActionConquerCityOfEnemy;              
  float coefActionConquerCityOfWar;
  int coefBetrayalAllyToTruce;
  int coefBetrayalAllyToEnemy;
  int coefBetrayalAllyToWar;
  int coefBetrayalTruceToEnemy;
  int coefBetrayalTruceToWar;
  int coefBetrayalEnemyToWar;
} TCoefsRecord;

} // end of namespace "diplomacy_ns"
} // end of namespace "ai_ns"

#endif

