/**
  @file /ai/Strategizer/expansion.h
  @brief Header modulu expanze AI.
  
  Header modulu expanze AI.
  
  @author PZ
  @version 0.1
*/

#ifndef __PETR_ZITA_STRATEGY_EXPANSION__
#define __PETR_ZITA_STRATEGY_EXPANSION__

#pragma warning( disable : 4290 )

#include <iostream>
#include <vector>
#include <list>

#include "ai/aiscriptids.h"
#include "ai/aiscriptcaller.h"
#include "ai/Strategizer/planning.h"
#include "ai/Strategizer/strategytypes.h"
#include "ai/Strategizer/strategyserialization.h"
#include "common/Interface.h"
#include "common/exc.h"
#include "common/Msg.h"
#include "common/Log.h"
#include "common/Interface.h"
#include "world/world_messages.h"
#include "world/useful.h"
#include "world/plan/plan.h"
#include "world/buildings/building.h"
#include "world/units/unit.h"

namespace ai_ns {
namespace strategizer_ns {

#ifdef AI_LOGGING
  /** Definice logovaciho souboru s procesy staveb a oprav AI hracu. */
  static TLog LogAIDesires("ai_desires.log");
#endif

/** Datovy typ obsahujici informace o poctech urciteho typu budovy. */
typedef struct type_building_statitics
{
  int build_order;                                              ///< implicitni poradi stavby budovy
  int building_type;                                            ///< typ budovy
  int building_num;                                             ///< pocet vyskytu budovy
  bool operator<(const struct type_building_statitics& tbs);    ///< operator pro trideni statistik budov
} TBuildingStatistic;

/** 
  Trida pro zpravu veskere expanze (staveni, opravy, rekrutace stavitelu, rozdelovani penez) AI hrace.
*/
class CStrategyExpansionEngine
{
  public:
    /**
      Konstruktor.
      @param WorldPointers        sada ukazatelu zpristupnujici veskere struktury WORLDu
      @param SS                   ukazatel na instanci komponenty pro ukladani AI dat
    */
    CStrategyExpansionEngine(TPacket_AI_LetsGo* WorldPointers,CStrategySerialization* SS);

    /** Destruktor. */
    ~CStrategyExpansionEngine();

    float MoneyRatioAttack;         ///< pomer novych penez AI hrace vynalozenych imlicitne na utocnou snahu vuci celkovemu mnozstvi novych penez
    float MoneyRatioDefence;        ///< pomer novych penez AI hrace vynalozenych imlicitne na obranu vuci celkovemu mnozstvi novych penez
    float MoneyRatioExpansion;      ///< pomer novych penez AI hrace vynalozenych imlicitne na expanzi vuci celkovemu mnozstvi novych penez

    /**
      Nastavi aktualniho AI hrace.
      @param passed_player_ID         World ID AI hrace
    */
    void setPlayer(int passed_player_ID);

    /** Provede rekrutaci novych stavitelu, zpracuje procesy staveb a oprav, naplanuje stavbu mosttu a vycvici jednotky pro obranu uzemi a statickych entit AI hrace. */
    void updateExpansion();

    /** Provede rozdeleni aktualnich penez na ucely obrany, utoceni a expanze. */
    void updateDisponibleMoney();

    /** Provede leceni zranenych muzu s jednotkami, jenz tak mohou aktualne ucinit. */
    void processHealingWithUnits();

    /** Provede rekrutaci novych muzu s jednotkami, jenz tak mohou aktualne ucinit. */
    void processRecruitingWithUnits();

    /** Vyjde s nestavicimi ci neopravujicimi jednotkami ze svych budov, aby zde bylo mozne produkovat nove jednotky. */
    void getUnitsOutOfBuildings();

    /** 
      Osetruje zvlastni pripad hry, kdy ma AI hrac k dispozici pouze jednotky urcene pro expanzi (napriklad pouze stavitele).
      throw E_8K_AI_Strategy_CantFindSelectedExpansionUnit
    */
    void processOnlyExpansionUnitsState() throw(E_8K_AI_Strategy_CantFindSelectedExpansionUnit);

    /** 
      Zjisti, zdali AI hrac vlastni nejaka mesta.
      return bool             priznak, zdali AI hrac vlastni nejaka mesta
    */
    bool hasCities();

    /** 
      Zjisti, zdali AI hrac vlastni nejake produkcni budovy.
      return bool             priznak, zdali AI hrac vlastni nejake produkcni budovy
    */
    bool hasProductionBuildings();

    /** 
      Zjisti, zdali ma AI hrac k dispozici nejake jednotky vhodne pro utok.
      return bool             priznak, zdali ma AI hrac k dispozici nejake jednotky vhodne pro utok
    */
    bool hasAttackableUnits();

    /** 
      Zjisti, zdali ma AI hrac nejakeho stavitele.
      return bool             zdali ma AI hrac nejakeho stavitele
    */
    bool hasBuilder();

  private:
    TPacket_AI_LetsGo* SIP;                 ///< sada ukazatelu zpristupnujici veskere struktury WORLDu.
    int playerID;                           ///< World ID AI hrace
    CStrategySerialization* SS;             ///< ukazatel na instanci komponenty pro ukladani AI dat

    int SecondBuilderBorder;                ///< koeficient udavajici hranici poctu produkcnich budov, pri ktere se vyplati stavet druheho stavitele
    int BuildingsPerBuilder;                ///< koeficient udavajici doporuceny pocet produkcnich budov na stavitele
    int MaxProductionBuildingsPerKingdom;   ///< koeficient udavajici maximalni pocet produkcnich budov na kralovstvi
    char* BuildOrder;                       ///< retezec s informacemi o implicitnim poradi staveni budov

    std::list<TBuildingStatistic> BuildingTemplateNumbers;    ///< sablona koeficientu pro vyvazeny pocet budov
    std::list<TBuildingStatistic> BuildingStatistics;         ///< zaznamy o poctech budov

    /** Nacte specificke koeficienty pro komponentu expanze z TCL skriptu. */
    void loadExpansionCoefs();

    /** Zpracovava vsechny procesy staveb a oprav pro AI hrace. */
    void processAllBuildsAndRepairs();

    /** 
      Zpracovava predany proces stavby.
      @param build_desire           proces stavby, jenz je urcen k zpracovani
    */
    void processBuild(TAIBuildDesireInfo* build_desire);

    /** 
      Zpracovava predany proces opravy.
      @param repair_desire           proces opravy, jenz je urcen k zpracovani
    */
    void processRepair(TAIRepairDesireInfo* repair_desire);

    /** Naverbuje jednotky v budovach pro ucely braneni sveho uzemi a statickych entit. */
    void recruitDefence();

    /** Naplanuje mozne procesy oprav svych budov. */
    void planRepairs();

    /** 
      Rozhodne, zdali je vhodne stavet novou produkcni budovu.
      throw E_8K_AI_Strategy_InvalidBuildingStatistics
      return bool           priznak, zdali je vhodne stavet novou produkcni budovu
    */
    bool shouldBuildNewBuildings() throw(E_8K_AI_Strategy_InvalidBuildingStatistics);

    /** Naplanuje mozne procesy oprav svych budov. */
    void planNewBuildings();

    /** 
      Nalezne nejblizsiho nezaukolovaneho stavitele od zadane pozice.
      @param hexid          ID hexu pozice, od ktere stavitele vyhledavam
      return int            World ID nejblizsiho volneho stavitele
    */
    int getNearestAccessibleFreeBuilderFromHex(int hexid);

    /** 
      Urci nahodne typ jednotky, jenz bude vyrobena pro ucel obrany uzemi AI hrace a jeho statickych entit.
      return int            typ defenzivni jednotky, jenz bude vyrobena
    */
    int selectToProduceRandomPossibleUnitForDefence();

    /** Spocita aktualni pocty budov a zanese je do vnitrnich struktur. */
    void countBuildingStatistics();

    /** 
      Zjisti implicitni poradi stavby dane budovy.
      @param building_type          typ budovy, pro niz se zjistuje poradi stavby
      throw E_8K_AI_Strategy_InvalidBuildOrder
      return int                    implicitni poradi stavby dane budovy
    */
    int getBuildingOrder(int building_type) throw(E_8K_AI_Strategy_InvalidBuildOrder);

    /** 
      Spocita pocet kralovstvi, ktere ma AI hrac aktualne v moci.
      return int            pocet kralovstvi, ktere ma AI hrac aktualne v moci
    */
    int countMyKingdoms();

    /** 
      Zjisti typ aktualne nejpotrebnejsi produkcni budovy, ktera se bude stavet.
      throw E_8K_AI_Strategy_InvalidBuildingStatistics
      return int                    typ aktualne nejpotrebnejsi produkcni budovy
    */
    int getMostNeededBuilding() throw(E_8K_AI_Strategy_InvalidBuildingStatistics);

    /** 
      Zjisti, zdali je vhodne vycvicit dalsiho stavitele.
      return bool             priznak, zdali je vhodne vycvicit dalsiho stavitele
    */
    bool needToRecruitBuilder();

    /** 
      Naplanuje vycvik stavitele.
      return bool             priznak, zdali byl vycvik stavitele uspesne naplanovan
    */
    bool recruitBuilder();

    /** Naplanuje stavbu mostu, pokud se nejaky vyplati stavet. */
    void buildBridge();

    /** 
      Spocita cenu vsech nezapocatych planovanych procesu staveb.
      return int           celkova cena vsech nezapocatych planovanych procesu staveb
    */
    int countPlannedBuildsPrice();

    /**
      Spocita cenu vsech nezapocatych planovanych procesu oprav.
      return int           celkova cena vsech nezapocatych planovanych procesu oprav
    */
    int countPlannedRepairsPrice();
};

} // end of namespace "strategizer_ns"
} // end of namespace "ai_ns"

#endif

