/**
  @file /ai/Strategizer/strategyserialization.h
  @brief Header tridy ukladani dat pro AI hrace.
  
  Header tridy ukladani dat pro AI hrace.
  
  @author PZ
  @version 0.1
*/

#ifndef __PETR_ZITA_STRATEGY_SERIALIZATION__
#define __PETR_ZITA_STRATEGY_SERIALIZATION__

#pragma warning( disable : 4290 )

#include <iostream>
#include <vector>
#include <list>
#include <string.h>

#include "ai/ai_makra.h"
#include "ai/Strategizer/strategytypes.h"
#include "common/Interface.h"
#include "common/exc.h"
#include "common/types.h"
#include "common/mm.h"
#include "common/Msg.h"
#include "common/Log.h"
#include "common/Interface.h"
#include "world/world_messages.h"
#include "world/useful.h"
#include "world/plan/plan.h"
#include "world/buildings/building.h"
#include "world/units/unit.h"

namespace ai_ns {
namespace strategizer_ns {

/** 
  Trida pro nacitani, ukladani a praci s vnitrni reprezentaci dat AI pro urciteho hrace.
*/
class CStrategySerialization
{
  public:
    /** 
      Konstruktor.
      @param WorldPointers          sada ukazatelu na struktury Worldu
    */
    CStrategySerialization(TPacket_AI_LetsGo* WorldPointers);

    /** Destruktor. */
    ~CStrategySerialization();
    
    /** 
      Metoda vrati zakladni AI informace o jednotce s danym World ID.
      @param unit_world_id        World ID jednotky
      @return                     (TAIUnitInfo*) zakladni AI informace o jednotce s danym World ID
    */
    TAIUnitInfo* findUnitInfoByWorldID(int unit_world_id);
    

    int MoneyAttack;            ///< penize, jenz ma AI hrac pridelen na utok      
    int MoneyDefence;           ///< penize, jenz ma AI hrac pridelen na obranu
    int MoneyExpansion;         ///< penize, jenz ma AI hrac pridelen na expanzi (staveni, rekrutaci, atd.)

    std::UNIT_AI_INFO_CONTAINER<TAIUnitInfo> UnitsInfo;                             ///< kontajner se zaznamy informaci o jednotkach AI hrace
    std::ATTACKS_AI_INFO_CONTAINER<TAIAttackInfo> AttacksInfo;                      ///< kontajner se zaznamy informaci o utocnych snahach AI hrace
    std::BUILD_DESIRES_AI_INFO_CONTAINER<TAIBuildDesireInfo> BuildDesiresInfo;      ///< kontajner se zaznamy informaci o stavbach novych budov AI hrace
    std::REPAIR_DESIRES_AI_INFO_CONTAINER<TAIRepairDesireInfo> RepairDesiresInfo;   ///< kontajner se zaznamy informaci o opravach existujicich budov AI hrace


    /** 
      Metoda nacte veskere informace, jenz si AI pro daneho hrace uklada do vnitrnich struktur.
      @param data                 retezec AI dat zpristupneny Worldem
      @throw                      E_8K_AI_Strategy_AIDataReadingError
    */
    void loadDataToInnerStructure(char* data) throw(E_8K_AI_Strategy_AIDataReadingError);

    /** 
      Metoda ulozi data AI z vnitrnich struktur pro daneho hrace do retezce, jenz je pak predan Worldu a ulozen.
      @return                     (char*) retezec s AI daty
    */
    char* saveDataFromInnerStructure();

    /** 
      Metoda nastavi defaultni nastaveni vnitrnich struktur AI dat.
    */
    void defaultInnerStructureInit();

    /** 
      Metoda vymaze data ve vnitrnich strukturach AI dat.
    */
    void clearInnerStructure();

  private:    
    TPacket_AI_LetsGo* SIP;       ///< sada ukazatelu na struktury Worldu, ktere jsou predany v konstruktoru

    /** 
      Metoda prida cislo na konec retezce predaneho.
      @param added_int                  pridavana ciselna hodnota
      @param buffer                     retezec, ke kteremu cislo pridavam
      @param size                       dosavadni delka retezce
    */
    void addIntValueToString(int added_int,char* buffer,size_t &size);

    /** 
      Metoda prida znak na konec retezce predaneho.
      @param added_int                  pridavany znak
      @param buffer                     retezec, ke kteremu znak pridavam
      @param size                       dosavadni delka retezce
    */
    void addCharToString(char added_char,char* buffer,size_t &size);

    /** 
      Metoda prida cislo na konec retezce predaneho.
      @param data                             zdrojovy retezec, ze ktereho cislo ctu
      @param size                             aktualni pozice v retezci, od ktere cislo ctu
      @param separator                        znak oddelovace, jenz jenz znaci konec cisla
      @param bigstrlen                        celkova delka zdrojoveho retezce
      @throw                                  E_8K_AI_Strategy_AIDataReadingError
      @return                                 nactene cislo
    */
    int readNumWithSeparator(char* data,size_t &size,char separator,size_t bigstrlen) throw(E_8K_AI_Strategy_AIDataReadingError);
};


} // end of namespace "strategizer_ns"
} // end of namespace "ai_ns"

#endif

