/**
  @file /ai/Strategizer/strategytypes.h
  @brief Header s obecnymi typy AI strategickeho modulu.
  
  Header s obecnymi typy AI strategickeho modulu.
  
  @author PZ
  @version 0.1
*/

#ifndef __PETR_ZITA_STRATEGY_TYPES__
#define __PETR_ZITA_STRATEGY_TYPES__

#define UNIT_AI_INFO_CONTAINER vector
#define ATTACKS_AI_INFO_CONTAINER vector
#define ATTACK_SLOTS_CONTAINER vector
#define BUILD_DESIRES_AI_INFO_CONTAINER vector
#define REPAIR_DESIRES_AI_INFO_CONTAINER vector
#define UNIT_PRODUCING_AI_INFO_CONTAINER vector

namespace ai_ns {
namespace strategizer_ns {

/** Datovy typ urcujici stav branici jednotky, respektive zpusob jakym braneni provadi. */
typedef enum
{
  NoDefenceDesire=0,    ///< branici jednotka zatim nema prirazenou cinnost
  DefendInside,         ///< jednotka brani statickou entitu uvnitr
  DefendOutside,        ///< jednotka brani statickou entitu venku
  DefendCharging        ///< jednotka vyrazi na ztec proti utocnikovi
} TDefendingUnitDesire;

/** Datovy typ urcujici globalni ucel jednotky pro AI. */
typedef enum
{
  NoGlobalDesire=0,     ///< jednotka nema prirazenou zadnou cinnost
  Defence,              ///< jednotka je urcena na obranu
  Attack_Unassigned,    ///< jednotka je urcena na utok, ale jeste neni prirazena zadne utocne skupine
  Attack_Assigned,      ///< jednotka je urcena na utok a je jiz prirazena nejake utocne skupine
  Expansion             ///< jednotka je urcena k expanzi
} TUnitGlobalDesire;

/** Datovy typ urcujici prubeh umyslu stavby budovy stavitelem. */
typedef struct type_TAIBuildDesireInfo
{
  bool builder_assigned;      ///< priznak, zdali je stavitel prirazen
  int builder_id;             ///< IDcko prirazeneho stavitele
  int building_type;          ///< typ stavene budovy
  int build_target_point;     ///< misto stavby (respektive misto, ze ktereho bude stavitel stavet)
  int bridge_placement;       ///< misto stavby, pokud se jedna o most
  int building_orientation;   ///< orientace budovy
  bool started_to_build;      ///< priznak zdali jiz zacal stavitel budovu stavet
  bool done;                  ///< priznak dokonceni stavby
} TAIBuildDesireInfo;

/** Datovy typ urcujici prubeh umyslu opraveni budovy stavitelem. */
typedef struct type_TAIRepairDesireInfo
{
  bool builder_assigned;    ///< priznak, zdali je stavitel prirazen
  int builder_id;           ///< IDcko prirazeneho stavitele
  int repair_target_point;  ///< misto poskozene budovy
  bool started_to_repair;   ///< priznak zdali jiz zacal stavitel budovu opravovat
  bool done;                ///< priznak dokonceni opravy
} TAIRepairDesireInfo;

/** Datovy typ urcujici informace o jedne jednotce v utocne skupine. */
typedef struct
{
  bool assigned;              ///< priznak, zdali jiz byla pridelena jednotka
  int needed_unit_type;       ///< typ utocne jendotky
  int assigned_unit_id;       ///< world ID prirazene jednotky
  int unit_position;          ///< aktualni pozice prirazene jednotky
  bool synchronized;          ///< priznak, zdali jiz jednotka dorazila k mistu synchronizace
} TAttackingUnitSlot;

/** Datovy typ urcujici stav utoku. */
typedef enum
{
  CannotBeSynchronized=0,       ///< synchronizacni bod nelze urcit nebo neni dostupny - obsadim vsechny attack sloty a pak prejdu do Charging
  Synchronizating,              ///< jendotky se synchronizuji k synchronizacnimu bodu
  Charging,                     ///< jednotky utoci
  Done                          ///< utok byl dokoncen
} TAttackState;

/** Datovy typ urcujici cil utoku. */
typedef enum
{
  ATCity=0,     ///< cilem utoku je mesto
  ATBuilding,   ///< cilem utoku je budova
  ATUnit        ///< cilem utoku je jednotka
} TAttackTargetType;

/** Datovy typ obsahujici AI informace o utocne skupine a jejim pocinani. */
typedef struct type_TAIAttackInfo
{
  std::ATTACK_SLOTS_CONTAINER<TAttackingUnitSlot> units_for_attack; ///< jednotky zarazene pro utok
  TAttackState attack_state;                                        ///< stav utoku
  int synchronization_point;                                        ///< bod, kde se utocici jednotky synchronizuji
  int target_point;                                                 ///< cil utoku
  int target_unit_id;                                               ///< IDcko cilove jednotky utoku
  TAttackTargetType attack_target_type;                             ///< typ cile utoku
} TAIAttackInfo;

/** Datovy typ obsahujici AI informace o kazde jednotce. */
typedef struct
{
  int world_id;                     ///< world ID jednotky
  int unit_type;                    ///< typ jednotky
  TUnitGlobalDesire global_desire;  ///< prirazeny ucel jednotky
} TAIUnitInfo;

/** Datovy typ obsahujici koeficienty specificke pro Strategicky engine AI. */
typedef struct type_strategy_coefs
{
  float AttackingGroupStrengthRatio;  ///< koeficient udavajici kolikrat silnejsi musi byt sila utocniku nez cizi obrany
  float DefaultAttackIncomeRatio;     ///< defaultni hodnota pomeru rozdeleni celkovych penez na utoceni
  float DefaultDefenceIncomeRatio;    ///< defaultni hodnota pomeru rozdeleni celkovych penez na obranu
  float DefaultExpansionIncomeRatio;  ///< defaultni hodnota pomeru rozdeleni celkovych penez na expanzi
} TStrategyCoefs;

/** Datovy typ pro filtr specifikujici pozadovany cil utoku. */
typedef enum
{
  ATF_Any_Static,       ///< je jedno, zdali se pro utok vyhleda budova ci mesto
  ATF_Building_Only,    ///< pro utok se vyhledava pouze budova
  ATF_City_Only         ///< pro utok se vyhledava pouze mesto
} TAttackTargetFilter;

} // end of namespace "strategizer_ns"
} // end of namespace "ai_ns"

#endif

