/**
  @file /ai/ai.h
  @brief Hlavni header modulu umele inteligence.
  
  Header, ktery inicializuje vsechny moduly umele inteligence, zprostredkovava komunikaci mezi nimi a taktez komunikuje s okolnim svetem pomoci systemu zprav.
  
  @author PZ
  @version 0.1
*/

#ifndef __PETR_ZITA_AI_ENGINE__
#define __PETR_ZITA_AI_ENGINE__

#include "ai/ai_makra.h"
#include "ai/Diplomacy/diplomacy.h"
#include "ai/PathFind/pathfind.h"
#include "ai/Strategizer/strategizer.h"
#include "ai/MapAnalyzer/mapanalyzer.h"

/** Prostor jmen obalujici veskere moduly AI. */
namespace ai_ns {

/** Trida pro AI engine - shromazduje a instancuje vsechny casti AI z jednotlivych modulu. */
class CAIEngine
{
public:
  /** Konstruktor */
  CAIEngine();

  /** Destruktor */
  ~CAIEngine();

private:
  /** Modul Pathfind */
  pathfind_ns::CPathFindEngine* PathFindEngine;

  /** Modul Diplomacie */
  diplomacy_ns::CDiplomacyEngine* DiplomacyEngine;

  /** Modul Strategicky planovac */
  strategizer_ns::CStrategizerEngine* StrategyEngine;

  /** Modul Analyzator mapy */
  mapanalyzer_ns::CMapAnalyzerEngine* MapAnalyzerEngine;
};

/** Konkretni instance CAIEngine, ktera je vytvorena pri spusteni hry. */
extern CAIEngine* AIEngine;

/** Inicializacni funkce volana v 8Kingdoms.cpp. */
void AIInit();

/** Destrukcni funkce volana v 8Kingdoms.cpp. */
void AIDestroy();

} // end of namespace "ai_ns"

#endif

