/** @file /common/TCL/tcl_var.h

	Reprezentace strukturovanych promennych z TCL skriptu, jejich nacitani a ukladani 
    do prostredi interpretu

    @author Petr Wolf
*/

#ifndef COMMON_TCL_TCL_VAR_H
#define COMMON_TCL_TCL_VAR_H

#include <map>
#include <vector>
#include <string>

enum TCL_VAR_TYPE
{
    // zakladni typy
    TVT_INT          ///< cele cislo
   ,TVT_FLOAT        ///< desetinne cislo
   ,TVT_STRING       ///< textovy retezec

    // typ list - spojovy seznam
   ,TVT_INT_LIST     ///< spojovy seznam \c int hodnot
   ,TVT_FLOAT_LIST   ///< spojovy seznam \c float hodnot
   ,TVT_STRING_LIST  ///< spojovy seznam \c string hodnot
   
    // typ array - asociativni pole 
   ,TVT_INT_ARRAY    ///< asociativni pole \c int hodnot
   ,TVT_FLOAT_ARRAY ///< asociativni pole \c float hodnot 
   ,TVT_STRING_ARRAY ///< asociativni pole \c string hodnot

    // typy popisujici objekty ve hre
   ,TVT_UNIT         ///< jednotka, obdoba LIVING_UNIT
   ,TVT_BUILDING     ///< budova, obdoba LIVING_BUILDING
   ,TVT_HEX          ///, hex, obdoba HEX
};

// datove typy pro spojove seznamy
typedef std::vector<int> TTCL_Int_List;
typedef std::vector<double> TTCL_Float_List;
typedef std::vector<std::string> TTCL_String_List;

// datove typy pro asociativni pole
typedef std::map<std::string, int> TTCL_Int_Array;
typedef std::map<std::string, double> TTCL_Float_Array;
typedef std::map<std::string, std::string> TTCL_String_Array;

#endif
/*****************************************************************************/
