/**
******************************************************************************

  @file /common/rm/rmadvxmli.h
  @brief Interface k RMadvxml
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef _RMuniti_
#define _RMuniti_

#include "common/rm/rmadvxml.h"


namespace rm
{

/// Interface k RMadvxml
template <class T>
class TRM_advxml_i
{
  public:
/** @name Nacitani/uvolnovani/ziskani dat */
//@{
    /** Na misto kam ukazuje data priradi odkaz na data z RM. Data se tedy nekopiruji */
    void get(T ** data,int id);

    /** Nacte okamzite data, ve stejnem vlakne */
    void loadNow(int id);

    /** Prida pozadavek na nacteni */
    void request(int id); 

    /** Prida pozadavek na nacteni a zaroven prida pozadaven na nacteni na soubory, na ktery odkazuje */
    void requestDep(int id); 

    /** Zpracuje pozadavky na nacteni */
    void loadBuffer();

    /** Uvolni nactene data z pameti */
    void freeData(int id); 

    /** Naplni pole ids, takze ids[i] bude rovno 1, pokud data s indexem i existuje (nemusi byt nactena), jinak 1 */
    void getIds(DA<int> * ids); 

    /** Vraci pocet vsech nactednych objektu */
    int getCount(); 
    
    /** Vraci pocet doposud nactenych objektu */
    int getLoaded(); 

    /** Vraci vsechny data, v poli serazene od 0 do poctu nactenych_objektu-1 */
    int getAll(T ** datas,int size);

    /** Na misto kam ukazuje unit priradi odkaz na pole dat z RM */ 
    void getAll(DA<T *> ** datas);
//@}

/** @name Pridavani, modifikace a mazani dat */
//@{
    /** Prida data.
        Pokud je data->id rovno -1, pak prida na prvni volne misto. Predpoklada, ze jmeno je jiz v RMtxt zapasno.
        @param data data pridavanych dat
        @param startid index od ktereho hledat volny
        @param file jmeno souboru, kde budou data ulozeny
    */
    void add(T * data,int startid,char * file);

    /** Prida data a navic prida retezec do RMtxt.
        Pokud je data->id rovno -1, pak prida na prvni volne misto. 
        @param data vlastni data
        @param startid index od ktereho hledat volny
        @param file jmeno souboru, kde budou data ulozeny
        @param name nazev pridavaneho objektu
        @param startidname index, od ktereho hledat volne misto pro retezec, pokud data->nameid je rovno -1 (jinak se vlozi na toto misto).
    */
    void addDep(T * data,int startid,char * file,char * name,int startidname); 

    /** Odebere objekt */
    void del(int id); 

    /** Odebere objekt, navic odebere i retezec jmena z RMtxt */
    void delDep(int id);

    /** Modifikuje jiz zapsane data */
    void modify(T * data);
//@}

    /** Konstruktor, ktery musi obsahovat odkaz na RM, ke kteremu toto bude interface */
    TRM_advxml_i(TRM_advxml * globalrm); 

    /** Destruktor */
    ~TRM_advxml_i();
  private:
    /** RM ke kteremu je tento interface */
    TRM_advxml * rmadvxml; 
};

template <class T>
void TRM_advxml_i<T>::loadNow(int id)
{
  this->rmadvxml->loadData(id);
}

template <class T>
void TRM_advxml_i<T>::request(int id)
{
  this->rmadvxml->request(id);
}

template <class T>
void TRM_advxml_i<T>::requestDep(int id)
{
  this->rmadvxml->requestDep(id);
}

template <class T>
void TRM_advxml_i<T>::loadBuffer()
{
  this->rmadvxml->loadBuffer();
}

template <class T>
void TRM_advxml_i<T>::freeData(int id)
{
  this->rmadvxml->freeData(id);
}

template <class T>
int TRM_advxml_i<T>::getCount()
{
  return this->rmadvxml->getCount();
}

template <class T>
int TRM_advxml_i<T>::getLoaded()
{
  return this->rmadvxml->getLoaded();
}

template <class T>
void TRM_advxml_i<T>::add(T * data,int startid,char * file)
{
  this->rmadvxml->add(data,startid,file);
}

template <class T>
void TRM_advxml_i<T>::addDep(T * data,int startid,char * file,char * name,int startidname)
{
  this->rmadvxml->addDep(data,startid,file,name,startidname);
}

template <class T>
void TRM_advxml_i<T>::del(int id)
{
  this->rmadvxml->del(id);
}

template <class T>
void TRM_advxml_i<T>::delDep(int id)
{
  this->rmadvxml->delDep(id);
}

template <class T>
void TRM_advxml_i<T>::modify(T * data)
{
  this->rmadvxml->modify(data);
}

template <class T>
void TRM_advxml_i<T>::get(T ** data,int id)
{
  this->rmadvxml->get((void **)data,id);
}

template <class T>
void TRM_advxml_i<T>::getAll(DA<T *> ** datas)
{
  this->rmadvxml->getAll((DA<void*>**)datas);
}
/*
void TRM_advxml_i::set(DA<T *> * datas)
{
  this->rmadvxml->setAll((DA<void*>*)datas);
}
*/
template <class T>
void TRM_advxml_i<T>::getIds(DA<int> * ids)
{
  this->rmadvxml->getIds(ids);
}

template <class T>
int TRM_advxml_i<T>::getAll(T ** datas,int size)
{
  DA<void *> * daids;
  int j=1,max;
  this->rmadvxml->getAll(&daids);
  max=daids->getMaxId();
  int i;
  for (i=0;i<size;i++)
  {
    if (j==(max+1)) // neni tam vic indexu nez n
    {
      break;
    }
    while ((j<=max) && ((*daids)[j]==NULL))
      j++;
    if ((*daids)[j]!=NULL) // kvuli poslednimu prvku ids[max]
    {
      datas[i]=(T*) (*daids)[j];
      j++;
    }
  }
  if (j!=(max+1))
    return i;
  else
    return i-1;
}

template <class T>
TRM_advxml_i<T>::TRM_advxml_i(TRM_advxml * globalrmadvxml)
{
  this->rmadvxml=globalrmadvxml;
  GLOBALLOGID(PRIORITY_CONFIRM ,"RMadvxmli was created");
}

template <class T>
TRM_advxml_i<T>::~TRM_advxml_i()
{
  GLOBALLOGID(PRIORITY_CONFIRM ,"RMadvxmli was destroyed");
}


} // namespace rm

#endif // indef _RMuniti_
/*****************************************************************************/
