/**
******************************************************************************

  @file /common/rm/rmconf.h
  @brief RM pristupujici ke konfiguracnim souborum
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef _RMCONF_
#define _RMCONF_

#include "common/rm/rm.h"


namespace rm
{

/// Struktura profilu hrace
struct TProfile
{  
  int id;                     ///< Index struktury
  int language_id;            ///< Language for the user - IS NOT USED! Default_language is used instead of this value
  int speed;                  ///< Nastaveni rychlosti hry
  int sfx_volume;             ///< Hlasitost zvuku
  int music_volume;           ///< Hlasitost hudby
  int fullscreen;             ///< Velikost obrazovky
  int announce;               ///< Zda hru oznamovat
  int autosave;               ///< Whether save game each turn
  char name[K8_MAX_NAME_LEN]; ///< Jmeno profilu
};

/// Nastavena konfigurace pro GUI
struct TGUIconfig
{
};

/** Prevod XML do struktury profilu.
    Data se budou alokovat
    @param data vytupni data
    @param xml vstupni data
    @param id index, jelikoz v XML souboru se nevyskytuje
*/
void xmlToProfile(TProfile ** data,TXMLdata * xml,int id); 

/** Prevod struktury profilu do XML
    @param xml vystupni data
    @param data vstupni data
*/
void profileToXML(TXMLdata * xml,TProfile * data);

/** Odstrani strukturu data z pameti.
    Dealokuje i pointer data a nastavi ho na NULL. */
void freeProfileStruct(TProfile ** data);


/// Trida Resource Manager pro konfiguracni soubory
class TRM_conf:public TRM
{
  public:
    TRM_conf(); ///< Konstruktor

    /** Destruktor.
        Dochazi k zapisu konfiguracniho souboru na disk.
    */
    ~TRM_conf();
    void readConfigFile(); ///< Nacte config.xml
    void saveConfigFile(); ///< Ulozi config.xml
    /** Ziska polozku z configu.
        Pole na nez se odkazuje output muze zvetsit svoji velikost.
        @param output je pointer na pole, kam se ulozi vysledek dotazu
        @param entry udava jmeno tagu
        @param size obsahuje (a nastavuje) velikost output
    */
    int getFromConfig(char ** output, char * entry, size_t &size); 

    /** Nastavi polozku z configu.
        @param entry udava jmeno tagu
        @param data obsahuje data k zapsani
    */
    void setToConfig(char * entry, char * data); 

    /** Precte polozku z files.
        Pole na nez se odkazuje output muze zvetsit svoji velikost.
        @param output e pointer na pole, kam se ulozi vysledek dotazu
        @param entry udava jmeno tagu
        @param size obsahuje (a nastavuje) velikost output
    */
    int getFromFiles(char ** output, char * entry, size_t &size);

    /** Prazdna funkce, jelikoz tento RM loadData nepouziva */
    void loadData(int id) {}; 

    /** Prazdna funkce, jelikoz tento RM loadData nepouziva */
    void freeData(int id) {}; 

    /** Ziskani profilu z konfiguracniho souboru 
        @param profiles je ukazatel na ukazatel na pole profilu
    */
    void getProfiles(DA<TProfile*> ** profiles);

//    void saveConfigBeforeExit();
  private:
    TXMLdata * config; ///< Vlastni obsah configuracniho souboru
    int waseddited;    ///< Promenna udrzujici informaci o tom, zda byla nejaka hodnota menena
    DA<TProfile*> * profile; ///< Pole profilu

};

}

#endif // define RMCONF
/*****************************************************************************/
