/**
******************************************************************************

  @file /common/rm/rmconfi.h
  @brief Interface k RMconf
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef _RMCONFi_
#define _RMCONFi_

#include "common/rm/rmconf.h"


namespace rm
{

/// Interface k RMconf.
class TRM_conf_i
{
  public:
    /** @brief Konstruktor 
        @param globalrmconf se odkazuje na RM, ke kteremu delam RMi
        */
    TRM_conf_i(TRM_conf * globalrmconf); 
    ~TRM_conf_i(); ///< Destruktor

    int getDefaultLanguage();  ///< Ziska defaultni jazyk
    char *         getPack();  ///< Ziska balicek
    char *    getSoundFile();  ///< Ziska sounds.xml
    char * getLanguageFile();  ///< Ziska languages.xml
    char *   getRasterFile();  ///< Ziska rasters.xml
    char *   getSymbolFile();  ///< Ziska rasters.xml
    char *     getUnitFile();  ///< Ziska units.xml
    char * getBuildingFile();  ///< Ziska buildings.xml
    char *    getSpellFile();  ///< Ziska spells.xml
    char *    getBonusFile();  ///< Ziska bonuses.xml
    char *    getModelFile();  ///< Ziska maps.xml
    char *    getOtherFile();  ///< Ziska others.xml

    void getProfiles(DA<TProfile*> ** profiles); ///< Ziska profily
    void deleteProfile(int id); ///< Smaze profil s indexem id
    int addProfile(TProfile * profile); ///< Prida profil na prvni volne misto
  
    void setLastUsedProfile(int id); ///< Nastavi posledni pouzivany profil v aplikaci

    /** Vraci naposledy pouzity profil, pokud takovy profil existuje. 
        V pripade neexistence a nebo pokud dana informace v konfiguracnim souboru neexistuje vraci -1.
    */
    int getLastUsedProfile(); 

    /** Nastavi vychozi jazyk hry */
    void setDefaultLanguage(int langId);

    /** Ziska 1 pokud je nastaveno zobrazovani pres celou obrazovku, 0 pokud ne */
    int getDefaultFullscreen();

    /** Nastavi rozmer obrazovky */
    void setDefaultFullscreen(int fullscr);

    /** Nastavi index rozliseni */
    void setDefaultResolution(int sizeId);

    /** Ziska index rozliseni */
    int getDefaultResolution();

    void getGUIconfig(TGUIconfig * cfg);
  private:
    TRM_conf *   rmconf;  ///< RM ke kteremu budu delat interface
    size_t   sizeofdata;  ///< Velikost promenne data
    char *         data;  ///< Pomocna promenna, abych mohl vracet char
    size_t sizeofk8pack;  ///< Velikost k8pack
    char *       k8pack;  ///< Pomocna promenna, abych mohl vracet char
    int  changeinconfig;  ///< Probehla zmena v config.xml
};

} // namespace

#endif // RMCONFi
/*****************************************************************************/

