/**
******************************************************************************

  @file /common/rm/rmmap.h
  @brief RM pristupujici k mapam
  
  @author Vta
  @version 1.0

******************************************************************************/
#ifndef _RMmap_
#define _RMmap_

#include "common/rm/rm.h"
#include "common/rm/rmconfi.h"
#include "common/rm/rmtxti.h"
#include "common/rm/rmadvxmli.h"
#include "common/rm/rmbuildingi.h"
#include "world/plan/plan.h"
#include "world/weather.h"
//#include "ai/Diplomacy/diptypes.h"

//using namespace World;
//using namespace ai_ns;
//using namespace diplomacy_ns;

namespace rm
{
#define MAP_NAME_LEN 30      ///< Maximalni delka jmena mapy
#define MAP_DESC_LEN 300     ///< Maximlani delka popisu mapy

#define MAP_TYPES 2          ///< Pocet ruznych zarazeni map
#define MAP_TYPE_NEW_GAME 0  ///< Zarazeni nova mapa
#define MAP_TYPE_SAVE_GAME 1 ///< Zarazeni ulozena mapa

/// Informace o aktualnim stavu
struct TStateInfo
{
  World::WEATHER_STATE weather;  ///< Aktualni pocasi
  int duration;           ///< Doba, po kterou trva aktualni pocasi
  int player_on_turn;     ///< Hrac na tahu
  int turn_id;            ///< Cislo kola
};

/// Informace o jednom meste
typedef World::TOWN TCityInfo;

/// Informace o jedne roli
struct TRoleInfo
{
  int id;                                     ///< Index role
  char name[K8_MAX_NAME_LEN];                 ///< Jmeno role
  int money;                                  ///< Aktualni stav penez
  int state;                                  ///< Stav hrace - zivy, mrtvy...
  ai_ns::diplomacy_ns::TDipRelation * (relations[MAX_PLAYERS+1]); ///< Pole vztahu. Index 0 neni pouzivan. NULL znamena ze role neexistuje.
  World::PLAYER_STATISTICS stats;                          ///< Statistika
//  int * (belief[MAX_PLAYERS+1]); // duvera k  hraci
};

/// Informace o jednom kralovstvi
typedef World::KINGDOM TKingdomInfo;

/// Obecne informace o mape
struct TMapInfo 
{
  int id;                     ///< Index mapy
  int sizex;                  ///< Sirka mapy
  int sizey;                  ///< Vyska mapy
  int players;                ///< Pocet hracu
  TRoleInfo * (roles[MAX_PLAYERS+1]); ///< Pole roli. Index 0 neni pouzivan. NULL znamena ze role neexistuje.
  char file[K8_MAX_FILE_NAME_LEN];    ///< Jmeno souboru mapy
  char name[MAP_NAME_LEN];            ///< Jmeno mapy
  char description[MAP_DESC_LEN];     ///< Popisek mapy
};

/// Plan AI ulozeny v mape
struct TAIPlan
{
  char * (plan[MAX_PLAYERS+1]);
};

// ! ! v cele aplikaci musi existovat prave jedna instance tridy TRM_map ! !
/// RM pristupujici k mapam
class TRM_map:public TRM
{
	public:
/** @name Nacitani a uvolnovani dat */
//@{
    /** Metoda na nacteni jedne mapy */
    void loadMap(int id); 

    /** Metoda na nacteni jedne mapy */
    void loadMap(char * filename,int maptype);

    /** Metoda na nacteni jedne mapy */
    void loadData(int id); 

    /** Odstrani nactenou mapu z pameti */
    void freeMap(); 

    /** Prazdna metoda kvuli predkovi */
    void freeData(int id) {}; 

    /** Oznaceni mapy pro pozdejsi vyber */
    void selectMap(int id);

 //@}

/** @name Ziskani a vraceni dat */
//@{
    /** Ziska ukazatel na hex s danym id z dat v RM */
    World::HEX * getHex(int id); 

    /** Ziska hexy. Pred koncem programu je nutne volat \ref setHexes(World::HEX ** map)
        @param map je ukazatel na pole ukazatelu na struktury, predava se odkaz na data v RM
    */
    int getHexes(World::HEX *** map);

    /** Ziska historickou mapu - informace ktery hrac videl jakou cast mapy. Pred koncem programu je nutne volat \ref setHistoryHexes(World::HISTORY_HEX *** hmap)
        @param hmap je ukazatel na pole ukazatelu na pole (pro kazdeho hrace jedna mapa) ukazatelu na struktury (vlastni data), predava se odkaz na data v RM
    */
    void getHistoryHexes(World::HISTORY_HEX **** hmap);

    /** Ziska zivouci jednotky. Predava se odkaz na data v RM. Pred koncem programu je nutne volat \ref setLivingUnits, do te doby nelze data znovu ziskat */
    void getLivingUnits(DA<World::LIVING_UNIT *> ** units);

    /** Ziska jednu jednotku. Predava se odkaz na zaznam v poli z RM */
    void getLivingUnit(World::LIVING_UNIT ** unit,int id); 

    /** Ziska zivouci budovy. Predava se odkaz na data v RM. Pred koncem programu je nutne volat \ref setLivingBuildings, do te doby nelze data znovu ziskat */
    void getLivingBuildings(DA<World::LIVING_BUILDING *> ** buildings);

    /** Ziska jednu budovu. Predava se odkaz na zaznam v poli z RM */
    void getLivingBuilding(World::LIVING_BUILDING ** building,int id);

    /** Ziska informace o kralovstvi. Predava se odkaz na data v RM. Pred koncem programu je nutne volat \ref setKingdoms, do te doby nelze data znovu ziskat */
    void getKingdoms(TKingdomInfo *** kingdoms);

    /** Ziska informace o mestech. Predava se odkaz na data v RM. Pred koncem programu je nutne volat \ref setCities, do te doby nelze data znovu ziskat */
    void getCities(DA<TCityInfo *> ** cities); 

    /** Ziska informace o aktualnim stavu. Predava se odkaz na data v RM. Pred koncem programu je nutne volat \ref setCurrentState, do te doby nelze data znovu ziskat */
    void getCurrentState(TStateInfo ** state);

    /** Ziska plan AI */
    void getAIPlan(TAIPlan ** plan);


    /** Zaznamena predtim odebrane hexy. \see getHexes(HEX *** map) */
    void setHexes(World::HEX ** map); 
    
    /** Vraci predtim odebrane budovy */
    void setLivingBuildings(DA<World::LIVING_BUILDING *> * buildings);

    /** Vraci predtim odebrane jednotky */
    void setLivingUnits(DA<World::LIVING_UNIT *> * units);

    /** Vraci predtim odebrane informace o kralovstvi */
    void setKingdoms(TKingdomInfo ** kingdoms);

    /** Vraci predtim odebrane informace o mestech */
    void setCities(DA<TCityInfo *> * cities);

    /** Vraci predtim odebrane informace o historicke mape */
    void setHistoryHexes(World::HISTORY_HEX *** hmap);

    /** Vraci informace o aktualnim stavu */
    void setCurrentState(TStateInfo * state);

    /** Vraci plan AI */
    void setAIPlan(TAIPlan * plan);

//@}

/** @name Informace o mapach */
//@{
    /** Nacte aktualni informace o danem typu map. Puvodni informace se smazou a jsou nacteny vsechny znovu.
        @param maptype nabyva bud \ref MAP_TYPE_NEW_GAME nebo \ref MAP_TYPE_SAVE_GAME
        @param refresh zda se maji informace nacitat znovu (pokud 1), pokud uz byly nactene
    */
    void loadMapsInfo(int maptype,int refresh); 

    /** Ziska informace o mape. Jelikoz je index jednoznacny pro nove i ulozene mapy, staci k identifikaci jeden index id. */
    void getMapInfo(TMapInfo ** info,int id); 

    /** Ziska informace o mapach nebo ulozenych hrach. Je nutne to povazovat za readonly data. */
    void getMapsInfo(DA<TMapInfo*> ** info,int maptype);

    /** Vraci pocet map k dispozici (podle poctu nactenych hlavicek map) */
    int getMapsCount(int maptype);

    /** Vraci nejvetsi pouzite id */
    int getMapsMaxId(int maptype);

    /** Vraci jestli je mapa uz nactena (this->gameloaded) */
    int isMapLoaded(); 

    /** Vraci id aktualne nactene mapy */
    int idOfLoadedMap();
//@}

/** @name Pridavani a mazani map */
//@{
    /** Prida mapu (zaradi se mezi ostatni). Data se kopiruji. Vraci id, na ktere se mapa pridala. */
    int addMap(TMapInfo * info,int maptype); 

    /** Zapise mapu do jejiho souboru.
        @param maptype pouze urcuje, jestli bude zapsana mezi nove mapy nebo rozehrane hry
        @param newname jmeno (noveho) souboru, do ktereho se ma mapa ulozit. Pokud je newname prazdne (tj. newname[0] je 0), ulozi se to do pudovniho souboru.
    */
    void saveMap(int maptype,char * newname);

    /** Smaze ze seznamu map i z disku */
    void delMap(int id);

//@}

    /** Konstruktor s interafacem pro konfiguracni soubory */
    TRM_map(TRM_conf_i * rmc);

    /** Destruktor */
    ~TRM_map();
	private:
    /** Zapomene (smaze) nactene informace o mapach. 
        @param maptype nabyva bud \ref MAP_TYPE_NEW_GAME nebo \ref MAP_TYPE_SAVE_GAME
    */
    void forgetMapsInfo(int maptype); 

    /** Overi existenci a korekntost vlastnika (role) nejakeho objektu ve hre. Pri neopravnenem vyskytu ohlasi vyjimku */
    void checkExistingOwner(int ownerid,int mapid);

    /** Vraci 1 pokud role existuje, jinak 0. Slouzi pouze k dotazu na existenci a ne ke kontrole. */
    int existsOwner(int ownerid,int mapid);

    /** Zapise aktualni mapu do XML struktury */
    void fillMap(TXMLdata * map);

    /** Interface pro konfiguracni soubory */
    TRM_conf_i * rmconfi;

    /** Historie hexu mapy pro kazdeho hrace */
    World::HISTORY_HEX *** histhexs;

    /** Hexy mapy */
    World::HEX ** hexs;

    /** Pocet hexu */
    int hexnum;

    /** Plan AI */
    TAIPlan * aiplan;

    /** Aktualni stav jednotek */
    DA<World::LIVING_UNIT *> * lunits;

    /** Aktualni stav budov */
    DA<World::LIVING_BUILDING *> * lbuildings;

    /** Zdali je mapa nactena (==1), nenactena (==0), ve stavu zpracovavani pri pridavani nove mapy (==2), oznacena (==3) */
    int maploaded; 

    /** Id aktualne nactene mapy */
    int idofmap;

    /** Informace o mapach - pole ukazatelu na pole */
    DA<TMapInfo *> * mapsinfo[MAP_TYPES];

    /** Informace o mestech */
    DA<TCityInfo *> * cities; 

    /** Pole ukazatelu na informace o kralovstvich */
    TKingdomInfo ** kingdoms;

    /** Informace o aktualnim stavu */
    TStateInfo * current_state;
};

} // namespace

#endif // define _RMmap_
/*****************************************************************************/
