/**
******************************************************************************

  @file /common/rm/rmmodel.h
  @brief RM pristupujici k modelum
  
  @author Vta
  @version 1.0

******************************************************************************/
#ifndef _RMmodel_
#define _RMmodel_

#include <SDL/SDL.h>

#include "common/rm/rm.h"
#include "common/rm/rmconfi.h"

#ifndef _K_EDITOR_
  #include "gui/model/reader_ase.h"
#else
  #define TModel int
#endif // ifndef

namespace rm
{

// ! ! v cele aplikaci musi existovat prave jedna instance tridy TRM_model ! !
/// RM pristupujici k modelum
class TRM_model:public TRM
{
	public:
/** @name Nacteni a uvolneni dat */
//@{
    /** Odstani nacteny model z pameti (z this->models) */
    void freeData(int id); 

    /** Odstani nacteny model z pameti (z this->models) */
    void freeModel(int id); 

    /** Nacte model ze souboru do models */
    void loadModel(int id);

    /** Nacte model ze souboru do models */   
    void loadData(int id); 
//@}

/** @name Pridani a mazani modelu */
//@{

    /** Prida model. Pokud je id rovno -1, najde prvni volnou pozici od startid, jinak zapise na danou pozici.
        Pridava jen odkaz na soubory s modelem, nikoli vlastni data modelu.
        @param id index na ktery ma byt model vlozen nebo -1
        @param startid index od ktereho se ma hledat volne misto, pokud je id rovno -1
        @param filenames pole jmen souboru, prvni je hlavni soubor, nasleduji sekundarni
        @param size pocet jmen souboru v filenames
        @return index, kam byl model pridan
    */
    int addModel(int id, int startid, char **filenames,int size); 

    /** Prida do modelu dalsi souboru. Id modelu musi byt specifikovano presne */  
    void addToModel(int id, char * secondary_file); 

    /** Smaze(odebere) model */
    void delModel(int id);

//@}

/** @name Informace o modelech */
//@{

    /** Vraci nejvetsi pouzity index ze vsech indexu modelu */
    int getMaxId();

    /** Vraci pocet vsech modelu */
    int getModelsCount(); 

    /** Vraci pocet doposud nactenych modelu */
    int getModelsLoaded(); 

//@}

/** @name Ziskani dat */
//@{

    /** Vraci model jehoz index je id*/
    #ifndef _K_EDITOR_
      gui::TModel * getModel(int id);
    #else
      TModel * getModel(int id);
    #endif

    /** Do ukazatele models priradi this->models, vraci MaxId() */
    #ifndef _K_EDITOR_
      int getModels(DA<gui::TModel *> ** models);
    #else
      int getModels(DA<TModel *> ** models);
    #endif
//@}

    /** Konstruktor s interafacem pro konfiguracni soubory */
    TRM_model(TRM_conf_i * rmc); 

    /** Destruktor */
    ~TRM_model();
	private:
    /** Interface pro konfiguracni soubory */
    TRM_conf_i * rmconfi;

    /** Vytvori a naplni TModel krome surface */
    void createRecord(int id);

    /** Modele s indexem i bude nacten do models[i] */
    #ifndef _K_EDITOR_
      DA<gui::TModel *> * models;
    #else
      DA<TModel *> * models;
    #endif
};

} // namespace

#endif // define _RMmodel_
/*****************************************************************************/
