/**
******************************************************************************

  @file /common/rm/rmmodeli.h
  @brief Interface k RMmodel
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef _RMmodeli_
#define _RMmodeli_

#include "common/rm/rmmodel.h"


namespace rm
{

/// Interface k RMmodel
class TRM_model_i
{
  public:
/** @name Nacteni a uvolnovani modelu */
//@{
    /** Odstani nacteny model z pameti */
    void freeModel(int id);

    /** Nacte z disku model */
    void loadNow(int id); 

    /** Prida pozadavek na naceteni modelu */
    void request(int id); 

    /** Zpracuje pozadavku na nacteni modelu */
    void loadBuffer(void); 
//@}

/** @name Pridavani a mazani modelu */
//@{
    /** Prida model. Pokud je id rovno -1, najde prvni volnou pozici od startid, jinak zapise na danou pozici.
        Pridava jen odkaz na soubory s modelem, nikoli vlastni data modelu.
        @param id index na ktery ma byt model vlozen nebo -1
        @param startid index od ktereho se ma hledat volne misto, pokud je id rovno -1
        @param filenames pole jmen souboru, prvni je hlavni soubor, nasleduji sekundarni
        @param size pocet jmen souboru v filenames
        @return index, kam byl model pridan
    */
    int addModel(int id,int startid,char ** filenames,int size); 

    /** Prida model pouze s odkazem na hlavni soubor
        @param id index na ktery ma byt model vlozen nebo -1
        @param startid index od ktereho se ma hledat volne misto, pokud je id rovno -1
        @param main_file jmeno hlavniho souboru s modelem 
    */
    int addModel(int id,int startid,char * main_file); 

    /** Prida do modelu dalsi souboru. Id modelu musi byt specifikovano presne */  
    void addToModel(int id,char * secondary_file);
 
    /** Smaze model z indexoveho souboru */
    void delModel(int id);
//@}

/** @name Informace o modelech */
//@{
    /** Vraci nejevsti pouzite id v modelech */
    int getMaxId(); 

    /** Vraci pocet vsech modelu */
    int getModelsCount(); 

    /** Vraci pocet doposud nactenych modelu - tj. vhodne pro getModels */
    int getModelsLoaded(); 
//@}

/** @name Ziskani dat */
//@{
    /** Na misto kam ukazuje models priradi primo odkaz na data v RM.
        @return maximalni pouzity index
    */
    #ifndef _K_EDITOR_
      int getModels(DA<gui::TModel *> ** models);
    #else
      int getModels(DA<TModel *> ** models);
    #endif

    /** Ziska modely az do poctku size. Kolik jich je, lze zjistit predem pomoci getMaxId() */
    #ifndef _K_EDITOR_
      int getModels(gui::TModel ** models, int size);
    #else
      int getModels(TModel ** models, int size);
    #endif

    /** Vrati model s id a priradi ji prioritu */
    #ifndef _K_EDITOR_
      gui::TModel * get(int id);
    #else
      TModel * get(int id);
    #endif
//@}

/** @name Prace se skupinami */
//@{
    /** Uvolni modely z posledni skupiny */
    int freeLastGroup(); 

    /** Odstrani konkreni skupinu modelu */
    void freeModelsHavingGroup(int group); 

    // Zmeni (zvetsi) aktualni pouzitou skupinu */
    int startNewGroup(); 

    // Vraci nejvetsi (aktualni) pouzitou skupinu */
    int getCurrentGroup(); 

    /** Vraci skupinu modelu, ktery ma index id */
    int getGroupOfModel(int id); 
//@}

    /** Konstruktor, ktery musi obsahovat odkaz na RM, ke kteremu toto bude interface */
    TRM_model_i(TRM_model * globalrmmodel); 

    /** Destruktor */
    ~TRM_model_i();
  private:
    /** RM ke kteremu je tento interface */
    TRM_model * rmmodel; 
};

} // namespace rm

#endif // indef _RMmodeli_

/*****************************************************************************/
