/**
******************************************************************************

  @file /common/rm/rmrast.h
  @brief RM pristupujici k rasterum
  
  @author Vta
  @version 1.0

******************************************************************************/
#ifndef _RMraster_
#define _RMraster_

#include "common/rm/rm.h"

#ifndef _K_EDITOR_
  #include "gui/common/TGA_lib.h"
#endif

#include "common/rm/rmconfi.h"

namespace rm
{

#define NUMBER_OF_TYPES 2     ///< Pocet ruznych typu

#define RASTER_BMP_NAME "bmp" ///< Oznaceni typu bitmap
#define RASTER_BMP_CODE 1     ///< Kod soboru typu bitmap
#define RASTER_TGA_NAME "tga" ///< Oznaceni typu tga
#define RASTER_TGA_CODE 2     ///< Kod soboru typu tga

/// Vlastni data o jednom rasteru
struct TRaster
{
  SDL_Surface * surface;                 ///< Vlastni data
  int type;                              ///< Druhy rastereru to je
  int id;                                ///< Id daneho rasteru
  char szFilename[K8_MAX_FILE_NAME_LEN]; ///< Jmeno souboru, ze ktereho byl raster nacten
};


// ! ! v cele aplikaci musi existovat prave jedna instance tridy TRM_raster ! !
/// RM pristupujici k rasterum
class TRM_raster:public TRM
{
	public:
/** @name Nacitani a uvolnovani rasteru */
//@{
    /** odstani nactenou raster i celou TRaster z pameti */
    void freeData(int id);

    /** Odstani nacteny raster z pameti (z rasters), odtrani i celou TRaster pokud total */
    void freeRaster(int id, int total); 

    /** Nacte raster */
    void loadRaster(int id); 

    /** Nacter raster */
    void loadData(int id); 
//@}

/** @name Ziskani dat */
//@{
    /** Vraci odkaz na raster s indexem id z RM */
    TRaster * getRaster(int id); 

    /** Priradi do rsts this->rasters, vraci MaxId() */
    int getRasters(DA<TRaster *> ** rsts); 
//@}


/** @name Pridani/mazani rasteru */
//@{
    /** Prida raster.
        Pokud je id rovno -1 najde prvni volny index, jinak zapise na dany
        @return index, na ktery byl raster zapsan
    */
    int addRaster(SDL_Surface * newraster,int newid,char *filename,int type);

    /** Smaze (odebere) raster */
    void delRaster(int id); 
//@}

/** @name Informace o rasterech */
//@{
    /** Ziska index rasteru podle jeho souboru */
    int getRasterId(const char *name); 

    /** Vraci nejevsti index rasterechu. Tj. max z id v this->index a this->rasters */
    int getMaxId(); 

    /** Vraci pocet vsech rasteru */
    int getRastersCount(); 

    /** Vraci pocet doposud nactenych rasteru */
    int getRastersLoaded(); 
//@}

    /** Konstruktor s interafacem pro konfiguracni soubory */
    TRM_raster(TRM_conf_i * rmc); 

    /** Destruktor */
    ~TRM_raster(); 
	private:
    /** Vytvori a naplni TRaster krome surface */
    void createRecord(int id); 

    /** Interface pro konfiguracni soubory */
    TRM_conf_i * rmconfi; 

    /** Vlastni data */
    DA<TRaster *> * rasters; 
};

} // namespace

#endif // define _RMraster_
/*****************************************************************************/
