/**
******************************************************************************

  @file /common/rm/rmrasti.h
  @brief Interface k RMraster
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef _RMrasteri_
#define _RMrasteri_

#include "common/rm/rmrast.h"


namespace rm
{

/// Interface k RMraster
class TRM_raster_i
{
  public:
/** @name Pridani rasteru */
//@{
    /** Prida novy raster do RM - primo odkaz raster (struktura se nekopiruje). Tato zmena se nezapise do indexoveho souboru */
    int addRaster(SDL_Surface * newraster,int type); 

    /** Prida novy raster na trvalo. Provedou se i zmeny v indexovem souboru */
    int addRasterPremanent(SDL_Surface * newraster,int newid,char *filename,int type); 
//@}

/** @name Nacitani a uvolnovani rasteru */
//@{
    /** Odstani nacteny raster z pameti*/
    void freeRaster(int id);

    /** Nacte okamzite jeden raster */
    void loadNow(int id);

    /** Nacte jeden raster ve vlakne */
    void loadThr(int id);

    /** Nacte z disku rastery s indexy v poli ids, ktere ma velikost size */
    void loadThr(int *ids,int size); 

    /** Nacte z disku raster zadany jmenem souboru */
    int loadNow(const char *name); 

    /** Prida pozadavek na nacteni rasteru specifikovany jeho cestou */
    int request(const char *name); 

    /** Prida pozadavek na nacteni rasteru s indexem id */
    void request(int id);

    /** Zpracuje pozadavky na nacteni */
    void loadBuffer(void);
//@}

/** @name Prace se skupinami */
//@{
    /** Zmeni (zvetsi) aktualni pouzitou skupinu */
    int startNewGroup();

    /** Vraci nejvetsi (aktualni) pouzitou skupinu */
    int getCurrentGroup(); 

    /** Vraci skupinu rasteru s indexem id */
    int getGroupOfRaster(int id); 

    /** Odstrani rastery z posledni skupiny, vraci index predposleni skupiny (tj. posledni po uvolneni) */
    int freeLastGroup(); 

    /** Odstrani z pameti vsechny bitmapy, ktere jsou ve skupine group */
    void freeRasterHavingGroup(int group); 
//@}

/** @name Informace o rasterech */
//@{
    /** Vraci pocet vsech rasteru */
    int getRastersCount(); 

    /** Vraci pocet doposud nactenych rasteru */
    int getRastersLoaded(); 

    /** Vraci nejevsti pouzite id v rasterech */
    int getMaxId(); 
//@}

/** @name Ziskani dat */
//@{
    /** Ziska vsechny rastery. Kolik jich je se zjisti pomoci getMaxId()*/
    int getRasters(TRaster ** rasters, int size); 

    /** Vraci vsechny rastery v poli. Bere se odkaz primo na data v RM */
    int getRasters(DA<TRaster *> ** rsts);

    /** Vrati raster s id */
    TRaster * get(int id); 
//@}      

    /** Konstruktor, ktery musi obsahovat odkaz na RM, ke kteremu toto bude interface */
    TRM_raster_i(TRM_raster * globalrmraster); 

    /** Destruktor */
    ~TRM_raster_i(); 
  private:
    /** RM ke kteremu je tento interface */
    TRM_raster * rmraster;
};

/// Informace o nacitanych rasterch v jinem vlakne
struct loadinthread
{
  int *ids; ///< Pole indexu pozadovanych rasteru
  int size; ///< Velikost pole ids
};

} // namespace rm

#endif // indef _RMrasteri_
/*****************************************************************************/
