/**
******************************************************************************

  @file /common/rm/rmsound.h
  @brief RM pristupujici ke zvukum
  
  @author Vta
  @version 1.0

******************************************************************************/
#ifndef _RMsound_
#define _RMsound_

#include "common/rm/rm.h"
#include "common/rm/rmconfi.h"
#include "common/rm/rmtxti.h"
#ifndef _K_EDITOR_
  #include <SDL/SDL_mixer.h>
#endif

namespace rm
{
#define MULTILANGUAGE_VOICES 1        ///< Whether to use multilanguage voices

#define STATUS_ENDING 0               ///< Signalizace, ze RM ukoncuje svoji cinnost
#define STATUS_RUNNING 1              ///< Signalizace, ze RM jsou v radnem stavu
#define STATUS_PREPARE 4              ///< Signalizace, ze se zacina pripravovat k cinnosti

/// Struktura udrzujici data a informace o zvuku
struct TSound
{
  int type;    ///< Typ zvuku @sa SOUND_MUSIC_CODE, SOUND_SAMPLE_CODE
  int playlist;///< In which playlist (in case of music) it is
  int id;      ///< Id daneho zvuku
  void * data; ///< Vlastni data zvuku
};

/** Function, which be called in separate thread which will check if the music keep playing */
int checkMusicPlaying_thread(void * data); 

#define NUMBER_OF_SOUND_TYPES 2 ///< pocet ruznych typu

#define SOUND_MUSIC "music"     ///< oznaceni typu hudba (napr. mod)
#define SOUND_MUSIC_CODE 1      ///< kod typu hudba
#define SOUND_SAMPLE "sample"   ///< oznaceni typu sample (napr. wav)
#define SOUND_SAMPLE_CODE 2     ///< kod typu sample

// ! ! v cele aplikaci musi existovat prave jedna instance tridy TRM_sound ! !
/// RM pristupujici ke zvukum
class TRM_sound:public TRM
{
	public:
/** @name Nacteni a uvolneni zvuku */
//@{

    /** Start playing music from group playlist */
    void playMusic(int playlist);

    /** Odstani nacteny zvuk z pameti */
    void freeData(int id); 

    /** Odstani nacteny zvuk z pameti */
    void freeSound(int id); 

    /** Nacte zvuk ze souboru do this->sounds */
    void loadSound(int id);

    /** Nacte zvuk ze souboru do this->sounds */
    void loadData(int id); 
//@}

/** @name Ziskani zvuku */
//@{
    
    /** Ziska zvuk. V pripade typu "music" probiha vlastni nacteni, jelikoz v jednu chvili muze byt nactena pouze jedna */
    TSound * getSound(int id);

    /** Priradi do snds this->sounds, vraci MaxId() */
    int getSounds(DA<TSound *> ** snds); 
//@}

/** @name Pridani a mazani zvuku */
//@{
    /** Prida novy zvuk do pameti.
        Pokud je id rovno -1 najde prvni volnou pozici, jinak zapise na danou pozici
        @return index, na ktery se vlozilo
    */
    int addSound(int newid, char *filename,int type); 

    /** Smaze (odebere) zvuk */
    void delSound(int id);
//@}
   
/** @name Informace o zvucich */
//@{
    /** Vraci nejevsti pouzite id ve zvucich - tj. maximalni index v this->index a this->sounds */
    int getMaxId(); 

    /** Vraci pocet vsech zvuku */
    int getSoundsCount();

    /** Vraci pocet doposud nactenych zvuku */
    int getSoundsLoaded(); 
//@}

    /** Konstruktor s interafacem pro konfiguracni soubory */
    TRM_sound(TRM_conf_i * rmc,TRM_txt_i * rmt); 

    /** Destruktor */
    ~TRM_sound();

  protected:
    /** Id posledni nahrane music, abych vedel, kterou mam umlcet */
    int lastMusId; 

    /** Id of Group of music I am playing */
    int playingGroup;

    /** Interface pro konfiguracni soubory */
    TRM_conf_i * rmconfi; 

    /** Interface to read current language (for different voices in different languages) */
    TRM_txt_i * rmtxti; 

    /** Vlastni data */
    DA<TSound *> * sounds; 

    /** Thread id which will check if the music keep playing */
    SDL_Thread * checkMusicPlaying; 

    /** State of the RMsound, so I could end the thread if ending */
    int status; 

    /** Pomocny semafor pri cekani na vytvoreni vlakna */
    SDL_sem *sem;

    /** Funkce odesilajici zpravy z fronty zprav */
    friend int checkMusicPlaying_thread(void * data); 
};

} // namespace

#endif // define _RMsound_
/*****************************************************************************/
