/**
******************************************************************************

  @file /common/rm/rmsymbol.h
  @brief RM pristupujici k symbolum
  
  @author Vta
  @version 1.0

******************************************************************************/
#ifndef _RMsymbol_
#define _RMsymbol_

#include "common/rm/rm.h"
#include "common/rm/rmconfi.h"
#include "common/rm/rmrasti.h"


namespace rm
{

/// Informace o jednom symbolu
struct TSymbolData
{
  int id;         ///< Id symbolu 
  int groupId;    ///< Skupina ve ktere se nachazi
  int rasterId;   ///< Id rasteru
  int x;          ///< Xova pozice horniho leveho rohu
  int y;          ///< Yova pozice horniho leveho rohu
  int width;      ///< Sirka
  int height;     ///< Vyska
};


// ! ! v cele aplikaci musi existovat prave jedna instance tridy TRM_symbol ! !
/// RM pristupujici k symbolum
class TRM_symbol:public TRM
{
	public:

/** @name Nacteni a uvolneni modelu */
//@{

    /** Nacte sadu symbolu do pameti */
    void loadSymbols(int id); 

    /** Nacte sadu symbolu \see loadSymbols() */
    void loadData(int id); 

    /** Provede pozadavek na nacteni sady symbolu i rasteru (primo do RMrasteri), ke kteremu se nacitana sada symbolu vaze */
    void requestSymbolsDep(int id); 

    /** Uvolni nactenou sadu z pameti */
    void freeSymbols(int id); 

    /** Uvolni nactenou sadu z pameti */
    void freeData(int id); 
//@}


/** @name Ziskani modelu */
//@{
    /** Ziska jeden symbol */
    void getSymbol(TSymbolData ** sym,int symgroupid,int symid); 

    /** Ziska sadu symbolu */
    void getSymbols(DA<TSymbolData*> * sym,int id); 

    /** Ziska sadu symbolu tak, ze symboly budou v poli primo za sebou nehlede na jejich id 
        @param sym pole ukazatelu
        @param id id sady
        @param size maximalni pocet symbolu, ktere se vejdou do sym
    */
    void getSymbols(TSymbolData ** sym,int id,int size);
//@}

/** @name Informace modelu */
//@{
    /** Vraci pocet vsech symbolu v sade s indexem id (vhode pro vstup getSymbols) */
    int getSymbolsCount(int id); 

    /** Vraci pocet vsech sad symbolu */
    int getGroupsOfSymbolsCount(); 

    /** Vraci pocet doposud nactenych sad symbolu */
    int getGroupsOfSymbolsLoaded(); 
//@}

/** @name Pridavani a mazani modelu */
//@{
    /** Prida symbol. Id je nutne zadat, data se kopiruji */
    void addSymbol(TSymbolData * sym); 

    /** Odebere symbol ze skupiny */
    void delSymbol(int group,int id);

    /** Modifikuje jiz zapsany symbol, prekopiruje data */
    void modifySymbol(TSymbolData * sym); 

    /** Prida sadu symbolu
        @param group id nove skupiny nebo -1
        @param file jmeno souboru, kde budou vlastni data
        @param rasterId id rasteru, ke kteremu se tato sada vaze
    */
    void addGroup(int group,char * file,int rasterId); 

    /** Smaze celou skupinu */
    void delGroup(int group); 
//@}

    /** Konstruktor s interafacem pro konfiguracni soubory */
    TRM_symbol(TRM_conf_i * rmc); 

    /** Destruktor */
    ~TRM_symbol();
	private:
    /** Vrati raster pro dany soubor symbolu */
    int getRaster(int id); 

    /** Zapise skupinu do jejiho souboru */
    void saveGroupToFile(int groupId); 

    /** Slozi do path celou cestu k souboru dane sady symbolu */
    void getFileOfSymbol(int id,char * path,int pathsize);

    /** Interface pro konfiguracni soubory */
    TRM_conf_i * rmconfi; 

    /** Vlastni data. Pole sad, ktera kazda obsahuje pole symbolu. */
    DA<DA<TSymbolData*>*> * symbols; 
};

} // namespace

#endif // define _RMsymbol_
/*****************************************************************************/
