/**
******************************************************************************

  @file /common/rm/rmsymboli.h
  @brief Interface k RMsymbol
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef _RMsymboli_
#define _RMsymboli_

#include "common/rm/rmsymbol.h"


namespace rm
{

///  Interface k RMsymbol
class TRM_symbol_i
{
  public:
/** @name Ziskani modelu */
//@{

    /** Ziska jeden symbol */
    void getSymbol(TSymbolData ** sym,int symgroupid,int symid); 

    /** Ziska sadu symbolu */
    void getSymbols(DA<TSymbolData*> * sym,int id);

    /** Ziska sadu symbolu tak, ze symboly budou v poli primo za sebou nehlede na jejich id 
        @param sym pole ukazatelu
        @param id id sady
        @param size maximalni pocet symbolu, ktere se vejdou do sym
    */
    void getSymbols(TSymbolData ** sym,int id,int size); 
//@}

/** @name Nacteni a uvolneni modelu */
//@{

    /** Nacte okamzite sadu symbolu, ve stejnem vlakne */
    void loadNow(int id); 

    /** Prida pozadavek na nacteni */
    void request(int id);

    /** Prida pozadavek na nacteni a zaroven prida pozadavek na nacteni rasteru, na ktery odkazuje */
    void requestDep(int id); 

    /** Zpracuje pozadavky na nacteni sad symbolu */
    void loadBuffer(); 

    /** Uvolni nactenou sadu symbolu z pameti */
    void freeSymbols(int id); 
//@}

/** @name Informace o modelech */
//@{
    /** Vraci pocet vsech sad symbolu */
    int getSymbolsCount(int id); 

    /** Vraci pocet doposud nactenych sad symbolu */
    int getGroupsOfSymbolsCount();

//@}

/** @name Pridani a mazani modelu */
//@{

    /** Prida symbol. Id je nutne zadat, data se kopiruji */
    void addSymbol(TSymbolData * sym); 
    
    /** Odebere symbol ze sady */
    void delSymbol(int group,int id); 

    /** Modifikuje jiz zapsany symbol. Data se kopiruji */
    void modifySymbol(TSymbolData * sym); 

    /** Prida sadu symbolu
        @param group id nove skupiny nebo -1
        @param file jmeno souboru, kde budou vlastni data
        @param rasterId id rasteru, ke kteremu se tato sada vaze
    */
    void addGroup(int group,char * file,int rasterId); 

    /** Smaze celou sadu */
    void delGroup(int group); 
//@}

    /** Konstruktor, ktery musi obsahovat odkaz na RM, ke kteremu toto bude interface */
    TRM_symbol_i(TRM_symbol * globalrmsymbol); 
    /** Destruktor */
    ~TRM_symbol_i(); 
  private:
    /** RM ke kteremu je tento interface */
    TRM_symbol * rmsymbol; 
};


} // namespace rm

#endif // indef _RMsymboli_
/*****************************************************************************/
