/**
******************************************************************************

  @file /common/rm/rmtxt.h
  @brief RM pristupujici k retezcum ve hre
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef RMTXT
#define RMTXT

#include "common/rm/rm.h"
#include "common/rm/rmconfi.h"


namespace rm
{

// ! ! v cele aplikaci musi existovat prave jedna instance tridy TRM_txt ! !
/// RM pristupujici k retezcum ve hre
class TRM_txt:public TRM
{
	public:
    /** Konstruktor obsahuje odkaz na RMconf, ktery v destruktoru i znici */
    TRM_txt(TRM_conf_i * rmc);

    /** Destruktor */
    ~TRM_txt();

/** @name Informace o datech */
//@{

    /** Vrati nejvetsi index, ktery se u retezcu vyskytuje */
    int getNumOfTexts(); 

    /** Vrati nejvetsi index jazyka, ktery je k dispozici*/
    int getNumOfLanguages();

    /** Naplni pole intu, kde na indexu i bude 1, pokud se text s indexem i v jazyce vyskytuje, jinak 0. 
        @return Vraci nejvetsi pouzity index. 
    */
    int getAllTextIds(DA<int> * ids);  
//@}

/** @name Prace s jazykovymi sadami */
//@{
    /** Ziska vsechny mozne jazykove sady nahrane v tomto systemu */
    int getAllLanguages(DA<char*> * ids,char * subtag);  

    /** Nacte jazykovou sadu s indexem id, (pouze) kdyz neexistuje, vytvori soubor, s nazvem language v prislusnem adresari */
    int inputLangDataRaw(int langid,char * language,char * code); 

    /** Prida dalsi jazyk na prvni volne misto, ktere najde */
    int addLangDataRaw(char * language,char * code); 

    /** Smaze jazyk s kodem iLangCode */
    void delLang(int iLangCode); 

    /** Vraci id nacteneho jazyka */
    int loadedLanguage(void); 
    
    /** Vraci jmeno file aktualne nactene jazykove sady - nikoliv languages.xml */
    int fileOfLoadedLanguage(char * output,size_t size); 

    /** Zapise jazyk (obsah indexLang) do souboru
        Interface to pak omezi, ze se zapise jen do nacteneho souboru (nebo vytvoreneho pomoci RAW metody)
    */
    void writeLangToFile(char *file); 

    /** Nacte jazyk s indexem id */
    int inputLangData(int langid);

    /** Nacte jazyk s id - metoda vsech RM */
    void loadData(int langid);
    
    /** Free zde neni treba, free nelze provest - v RMtxt musi byt v kazde chvili nacten nejaky jazyk */
    void freeData(int id) {}; 
//@}

/** @name Prace s jednotlivymi retezci */
//@{
    /** Zapise retezec s danym indexem do jazyka */
    void setData(char * data,int id); 

    /** Odstrani retezec z jazyka */
    void delData(int id); 
    
    /** Prida retezec do vsech jazyku.
        Pokud je id rovno -1, najde prvni volny index od startid
        @return index, kam se retezec pridal
    */
    int addData(char * data,int id,int startid); 

    /** Ziska data (retezec) */
    int getLangData(char * output,int size,int id);

    /** Vrati primo odkaz na data (retezec) v RM */
    char * getLangDataRaw(int id);

    /** Vraci jmeno jazyka, velikost outputu se muze menit. Info=1 gets name, else gets code */
    int getLangId(char ** output, size_t &size, int id, int info=1); 
//@}
	private:

    /** Narozdil od indexu, tady jsou ukladany vsechny texty dane sady */
    TXMLdata * texts; 

    /** Index nacteneho jazyka */
    int langloaded;

    /** Interface ke konfiguracnim souborum */
    TRM_conf_i * rmconfi;
};

} // namespace

#endif // define RMTXT
/*****************************************************************************/
