/**
******************************************************************************

  @file /common/rm/rmtxti.h
  @brief Interface k RMtxt, definice struktur pro vmnu dat
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef RMTXTi
#define RMTXTi

#include "common/rm/rmtxt.h"

// toto bude obsluhovat pristup k RMtxt. Interface neresi multithreadove pristupy,
// toto resi az RM.
// je tedy nutne mit pro kazdou praci vlastni interface (je to prevazne kvuli metode get(int))
// RMi se nesmi sdilet mezi vlakny, v pripade potreby neni problem poslat zpravu pro novy RMi


namespace rm
{

#define MAX_LANGUAGE_NAME_LENGTH 30  ///< Maximalni delka nazvu jazyka
#define MAX_LANGUAGE_CODE_LENGTH 3   ///< Maximalni delka kodu jazyka

/// Struktura udrzujici informace o jednom jazyce
struct TLang
{
	int iLangCode; ///< Id jazykove sady
  char szCode[MAX_LANGUAGE_CODE_LENGTH]; ///< Kod jazyka
  char szName[MAX_LANGUAGE_NAME_LENGTH]; ///< Jmeno jazyka
};

/// Interface k RMtxt
class TRM_txt_i // CLSDBG1
{
  public:
    /// Konstruktor, ktery musi obsahovat odkaz na RM, ke kteremu toto bude interface
    TRM_txt_i(TRM_txt * globalrmtxt); 

    /// Destruktor
    ~TRM_txt_i(); 

/** @name Ziskani dat */
//@{
    /** Ziska (prekopiruje) data do velikosti size.  \see get(char * buffer,int size, int id)
        @param data2 vystpuni parametr
        @param size maximalni delka retezce, ktery lze do data2 ulozit, vcetne koncove nuly
        @param id index retezces
    */
    wchar_t * getWChar(wchar_t * data2,int size,int id); 

    /** Ziska (prekopiruje) data, pokud by se nevesela, data2 bude zvetseno.
        @param data2 vystpuni parametr
        @param size puvodni velikost pole data2
        @param id index retezces
    */
    wchar_t * getWCharResize(wchar_t ** data2,int &size,int id);

    /** Ziska odkaz na data primo v RM (nekopiruje se) */
    char * getRaw(int id); 

    /** Ziska retezec s daty (texty) do this->data, vraci obsah this->data */
    char * get(int id); 
    
    /** Ziska retezec s daty (texty) do this->data, delky maximalne maxsize (na pozici maxsize-1 bude umistena vzdy 0). Vraci obsah this->data */
    char * get(int id,int maxsize); 

    /** Ziska retezec s daty o velikosti maximalne size. \see getWChar(wchar_t * data2,int size,int id)  */
    int get(char * buffer,int size, int id);

//@}

/** @name Pridavani nebo mazani dat */
//@{
    /** Smaze retezec s indexe int id */
    void delString(int id); 
    
    /** Prida retezec do vsech jazyku, pokud id==-1 najde prvni volne misto od startid */
    int addString(char * data,int id,int startid); 

    /** Nastavi (zapise) data na zadany index */
    void set(char * buffer, int id); 

    /** Zapise nactenou jazykovou sadu, mozna upravenou pomoci set, do prislusneho souboru, ze ktereho byla nactena. */
    void writeToFile(void); 

    /** Prida jazykovou sadu s nazvem lang->szName, a code lang->szCode, id se ignoruje.
        Prida se na prvni mozne volne misto - tj. zaplni se prvni volne misto.
    */
		void addLang(TLang* lang); 

    /** odstrani jazyk s iLangCode */
    void delLang(int iLangCode);

//@}

/** @name Informace o datech */
//@{
    /** Vrati pocet retezcu ve zvolene znakove sade. Pozn. Vsechny znakove sady by mely obsahovat shodny pocet retezcu. */
		int getStringsCount(); 
    
    /** Naplni pole ids indexy v jazykove sady se vyskytujicich retezcu
        @return presny pocet retezcu. */
    int getIds(DA<int> * ids);

    /** Naplni pole ids indexy v jazykove sady se vyskytujicich retezcu
        @return presny pocet vracenych retezcu. */
    int getIds(int* ids, int n);

    /** Naplni (prekopiruje) informace do pole struktur.
        O alokaci, dealokaci se stara volajici, pripadne staticky
        @return presny pocet zapsanych jazyku
    */
 		int getLangs(TLang* langs, int n);

    /** Naplni (prekopiruje) informace o dostupnych jazycich. */
 		void getLangs(DA<TLang*>* langs);

    /** Naplni (prekopiruje) nazvy dostupnych jazyku. */
    void getLangNames(DA<char*>* langs);

    /** Naplni (prekopiruje) nazev kodu jazyku s lang_id. */
    void getLangCode(char ** code,size_t &size,int lang_id);

    /** Vrati pocet dostupnych jazyku */
    int getLangsCount();

//@}

/** @name Nacitani dat */
//@{

    /** Pozadavek na nacteni jazykove sady pres frontu pozadavku.
        Nema smysl zadavat vic nez 1 jazyk, lze tedy pouzit pouze, kdyz chci mit kontrolu nad tim, kdy se ma zacit nacitat
    */
    void request(int iLangCode);

    /** Zpracuje pozadavky k nacteni */
    void loadBuffer(void); 

    /** Nacte okamzite jazykovou sadu do pameti */
    void selectLangNow(int iLangCode); 

    /** Nacte jazykovou sadu v jinem vlakne */
    void selectLangThr(int iLangCode); 

    /** Vraci id nacteneho jazyka */
    int loadedLanguage(void); 

//@}
  private:
    /** Vraci jmeno souboru jazykove sady, ktera byl nactena - toto slouzi pouze pri vytvareni souboru
        Uzivatelum je tato hodnota nepristupna, jelikoz ji nepotrebuji znat. Oni pristupuji k jazykum za pomoci id
    */
    char * loadedLang(void); 

    /** Velikost pole data */
    int sizeofdata; 

    /** Pomocny retezec, aby nektere fce mohly vrace char * */
    char *data;

    /** RM ke kteremu je tento interface */
    TRM_txt * rmtxt;
};

} // namespace

#endif // indef RMTXTi

/*
Example:
1) Vytvoreni + Get + Zniceni

  TRM_txt_i * RMtxti=(TRM_txt_i *)KSendMessage(RQUEUE,MSG_GET_RM_TXT_I,NULL,NULL,NULL);
  wprintf(L"%s \n",RMtxti->getRaw(1)); // vypise primo obsah kus pameti meho RM
  wprintf(RMtxti->get(1)); // kopiruje do this->data, vypise cele
  wprintf(RMtxti->get(1,3)); // kopiruje do this->data, vypise prvni dva znaky
  char text[10];
  RMtxti->get(text,10,1); // do test nakopiruje nejvyse 9 znaku z 5. retezce, 10. znak (=text[9]) bude 0
  delete(RMtxti);

2) Prace s TLang a getLangs(TLang* langs, int n); // ziskani vsech jazyku, ktere jsou k dispozici

  TLang *langs=(TLang*) KMemAlloc(sizeof (TLang)*RMtxti->getLangCount());
  RMtxti->getLangs(langs,RMtxti->getLangCount());
  for (i=0;i<RMtxti->getStrings();i++)
  {
    WideChartoUTF8(langs[i].szName, output, 1000);  
    printf("\n%i %s",langs[i].iLangCode,output);
  }
  KMemFree(langs);

3) Prace s getIds(int* ids, int n); // pravdepodobne jen pro editor, ziskani vsech retezcu

  int *test=(int*) KMemAlloc(sizeof(int)*RMtxti->getStrings());
  int RMtxti->getIds(test,RMtxti->getStrings());
  for (int i=0;i<RMtxti->getStrings();i++)
  {
    printf("\n%i Je tu",test[i]);  
  }
  KMemFree(test);

*/
/*****************************************************************************/
