/**
******************************************************************************

  @file /common/rm/rmxmli.h
  @brief Interface k RMxml
  
  @author Vta
  @version 1.0

******************************************************************************/

#ifndef _RMxmli_
#define _RMxmli_

#include "common/rm/rmxml.h"


namespace rm
{

/// Interface k RMxml
template <class T>
class TRM_xml_i
{
  public:
/** @name Ziskani dat */
//@{
    /** Ziska do *data odkaz na strukturu s id z RM. Data se nekopiruji. */
    void get(T ** data,int id); 

    /** Ziska vsechna data.
        Do ukazatele, na ktery ukazuje data se priradi ukazatel na pole, ktere je v RMxmli.
    */
    void getAll(DA<void*> ** data);

    /** Vyplni pole ids tak, ze 1 je na i-te pozici, pokud existuji data s indexem i */
    void getIds(DA<int> * ids); 
//@}

/** @name Nacitani/uvolnovani dat */
//@{

    /** Nacte okamzite, ve stejnem vlakne */
    void loadNow(int id); 

    /** Prida pozadavek na nacteni */
    void request(int id);

    /** Zpracuje pozadavky na nacteni */ 
    void loadBuffer();

    /** Uvolni nactene data z pameti */
    void freeData(int id); 
//@}

/** @name Pridavani, mazani a modifikace */
//@{
    /** Prida dalsi data 
        @param data vlastni data
        @param structid typ dat \ref STRUCT_TCL, \ref STRUCT_TERRAIN n
        @param id pozice na kterou se data maji pridat, nebo -1, pokud na tom nezalezi
        @param startid pokud je id rovno -1, tento parametr urcuje, odkud se zacne hledat volny index
        @param file jmeno souboru, kde budou data ulozana
        @return Pozice, kam se data pridala
    */
    int add(T * data,int structid,int id,int startid,char * file);

    /** Odstrani data s indexem id */
    void del(int id);

    /** Modifikuje jiz zapsana data s indexem id */
    void modify(T*data, int id);
//@}

/** @name Ziskani informaci o datech */
//@{
    /** Vraci pocet vsech zaznamu v indexovem souboru */
    int getCount();

    /** Vraci pocet doposud nactenych dat */
    int getLoaded();
//@}

    /** Konstruktor, ktery musi obsahovat odkaz na RM, ke kteremu toto bude interface */
    TRM_xml_i(TRM_xml * globalrmxml); 

    /** Destruktor */
    ~TRM_xml_i();

  private:

    /** RM ke kteremu je tento interface */
    TRM_xml * rmxml; 
};

template <class T>
void TRM_xml_i<T>::loadNow(int id)
{
  this->rmxml->loadXML(id);
}

template <class T>
void TRM_xml_i<T>::request(int id)
{
  this->rmxml->request(id);
}

template <class T>
void TRM_xml_i<T>::loadBuffer()
{
  this->rmxml->loadBuffer();
}

template <class T>
void TRM_xml_i<T>::freeData(int id)
{
  this->rmxml->freeData(id);
}

template <class T>
int TRM_xml_i<T>::getCount()
{
  return this->rmxml->getCount();
}

template <class T>
int TRM_xml_i<T>::getLoaded()
{
  return this->rmxml->getLoaded();
}

template <class T>
int TRM_xml_i<T>::add(T * data,int structid,int id,int startid,char * file)
{
  return this->rmxml->add(data,structid,id,startid,file);
}

template <class T>
void TRM_xml_i<T>::del(int id)
{
  this->rmxml->del(id);
}


template <class T>
void TRM_xml_i<T>::modify(T * data,int id)
{
  this->rmxml->modify(data,id);
}

template <class T>
void TRM_xml_i<T>::get(T ** data,int id)
{
  this->rmxml->get((void**)data,id);

}

template <class T>
void TRM_xml_i<T>::getAll(DA<void*> ** data)
{
  this->rmxml->getAll(data);
}

template <class T>
void TRM_xml_i<T>::getIds(DA<int> * ids)
{
  this->rmxml->getIds(ids);
}
/*
template <class T>
void TRM_xml_i<T>::setAll(DA<void*> * data)
{
  this->rmxml->setAll(data);
}
*/
template <class T>
TRM_xml_i<T>::TRM_xml_i(TRM_xml * globalrmxml)
{
  this->rmxml=globalrmxml;
  GLOBALLOGID(PRIORITY_CONFIRM ,"RMxmli was destroyed");
}

template <class T>
TRM_xml_i<T>::~TRM_xml_i()
{
  GLOBALLOGID(PRIORITY_CONFIRM,"RMXmli was destroyed");
}

} // namespace rm

#endif // indef _rmxmli_
/*****************************************************************************/
