/******************************************************************************

  @file /common/uComputeBorders.h
  @brief Zdrojovy kod pro zaplavovy vypocet uzemi.


  @author PZ
  @version 0.1
 
******************************************************************************/

#include "uComputeBorders.h"


using namespace std;

bool validCoordinates(int x, int y)
{
  if (((x>=0) && (x<mapWidth)) && ((y>=0) && (y<mapHeight)))
  {
    return true;
  }
  return false;
}

list<THexOwnership> chooseNeighbours(int sourX,int sourY)
{
  list<THexOwnership> rnl;
  THexOwnership tempsou;

  tempsou.x=sourX;
  tempsou.y=sourY+2;
  if (validCoordinates(tempsou.x,tempsou.y))
  {
    rnl.push_front(tempsou);
  }

  tempsou.y-=4;
  if (validCoordinates(tempsou.x,tempsou.y))
  {
    rnl.push_front(tempsou);
  }

  tempsou.y++;
  if (sourY%2) tempsou.x--;

  if (validCoordinates(tempsou.x,tempsou.y))
  {
    rnl.push_front(tempsou);
  }

  tempsou.x++;
  if (validCoordinates(tempsou.x,tempsou.y))
  {
    rnl.push_front(tempsou);
  }

  tempsou.y+=2;
  if (validCoordinates(tempsou.x,tempsou.y))
  {
    rnl.push_front(tempsou);
  }

  tempsou.x--;
  if (validCoordinates(tempsou.x,tempsou.y))
  {
    rnl.push_front(tempsou);
  }

  return rnl;
}

void computeBorders(int* entityMask,int height,int width,int entitiesnum,THexOwnership* centers,bool CanHaveSameCenterEntityNumber) throw (E_8K_Editor_InvalidCenters)
{
  mapHeight=height;
  mapWidth=width;
  
  // check validity of kingdom centers
  int i,j;
  for(i=0;i<entitiesnum;i++)
  {
    if (!validCoordinates(centers[i].x,centers[i].y))
    {
      THROW(E_8K_Editor_InvalidCenters,"Invalid centers coordinates.");
    }
    for(j=0;j<entitiesnum;j++)
    {
      if (i!=j)
      {
        if ((centers[i].x==centers[j].x) && (centers[i].y==centers[j].y))
        {
          THROW(E_8K_Editor_InvalidCenters,"Same centers coordinates.");
        }
        if ((!CanHaveSameCenterEntityNumber) && (centers[i].entityNumber==centers[j].entityNumber))
        {
          THROW(E_8K_Editor_InvalidCenters,"Two same entity numbers.");
        }
      }
    }
  }

  for(i=0;i<mapWidth;i++)
  {
    for(j=0;j<mapHeight;j++)
    {
      entityMask[i + j*width]=-1;
    }
  }

  THexOwnership tho;
  THexOwnership thonb;
  deque<THexOwnership> HOQueue;
  HOQueue.clear();

  for(i=0;i<entitiesnum;i++)
  {
    tho.x=centers[i].x;
    tho.y=centers[i].y;
    tho.entityNumber=centers[i].entityNumber;
    entityMask[tho.x + tho.y*width]=tho.entityNumber;
    HOQueue.push_back(tho);
  }

  list<THexOwnership> NeighboursList;
  NeighboursList.clear();

  while (!HOQueue.empty())
  {
    tho=HOQueue.front();
    NeighboursList.clear();
    NeighboursList=chooseNeighbours(tho.x,tho.y);
    while (!NeighboursList.empty())
    {
       thonb=NeighboursList.front();
       if (entityMask[thonb.x + thonb.y*width]==-1)
       {
         tho.x=thonb.x;
         tho.y=thonb.y;
         entityMask[tho.x + tho.y*width]=tho.entityNumber;
         HOQueue.push_back(tho);
       }
       NeighboursList.pop_front();
    }
    HOQueue.pop_front();
  }
}


