/**
*******************************************************************************

  @file /gui/common/ActiveObjects.cpp
  @brief okna, tlacitka, scrollbary,...
  
  @author Pavel
  @version 0.1
 
******************************************************************************/

#include "gui/common/ActiveObjects.h"

#include "gui/GUI.h"
#include "gui/common/BasicFonts.h"
#include "common/rm/rminit.h"
#include "gui/common/Sounds.h"
#include "gui/common/Defs.h"
#include "gui/engine/TextureManager.h"

#include <GL/gl.h>
#include <GL/glu.h>

namespace gui{

/******************************************************************************************/
/// definice zakladnich barev
const float bound_col_light[4] = {0.75f, 0.75f, 0.75f, 1};
const float bound_col_dark[4] = {0.25f, 0.25f, 0.25f, 1};
const float back_col[4] = {0.0f, 0.0f , 0.0f, 0.25f};
const float back_col_disabled[4] = {0.0f, 0.0f , 0.0f, 0.125f};
const float focus_col[4] = {0.0f, 0.0f , 0.0f, 0.5f};
const int ao_font = 0;

#define AO_SCROLLBAR_SIZE  19
#define AO_SCROLLBAR_STEP  1
#define ICON_BMP_SIZE  256.0f
#define CURSOR_WIDTH  32
#define CURSOR_HEIGHT  32
#define AO_HELP_DELAY 700

/******************************************************************************************/
/// pomocne promenne, funkce a makra
/// zjisti jestli je mys nad danym objektem
#define MOUSEOVERME (TaoKernel::aoKernel->parentover&&(input->mousepos.x>=x)&&(input->mousepos.y>=y)&&(x+width>input->mousepos.x)&&(y+height>input->mousepos.y))
/// nastavi souradnice bitmapy pozadi podle pozice na obrazovce
#define myTexCoord(x,y) glTexCoord2f((x)/256.0f,(y)/256.0f)

  
void glRectUp(int x, int y, int width, int height)
{
  glRecti(x,y,x+width,y+1);
  glRecti(x,y,x+1,y+height);
}
void glRectDown(int x, int y, int width, int height)
{
  glRecti(x,y+height-1,x+width,y+height);
  glRecti(x+width-1,y,x+width,y+height);
}
void glRectEmpty(int x, int y, int width, int height)
{
  glRecti(x,y,x+width,y+1);
  glRecti(x,y,x+1,y+height);
  glRecti(x,y+height-1,x+width,y+height);
  glRecti(x+width-1,y,x+width,y+height);
}

//TODO: nefunguje na samotnem souboru "testmapa11.xml".. vrati prazdny retezec!!
void getRawFile(char *dest, char *source)
{
  size_t f;
  for (f = strlen(source) - 1; f >= 0; f--)
    if ((source[f] == '\\') || (source[f] == '/')) break;
  strcpy(dest, source + f + 1);
  for (f = strlen(dest) - 1; f >= 0; f--)
    if (dest[f] == '.') {
      dest[f] = 0;
      return;
    }
}

/******************************************************************************************/

void TActiveObject::restart()
{
  state &= (AO_CHECKED | AO_SELECTED);
}

void TActiveObject::show()
{
  restart();
  active = AO_ACTIVE;
}

void TActiveObject::hide()
{
  releaseKeyboardInput();
  active = AO_PASIVE;
  restart();
}

void TActiveObject::showToggle()
{
  if (active == AO_PASIVE) show();
  else hide();
}

void TActiveObject::grabKeyboardInput()
{
  TaoKernel::aoKernel->keyboardgraber = this;
}

void TActiveObject::releaseKeyboardInput()
{
  if (TaoKernel::aoKernel->keyboardgraber == this)
    TaoKernel::aoKernel->keyboardgraber = NULL;
}

bool TActiveObject::mayIUseKeyboardInput()
{
  if ( (TaoKernel::aoKernel->keyboardgraber == this)
    || (TaoKernel::aoKernel->keyboardgraber == NULL) ) return true;
  return false;
}

void TActiveObject::setHelp(wchar_t *text)
{
  int alen, mlen, lines, i;
  TLabel *label = new TLabel(text, 1, 1);
  if (helpbar != NULL) {
    if (TaoKernel::aoKernel->activeHelpBar == helpbar) TaoKernel::aoKernel->setActiveHelpBar(NULL);
    delete helpbar;
  }
  mlen = 0;
  alen = 0;
  lines = 1;
  i = 0;
  while (text[i] != L'\0') {
    if (text[i] == L'\n') {
      if (alen > mlen) mlen = alen;
      lines++;
      alen = 0;
    }
    alen++;
    i++;
  }
  if (alen >= mlen) mlen = alen;
  helpbar = new TGroupBar(AO_NORMAL, 0, 0, 8 * mlen + 2, 2 + 16 * lines);
//  helpbar = new TGroupBar(AO_NORMAL, 0, 0, 8 * wcslen(text) + 2, 18);
  helpbar->Add(label);
}

void TActiveObject::addHelpLine(wchar_t *text)
{
  TLabel *label;
  int len;
  if (helpbar == 0) setHelp(text);
  else {
    label = new TLabel(text, 1, helpbar->height - 1);
    helpbar->Add(label);
    helpbar->height += 16;    
    len = (int)wcslen(text);
    if (len * 8 > 2 + helpbar->width) helpbar->width = 8 * len + 2;
  }
}

void TActiveObject::addHelpWord(wchar_t *text)
{
  TLabel *label;
  if (helpbar == 0) setHelp(text);
  else {
    label = new TLabel(text, helpbar->width - 1, 1);
    helpbar->Add(label);
    helpbar->width += 8 * (int)wcslen(text);
  }
}

void TActiveObject::setHelp(TBar *_helpbar)
{
  if (helpbar) {
    if (TaoKernel::aoKernel->activeHelpBar == helpbar) TaoKernel::aoKernel->setActiveHelpBar(NULL);
    delete helpbar;
  }
  helpbar = _helpbar; 
}

void TActiveObject::delHelp()
{
  if (helpbar) {
    if (TaoKernel::aoKernel->activeHelpBar == helpbar) TaoKernel::aoKernel->setActiveHelpBar(NULL);
    delete helpbar;
    helpbar = 0;
  }
}

TActiveObject::TActiveObject()
{
  helpbar = 0;
  active = AO_ACTIVE;
  state = 0;
}

TActiveObject::~TActiveObject()
{
  if (helpbar && TaoKernel::aoKernel->activeHelpBar == helpbar) TaoKernel::aoKernel->setActiveHelpBar(NULL);
  if (helpbar) delete helpbar;
  releaseKeyboardInput();
}


/******************************************************************************************/
/// mozne hodnoty _style = AO_NORMAL nebo AO_TOOLBAR
TButton::TButton(int _style, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl)
{
  x=_x;y=_y;width=_width;height=_height;
  light_col[0] = _rl;
  light_col[1] = _gl;
  light_col[2] = _bl;
  light_col[3] = 1;
  dark_col[0] = _rl/2;
  dark_col[1] = _gl/2;
  dark_col[2] = _bl/2;
  dark_col[3] = 1;
  style = _style;  
  state = 0;
  active = AO_ACTIVE;
  appearence=1;
  hwtid = 0;
  enabled = true;
  shortcutKey = (SDLKey)0;
}

TButton::TButton(int _style, int _x, int _y, int _width, int _height)
{
  x=_x;y=_y;width=_width;height=_height;
  light_col[0] = bound_col_light[0];
  light_col[1] = bound_col_light[1];
  light_col[2] = bound_col_light[2];
  light_col[3] = bound_col_light[3];
  dark_col[0] = bound_col_dark[0];
  dark_col[1] = bound_col_dark[1];
  dark_col[2] = bound_col_dark[2];
  dark_col[3] = bound_col_dark[3];
  style = _style;  
  state = 0;
  active = AO_ACTIVE;
  appearence=1;
  hwtid = 0;
  enabled = true;
  shortcutKey = (SDLKey)0;
}
/// Zpracuje jeden vstup
int TButton::workInput(INPUT *input)
{
  int ret=0;
  state &= AO_CLEAR_CLICKS;
  if (active == AO_PASIVE) return 0;
  if ((!enabled) || TaoKernel::aoKernel->cursor->wait) {
    state &= (AO_CHECKED | AO_SELECTED);
    return 0;
  }
  if ((MOUSEOVERME) && (!TaoKernel::aoKernel->ao_worked)) state |= AO_OVER; else state &= AO_NOT_OVER;
  switch (input->type){
  case INPUT_MOUSE_MOTION:
    break;
  case INPUT_MOUSE_DOWN:
    if (input->button == SDL_BUTTON_LEFT) {
      state |= AO_LEFT;
      if ((state&AO_OVER)!=0) state |= AO_FOCUS;
                else state &= AO_NOT_FOCUS;
    } else
    if (input->button == SDL_BUTTON_RIGHT) {
      state |= AO_RIGHT;
      if ((state&AO_OVER)!=0) state |= AO_FOCUS;
                else state &= AO_NOT_FOCUS;
    }
    if ((state&AO_OVER)!=0) ret = 1;
    break;
  case INPUT_MOUSE_UP:
    if (input->button == SDL_BUTTON_LEFT)
    {
      state &= AO_NOT_LEFT;
      if (((state & AO_FOCUS) != 0) && ((state & AO_OVER) != 0)) state |= AO_CLICKED;
    } else
    if (input->button == SDL_BUTTON_RIGHT)
    {
      state &= AO_NOT_RIGHT;
      if ((state & AO_OVER) != 0) state |= AO_RIGHTCLICKED;
    }
    break;
  case INPUT_KEY_DOWN:
    if ((shortcutKey > 0) && (input->key.sym == shortcutKey) && mayIUseKeyboardInput()) {
      if ((modificator == KMOD_NONE) || ((SDL_GetModState() & modificator) != 0) )
      state |= AO_CLICKED;
    }
    break;
  }
  appearence = 1;
  switch (style){
  case AO_NORMAL:
    if ((state & AO_PUSHED)==AO_PUSHED) appearence = 2;
    break;
  case AO_TOOLBAR:
    if (((state & AO_OVER) != 0) && ((state & AO_LEFT) == 0)) appearence = 2; else
    if ((state & AO_PUSHED) == AO_PUSHED) appearence = 3; else
    if ((state & (AO_FOCUS | AO_LEFT)) == (AO_FOCUS | AO_LEFT)) appearence = 2;
    break;
  }
  return ret;
}

void TButton::setShortcutKey(SDLKey _shortcutKey, wchar_t *shortcutKeyName, SDLMod _modificator)
{
  wchar_t wtemp[LABEL_MAX_LENGTH];

  shortcutKey = _shortcutKey;
  modificator = _modificator;
  if (shortcutKeyName != NULL) {
    if (helpbar) snwprintf(wtemp, LABEL_MAX_LENGTH, L" (%s)", shortcutKeyName);
      else snwprintf(wtemp, LABEL_MAX_LENGTH, L"(%s)", shortcutKeyName);
    addHelpWord(wtemp);
  } else {
    if ((shortcutKey >= SDLK_a) && (shortcutKey <= SDLK_z)) {
      if (helpbar) snwprintf(wtemp, LABEL_MAX_LENGTH, L" (%lc)", shortcutKey - SDLK_a + L'A');
        else snwprintf(wtemp, LABEL_MAX_LENGTH, L"(%lc)", shortcutKey - SDLK_a + L'A');
      addHelpWord(wtemp);
    }
  }
}

void TButton::drawObject()
{
  int i;
  if (active==AO_ACTIVE) {
    if (helpbar && MOUSEOVER(this)) TaoKernel::aoKernel->setActiveHelpBar(helpbar);
    glColor4fv(back_col);
    if (enabled) glRecti(x+1, y+1, x+width-1, y+height-1);
    switch (style){
    case AO_NORMAL:
      if (appearence == 1){
        glColor4fv(light_col);
        for (i = 0; i < 2; i++)
          glRectUp(x+i,y+i,width-2*i,height-2*i);
        glColor4fv(dark_col);
        for (i = 0; i < 2; i++)
          glRectDown(x+i,y+i,width-2*i,height-2*i);
      } else if (appearence == 2)
      {
        glColor4fv(dark_col);
        glRectUp(x,y,width,height);
        glRectDown(x,y,width,height);
      }
    break;
    case AO_TOOLBAR:
      if (appearence == 1) return;
      if (appearence == 3) glColor4fv(dark_col);
                else glColor4fv(light_col);
      for (i = 0; i < 1; i++)  glRectUp(x+i,y+i,width-2*i,height-2*i);
      if (appearence == 3) glColor4fv(light_col);
              else glColor4fv(dark_col);
      for (i = 0; i < 1; i++)  glRectDown(x+i,y+i,width-2*i,height-2*i);
      break;
    case AO_UNPRESSABLE:    
      glColor4fv(light_col);
      for (i = 0; i < 2; i++)
        glRectUp(x+i,y+i,width-2*i,height-2*i);
      glColor4fv(dark_col);
      for (i = 0; i < 2; i++)
        glRectDown(x+i,y+i,width-2*i,height-2*i);
      break;
    }  // switch (style)

  }  // if (active == AO_ACTIVE)
}

TButton::~TButton()
{
}

void TButton::move(int _x, int _y)
{
  x = _x;
  y = _y;
}

/******************************************************************************************/
TTextButton::TTextButton(int _style, wchar_t *_text, int _x, int _y, 
             int _width, int _height, float _rl, float _gl, float _bl) :
              TButton(_style, _x, _y, _width, _height, _rl, _gl, _bl)
{
  label = new TLabel(_text, _x+2, _y+2);
}

TTextButton::TTextButton(int _style, wchar_t *_text, int _x, int _y, 
             int _width, int _height) :
              TButton(_style, _x, _y, _width, _height)
{
  label = new TLabel(_text, x+(width-(int)wcslen(_text)*getFontWidth(ao_font))/2,
    y+(height-getFontHeight(ao_font))/2);
}

TTextButton::~TTextButton()
{
  delete label;
}

void TTextButton::move(int _x, int _y)
{
  label->x += (_x - x);
  label->y += (_y - y);
  TButton::move(_x, _y);
}

void TTextButton::drawObject()
{
  if (active == AO_PASIVE) return;

  TButton::drawObject();
  if PUSHED(this) {
    label->x++;
    label->y++;
  }
  label->drawObject();
  if PUSHED(this) {
    label->x--;
    label->y--;
  }

  if (!enabled) {
    glColor4fv(back_col_disabled);
    glRecti(x+1, y+1, x+width-1, y+height-1);
  }
}

/******************************************************************************************/

TSoundedTextButton::TSoundedTextButton(int _style, wchar_t *_text, int _x, int _y, 
                                       int _width, int _height, 
                                       int _onclick_sample)
:TTextButton(_style, _text, _x, _y, _width, _height)
{
  onclick_sample = _onclick_sample;
}

int TSoundedTextButton::workInput(INPUT *input){
  int ret = TButton::workInput(input);

  switch (input->type){
    case INPUT_MOUSE_DOWN:
      if(((state&AO_OVER)!=0) && TSoundInterface::si)
        TSoundInterface::si->playSample(onclick_sample); 
    break;
  }
  return ret;
}


/******************************************************************************************/
TIconButton::TIconButton(int _style, rm::TSymbolData* _data, int _x, int _y, 
              int _width, int _height, float _rl, float _gl, float _bl) :
              TButton(_style, _x, _y, _width, _height, _rl, _gl, _bl)
{
  //do prostred tlacitka
  icon = new TIcon(_data, x+(width-_data->width)/2,
                          y+(height-_data->height)/2 );
}

TIconButton::TIconButton(int _style, rm::TSymbolData* _data, int _x, int _y, 
              int _width, int _height) :
              TButton(_style, _x, _y, _width, _height)
{
  //umisti doprostred tlacitka
  icon = new TIcon(_data, x+(width-_data->width)/2,
                          y+(height-_data->height)/2 );
}

void TIconButton::drawObject()
{
  if (active == AO_PASIVE) return;
  TButton::drawObject();
  if PUSHED(this) {
    icon->x++;
    icon->y++;
  }
  icon->drawObject();
  if PUSHED(this) {
    icon->x--;
    icon->y--;
  }

  if (!enabled) {
    glColor4fv(back_col_disabled);
    glRecti(x+1, y+1, x+width-1, y+height-1);
  }
}

TIconButton::~TIconButton()
{
  delete icon;
}
/******************************************************************************************/
void TLabel::drawObject()
{
  if (active == AO_PASIVE) return;
  glColor4fv(light_col);
    SelectFont(font);
  DrawText(TaoKernel::aoKernel->parentpos.x+x,SCREEN_HEIGHT-(TaoKernel::aoKernel->parentpos.y+y)-getFontHeight(font),text);
}

int TLabel::workInput(INPUT *input)
{
  return 0;
}

TLabel::TLabel(wchar_t *_text, int _x, int _y, float _rl, float _gl, float _bl)
{
  x=_x;y=_y;width=0;height=0;
  light_col[0] = _rl;
  light_col[1] = _gl;
  light_col[2] = _bl;
  light_col[3] = 1;
  state = 0;
  font = ao_font;
  active = AO_ACTIVE;

  wcsncpy(text, _text, LABEL_MAX_LENGTH);
  text[LABEL_MAX_LENGTH] = L'\0';
}

void TLabel::setColor(float r, float g, float b, float alfa)
{
  light_col[0] = r;
  light_col[1] = g;
  light_col[2] = b;
  light_col[3] = alfa;
}

TLabel::TLabel(wchar_t *_text, int _x, int _y)
{
  x=_x;y=_y;width=0;height=0;
  light_col[0] = bound_col_light[0];
  light_col[1] = bound_col_light[1];
  light_col[2] = bound_col_light[2];
  light_col[3] = bound_col_light[3];
  state = 0;
  font = ao_font;
  active = AO_ACTIVE;

  wcsncpy(text, _text, LABEL_MAX_LENGTH);
  text[LABEL_MAX_LENGTH] = L'\0';
}

TLabel::TLabel(wchar_t *_text, int _x, int _y, int _w, int _h, int _alignment){
  x=_x;y=_y;width=0;height=0;
  light_col[0] = bound_col_light[0];
  light_col[1] = bound_col_light[1];
  light_col[2] = bound_col_light[2];
  light_col[3] = bound_col_light[3];
  state = 0;
  font = ao_font;
  active = AO_ACTIVE;

  wcsncpy(text, _text, LABEL_MAX_LENGTH);
  text[LABEL_MAX_LENGTH] = L'\0';

  //uprav x, y podle zarovnani
  //TODO: ruzne zpusoby zarovnani(vlevo, vpravo, nastred, preteceni, zalomeni,..
  if(_alignment){
    int realw, realh;
    GetTextDim ( text, &realw, &realh );
    //
    x += (_w - realw)/2;
    y += (_h - realh)/2;   
  }
}

TLabel::TLabel(int _x, int _y, wchar_t *format,...)
{
  x=_x;y=_y;width=0;height=0;
  light_col[0] = bound_col_light[0];
  light_col[1] = bound_col_light[1];
  light_col[2] = bound_col_light[2];
  light_col[3] = bound_col_light[3];
  state = 0;
  font = ao_font;
  active = AO_ACTIVE;

  //
  va_list args;
  va_start( args, format );
  //
  vsnwprintf( text, LABEL_MAX_LENGTH ,format, args );
}


/******************************************************************************************/

TIcon::TIcon(rm::TSymbolData* _data, int _x, int _y)
{
  //screen pos
  x = _x;
  y = _y;
  data = NULL;
  width = 0;
  height = 0;
    
  //nastav ikonu
  setIcon(_data);  
  
  //nastav barvu
  setColor(1, 1, 1);
}

void TIcon::drawObject()
{
  if (active == AO_PASIVE) return;
  if (helpbar && MOUSEOVER(this)) TaoKernel::aoKernel->setActiveHelpBar(helpbar);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, hwtidIconTexture);
  glColor3f(cr,cg,cb);
  glBegin(GL_QUADS);
    glTexCoord2f(tx1, ty1);
    glVertex2i(x, y);
    glTexCoord2f(tx2, ty1);
    glVertex2i(x+width,y);
    glTexCoord2f(tx2, ty2);
    glVertex2i(x+width, y+height);
    glTexCoord2f(tx1, ty2);
    glVertex2i(x, y+height);
  glEnd();
  glDisable(GL_TEXTURE_2D);
}

int TIcon::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  switch (input->type){
    case INPUT_MOUSE_MOTION:
      if ((MOUSEOVERME) && (!TaoKernel::aoKernel->ao_worked)) state |= AO_OVER; 
        else state &= AO_NOT_OVER;
      break;
  }
  return 0;
}

void TIcon::setIcon(rm::TSymbolData* _data)
{
  if(data == _data)return;
  width = _data->width;
  height = _data->height; 
  data = _data;

  //raster
  rm::TRM_raster_i* rmr = (rm::TRM_raster_i*) KSendGlobalMessage(MSG_GET_RM_RASTER_I,MOD_GUI,MOD_RM,NULL);
  rm::TRaster* raster = rmr->get(_data->rasterId);
  delete rmr;

  //texture pos;
  tx1 = ((float)_data->x)/raster->surface->w;
  ty1 = ((float)_data->y)/raster->surface->h;
  tx2 = tx1 + ((float)width)/raster->surface->w;
  ty2 = ty1 + ((float)height)/raster->surface->h;

  //textura
  hwtidIconTexture = 0xFFFFFFFF;
  TTextureManager::tm->loadTextureQuery(_data->rasterId, &hwtidIconTexture, TEXFLAG_PRIORITY_NORMAL, GL_RGBA );
//  if(hwtidIconTexture == 0xFFFFFFFF)TextureManager->loadTextures();
}

void TIcon::setColor(float r, float g, float b)
{
  cr=r;cg=g;cb=b;
}

/******************************************************************************************/

TIconExt::TIconExt( rm::TSymbolData *_data, int _x, int _y, 
                    float bg_r, float bg_g, float bg_b,
                    int _border, float b_r, float b_b, float b_g)                                
:TIcon(_data, _x, _y)
{
  border = _border;
  //
  bg[0] = bg_r;
  bg[1] = bg_g;
  bg[2] = bg_b;
  //
  border_col[0] = b_r;
  border_col[1] = b_g;
  border_col[2] = b_b;
}

void TIconExt::drawObject()
{ 
  glColor3fv(bg);
  glBegin(GL_QUADS);
    glVertex2i(x, y);
    glVertex2i(x+width,y);
    glVertex2i(x+width, y+height);
    glVertex2i(x, y+height);
  glEnd();  
  
  TIcon::drawObject();
  
  glColor3fv(border_col);
  glLineWidth((GLfloat)border);
  glBegin(GL_LINE_LOOP);
    glVertex2i(x, y);
    glVertex2i(x+width,y);
    glVertex2i(x+width, y+height);
    glVertex2i(x, y+height);
  glEnd();  
  glLineWidth(1);
}

/******************************************************************************************/
void TBar::Add(TActiveObject* ao){
  vActObj.push_back(ao);
}

int TBar::Del(TActiveObject* ao){
  TvActObj::iterator it;
  it=vActObj.begin();
  while(it!=vActObj.end()){
    if(*it==ao){
      vActObj.erase(it);
      return 0;
    }
    it++;
  }
  return 1;
}

void TBar::setForground(TActiveObject* ao)
{
  Del(ao);
  Add(ao);
}


void TBar::drawObject()
{
  if (active == AO_PASIVE) return;
  if (helpbar && MOUSEOVER(this)) TaoKernel::aoKernel->setActiveHelpBar(helpbar);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glTranslatef((float)x,(float)y,0);
  TaoKernel::aoKernel->parentpos.x+=x;
  TaoKernel::aoKernel->parentpos.y+=y;
  for(TvActObj::iterator it= vActObj.begin();it!=vActObj.end();it++)
    (*it)->drawObject();
  glPopMatrix();
  TaoKernel::aoKernel->parentpos.x-=x;
  TaoKernel::aoKernel->parentpos.y-=y;
}

int TBar::workInput(INPUT *input)
{
  bool grabedKey = false;
  if (active == AO_PASIVE) return 0;
  if ((!TaoKernel::aoKernel->isKeyboardFree()) &&
    mayIUseKeyboardInput() )
  {
    grabedKey = true;
    releaseKeyboardInput();
  }
  int ret=0, i, _po=TaoKernel::aoKernel->parentover, _state=0;
  TaoKernel::aoKernel->parentover=MOUSEOVERME;
  TActiveObject* ao;
  if (vActObj.begin() != vActObj.end()){
    input->mousepos.x-=x;
    input->mousepos.y-=y;

    TvActObj::iterator it = vActObj.end(); it--;
    for(;it!=vActObj.begin();it--) {
      i = (*it)->workInput(input);
      if (i!=0){
        ao = *it;
        ret = i;
      }
      if (CLICKED((*it))) {
        _state = _state | AO_CLICKED;
      }
    }
    i = (*vActObj.begin())->workInput(input);
    if (i != 0){
      ao = *vActObj.begin();
      ret = i;
    }
    if (CLICKED((*vActObj.begin()))) _state = _state | AO_CLICKED;
    if (ret == 2){
      _state |= AO_CHANGED;
      for(TvActObj::iterator it= vActObj.begin();it!=vActObj.end();it++){
        if ((*it != ao)) (*it)->state &= (255-AO_SELECTED);
      }
      ret=0;
    }
    input->mousepos.x+=x;
    input->mousepos.y+=y;
  }
  TaoKernel::aoKernel->parentover=_po;
  if (!ret) TButton::workInput(input);
  state |= _state;
  if (grabedKey) grabKeyboardInput();
  return ret;  
}

TBar::TBar(int _style, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl):
              TButton(_style, _x, _y, _width, _height, _rl, _gl, _bl){}

TBar::TBar(int _style, int _x, int _y, int _width, int _height):
              TButton(_style, _x, _y, _width, _height){}

TBar::~TBar()
{
  for(TvActObj::iterator it= vActObj.begin();it!=vActObj.end();it++) delete (*it);
  while (vActObj.begin() != vActObj.end()) vActObj.erase(vActObj.begin());
}

void TBar::DelNFreeAll()
{
  for(TvActObj::iterator it= vActObj.begin();it!=vActObj.end();it++) delete (*it);
  while (vActObj.begin() != vActObj.end()) vActObj.erase(vActObj.begin());
}

void TBar::restart()
{
  for (TvActObj::iterator it = vActObj.begin(); it!=vActObj.end(); it++) 
    (*it)->restart();
  TButton::restart();

}

/******************************************************************************************/
void TWindow::drawObject()
{
  if (active == AO_PASIVE) return;
  glColor4fv(light_col);
  glRectEmpty(x,y,width,height);
  if (hwtid == 0){
    glColor4fv(back_col);
    glRecti(x+1, y+1, x+width-1, y+height-1);
  } else {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, hwtid);
    glColor3f(1,1,1);
    glBegin(GL_QUADS);
    if (style == AO_MOVABLE){
      myTexCoord(1, 1);
      glVertex2i(x+1,y+1);
      myTexCoord(width-1, 1);
      glVertex2i(x+width-1,y+1);
      myTexCoord(width-1, height-1);
      glVertex2i(x+width-1,y+height-1);
      myTexCoord(1, height-1);
      glVertex2i(x+1,y+height-1);
    } else {
      myTexCoord(TaoKernel::aoKernel->parentpos.x+x+1,TaoKernel::aoKernel->parentpos.y+y+1);
      glVertex2i(x+1,y+1);
      myTexCoord(TaoKernel::aoKernel->parentpos.x+x+width-1,TaoKernel::aoKernel->parentpos.y+y+1);
      glVertex2i(x+width-1,y+1);
      myTexCoord(TaoKernel::aoKernel->parentpos.x+x+width-1,TaoKernel::aoKernel->parentpos.y+y+height-1);
      glVertex2i(x+width-1,y+height-1);
      myTexCoord(TaoKernel::aoKernel->parentpos.x+x+1,TaoKernel::aoKernel->parentpos.y+y+height-1);
      glVertex2i(x+1,y+height-1);
    }
    glEnd();
    glDisable(GL_TEXTURE_2D);
  }
  TBar::drawObject();  
  
}

int TWindow::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  if ((state&(AO_LEFT|AO_FOCUS))!=(AO_LEFT|AO_FOCUS)) {  
    click.x = input->mousepos.x-x;
    click.y = input->mousepos.y-y;
  }
  int ret = TBar::workInput(input);
  if (MOUSEOVERME) {
    if ((input->type == INPUT_MOUSE_DOWN) && (!TaoKernel::aoKernel->ao_worked)) ret = 1;
    TaoKernel::aoKernel->ao_worked=1;
  }
  if ((style==AO_MOVABLE)&&(state&(AO_LEFT|AO_FOCUS))==(AO_LEFT|AO_FOCUS)){
    x = input->mousepos.x-click.x;
    y = input->mousepos.y-click.y;
  }
  return ret;
}

/// konstruktor, mozne hodnoty _style = AO_MOVABLE, AO_UNMOVABLE
TWindow::TWindow(int _style, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl):
              TBar(_style, _x, _y, _width, _height, _rl, _gl, _bl){}

TWindow::TWindow(int _style, int _x, int _y, int _width, int _height):
              TBar(_style, _x, _y, _width, _height){}

TWindow::~TWindow()
{
  if (TaoKernel::aoKernel->modalWindow == this) TaoKernel::aoKernel->modalWindow = NULL;
}

void TWindow::setTexture(HWTID _hwtid)
{
  hwtid = _hwtid;
}

void TWindow::setBgImage(int bid){
  TTextureManager::tm->loadTextureQuery(bid, &hwtid);
}

void TWindow::showModal()
{
  if (TaoKernel::aoKernel->modalWindow != NULL) TaoKernel::aoKernel->modalWindow->hide();
  TaoKernel::aoKernel->modalmod = false;
  grabKeyboardInput();
  TaoKernel::aoKernel->modalWindow = this;
  show();
}

void TWindow::hide()
{
  if (TaoKernel::aoKernel->modalWindow == this) TaoKernel::aoKernel->modalWindow = NULL;
  TActiveObject::hide();
}


/******************************************************************************************/
/// konstruktor, mozne hodnoty _checked = AO_YES, AO_NO
TCheckBox::TCheckBox(int _checked, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl):
              TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdCross, _x, _y, _width, _height, _rl, _gl, _bl)
{
  if (_checked != 0) _checked = 1;
  state = AO_CHECKED*_checked;
}

TCheckBox::TCheckBox(int _checked, int _x, int _y, int _width, int _height):
              TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdCross, _x, _y, _width, _height)
{
  if (_checked != 0) _checked = 1;
  state = AO_CHECKED*_checked;
}

int TCheckBox::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  int ret = TButton::workInput(input);
  if (CLICKED(this)) state = state ^ AO_CHECKED;
  return ret;
}

void TCheckBox::drawObject()
{
  if (active == AO_PASIVE) return;
  glColor4fv(light_col);
  glRectDown(x,y,width,height);
  glColor4fv(dark_col);
  glRectUp(x,y,width,height);
  if (PUSHED(this)){
    glColor4fv(back_col);
    glRecti(x+1,y+1,x+width-1,y+height-1);
  }
  if ((state&AO_CHECKED)==AO_CHECKED) {
    icon->drawObject();
  }
}
/******************************************************************************************/
/// konstruktor, mozne hodnoty _checked = AO_YES, AO_NO

TRadioBox::TRadioBox(int _selected, int _x, int _y, int _width, int _height)
: TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdRadioUnchecked, _x, _y, _width, _height)
{
  if (_selected == AO_UNSELECTABLE) {
    style = AO_UNSELECTABLE; 
    _selected = 0;
  } else style = AO_NORMAL;

  itselected = TaoKernel::aoKernel->sdRadioChecked;
  itover = TaoKernel::aoKernel->sdRadioUnchecked;
  itnone = TaoKernel::aoKernel->sdRadioUnchecked;

  if (_selected != 0) _selected = 1;
  state = AO_SELECTED * _selected;
  if (_selected) icon->setIcon(itselected);
}

void TRadioBox::setState(int newstate)
{
  if (newstate == 0) {
    state = 0;
    icon->setIcon(itnone);
  } else {
    state = AO_SELECTED;
    icon->setIcon(itselected);
  }
}


TRadioBox::TRadioBox(int _selected, int _x, int _y, int _width, int _height,
    rm::TSymbolData* _itnone, rm::TSymbolData* _itselected, rm::TSymbolData* _itover)
:  TIconButton(AO_NORMAL, _itnone, _x, _y, _width, _height)
{
  if (_selected == AO_UNSELECTABLE) {
    style = AO_UNSELECTABLE; 
    _selected = 0;
  } else style = AO_NORMAL;

  itselected = _itselected;
  itover = _itover;
  itnone = _itnone;
  
  if (_selected != 0) _selected = 1;
  state = AO_SELECTED * _selected;
  if (_selected) icon->setIcon(itselected);

}


int TRadioBox::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  int ret = TButton::workInput(input);
  if (CLICKED(this) && (style == AO_NORMAL)) {
    state |= AO_SELECTED;
    return 2;
  }

  return ret;
}

void TRadioBox::drawObject()
{
  if (active == AO_PASIVE) return;
  if (helpbar && MOUSEOVER(this)) TaoKernel::aoKernel->setActiveHelpBar(helpbar);

  if SELECTED(this) icon->setIcon(itselected);
    else if PUSHED(this) icon->setIcon(itover);
      else icon->setIcon(itnone);
  
  icon->drawObject();
      /*  glColor3f(1,1,1);
  glBegin(GL_LINES);
    glColor4fv(dark_col);
    glVertex2i(x+width/2,y);
    glVertex2i(x,y+height/2);
    glColor4fv(light_col);
    glVertex2i(x+width/2,y);
    glVertex2i(x+width,y+height/2);
    glColor4fv(dark_col);
    glVertex2i(x+width/2,y+height);
    glVertex2i(x,y+height/2);
    glColor4fv(light_col);
    glVertex2i(x+width/2,y+height);
    glVertex2i(x+width,y+height/2);
  glEnd();

  if (PUSHED(this)){
    glBegin(GL_POLYGON);
      glColor4fv(dark_col);  
      //glColor3f(2*rd/3,2*gd/3,2*bd/3);
      glVertex2i(x+width/2,y+1);
      glVertex2i(x+1,y+height/2);
      glVertex2i(x+width/2,y+height-1);
      glVertex2i(x+width,y+height/2);
    glEnd();
  }
  if ((state&AO_CHECKED)==AO_CHECKED) {
    glBegin(GL_POLYGON);
      glColor4fv(light_col);
      glVertex2i(x+width/2,y+2);
      glVertex2i(x+3,y+height/2);
      glVertex2i(x+width/2,y+height-2);
      glVertex2i(x+width-2,y+height/2);
    glEnd();
  }*/
}
/******************************************************************************************/
void TGroupBar::drawObject()
{
  if (active == AO_PASIVE) return;
  if (style != AO_HIDDEN){
    glColor4fv(dark_col);
    glRectUp(x,y,width,height);
    glRectDown(x,y,width,height);
    glColor4fv(back_col);
    glRecti(x+1, y+1, x+width-1, y+height-1);
  }
  TBar::drawObject();  
}

/// konstruktor, mozne hodnoty _style = AO_NORMAL, AO_HIDDEN
TGroupBar::TGroupBar(int _style, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl):
              TBar(_style, _x, _y, _width, _height, _rl, _gl, _bl){}

TGroupBar::TGroupBar(int _style, int _x, int _y, int _width, int _height):
              TBar(_style, _x, _y, _width, _height){}
/******************************************************************************************/
/// konstruktor, mozne hodnoty _type = AO_HORIZONTAL, AO_VERTICAL
TScrollBar::TScrollBar(int _type, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl):
              TBar(AO_HIDDEN, _x, _y, _width, _height, _rl, _gl, _bl)
{
  orient = _type;
  Range = 100;
  Pos = 0;
  Size = 100;
  if (orient == AO_HORIZONTAL){
    // obrazek sipky doleva
    A = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdLeftArrow, 0, 0,_height,_height, _rl, _gl, _bl);
    // obrazek sipky doprava
    B = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdRightArrow , _width-_height, 0,_height,_height, _rl, _gl, _bl);
    M = new TScrollButton(AO_HORIZONTAL, _height, 0, 5, _height, _height, _width-2*_height, _rl, _gl, _bl);
  } else
  if (orient == AO_VERTICAL){
    // obrazek sipky nahoru
    A = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdUpArrow, 0, 0,_width,_width, _rl, _gl, _bl);
    // obrazek sipky dolu
    B = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdDownArrow, 0, _height-_width,_width,_width, _rl, _gl, _bl);
    M = new TScrollButton(AO_VERTICAL, 0, _width, _width, 5, _width, _height-2*_width, _rl, _gl, _bl);
  }
  CompSize();
  Add(A);
  Add(B);
  Add(M);
}

TScrollBar::TScrollBar(int _type, int _x, int _y, int _width, int _height):
              TBar(AO_HIDDEN, _x, _y, _width, _height)
{
  orient = _type;
  Range = 100;
  Pos = 0;
  Size = 100;

  if (orient == AO_HORIZONTAL){
    // obrazek sipky doleva
    A = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdLeftArrow, 0, 0,_height,_height);
    // obrazek sipky doprava
    B = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdRightArrow, _width-_height, 0,_height,_height);
    M = new TScrollButton(AO_HORIZONTAL, _height, 0, 5, _height, _height, _width-2*_height);
  } else
  if (orient == AO_VERTICAL){
    // obrazek sipky nahoru
    A = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdUpArrow, 0, 0,_width,_width);
    // obrazek sipky dolu
    B = new TIconButton(AO_NORMAL, TaoKernel::aoKernel->sdDownArrow, 0, _height-_width,_width,_width);
    M = new TScrollButton(AO_VERTICAL, 0, _width, _width, 5, _width, _height-2*_width);
  }
  CompSize();
  Add(A);
  Add(B);
  Add(M);
}

void TScrollBar::drawObject()
{
  if (active == AO_PASIVE) return;
  if (orient == AO_HORIZONTAL)
  {
    glColor4fv(dark_col);
    glBegin(GL_LINES);
      glVertex2i(x+height,y+1);
      glVertex2i(x+width-height,y+1);
      glVertex2i(x+height,y+height);
      glVertex2i(x+width-height,y+height);
    glEnd();
  }
  if (orient == AO_VERTICAL)
  {
    glColor4fv(dark_col);
    glBegin(GL_LINES);
      glVertex2i(x+1,y+width);
      glVertex2i(x+1,y+height-width);
      glVertex2i(x+width,y+width);
      glVertex2i(x+width,y+height-width);
    glEnd();
  }
  TBar::drawObject();
}

int TScrollBar::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  int _sp,sp,ret;
  if (orient == AO_HORIZONTAL) _sp = M->x-height;
  if (orient == AO_VERTICAL) _sp = M->y-width;
  ret = TBar::workInput(input);
  if (MOUSEOVERME) {
    if ((input->type == INPUT_MOUSE_DOWN) && (!TaoKernel::aoKernel->ao_worked)) ret = 1;
  }
  if (orient == AO_HORIZONTAL) {
    sp = M->x-height;
    if (sp != _sp) 
    {
      Pos = ((M->x-height)*(Range-Size+1))/(width-2*height-M->width+1);
      if (sp == width-2*height-M->width) Pos = Range-Size;
      CompPos();
    }
  }
  if (orient == AO_VERTICAL) {
    sp = M->y-width;
    if (sp != _sp) 
    {
      Pos = ((M->y-width)*(Range-Size+1))/(height-2*width-M->height+1);
      if (sp == height-2*width-M->height) Pos = Range-Size;
      CompPos();
    }
  }
  if ( (PUSHED(A) || PUSHED(B)) && (input->type== INPUT_NO) ){
    if (PUSHED(A) && (Pos > 0)) Pos -= AO_SCROLLBAR_STEP;
    if (PUSHED(B) && (Pos < Range-Size)) {
      Pos += AO_SCROLLBAR_STEP;
      if (Pos >= Range - Size) Pos = Range-Size;
    }
    if (Pos < 0 ) Pos = 0;
    CompPos();
  }
  return ret;
}

void TScrollBar::CompSize()
{
  int swp;
  if (orient == AO_HORIZONTAL)
  {
    M->height = height;
    if (Range > Size){
      swp = ((width-2*height)*Size)/Range;
      if (swp < 5) swp = 5;
      M->width = swp;
    } else  M->width=width-2*height;
  }
  if (orient == AO_VERTICAL)
  {
    M->width = width;
    if (Range > Size){
      swp = ((height-2*width)*Size)/Range;
      if (swp < 5) swp = 5;
      M->height = swp;
    } else M->height=height-2*width;
  }
}

// predpoklada spravne nastavenou size
void TScrollBar::CompPos()
{
  if (orient == AO_HORIZONTAL)
  {
    M->y = 0;
    if (Range <= Size) M->x=height; else
    M->x = height+((width-2*height-M->width)*Pos)/(Range-Size);
  }
  if (orient == AO_VERTICAL)
  {
    M->x = 0;
    if (Range <= Size) M->y=width; else
    M->y = width+((height-2*width-M->height)*Pos)/(Range-Size);
  }
}

void TScrollBar::ChangeTo(int _Range, int _Size, int _Pos)
{
  if (_Range != AO_NOCHANGE) Range = _Range;
  if (_Size != AO_NOCHANGE) Size = _Size;
  if (_Pos != AO_NOCHANGE) Pos = _Pos;
  if (Pos > Range-Size) Pos = Range-Size;
  if (Range <= Size) Pos = 0;
  CompSize();
  CompPos();
}

int TScrollBar::range()
{
  return Range;
}

int TScrollBar::pos()
{
  return Pos;
}

int TScrollBar::size()
{
  return Size;
}
/******************************************************************************************/
TScrollButton::TScrollButton(int _type, int _x, int _y, int _width, int _height, int _winstart, int _winsize,
               float _rl, float _gl, float _bl):
              TButton(AO_UNPRESSABLE, _x, _y, _width, _height, _rl, _gl, _bl)
{
  orient = _type;
  winstart = _winstart;
  winsize = _winsize;
}

TScrollButton::TScrollButton(int _type, int _x, int _y, int _width, int _height, int _winstart, int _winsize):
              TButton(AO_UNPRESSABLE, _x, _y, _width, _height)
{
  orient = _type;
  winstart = _winstart;
  winsize = _winsize;
}

int TScrollButton::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  int i;
  if ((state&(AO_LEFT|AO_FOCUS))!=(AO_LEFT|AO_FOCUS)) {  
    click.x = input->mousepos.x-x;
    click.y = input->mousepos.y-y;
  }
  int ret=TButton::workInput(input);
  if ((state&(AO_LEFT|AO_FOCUS))==(AO_LEFT|AO_FOCUS)){
    if (orient == AO_HORIZONTAL){
      i = input->mousepos.x-click.x;
      if (i < winstart) i = winstart;
      if (i + width > winstart+winsize) i = winstart+winsize-width;
      x = i;
    }
    if (orient == AO_VERTICAL){
      i = input->mousepos.y-click.y;
      if (i < winstart) i = winstart;
      if (i + height > winstart+winsize) i = winstart+winsize-height;
      y = i;
    }
  }
  return ret;

}
/******************************************************************************************/
/// konstruktor, mozne hodnoty _style = AO_NORMAL
TListBar::TListBar(int _style, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl):
  TBar(_style, _x, _y, _width, _height, _rl, _gl, _bl)
{
  scroll = new TScrollBar(AO_VERTICAL,_width-AO_SCROLLBAR_SIZE-1,1,AO_SCROLLBAR_SIZE,_height-2,_rl,_gl,_bl);
  bar = new TBar(AO_HIDDEN,1,1,_width-AO_SCROLLBAR_SIZE-2,1,_rl,_gl,_bl);
  scroll->ChangeTo(0,_height-2,0);
  TBar::Add(scroll);
  TBar::Add(bar);
  pixellength=0;
}

TListBar::TListBar(int _style, int _x, int _y, int _width, int _height):
  TBar(_style, _x, _y, _width, _height)
{
  scroll = new TScrollBar(AO_VERTICAL,_width-AO_SCROLLBAR_SIZE-1,1,AO_SCROLLBAR_SIZE,_height-2);
  bar = new TBar(AO_HIDDEN,1,1,_width-AO_SCROLLBAR_SIZE-2,1);
  scroll->ChangeTo(0,_height-2,0);
  TBar::Add(scroll);
  TBar::Add(bar);
  pixellength=0;
}

void TListBar::drawObject()
{
  if (active == AO_PASIVE) return;
  if (style != AO_HIDDEN){
    glColor4fv(dark_col);
    glRectUp(x,y,width,height);
    glRectDown(x,y,width,height);
    glColor4fv(back_col);
    glRecti(x + 1, y + 1, x + width - 1 - AO_SCROLLBAR_SIZE ,y + height - 1);

  }
  bar->y=1-scroll->pos();

  glEnable(GL_SCISSOR_TEST);
  glScissor(TaoKernel::aoKernel->parentpos.x+x+1,SCREEN_HEIGHT-(TaoKernel::aoKernel->parentpos.y+y+1+height-2),width-2, height-2);
  TBar::drawObject();
  glDisable(GL_SCISSOR_TEST);

}

int TListBar::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  int ret=TBar::workInput(input);
  if (MOUSEOVERME) {
    if ((input->type == INPUT_MOUSE_DOWN) && (!TaoKernel::aoKernel->ao_worked)) ret = 1;
  }
  if CHANGED(bar) state |= AO_CHANGED;
  return ret;
}


void TListBar::Add(TActiveObject* ao)
{
  ao->width=width-2-AO_SCROLLBAR_SIZE;
  ao->x=0;
  ao->y=pixellength;
  pixellength+=ao->height;
  bar->Add(ao);
  bar->height=pixellength;
  scroll->ChangeTo(pixellength,AO_NOCHANGE, AO_NOCHANGE);
}

int TListBar::Del(TActiveObject* ao)
{
  if (bar->Del(ao) == 0){
    TvActObj::iterator it;
    it=bar->vActObj.begin();
    while(it!=bar->vActObj.end()){
      if((*it)->y > ao->y) (*it)->y -= ao->height;
      it++;
    }
    pixellength -= ao->height;
    scroll->ChangeTo(pixellength,AO_NOCHANGE, AO_NOCHANGE);
    return 0;
  }
  return 1;
}

int TListBar::getSelection()
{
  TvActObj::iterator it;
  int i = 0;

  for (it = bar->vActObj.begin();  it!=bar->vActObj.end(); it++)
  {
    if (SELECTED((*it))) return i;
    i++;
  }
  return -1;  
}

TActiveObject* TListBar::getSelectedObject(){
  TvActObj::iterator it;

  for (it = bar->vActObj.begin();  it!=bar->vActObj.end(); it++)
  {
    if (SELECTED((*it))) return *it;
  }
  return NULL;  
}

void TListBar::DelNFreeAll()
{
  pixellength=0;
  scroll->ChangeTo(0,height-2,0);
  bar->DelNFreeAll();
}

void TListBar::deselectAll()
{
  TvActObj::iterator it;
  for (it = bar->vActObj.begin();  it!=bar->vActObj.end(); it++) 
    (*it)->state &= AO_NOT_SELECTED;
}

/******************************************************************************************/
/// konstruktor, mozne hodnoty _checked = AO_YES, AO_NO
TItem::TItem(int _selected, int _height, float _rl, float _gl, float _bl)
      : TBar(AO_HIDDEN, 0, 0, 1, _height, _rl, _gl, _bl)
{
  if (_selected == AO_UNSELECTABLE) {
    style = AO_UNSELECTABLE; 
    _selected = 0;
  } else style = AO_NORMAL;
  if (_selected != 0) _selected = 1;
  state = AO_SELECTED*_selected;
}

TItem::TItem(int _selected, int _height)
      : TBar(AO_HIDDEN, 0, 0, 1, _height)
{
  if (_selected == AO_UNSELECTABLE) {
    style = AO_UNSELECTABLE; 
    _selected = 0;
  } else style = AO_NORMAL;
  if (_selected != 0) _selected = 1;
  state = AO_SELECTED*_selected;
}

void TItem::setState(int newstate)
{
  if (newstate == 0) state = 0;
  else state = AO_SELECTED;
}

void TItem::Add(TActiveObject* ao){
  if(ao->x+ao->width > width)
    width = ao->x+ao->width;
  if(ao->y+ao->height > height)
    height = ao->y+ao->height;
    
  TBar::Add(ao);
}

int TItem::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  int ret = TBar::workInput(input);
  if ( (CLICKED(this)) && (style == AO_NORMAL) ) {
    state = state | AO_SELECTED;
    return 2;
  }
  return ret;
}

void TItem::drawObject()
{
  if (active == AO_PASIVE) return;
/*  glColor4fv(back_col);
  glRecti(x,y,x+width,y+height);*/
  if ((state&AO_SELECTED)==AO_SELECTED) {
    glColor4fv(back_col);
    glRecti(x,y,x+width,y+height);
  }
  TBar::drawObject();
}

/******************************************************************************************/

TLineTextField::TLineTextField(int _x, int _y, int _maxlength) : TButton(AO_NORMAL, _x, _y, 0, 0)
{
  maxlength = _maxlength;
  if (maxlength > LABEL_MAX_LENGTH) maxlength = LABEL_MAX_LENGTH;
  label = new TLabel(L"", _x+4, _y+2);
  editorpos=0;
  height = getFontHeight(label->font)+4;
  width = getFontWidth(label->font)*maxlength+8;
  hidden = false;
}

TLineTextField::TLineTextField(int _x, int _y, int _maxlength, float _rl, float _gl, float _bl)
: TButton(AO_NORMAL, _x, _y, 0, 0, _rl, _gl, _bl)
{
  maxlength = _maxlength;
  if (maxlength > LABEL_MAX_LENGTH) maxlength = LABEL_MAX_LENGTH;
  label = new TLabel(L"", _x+4, _y+2, _rl, _gl, _bl);
  editorpos=0;
  height = getFontHeight(label->font)+4;
  width = getFontWidth(label->font)*maxlength+8;
  hidden = false;
}

void TLineTextField::drawObject()
{
  if (!hidden) {
    glColor4fv(light_col);
    glRectDown(x,y,width,height);
    glColor4fv(dark_col);
    glRectUp(x,y,width,height);
  }
  glColor4fv(back_col);
  if FOCUSED(this) glColor4fv(focus_col);
  if (!hidden) glRecti(x+1,y+1,x+width-1,y+height-1);
  label->drawObject();
  if FOCUSED(this) glRecti(label->x+editorpos*getFontWidth(label->font),
    label->y, label->x+editorpos*getFontWidth(label->font)+1, label->y+getFontHeight(label->font));
}

int TLineTextField::workInput(INPUT *input)
{
  static wchar_t temp[LABEL_MAX_LENGTH+1];

  int ret = TButton::workInput(input);
  if CLICKED(this) {
    editorpos = (input->mousepos.x+4-x)/getFontWidth(label->font)-1;
    if (editorpos < 0) editorpos = 0;
    if (editorpos > (int)wcslen(label->text) ) editorpos = (int)wcslen(label->text);
  }
  if (FOCUSED(this) && mayIUseKeyboardInput())
  {
    if (input->type == INPUT_KEY_DOWN)
    {
      wchar_t ch = input->key.unicode;      
      if ( (ch >= 32) && (ch <= lastFontChar(0)) && 
        ((int)wcslen(label->text) < maxlength) ) {    
        wcscpy(temp, label->text+editorpos);
        label->text[editorpos] = input->key.unicode;
        wcscpy(label->text+editorpos+1, temp);
        editorpos++;
        state |= AO_CHANGED;
        return ret;
      }
      switch ( input->key.sym ){
      case SDLK_LEFT:
        if (editorpos > 0) editorpos--;
        break;
      case SDLK_RIGHT:
        if (editorpos < (int)wcslen(label->text)) editorpos++;
        break;
      case SDLK_BACKSPACE:
        if (editorpos > 0) {
          editorpos--;
          wcscpy(label->text+editorpos, label->text+editorpos+1);
          state |= AO_CHANGED;
        }
        break;
      case SDLK_DELETE:
        if (editorpos < (int)wcslen(label->text)) {
          wcscpy(label->text+editorpos, label->text+editorpos+1);
          state |= AO_CHANGED;
        }
        break;
      case SDLK_HOME:
        editorpos = 0;
        break;
      case SDLK_END:
        editorpos = (int)wcslen(label->text);
        break;

      }  // switch ( input->key )
    }  // if (input->type == INPUT_KEY_DOWN)
  }  // if FOCUSED(this) 
  return  ret;

}

void TLineTextField::setText(wchar_t *newtext)
{
  wcsncpy(label->text, newtext, maxlength);
  label->text[maxlength] = L'\0';
  editorpos=(int)wcslen(label->text);
}

wchar_t *TLineTextField::getText()
{
  return  label->text;
}

TLineTextField::~TLineTextField()
{
  delete label;
}

/******************************************************************************************/
TMultiLabel::TMultiLabel(int _style, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl)
: TBar(_style, _x, _y, _width, _height, _rl, _gl, _bl)
{
  rowcount = 0;
}

TMultiLabel::TMultiLabel(int _style, int _x, int _y, int _width, int _height)
: TBar(_style, _x, _y, _width, _height)
{
  rowcount = 0;
}

#define MULTILABEL_BORDER  4
void TMultiLabel::setText(wchar_t *text, int autoheight)
{
  int charinrow = (width - MULTILABEL_BORDER * 2) / getFontWidth(ao_font);
  int charinrowact;
  int tlength;
  TLabel *l;
  DelNFreeAll();
  rowcount = 0;
  while ((tlength = (int)wcslen(text)) > 0) {
    if (tlength < charinrow) {  
      l = new TLabel(L"", MULTILABEL_BORDER, MULTILABEL_BORDER + rowcount * getFontHeight(ao_font));
      wcsncpy(l->text, text, tlength);
      l->text[tlength] = L'\0';
      text += tlength;
      Add(l);
    } else {
      charinrowact = charinrow;
      while ( (text[charinrowact] != L'\0') && (text[charinrowact] != L' ')
        && (charinrowact > 0) ) charinrowact--;
      if (charinrowact == 0) charinrowact = charinrow;
      l = new TLabel(L"", MULTILABEL_BORDER, MULTILABEL_BORDER + rowcount * getFontHeight(ao_font));
      wcsncpy(l->text, text, charinrowact);
      l->text[charinrowact] = L'\0';
      if (text[charinrowact] == L' ') text++;
      text += charinrowact;
      Add(l);
    }
    rowcount++;
  }
  //automaticke nastaveni vysky
  if(autoheight)height = 2*MULTILABEL_BORDER + rowcount * getFontHeight(ao_font);
}

void TMultiLabel::setColor(float r, float g, float b, float alfa){
  for(TvActObj::iterator it = vActObj.begin();
      it != vActObj.end();
      it++)
  {
    ((TLabel*)(*it))->setColor(r, g, b, alfa);
  }
}

/******************************************************************************************/
TFrame::TFrame() : TBar(AO_HIDDEN,0,0,SCREEN_WIDTH,SCREEN_HEIGHT)
{
  TaoKernel::aoKernel->modalmod = false;
}

TFrame::~TFrame()
{  
  //TBar::~TBar(); 
}

void TFrame::drawObject()
{
  if (active == AO_PASIVE) return;

  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_LIGHTING);
  
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(0,SCREEN_WIDTH,SCREEN_HEIGHT,0);
  glDepthFunc(GL_ALWAYS);

  TaoKernel::aoKernel->parentpos.x=0;TaoKernel::aoKernel->parentpos.y=0;
  TaoKernel::aoKernel->setActiveHelpBarBlank();
  TBar::drawObject();
  if ( (TaoKernel::aoKernel->activeHelpBar) && (!TaoKernel::aoKernel->BlankedActiveHelpBar) &&
    (KTime() - TaoKernel::aoKernel->activeHelpBarSince >= AO_HELP_DELAY) )
  {
    if (TaoKernel::aoKernel->cursor->x <= (SCREEN_WIDTH/2))
      TaoKernel::aoKernel->activeHelpBar->x = TaoKernel::aoKernel->cursor->x + 22;
    else TaoKernel::aoKernel->activeHelpBar->x = TaoKernel::aoKernel->cursor->x - 
      TaoKernel::aoKernel->activeHelpBar->width + 2;
    TaoKernel::aoKernel->activeHelpBar->y = TaoKernel::aoKernel->cursor->y + 2;
    if (TaoKernel::aoKernel->activeHelpBar->y + TaoKernel::aoKernel->activeHelpBar->height > 
      SCREEN_HEIGHT) TaoKernel::aoKernel->activeHelpBar->y = SCREEN_HEIGHT - TaoKernel::aoKernel->activeHelpBar->height;
    TaoKernel::aoKernel->activeHelpBar->drawObject();

  }
  TaoKernel::aoKernel->cursor->drawObject();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

  glDisable(GL_BLEND);
}

int TFrame::workInput(INPUT *input)
{
  if (active == AO_PASIVE) return 0;
  if ((!TaoKernel::aoKernel->modalmod) && (TaoKernel::aoKernel->modalWindow != NULL))
  {
    // je potreba zrestartovat vsechny active objekty
    restart();
    Del(TaoKernel::aoKernel->modalWindow);
    Add(TaoKernel::aoKernel->modalWindow);
  } else if ((TaoKernel::aoKernel->modalmod) && (TaoKernel::aoKernel->modalWindow == NULL)) { 
    // je potreba zrestartovat vsechny active objekty
    if (vActObj.begin() != vActObj.end()){
      TvActObj::iterator it = vActObj.end(); it--;
      for(;it!=vActObj.begin();it--) (*it)->restart();
      (*vActObj.begin())->restart();
    }
  }
  if (TaoKernel::aoKernel->modalWindow == NULL) TaoKernel::aoKernel->modalmod = false;
  else TaoKernel::aoKernel->modalmod = true;
  
  int ret=0, i, _state=0;
  TaoKernel::aoKernel->ao_worked=0;
  TaoKernel::aoKernel->parentover=1;
  TActiveObject* ao;
  if (TaoKernel::aoKernel->modalmod) {
    input->mousepos.x-=x;
    input->mousepos.y-=y;

    TaoKernel::aoKernel->cursor->workInput(input);
    ret = TaoKernel::aoKernel->modalWindow->workInput(input);

    // je potreba zrestartovat vsechny active objekty
    if (TaoKernel::aoKernel->modalWindow == NULL) {
      if (vActObj.begin() != vActObj.end()){
        TvActObj::iterator it = vActObj.end(); it--;
        for(;it!=vActObj.begin();it--) (*it)->restart();
        (*vActObj.begin())->restart();
      }
    }

    input->mousepos.x+=x;
    input->mousepos.y+=y;
  } else {
    if (vActObj.begin() != vActObj.end()){
      input->mousepos.x-=x;
      input->mousepos.y-=y;

      TaoKernel::aoKernel->cursor->workInput(input);
      
      TvActObj::iterator it = vActObj.end(); it--;
      for(;it!=vActObj.begin();it--) {
        i = (*it)->workInput(input);
        if (i!=0){
          ao = *it;
          ret = i;
        }
        if (CLICKED((*it))) {
          _state = _state | AO_CLICKED;
        }
      }
      i = (*vActObj.begin())->workInput(input);
      if (i != 0){
        ao = *vActObj.begin();
        ret = i;
      }
      if (CLICKED((*vActObj.begin()))) _state = _state | AO_CLICKED;
      if (ret == 2){
        for(TvActObj::iterator it= vActObj.begin();it!=vActObj.end();it++){
          if ((*it != ao)) (*it)->state &= (255-AO_CHECKED);
        }
        ret=0;
      }
      input->mousepos.x+=x;
      input->mousepos.y+=y;
    }
    if (ret){
      Del(ao);
      Add(ao);
    }
  }
  if (!ret) TButton::workInput(input);
  state |= _state;
  return ret;  
}


/******************************************************************************************/
TCursor::TCursor()
: TIcon(TaoKernel::aoKernel->sdCurBasic, 0, 0)
{
  scrolling = true;
  wait = false;
}

void TCursor::setWait()
{
  wait = true;
  setIcon(TaoKernel::aoKernel->sdCurWait);
}

void TCursor::setBasic()
{
  wait = false;
  setIcon(TaoKernel::aoKernel->sdCurBasic);
}

int TCursor::workInput(INPUT *input)
{
  if (input->type == INPUT_MOUSE_MOTION){
    x = input->mousepos.x;
    y = input->mousepos.y;
    if (wait) {
      setIcon(TaoKernel::aoKernel->sdCurWait);
      return 0;
    }
    if (scrolling) {
      if (x == 0) setIcon(TaoKernel::aoKernel->sdCurLeft);
      else if (x == SCREEN_WIDTH - 1) {
        x = SCREEN_WIDTH - CURSOR_WIDTH;
        setIcon(TaoKernel::aoKernel->sdCurRight);
      } else if (y == 0) {
        setIcon(TaoKernel::aoKernel->sdCurUp);
      } else if (y == SCREEN_HEIGHT - 1){
        y = SCREEN_HEIGHT - CURSOR_HEIGHT;
        setIcon(TaoKernel::aoKernel->sdCurDown);
      } else setIcon(TaoKernel::aoKernel->sdCurBasic);
    } else setIcon(TaoKernel::aoKernel->sdCurBasic);
  }
  return 0;
}

/******************************************************************************************/
TaoKernel* TaoKernel::aoKernel = NULL;

rm::TSymbolData* TaoKernel::getSymbol(int group, int sid){
  rm::TSymbolData*  data;  
  if(!TaoKernel::aoKernel->rms)return NULL;
  TaoKernel::aoKernel->rms->getSymbol(&data, group, sid);
  return data;
}

void TaoKernel::init()
{
  // inicializace vnitrnich promennych
  ao_worked = 0;
  parentpos.x = 0;
  parentpos.y = 0;
  parentover = 1;

  // nacteni pozic hejblatek
  rms = (rm::TRM_symbol_i*) KSendGlobalMessage(MSG_GET_RM_SYMBOL_I, MOD_GUI, MOD_RM, NULL);

  //nahraj
  rms->getSymbol(&sdCurBasic, SYMBOLS_BASIC_ICONS, 13);
  rms->getSymbol(&sdCurUp,    SYMBOLS_BASIC_ICONS, 14);
  rms->getSymbol(&sdCurDown,  SYMBOLS_BASIC_ICONS, 15);
  rms->getSymbol(&sdCurLeft,  SYMBOLS_BASIC_ICONS, 16);
  rms->getSymbol(&sdCurRight, SYMBOLS_BASIC_ICONS, 17);
  rms->getSymbol(&sdCurWait, SYMBOLS_BASIC_ICONS, 101);
  rms->getSymbol(&sdCross,      SYMBOLS_BASIC_ICONS, 5);
  rms->getSymbol(&sdUpArrow,    SYMBOLS_BASIC_ICONS, 1);
  rms->getSymbol(&sdDownArrow,  SYMBOLS_BASIC_ICONS, 2);
  rms->getSymbol(&sdLeftArrow,  SYMBOLS_BASIC_ICONS, 3);
  rms->getSymbol(&sdRightArrow, SYMBOLS_BASIC_ICONS, 4);
  rms->getSymbol(&sdRadioUnchecked, SYMBOLS_BASIC_ICONS, 6);
  rms->getSymbol(&sdRadioChecked, SYMBOLS_BASIC_ICONS, 7);

  //kursor
  cursor = new TCursor();  

  // klavesnice je zatim pro vsechny
  keyboardgraber = NULL;

  BlankedActiveHelpBar = false;
  setActiveHelpBar(NULL);
  modalWindow = NULL;
}

void TaoKernel::done()
{
  //
  delete cursor;
  //
  delete rms; 
}

void TaoKernel::initAOKernel()
{
  TaoKernel::aoKernel = new TaoKernel();
  TaoKernel::aoKernel->init();
}

void TaoKernel::freeAOKernel()
{
  TaoKernel::aoKernel->done();
  delete TaoKernel::aoKernel;
  TaoKernel::aoKernel = NULL;
}


bool TaoKernel::isKeyboardFree()
{
  return (keyboardgraber == NULL);
}

void TaoKernel::setActiveHelpBar(TBar *_activeHelpBar)
{
  if ( (activeHelpBar != _activeHelpBar)) activeHelpBarSince = KTime();
  activeHelpBar = _activeHelpBar;
  BlankedActiveHelpBar = false;
}

void TaoKernel::setActiveHelpBarBlank()
{
  if (BlankedActiveHelpBar) activeHelpBar = NULL;
  BlankedActiveHelpBar = true;
}


}
