/**
*******************************************************************************

  @file /gui/common/ActiveObjects_ext.cpp
  @brief konzole, osd
  
  @author Vajicek
  @version 0.1
 
******************************************************************************/
#include "gui/common/ActiveObjects_ext.h"
//
#include "gui/common/BasicFonts.h"
#include "gui/common/Sounds.h"
#include "gui/common/Defs.h"
#include "gui/engine/TextureManager.h"

#include "common/Log.h"
#include "common/compatibility.h"
#include "common/utils.h"
#include "common/rm/rminit.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>

#define CW_WIDTH   400
#define CW_HEIGHT  400
#define CW_BORDER  3
#define CW_INPUTTF_LEN  40
#define CW_INPUTTF_H    20
#define CW_OUTPUTL_H    20

namespace gui{


/******************************************************************************/

void TConsoleWindow::consoleOutput(const char *format,... ){
  char buffer[MAX_OUTPUT_LINE_LEN+1];
  //
	va_list args;
	va_start( args, format );
	vsnprintf( buffer, MAX_OUTPUT_LINE_LEN, format, args );
	//
  wchar_t wbuffer[MAX_OUTPUT_LINE_LEN+1];
  UTF8toWideChar(buffer, wbuffer, MAX_OUTPUT_LINE_LEN);
  //
  consoleOutput(wbuffer);
}

void TConsoleWindow::consoleOutput(wchar_t *text){
  //
 	TItem* ti = new TItem(0, CW_OUTPUTL_H);
	lbOutput->Add(ti);
	TLabel* tl = new TLabel(text, 0, 0);
	ti->Add(tl);
}

void TConsoleWindow::consoleInput(char* str){
  //vyresi ruzny prikazy
  consoleOutput(str);
}

void TConsoleWindow::drawObject(){
  if(style!=AO_HIDDEN)TWindow::drawObject();
}

int TConsoleWindow::workInput(INPUT *input){
  int ret = TWindow::workInput(input);

  if( FOCUSED(ltfInput) && (input->key.sym == SDLK_RETURN)){
    //poslalni na vystup
    char buffer[MAX_OUTPUT_LINE_LEN+1];
    WideChartoUTF8(ltfInput->label->text, buffer, MAX_OUTPUT_LINE_LEN);
    if(strlen(buffer)){
      consoleInput(buffer);
      ltfInput->setText(L"");
    }
  }


  return ret;
}

TConsoleWindow::TConsoleWindow()
:TWindow(AO_MOVABLE, 0, 0, CW_WIDTH, CW_HEIGHT){

	hide();

	lbOutput = new TListBar(AO_NORMAL, CW_BORDER, CW_BORDER, CW_WIDTH-2*CW_BORDER, CW_HEIGHT - CW_INPUTTF_H - 3*CW_BORDER );
  Add(lbOutput);

  ltfInput = new TLineTextField(CW_BORDER, CW_HEIGHT - CW_BORDER - CW_INPUTTF_H, CW_INPUTTF_LEN);   
  Add(ltfInput);
}









/******************************************************************************/
TIME TTimeControledObject::getTime(){
  return (tTime = KTime());
}

TTimeControledObject::TTimeControledObject(){
  tLast = 0;
}






/******************************************************************************/

TOSD::TOSD(int _x, int _y, int _w, int _h){
	x=_x;y=_y;width=_w;height=_h;
  iClosed = 0;
}

int TOSD::workInput(INPUT *input){
  return 0;
}

void TOSD::showMsgW(wchar_t* msg, float _r, float _g, float _b){
  //
  TOSDMsg osdmsg;
  osdmsg.insert_time = getTime();
  wcsncpy(osdmsg.msg, msg, OSD_STR_MAX);
  osdmsg.priority = 1;
  osdmsg.fadeout = 1;
  osdmsg.r = _r;
  osdmsg.g = _g;
  osdmsg.b = _b;

  //
  if(!iClosed){
    vMsgs.push_back(osdmsg);
	  vMsgs.sort(std::greater<TOSDMsg>());
  }
}

void TOSD::showMsg(char* msg, float _r, float _g, float _b){
  //
  wchar_t wmsg[OSD_STR_MAX];
  UTF8toWideChar(msg, wmsg, OSD_STR_MAX);

  showMsgW(wmsg, _r, _g, _b);
}

void TOSD::drawObject(){
  int xx, yy;
  int fonth = getFontHeight(OSD_BASICFONT);

  //setrid
  if( (getTime() - tLast) > OSD_REFRESH_PERIOD ){
    tLast = tTime;

    //promazni
    for(  std::list<TOSDMsg>::iterator it = vMsgs.begin();
          it != vMsgs.end();
          it++)
    {
      if( (*it).fadeout == 0 ){
        vMsgs.erase(it);
        break;
      }//if   
    }//for
  }

  //nastav font a pocatecni pozici
  SelectFont( OSD_BASICFONT );
  xx = x + OSD_BORDER;
  yy = y + height - OSD_BORDER;
  for(  std::list<TOSDMsg>::iterator it = vMsgs.begin();
        it != vMsgs.end();
        it++)
  {
    //vypocet a updatuj pruhlednosti
    float coef = 
      (((int)tTime) - (*it).insert_time - OSD_SHOW_TIME) / ((float)OSD_FADEOUT_TIME);
    (*it).fadeout = 
      MAX(0,MIN(1, 1 - coef*coef*coef ));

    //vypis na obrazovku
    glColor4f((*it).r, (*it).g, (*it).b, (*it).fadeout);
    DrawText ( xx, SCREEN_HEIGHT-yy, (*it).msg );     

    //updatuj pozici
    yy -= fonth;
  }
}







/******************************************************************************/
void TWallpaper::drawObject(){
  float ratio_w = (width>height)?1:float(width)/height;
  float ratio_h = (width>height)?float(height)/width:1;
	glColor4f(WALLPAPER_BACKGROUND);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, hwtid);
	glBegin(GL_QUADS);  
    glTexCoord2f(0,0); 
    glVertex2i(x,y);
    glTexCoord2f(ratio_w,0); 
    glVertex2i(x+width,y);
    glTexCoord2f(ratio_w,ratio_h); 
    glVertex2i(x+width,y+height);  
    glTexCoord2f(0,ratio_h); 
    glVertex2i(x,y+height);  
  glEnd();
  glDisable(GL_TEXTURE_2D);
}

int TWallpaper::workInput(INPUT *input){
  return 0;
}

TWallpaper::TWallpaper(int bid, int _x, int _y, int _w, int _h){
	x=_x;y=_y;width=_w;height=_h;
	TTextureManager::tm->loadTextureQuery(bid, &hwtid);  
}







/******************************************************************************/
TScrollingTitles::TScrollingTitles(int _x, int _y, int _w, int _h, int speed){
	x=_x;y=_y;width=_w;height=_h;
  iPosition = 0;
  iSpeed = speed;
  tLast = getTime();
}

TScrollingTitles::~TScrollingTitles(){
  for(  std::vector<wchar_t*>::iterator it = vLines.begin();
        it != vLines.end();
        it++ )
    KReleaseWString(*it);
}

void TScrollingTitles::addLine(wchar_t* text){
  vLines.push_back( KInitWString(text) );
}

#define CT_LINE 10
#define CT_FADEOUT_RATIO 0.33f

void TScrollingTitles::drawObject(){

  //cas
  getTime();
  iPosition += (tTime-tLast);
  //pixel
  int yoff_pixel = iPosition/iSpeed;
  //radek
  int yoff_line = (int)floor(yoff_pixel/float(CT_LINE));
  
  //preskoc yoff_line radku
  std::vector<wchar_t*>::iterator it = vLines.begin();
  int pos = 0;
  while( (it != vLines.end()) && (pos<yoff_line) ){
    it++;
    pos++;
  }
  
  if(it == vLines.end())
    iPosition = -height*iSpeed;

  //vykresli height/CT_LINE radku
  int yy=-(yoff_pixel - pos*CT_LINE);
  while( it != vLines.end() ){ 
  
    if(yy<CT_FADEOUT_RATIO*height)
      glColor4f(1,1,1, (yy)/(CT_FADEOUT_RATIO*height) ); 
    else if(yy>(1-CT_FADEOUT_RATIO)*height)
      glColor4f(1,1,1, (height - yy)/(CT_FADEOUT_RATIO*height) ); 
    else      
      glColor4f(1,1,1,1); 
      
    DrawText ( x, SCREEN_HEIGHT - (y + yy), *it );
    yy+=10;
    it++;
  }

  //update
  tLast = tTime;
}

int TScrollingTitles::workInput(INPUT *input){
  return 0;
}






/******************************************************************************/
TIconTextButton::TIconTextButton(int _style, rm::TSymbolData* _data, wchar_t* t, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl)
:TIconButton(_style, _data, _x, _y, _width, _height, _rl, _gl, _bl)
{
	label = new TLabel(t, 
    x+(width-(int)wcslen(t)*getFontWidth(ICONTEXTBUTTON_DEFAULT_FONT))/2,
		y+height-getFontHeight(ICONTEXTBUTTON_DEFAULT_FONT));  
}

TIconTextButton::TIconTextButton(int _style, rm::TSymbolData* _data, wchar_t* t, int _x, int _y, int _width, int _height)
:TIconButton(_style, _data, _x, _y, _width, _height)
{
	label = new TLabel(t, 
    x+(width-(int)wcslen(t)*getFontWidth(ICONTEXTBUTTON_DEFAULT_FONT))/2,
		y+height-getFontHeight(ICONTEXTBUTTON_DEFAULT_FONT));  
}

void TIconTextButton::drawObject()
{
  TIconButton::drawObject();
	label->drawObject();
}

TIconTextButton::~TIconTextButton()
{
	delete label;
}









/******************************************************************************/
TProgressBar::TProgressBar(int _x, int _y, int _w, int _h, int max, int pos)
{
	x=_x;y=_y;width=_w;height=_h;
  iPosition = pos; 
  iMax = max;
}

TProgressBar::~TProgressBar()
{
}

void TProgressBar::set(int pos)
{
  iPosition = pos;
}

void TProgressBar::drawObject()
{
  float factor = iPosition/(float)iMax;
  
	glBegin(GL_QUADS);
		glVertex2i(x, y);
		glVertex2i(x+(int)(width*factor),y);
		glVertex2i(x+(int)(width*factor), y+height);
		glVertex2i(x, y+height);
	glEnd();	  
}

int TProgressBar::workInput(INPUT *input)
{
  return 0;
}









/******************************************************************************/
#define ERRWND_BORDER    13
#define ERRWND_WIDTH     360
#define ERRWND_HEIGHT    160
#define ERRWND_X         ((SCREEN_WIDTH-ERRWND_WIDTH)/2)
#define ERRWND_Y         ((SCREEN_HEIGHT-ERRWND_HEIGHT)/2)

TErrorWindow::TErrorWindow(int txtid, TErrorWndType ewt, bool showmodal)
: TWindow(AO_MOVABLE, ERRWND_X, ERRWND_Y, ERRWND_WIDTH, ERRWND_HEIGHT)
{
  //
  type = ewt;
  //
  wchar_t wtemp1[LABEL_MAX_LENGTH];
  wchar_t wtemp2[LABEL_MAX_LENGTH];
  rm::TRM_txt_i *RMtxti = (rm::TRM_txt_i *)KSendMessage(RQUEUE, MSG_GET_RM_TXT_I, MOD_GUI, MOD_RM, NULL);
  RMtxti->getWChar(wtemp1, LABEL_MAX_LENGTH, txtid);
  RMtxti->getWChar(wtemp2, LABEL_MAX_LENGTH, TEXT_OK);
  delete RMtxti;
  //zprava
  setupLayout(wtemp1, wtemp2);
  //
  if(showmodal)showModal();
}

TErrorWindow::TErrorWindow(wchar_t* msg, TErrorWndType ewt, bool showmodal)
: TWindow(AO_MOVABLE, ERRWND_X, ERRWND_Y, ERRWND_WIDTH, ERRWND_HEIGHT)
{
  //
  type = ewt;
  //
  if(type != ewtFatalError) {
    wchar_t wtemp1[LABEL_MAX_LENGTH];
    rm::TRM_txt_i *RMtxti = (rm::TRM_txt_i *)KSendMessage(RQUEUE, MSG_GET_RM_TXT_I, MOD_GUI, MOD_RM, NULL);
    RMtxti->getWChar(wtemp1, LABEL_MAX_LENGTH, TEXT_OK);
    delete RMtxti;
    //zprava
    setupLayout(msg, wtemp1);
  }
  else {
    setupLayout(msg, L"Abort");
  }
  //
  if(showmodal)showModal();
}

void TErrorWindow::setupLayout(wchar_t* msg, wchar_t* blab)
{
  TGroupBar* gmsg = new TGroupBar(AO_NORMAL, ERRWND_BORDER, ERRWND_BORDER, 
    ERRWND_WIDTH-2*ERRWND_BORDER, ERRWND_HEIGHT-3*ERRWND_BORDER - 35);
  Add(gmsg);
  //

  TMultiLabel* message = new TMultiLabel(AO_HIDDEN, ERRWND_BORDER, ERRWND_BORDER, 
    ERRWND_WIDTH-4*ERRWND_BORDER, ERRWND_HEIGHT-5*ERRWND_BORDER - 35);
  message->setText( msg ); 
  gmsg->Add(message);
  //  
  bok = new TSoundedTextButton(AO_NORMAL, blab, 
    (ERRWND_WIDTH-100)/2, ERRWND_HEIGHT-35-ERRWND_BORDER, 
    100, 35, SAMPLE_CLICK);
  Add(bok);
  bok->setShortcutKey(SDLK_RETURN);
  //pozadi
  setBgImage(RASTER_PANEL);
}

int TErrorWindow::workInput(INPUT *input)
{
  int ret = TWindow::workInput(input);
  //
  if( type == ewtFatalError && CLICKED(bok) ){        
    exit(-1);
  }
  //
  return ret;
}









/******************************************************************************/

TCurtain::TCurtain()
:TWindow(0,0,0,SCREEN_WIDTH,SCREEN_HEIGHT)
{
  anim = 0;
  dir = 0;
  pos = 0;
}

void TCurtain::fadeIn(int dur){
  anim = 1;
  dir = 1;
  iDur = dur;
  iStart = getTime();
  showModal();
}

void TCurtain::fadeOut(int dur){
  anim = 1;
  dir = 0;
  iDur = dur;
  iStart = getTime();
  showModal();
}

void TCurtain::drawObject(){
  //update
  if(anim){
    float t = (getTime() - iStart)/(float)iDur;    
    if( t >= 1 ){
      anim = 0;
      //zmen showModal na show
      hide();
      show();
    }
    
    //fadein    
    if(dir) pos = t;
    //fadeout
    else  pos = 1-t;
  }
  //vykresleni
	glBegin(GL_QUADS);
    glColor4f(0,0,0,pos);
		glVertex2i(0, 0);
		glVertex2i(SCREEN_WIDTH, 0);
		glVertex2i(SCREEN_WIDTH, SCREEN_HEIGHT);
		glVertex2i(0, SCREEN_HEIGHT);
	glEnd();	  
}

int TCurtain::workInput(INPUT *input){
  return 0;
}

}//namespace
