/**
*******************************************************************************

  @file /gui/common/ActiveObjects_ext.h
  @brief konzole, osd
  
  @author Vajicek
  @version 0.1
 
******************************************************************************/
#ifndef ACTIVEOBJECTS_EXT
#define ACTIVEOBJECTS_EXT

#include "common/mm.h"
#include "common/rm/rminit.h"
#include "gui/common/KTime.h"
#include "gui/common/ActiveObjects.h"
#include "gui/GUI.h"

#include <SDL/SDL.h>
#include <vector>
#include <list>
#include <functional>

namespace gui {

#define MAX_OUTPUT_LINE_LEN 256

/** okno pro uzivatelskou konzoli, snadne pro predefinovovani */
class TConsoleWindow : public TWindow {
public:
	TListBar *lbOutput;
	TLineTextField *ltfInput;
  //
  void consoleOutput(const char *format,... );
  virtual void consoleInput(char* str);
  void consoleOutput(wchar_t *text);
  //
	virtual void drawObject();
	virtual int workInput(INPUT *input);
	TConsoleWindow();
};

/** vrstvicka pro active objekty co pracuji s casem(napr. se nejak animuji) */
class TTimeControledObject //## 
{
public:
  TIME tTime;
  TIME tLast;
  TIME getTime();
  TTimeControledObject();
};

#define OSD_STR_MAX   255
#define OSD_BASICFONT 0
#define OSD_BORDER    5
#define OSD_REFRESH_PERIOD  100
#define OSD_COLOR1    1,0,1
#define OSD_COLOR_R    0.7
#define OSD_COLOR_G    0.7
#define OSD_COLOR_B    0.7
#define OSD_SHOW_TIME  10000
#define OSD_FADEOUT_TIME  3000

/** OnScreenDispaly */
class TOSD: public TActiveObject, public TTimeControledObject{
public:
  /** Struktura pro zachyceni zpravy na obrazovce */
  struct TOSDMsg{
    wchar_t msg[OSD_STR_MAX]; ///< Zprava
    int priority;     ///< Priorita
    int insert_time;  ///< Cas vlozeni
    float fadeout;    ///< Fade parametr
    float r;  ///< R barva
    float g;  ///< G barva
    float b;  ///< B barva
    /** Porovnavaci operator kvuli trizeni zprav jak jdou na obrazovku. */
    bool operator>(const TOSDMsg& a) const
    {
      return insert_time > a.insert_time; 
    }
  };

  /// Priznak zdali je zavren pristup, 0 = zpravy se nezarazuji, 1 = zpravy se zarazuji.
  int iClosed;
  /// Seznam zprav.
  std::list<TOSDMsg> vMsgs;

  /// Vystup a Vstup
  //@{
  /// Nic nedela.
	virtual int workInput(INPUT *input);
  /// Vykresli zpravy a jednou za OSD_REFRESH_PERIOD zmizele.
	virtual void drawObject();
  //@}
  ///Insertovani zprav
  //@{
  /// Vytvori a zatridi zpravy do seznamu zobrazovanych zprav.
  void showMsgW(wchar_t* msg, float _r = OSD_COLOR_R, float _g = OSD_COLOR_G, float _b = OSD_COLOR_B);
  /// Prekonveruje a vola showMsgW.
  void showMsg(char* msg, float _r = OSD_COLOR_R, float _g = OSD_COLOR_G, float _b = OSD_COLOR_B);
  /// Vlozi s priortou.
  void showMsg(int priority, wchar_t* msg,...);
  //@}
  /** Konstruktor.
    @param _x X souradnice.
    @param _y Y souradnice.
    @param _w Sirka.
    @param _h Vyska.
  */
  TOSD(int _x, int _y, int _w, int _h);
};


/// barva pozadi
#define WALLPAPER_BACKGROUND 1, 1, 1, 1

/** Obrazek na pozadi. */
class TWallpaper: public TActiveObject {
public:
	HWTID hwtid;  ///< Textura
  //
	virtual void drawObject();
	virtual int workInput(INPUT *input);
	TWallpaper(int bid, int _x, int _y, int _w, int _h);
};

/** Skrolujici titulky. */
class TScrollingTitles: public TActiveObject, public TTimeControledObject{
public:
  int iPosition;  ///< Pozice v titulcich.
  int iSpeed;     ///< Rychlost.
  std::vector<wchar_t*> vLines; ///< Radky titulku.
  /** Konstruktor.
    @param _x X pozice.
    @param _y Y pozice.
    @param _w Sirka.
    @param _h Vyska.
    @param speed Rychlost skrolovani.
  */
  TScrollingTitles(int _x, int _y, int _w, int _h, int speed);
  /** Destruktor. 
  */
  virtual ~TScrollingTitles();
  /** Pridej radek do titulku.
    @param text Radek titulku.
  */
	virtual void addLine(wchar_t* text);
  /** Vykresli titulky. 
  */
	virtual void drawObject();
	virtual int workInput(INPUT *input);
};


/// cislo fontu
#define ICONTEXTBUTTON_DEFAULT_FONT 0

/** Tlacitko s obrazkem a textem. */
class TIconTextButton : public TIconButton {
public:
  TLabel *label;
  TIconTextButton(int _style, rm::TSymbolData* _data, wchar_t* t, int _x, int _y, int _width, int _height, float _rl, float _gl, float _bl);
  TIconTextButton(int _style, rm::TSymbolData* _data, wchar_t* t, int _x, int _y, int _width, int _height);
  virtual void drawObject();
  ~TIconTextButton();
};

/** Progres bar. */
class TProgressBar: public TActiveObject{
public:
  int iPosition;  ///< aktualni pozice
  int iMax;       ///< maximalni pozice
  /** Konstruktor.
    @param _x Pozice X.
    @param _y Pozice Y.
    @param _w Sirka.
    @param _h Vyska.
    @param max Maximalni hodnota.
    @param pos Hodnota.
  */
  TProgressBar(int _x, int _y, int _w, int _h, int max, int pos);
  /** Destruktor. 
  */
  virtual ~TProgressBar();
  /** Nastavi pozici. 
    @param pos Pozice.
  */
	virtual void set(int pos);
  
	virtual void drawObject();
	virtual int workInput(INPUT *input);
};


/** Typ chyboveho okna */
enum TErrorWndType {
  ewtFatalError,  /// samo zavre program
  ewtNotice,      /// neobsluhuje vstupy
  ewtYesNo        /// dve tlacitka ano/ne(neimplementovano)
};

/** Univerzalni chybove okno */
class TErrorWindow: public TWindow {
public:
  /// type
  TErrorWndType type;
  /// ok
  TButton *bok;
  /** Vytvor layout
    @param msg Zprava.
    @param blab Navesti buttonu.
  */
  void setupLayout(wchar_t* msg, wchar_t* blab);
  /** Obsluha vstupu.
    @see TActiveObject
  */
	virtual int workInput(INPUT *input);
  /** Konstruktor.
    @param txtid Id zpravy.
    @param ewt Typ okna.
    @param showmodal Zobrazit jako modalni.
  */
	TErrorWindow(int txtid, TErrorWndType ewt, bool showmodal);
  /** Konstruktor.
    @param msg Text zpravy.
    @param ewt Typ okna.
    @param showmodal Zobrazit jako modalni.
  */
	TErrorWindow(wchar_t* msg, TErrorWndType ewt, bool showmodal);
};

/** Opona. */
class TCurtain: public TWindow, TTimeControledObject{
protected:
  float pos;  ///< poloha
  int anim;   ///< 1=on, 0=off
  int dir;    ///< 1=fadein, 0=fadeout
public:
  int iDur;   ///< trvani
  int iStart; ///< zacatek
  /** Konstruktor.
  */
	TCurtain();
  /** Ztmaveni(fadeout).
    @param dur Doba ztmaveni.
  */
  void fadeIn(int dur);
  /** Rozsviceni(fadein).
    @param dur Doba rozsviceni.
  */
  void fadeOut(int dur);
  /// Vykresleni a update.
	virtual void drawObject();
  /// Zpracovani.
	virtual int workInput(INPUT *input);
};

}	// namespace
#endif
