/**
*******************************************************************************

  @file /gui/common/BasicFonts.h
  @brief Zakladni fonty
  

  @author Vajicek
  @version 0.1
 
******************************************************************************/
#ifndef _BASICFONTS_H
#define _BASICFONTS_H

#include "common/types.h"

namespace gui{

/** Nastavi prostredi, souradnici, vykresli text a obnovi puvodni nastaveni.
  @param x X souradnice pocatku vystupu. 
  @param y Y souradnice pocatku vystupu.
  @param str Retezec.
*/
extern void DrawText ( int x, int y, const wchar_t *str );
/** Vykresli text.
  @param str Retezec.
  @param fontNumber Cislo fontu.
*/
extern void DrawText ( const wchar_t *str, int fontNumber );
/** Nastavi souradnice.
  @param x X souradnice pocatku vystupu. 
  @param y Y souradnice pocatku vystupu.
*/
extern void SetTextPosition ( int x, int y );
/** Nastavi prostredi rendereru.
  @param scr_width Sirka obrazovky.
  @param scr_height Vyska obrazovky.
*/
extern void SetTextCoords ( int scr_width, int scr_height );
/** Obnovi nastaveni 
*/
extern void UnsetTextCoords ();
/** Nakonfiguruje parametry sirky a vysky prostredi.
  @param scr_width Sirka obrazovky.
  @param scr_height Vyska obrazovky.
*/
extern void SetBasicFontsEnv ( int scr_width, int scr_height );
/** Zvoli font.
  @param fn Font.
*/
extern void SelectFont( int fn );
/** Zrusi fonty. 
*/
extern void UninitBasicFonts ( );
/** Inicializuje fonty.
  @param scr_width Sirka obrazovky.
  @param scr_height Vyska obrazovky.
*/
extern void InitBasicFonts ( int scr_width, int scr_height );
/** Vyska fontu v pixelech.
  @return Vyska fontu v pixelech.
*/
extern int getFontHeight(int f);
/** Vrati sirku fontu v pixelech 
  @return Sirka fontu v pixelech.
*/
extern int getFontWidth(int f);
/** Index do pole fontu v zavislosti na unicodu. 
  Ma vyznam pro	font 0, kde jsou pridelane hacky a carky, jinak vraci (BYTE)ch.
  @param ch Siroky znak.
  @param font Font.
  @return Index do pole fontu.  
*/
extern BYTE Uni2Byte(wchar_t ch, int font);
/** Posledni definovany znak daneho fontu.
  @param font Font.
  @return Posledni definovany znak daneho fontu.
*/
extern int lastFontChar(int font);
/** Velikost napisu.
  @param str Retezec.
  @param[out] w Vyska napisu.
  @param[out] h Sirka napisu.
*/
extern void GetTextDim ( const wchar_t *str, int *w, int *h );

}//namespace
#endif
