/**
*******************************************************************************

  @file /gui/common/Context.h
  @brief Kontext = to co je videt na obrazovce


  @author Vajicek
  @version 0.1
 
******************************************************************************/
#ifndef TCONTEXT_H
#define TCONTEXT_H
//
#include "gui/common/ActiveObjects.h"
#include "common/Msg.h"

#define CONTEXT_NAME_LEN 25

namespace gui{

/** Navratova hodnota od contextu pres screen a gui az do kralovstvi.
	iPrimary ma vyznam akce ktera se ma provest po navratu z kontextu,
	iSecondary je parametr teto akce. (Napr: iPrimary ... zacni hru
	iSecondary ... hru cislo 1) 
*/
enum TContextPrimaryParameter{ 
  context_undefined = -1, 
  context_quit, 
  context_disconnected, 
  context_defeated, 
  context_endgame, 

  //stav mainmenu kontextu
  context_profile,
  context_mainmenu,
  context_localmultigame,
  context_networkhostgame,
  context_networkjoingame,

  //typ hry
  context_localgame,
  context_networkgame
};

/** Trida ktera se pouziva pro predavani informaci mezi kontexty. */
class TContextParameter //##
{
public:
  /// Primarni paramater, casto urcuje vyznam sekundarniho parametru.
	TContextPrimaryParameter iPrimary;
  /// Sekundarni parametr.
	void *iSecondary;
  /** Prirazovaci operator, zkopiruje data. 
  */
	void operator=(TContextParameter sr) {iPrimary = sr.iPrimary; iSecondary = sr.iSecondary;}
  /** Nastavi defaultni hodnoty.
  */
	void setDefault() {iPrimary = context_undefined; iSecondary = 0; }
  /** Bezparametricky konstruktor.
  */
  TContextParameter(){setDefault();}
  /** Konstruktor.
    @param p Primarni parametr.
    @param s Sekundarni parametr.
  */
  TContextParameter(TContextPrimaryParameter p, void *s){iPrimary = p, iSecondary = s;}
};

/** Kontext.
  Predek pro vsechny typy obsahu obrazovek, slouzi jako kolekce
  pro vsechny objekty
*/
class TContext : public TFrame
{
public:
  /// Jmeno kontextu, pekna identifikace.
	char szName[CONTEXT_NAME_LEN];

  /// Zivotni procesy.
  //@{  
	/** Aktivuje kontext.
    Spusti animace, nastavi kameru.
    @param contextparameter Data predana kontextu pri aktivovani.
  */
	virtual void activateContext(TContextParameter contextparameter)=0;
	/** Deaktivace kontextu.
    Zastavi animace, zastavi kameru, ulozi nastaveni.
  */
	virtual void deactivateContext()=0;
	/** Vykresli kontext.
    Muze testovat jestli je aktivni,... v nejjednodussim pripade vola draw
  */
	virtual void drawContext();
  /** Vstup od uzivatele.
    Osetreni chyb.
    @see TFrame
  */  
  virtual int workInput(INPUT *input);  
  /** Konstruktor.
    vytvori se prisluzne objekt(mapa,..) a registruji se ve frontach
    zavola se po pridani do Screen, nejspis na zacatku programu, nemela by byt priliz narocna
    napriklad navaze se na ostatni screeny -> musi ji volat screen az bude vedet, ze jsou vsechny nahrany
    nekde v hlavnim programu
  */
	TContext();
	/** Destruktor, zrusi kontext, smaze/ulozi.
  */
	virtual ~TContext();
	/** Builder.
    @return Novy kontext.
  */
	static TContext* createContext(...);
  //@}
};


}//namespace
#endif
/*****************************************************************************/
