/**
******************************************************************************

  @file /gui/common/Sounds.cpp
  @brief Modul pro prehravani zvuku
 

  @author Vajicek
  @version 0.1

******************************************************************************/
#include "gui/common/Sounds.h"
#include "gui/common/Defs.h"
#include "gui/common/Defs.h"
#include "gui/engine/Camera.h"

using namespace std;
namespace gui{


/// callback ze sound systemu, ze zvuk na kanale dohral
void channelDone_callback(int channel) {
  TSoundInterface::si->channels[channel].playing = NULL;
}

/// callback ze sound systemu, ze hudba na kanale dohrala
void musicFinished_callback() {
}




//==========================================================================//
// TSoundInterface
//==========================================================================//

//////////////////////////////////////////////////////////////////////////////
// zvuky
//////////////////////////////////////////////////////////////////////////////

void TSoundInterface::playSampleSpatialConrolled(int rid, P3F* position, float latt){
  //
  int channel;
  Mix_Chunk* sample = SRM->getSample(rid);

  //
  channel = Mix_PlayChannel(-1, sample, 0);
  if( channel == -1 )return; 
  channels[channel].playing = sample;
  channels[channel].volume = MIX_MAX_VOLUME;
  channels[channel].iSpatialControlled = 1;
  CopyV3(channels[channel].p3fPosition.p, position->p);
  channels[channel].fLinearAttenuation = latt;
  Mix_Volume(channel, ROUND( channels[channel].volume * soundVolume ) );
}

void TSoundInterface::playSample(int rid){
  //
  int channel;
  Mix_Chunk* sample = SRM->getSample(rid);

  //
  channel = Mix_PlayChannel(-1, sample, 0);
  channels[channel].playing = sample;
  channels[channel].volume = MIX_MAX_VOLUME;
  channels[channel].iSpatialControlled = 0;
  Mix_Volume(channel, ROUND( channels[channel].volume * soundVolume ) );
}

void TSoundInterface::setSoundVolume(float vol){

  //
  soundVolume = vol;
   
  //
  for(int i = 0; i < channels_used; i++){
    if(channels[i].playing){
      Mix_Volume(i, ROUND(channels[i].volume * soundVolume) );
    }
  }
}

float TSoundInterface::getSoundVolume()
{
	return soundVolume;
}

void TSoundInterface::stopAllChannels(){
  Mix_HaltChannel(-1);
}






//////////////////////////////////////////////////////////////////////////////
// hudba
//////////////////////////////////////////////////////////////////////////////

int TSoundInterface::getMusicCount(){
  return 0;
}

char* TSoundInterface::getMusicName(int rid){
  return NULL;
}

void TSoundInterface::playMusic(int rid){

  if(playing_track_id == rid)return;
  
  //
//  SRM->loadNow(rid);
//  playing_track = SRM->getMusic(rid);
  this->playing_track_id = rid;

  //
  if(musicVolume>0){
//    Mix_PlayMusic(playing_track, -1);
    this->SRM->playMusic(this->playing_track_id);
    Mix_VolumeMusic(ROUND(musicVolume * MIX_MAX_VOLUME));
  }
}

void TSoundInterface::setMusicVolume(float vol){
  //nenulova hlasitost->nastav
  if( vol>0 ) {
    //byla nulova->zacni prehravat
    if( musicVolume==0 ){
//      Mix_PlayMusic(playing_track, -1);
      this->SRM->playMusic(this->playing_track_id);
    }
    Mix_VolumeMusic(ROUND(musicVolume * MIX_MAX_VOLUME));
  }
  //nulova hlasitost -> nic neprehravej
  else {
    this->SRM->playMusic(-1);
//    Mix_HaltMusic();
  }
  //
  musicVolume = vol;
}

float TSoundInterface::getMusicVolume()
{
	return musicVolume;
}

void TSoundInterface::stopMusic(){
  this->SRM->playMusic(0);
//  Mix_HaltMusic();
  playing_track = NULL;
  playing_track_id = -1;
}

void TSoundInterface::fadeoutMusic(int ms){
  this->SRM->playMusic(-ms);
//  Mix_FadeOutMusic(ms);
  playing_track = NULL;
  playing_track_id = -1;
}






//////////////////////////////////////////////////////////////////////////////
// kostrukce & destrukce
//////////////////////////////////////////////////////////////////////////////

void TSoundInterface::initChannels(int chc){
  channels_used = 16;

  //
  Mix_AllocateChannels(channels_used);
  
  //
  for(int i = 0; i < channels_used; i++){
    channels[i].volume = MIX_MAX_VOLUME;
    channels[i].id = i;
    channels[i].playing = NULL;
    channels[i].iSpatialControlled = 0;
    channels[i].fLinearAttenuation = 1;
  }

  //
  Mix_ChannelFinished(channelDone_callback);
}

TSoundInterface::TSoundInterface(){
  //
  playing_track = NULL;
  playing_track_id = -1;

  //inicializace SDL
  if( SDL_Init(SDL_INIT_AUDIO) < 0 ) {
    GLOBALLOG("Audio initialization failed: %s\n",SDL_GetError());
    THROW(E_8K_GUI,"Audio initialization failed.");
  }	

  //inicializace sound sytemu
  if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 1024)==-1) {
    GLOBALLOG("Mix_OpenAudio() failed! error: %d \n", Mix_GetError());
    THROW(E_8K_GUI,"Mix_OpenAudio() failed!");
  }

  //kanalnik
  initChannels(DEFAULT_CHANNELS_COUNT);

  //callback na prehravac hudby
  Mix_HookMusicFinished(musicFinished_callback);

  //resource manager
  SRM = NULL;
  if( !( SRM = (rm::TRM_sound_i*)KSendBcAnMsg(MSG_GET_RM_SOUND_I, 0)) )
    THROW(E_8K_GUI, "TSoundInterface::TSoundInterface(): failed to query rmsound interface\n");

  //
  musicVolume =  1;
  soundVolume =  1;
}

TSoundInterface::~TSoundInterface(){
  stopAllChannels();
  stopMusic();
  delete SRM;
}






//////////////////////////////////////////////////////////////////////////////
// staticky
//////////////////////////////////////////////////////////////////////////////

TSoundInterface* TSoundInterface::si = NULL;

void TSoundInterface::initSound(){
  TSoundInterface::si = new TSoundInterface;
}

void TSoundInterface::uninitSound(){
	if(TSoundInterface::si)
  	delete TSoundInterface::si;
}











//==========================================================================//
// TSoundInterfaceController
//==========================================================================//

void TSoundInterfaceController::update(int absoluteTime){

  if( (absoluteTime - iLastTime) < SOUNDINTERFACE_CONTROLER_UPDATE_PERIOD )
    return;

  //
  iLastTime = absoluteTime;

  //
  TChannel* ch;
  for(int i = 0; i < si->channels_used; i++){
    ch = si->channels + i;
    if(ch->iSpatialControlled){
      float dist = DISTV3(ch->p3fPosition.p, cam->p4fPos.p);

      //
      ch->volume = ROUND( MIX_MAX_VOLUME * ch->fLinearAttenuation/(ch->fLinearAttenuation+dist) );     

      //
      Mix_Volume(ch->id, ROUND(ch->volume * si->soundVolume) );     
    }
  }//for 
}

TSoundInterfaceController::TSoundInterfaceController(TCamera* _cam, TSoundInterface* _si){
  RTTID("sndint_c");
  cam = _cam;
  si = _si;
  iLastTime = KTime();
  activate(iLastTime);
}

}//namespace
/*****************************************************************************/

