/**
*******************************************************************************

  @file /gui/common/TGA_lib.cpp
  @brief Punkce pro nahravani rastru z .TGA


  @author Vajicek
  @version 0.1
 
******************************************************************************/

#include "gui/common/TGA_lib.h"
#include "common/exc.h"
#include "common/mm.h"

#include <stdio.h>
#include <stdlib.h>

namespace gui{
  
/** Hlavicka TGA */
struct TTGAHeader{
  BYTE IDLength;
  BYTE ColorMapType;
  BYTE ImageType;
  //ColorMapSpec
  unsigned short FirstEntryIndex;
  unsigned short ColorMapLength;
  BYTE ColorMapEntrySize;
  //ImageSpec
  unsigned short XorigImage;
  unsigned short YorigImage;
  unsigned short ImageWidth;
  unsigned short ImageHeight;
  BYTE PixelDepth;
  BYTE ImageDescriptor;
};

/** paticka TGA - rozrirena verze */
struct TTGAFooter{
  unsigned extensionAreaOffset[4];
  unsigned developerDirectoryOffset[4];
  char signature[16];
  char dot;
  BYTE terminator;
};

SDL_Surface* LoadTGA(const char* filename){
  FILE* f = NULL;
  SDL_Surface* s;
  TTGAHeader header;
  //
  f = fopen(filename, "rb");
  if(!f){
    THROW(E_8K_TGA_READER, "Error reading TGA; File not found");
  }
  //
  fread(&(header.IDLength)          , sizeof(BYTE), 1, f);
  fread(&(header.ColorMapType)      , sizeof(BYTE), 1, f);
  fread(&(header.ImageType)         , sizeof(BYTE), 1, f);
  fread(&(header.FirstEntryIndex)   , sizeof(unsigned short), 1, f);
  fread(&(header.ColorMapLength)    , sizeof(unsigned short), 1, f);
  fread(&(header.ColorMapEntrySize) , sizeof(BYTE), 1, f);
  fread(&(header.XorigImage)        , sizeof(unsigned short), 1, f);
  fread(&(header.YorigImage)        , sizeof(unsigned short), 1, f);
  fread(&(header.ImageWidth)        , sizeof(unsigned short), 1, f);
  fread(&(header.ImageHeight)       , sizeof(unsigned short), 1, f);
  fread(&(header.PixelDepth)        , sizeof(BYTE), 1, f);
  fread(&(header.ImageDescriptor)   , sizeof(BYTE), 1, f);
  //
  if(feof(f)){
    fclose(f);
    THROW(E_8K_TGA_READER, "Error reading TGA; Incomplete file - header");
  }

  unsigned int Rmask, Gmask, Bmask, Amask;
  unsigned int bytesPerColorMapEntry = 0;
  int bytesPerDataElement = header.PixelDepth>>3;
  //
  if( (header.ColorMapEntrySize == 15) ||
      (header.ColorMapEntrySize == 16) )
      bytesPerColorMapEntry = 2;
  if( (header.ColorMapEntrySize == 24) )
      bytesPerColorMapEntry = 3;
  if( (header.ColorMapEntrySize == 32) )
      bytesPerColorMapEntry = 4;

  //image id
  fseek(f, header.IDLength , SEEK_CUR); 



  Rmask = 0xFF000000;
  Gmask = 0x00FF0000;
  Bmask = 0x0000FF00;
  Amask = 0x000000FF;


  s = SDL_CreateRGBSurface( SDL_SWSURFACE, 
                            header.ImageWidth, header.ImageHeight, 
                            header.PixelDepth, 
                            Rmask, Gmask, Bmask, Amask);
  //read pallete
  if(header.ColorMapType){
    fseek(f, bytesPerColorMapEntry*header.ColorMapLength, SEEK_CUR); 
  }

  //read data
  if(header.ImageType > 3){
    fclose(f);
    THROW(E_8K_TGA_READER, "Error reading TGA; Compression not supported");
  }
  if( (header.ImageType == 2) || (header.ImageType == 1) || (header.ImageType == 3)){
    size_t count = header.ImageWidth*header.ImageHeight;
    if( count != fread(s->pixels, bytesPerDataElement, count, f) ){
      fclose(f);
      THROW(E_8K_TGA_READER, "Error reading TGA; Incomplete file - data");
    }
  }


  //
  fclose(f);
  return s;
}

}//namespace
/***/
/******************************************************************************/
