/**
*******************************************************************************

  @file /gui/common/chatbox.cpp
  @brief graficke rozhrani chatboxu
  

  @author Pavel
  @version 0.1
 
******************************************************************************/
#include "gui/common/chatbox.h"
#include "gui/common/ActiveObjects_ext.h"
#include "gui/common/Defs.h"
#include "net/comm.h"
#include "gui/GUI.h"

namespace gui{

#define MESSAGE_HEIGHT    22
#define MESSAGES_HEIGHT   (_height-MESSAGE_HEIGHT-16)
#define MESSAGES_Y        16
#define BORDER            2
#define CHAR_WIDTH        8
#define MESSAGE_LENGTH    28
#define MESSAGE_WIDTH     (MESSAGE_LENGTH*CHAR_WIDTH+BORDER*2)
#define BUTTON_WIDTH      80
#define BUTTON_X          (_width - BUTTON_WIDTH)
#define BUTTON_HEIGHT     20
#define BUTTON_Y          (_height - BUTTON_HEIGHT)

TChatBox::TChatBox(rm::TRM_txt_i *rmtxti, int _x, int _y, int _width, int _height, bool _isHost)
: TGroupBar(AO_HIDDEN, _x, _y, _width, _height)
{
  wchar_t wtemp[LABEL_MAX_LENGTH];

  lchat = new TLabel(rmtxti->getWChar(wtemp, LABEL_MAX_LENGTH, TEXT_CHAT),
      0, 0);
  Add(lchat);

  lbmessages = new TListBar(AO_NORMAL, 0, MESSAGES_Y, _width, MESSAGES_HEIGHT);
  Add(lbmessages);

  tfmessage = new TLineTextField(0, BUTTON_Y, MESSAGE_LENGTH);
  Add(tfmessage);

  bsend = new TSoundedTextButton(AO_NORMAL, rmtxti->getWChar(wtemp, LABEL_MAX_LENGTH, TEXT_SEND), 
		BUTTON_X, BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT, SAMPLE_CLICK);
  Add(bsend);

  bsend->setShortcutKey(SDLK_RETURN);
  
  isHost = _isHost;
  client_id = -1;
}



void TChatBox::receiveMsg(TPackage *p)
{

  if (p->msgTo() == TO_SERVER) {
    // server jen preposle zpravu vsem klientum
    p->send(TO_SERVER, TO_EVERYBODY);
  } else {
    if (client_id < 0)
    {
      // z prvni dorucene zpravy se dozvim svoje client_id
      client_id = p->msgTo();
    } /*else */{
      showPackage(p);
    }
  }


  delete p;
}

#define ITEM_HEIGHT   (32+4)

void TChatBox::showPackage(TPackage *p)
{
  char ctemp[(MESSAGE_LENGTH+2)*4], ctemp2[(MESSAGE_LENGTH+2)*4];
  wchar_t wtemp[(MESSAGE_LENGTH+2)*4];
  TItem *item;
  

  TMultiLabel *mlabel;
  mlabel = new TMultiLabel(AO_HIDDEN, 0, 0, 299, ITEM_HEIGHT);

  // sestaveni zpravy
  p->getChar(ctemp, MESSAGE_LENGTH, "name", 0, NULL);
  strcat(ctemp, ": ");
  p->getChar(ctemp2, (MESSAGE_LENGTH + 1) * 4, "msg", 0, NULL);
  strcat(ctemp, ctemp2);
  UTF8toWideChar(ctemp, wtemp, 2 * MESSAGE_LENGTH);  
  mlabel->setText(wtemp);
  if (mlabel->rowcount <= 1) mlabel->height -= 16;


  item = new TItem(AO_UNSELECTABLE, mlabel->height);
  item->Add(mlabel);

  lbmessages->Add(item);

  lbmessages->scroll->ChangeTo(AO_NOCHANGE, AO_NOCHANGE, lbmessages->scroll->range()
    - lbmessages->scroll->size());
}


int TChatBox::workInput(INPUT *input)
{
  int ret = TGroupBar::workInput(input);
  
  if CLICKED(bsend) { 
    if ((wcslen(tfmessage->label->text) > 0) && (client_id != -1) ){
      TPackage *mypacket = new TPackage(MSG_NET_CHAT);
      char ctemp[(MESSAGE_LENGTH+2)*4];

      strncpy(ctemp, TGUI::gui_inst->profile->name, MESSAGE_LENGTH);
      mypacket->setChar(ctemp, "name", 0, NULL);
    
      WideChartoUTF8(tfmessage->label->text, ctemp, (4*MESSAGE_LENGTH));
      mypacket->setChar(ctemp, "msg", 0, NULL);

      mypacket->setInt(-1, "red", 0, NULL);
      mypacket->setInt(-1, "green", 0, NULL);
      mypacket->setInt(-1, "blue", 0, NULL);

      mypacket->send(client_id, TO_SERVER);
      showPackage(mypacket);
      delete mypacket;
    
      tfmessage->setText(L"");
      tfmessage->state |= AO_FOCUS;
    }

  }
  
  return ret;
}

void TChatBox::directSend(char *name, char *msg)
{
  TPackage *mypacket = new TPackage(MSG_NET_CHAT);

  mypacket->setChar(name, "name", 0, NULL);
  mypacket->setChar(msg, "msg", 0, NULL);
  mypacket->setInt(-1, "red", 0, NULL);
  mypacket->setInt(-1, "green", 0, NULL);
  mypacket->setInt(-1, "blue", 0, NULL);

  if (isHost) mypacket->send(TO_SERVER, TO_EVERYBODY);
    else if (client_id != -1)
      mypacket->send(client_id, TO_SERVER);

  delete mypacket;

}

/******************************************************************************/

TOSDChatBox::TOSDChatBox(TOSD *_osd, int _x, int _y, int _msglength)
: TGroupBar(AO_HIDDEN, _x, _y, _msglength*8, 20)
{
  // ukazovatko, kde se muze psat
  caption = new TLabel(L">", 0, 0);
  Add(caption);
  // jediny prvek prijimajici vstupy
  line = new TLineTextField(8, 0, _msglength);
  Add(line);
  line->hidden = true;

  osd = _osd;
  // defaultne chat neni videt
  hide();

  client_id = -1;
  red = -1;
  green = -1;
  blue = -1;

}

void TOSDChatBox::setClientColor(int _client_id, int _red, int _green, int _blue)
{
  client_id = _client_id;
  red = _red;
  green = _green;
  blue = _blue;
  caption->setColor(red / 255.f, green / 255.f, blue / 255.f, 1);
  line->label->setColor(red / 255.f, green / 255.f, blue / 255.f, 1);
}


void TOSDChatBox::receiveMsg(TPackage *p)
{
  if (p->msgTo() == TO_SERVER) {
    // server jen preposle zpravu vsem klientum
    p->send(TO_SERVER, TO_EVERYBODY);
  } else {
    if (p->msgTo() == client_id)
    {
      showPackage(p);
    }
  }

  delete p;
}

void TOSDChatBox::showPackage(TPackage *p)
{
  char ctemp[(LABEL_MAX_LENGTH+2)*4], ctemp2[(LABEL_MAX_LENGTH+2)*4];
  int r, g, b;
  bool defaultcol;

  p->getChar(ctemp, LABEL_MAX_LENGTH * 4, "name", 0, NULL);
  strncat(ctemp, ": ", LABEL_MAX_LENGTH * 4);
  p->getChar(ctemp2, LABEL_MAX_LENGTH * 4, "msg", 0, NULL);
  strncat(ctemp, ctemp2, LABEL_MAX_LENGTH * 4);
  
  defaultcol = false;
  r = p->getInt("red", 0, NULL);
  if ( (r < 0) || (r > 255) ) defaultcol = true;
  g = p->getInt("green", 0, NULL);
  if ( (g < 0) || (g > 255) ) defaultcol = true;
  b = p->getInt("blue", 0, NULL);
  if ( (b < 0) || (b > 255) ) defaultcol = true;

  if (defaultcol) osd->showMsg(ctemp);
    else osd->showMsg(ctemp, r / 255.0f, g / 255.0f, b / 255.0f);
}

void TOSDChatBox::directSend(char *name, char *msg, int _red, int _green, int _blue)
{
  TPackage *mypacket = new TPackage(MSG_NET_CHAT);

  mypacket->setChar(name, "name", 0, NULL);
  mypacket->setChar(msg, "msg", 0, NULL);
  mypacket->setInt(_red, "red", 0, NULL);
  mypacket->setInt(_green, "green", 0, NULL);
  mypacket->setInt(_blue, "blue", 0, NULL);

  if (client_id != -1) mypacket->send(client_id, TO_SERVER);

  delete mypacket;
}

int TOSDChatBox::workInput(INPUT *input)
{
  int ret = TGroupBar::workInput(input);

  if (active == AO_ACTIVE) {
    if ( (input->type == INPUT_KEY_DOWN) && (line->mayIUseKeyboardInput()) )
    {
      if (input->key.sym == SDLK_ESCAPE){
        hide();
      } else 

      if (input->key.sym == SDLK_RETURN){
        if ((wcslen(line->label->text) > 0) ) {
          TPackage *mypacket = new TPackage(MSG_NET_CHAT);
          char ctemp[(LABEL_MAX_LENGTH+2)*4];

          strncpy(ctemp, TGUI::gui_inst->profile->name, LABEL_MAX_LENGTH);
          mypacket->setChar(ctemp, "name", 0, NULL);
    
          WideChartoUTF8(line->label->text, ctemp, (4*LABEL_MAX_LENGTH));
          mypacket->setChar(ctemp, "msg", 0, NULL);

          mypacket->setInt(red, "red", 0, NULL);
          mypacket->setInt(green, "green", 0, NULL);
          mypacket->setInt(blue, "blue", 0, NULL);

          if (client_id != -1) mypacket->send(client_id, TO_SERVER);
          
          showPackage(mypacket);
          delete mypacket;
    
          line->setText(L"");
          hide();

        }
      }
    
    } // if (input->type == INPUT_KEY_DOWN)

    line->state |= AO_FOCUS;
  } // if (active == AO_ACTIVE) {
  

  return ret;
}

void TOSDChatBox::show()
{
  if (active == AO_PASIVE) {
    line->grabKeyboardInput();
    line->setText(L"");
    TGroupBar::show();
  }
}

void TOSDChatBox::hide()
{
  line->releaseKeyboardInput();
  TGroupBar::hide();
}

} // namespace

