/**
*******************************************************************************

  @file /gui/common/init_env.cpp
  @brief Nizkourovnova inicializace UI, rutiny pro testovani
  

  @author Vajicek
  @version 0.1
 
******************************************************************************/

#include "config.h"
#include "gui/common/init_env.h"
#include "gui/common/KTime.h"
#include "common/Log.h"
#include "gui/common/mymath.h"

namespace gui{

int SwitchResolution(int width, int height, int bpp, int fs)
{
  const SDL_VideoInfo* info = NULL;
  int flags = SDL_OPENGL;
  if(fs)
  	flags |= SDL_FULLSCREEN;
  
  //zjisti video  
  info = SDL_GetVideoInfo();

  //prepni
  if( SDL_SetVideoMode( width, height, bpp, flags) == 0 ) 
  {  
    GLOBALLOG("Video mode set failed: %s\n",SDL_GetError() );
	  return -3;
  }
  return 0;
}

int Quit_filter(const SDL_Event *event)
{
  if (event->type==SDL_QUIT) // pouze Quit zdrz
  {
    //zabij bezici nahravani gui
    KSendMessage(RQUEUE, MSG_KILL_MAPLOADING, MOD_GUI, MOD_GUI, NULL);   
    //zabij bezici ai    
//    int i;
    int * i=(int*)KSendMessage(RQUEUE, MSG_AI_KILL_AI_PLANNING, MOD_GUI, MOD_STRATEGY, NULL);
    if (i==0)
      return 0;
    else
      return 1;
  }
  else // ostatni pust
  {
    return 1;
  }
}

int InitEnv(int _w, int _h, int _fullscreen)
{
  const SDL_VideoInfo* info = NULL;
  int width = _w;
  int height = _h;
  int bpp = 0;
	int ret=0;
  int flags = SDL_OPENGL;

  if(_fullscreen)
  	flags |= SDL_FULLSCREEN;

	//inicializace sdl
  if( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    GLOBALLOG("Video initialization failed: %s\n",SDL_GetError());
		return -1;
  }	

  //filter na zpravu o ukonceni(zabijeni vlaken)
  SDL_SetEventFilter(Quit_filter);
    
  //zjisti video
  info = SDL_GetVideoInfo();
  if( !info ) {
    GLOBALLOG("Video query failed: %s\n",SDL_GetError());
	  return -2;
  }
  
	//nastav opengl
	bpp = info->vfmt->BitsPerPixel;
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 24 );
  SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 8);
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  //titulek(takhle to staci)
  SDL_WM_SetCaption("8Kingdoms " VERSION, "8Kingdoms");

	//nastav rozliseni
  if( SDL_SetVideoMode( width, height, bpp, flags ) == 0 ) {
    GLOBALLOG("Video mode set failed: %s\n",SDL_GetError() );
	  return -3;
  }

  //vstup
  SDL_EnableUNICODE(1);
  SDL_ShowCursor(SDL_DISABLE);
  SDL_EnableKeyRepeat(300, 100);   

	//
	return 0;
}

void ShutdownEnv(){
	SDL_Quit();
}


//-----------------------------------------------------------------------------
#ifdef DEMO

int UserCamera(int keyscanrate)
{
  static MX4 cam;
  static int init = 0;
  static int time = 0;
  MX4 t,m;
  int knum;
  Uint8 *pkeys;

  //inicializace kamery
  if( !init ){
    SetMX4( cam,
            1, 0, 0,  0,
            0, 1, 0,  0,
            0, 0, 1, 0,
            0, 0, 0,  1);

    init = 1;
  }

	SDL_PumpEvents();
  pkeys = SDL_GetKeyState(&knum);	

  //zpomaleni
  int at = KTime();
  if( (at - time) > 1000.0/keyscanrate ) {
    time = at;

    //akce
    if(pkeys[SDLK_RIGHT]){
      AATOMX(t, 0.05f, 0, 1, 0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_LEFT]){
      AATOMX(t, -0.05f, 0, 1, 0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_s]){
      SetTransMX4(t,0,0,-0.1f);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_w]){
      SetTransMX4(t,0,0,0.1f);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_UP]){
      SetTransMX4(t,0,-0.1f,0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_DOWN]){
      SetTransMX4(t,0,0.1f,0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_a]){
      SetTransMX4(t,0.1f,0,0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_d]){
      SetTransMX4(t,-0.1f,0,0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_PAGEUP]){
      AATOMX(t, -0.05f, 1, 0, 0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_PAGEDOWN]){
      AATOMX(t, 0.05f, 1, 0, 0);
      MulMatrixMX4(m,cam,t);
      CopyMX4(cam,m);
    }
    if(pkeys[SDLK_ESCAPE])
      return 0;
  }
  
  glMultMatrixf(cam);
  return 1;
}

//-----------------------------------------------------------------------------

int GetKey(){
  SDL_Event event;
  if(SDL_PollEvent(&event)){

    if(event.type == SDL_KEYDOWN)
      return event.key.keysym.sym;
  }
  return -1;
  
}

//-----------------------------------------------------------------------------

void DrawAxis()
{
  glBegin(GL_LINES);
  glColor3f(1,0,0);
  glVertex3f(0,0,0);glVertex3f(1,0,0);
  glColor3f(0,1,0);
  glVertex3f(0,0,0);glVertex3f(0,1,0);
  glColor3f(0,0,1);
  glVertex3f(0,0,0);glVertex3f(0,0,1);
  glEnd();
}

void DrawGrid(int n = 25){
  float dist = 1;

  glBegin(GL_LINES);
  glColor3f(0,0.5f,0);

  for(int i=-n;i<n;i++){
    glVertex3f(i*dist,0,-n*dist);glVertex3f(i*dist,0,n*dist);
    glVertex3f(-n*dist,0,i*dist);glVertex3f(n*dist,0,i*dist);
  }
  glEnd();
}

//-----------------------------------------------------------------------------

void SetupPerspective(double av,double ar,double n,double f){
	glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
	gluPerspective(av,ar,n,f);
	glMatrixMode(GL_MODELVIEW);
}

#endif//DEMO
}//namespace
/*****************************************************************************/

