/**
*******************************************************************************

  @file /gui/common/minimap.cpp
  @brief mini-mapa


  @author Pavel
  @version 0.1

******************************************************************************/
#include "gui/common/minimap.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include "gui/common/mymath.h"
#include "gui/engine/SceneGeom.h"
#include "gui/model/guimap.h"
#include "gui/engine/SceneGeom.h"
#include "gui/engine/TextureManager.h"
#include "gui/common/colors.h"
#include "gui/GUI.h"
#include "gui/contexts/Game.h"

namespace gui{
using namespace World;

#define MINIMAP_BMP_WIDTH	256
#define MINIMAP_BMP_HEIGHT	256
#define MINIMAP_WIDTH		(width - 2)
#define MINIMAP_HEIGHT		(height - 2)
#define MINIMAP_COLOR_CHANELS	4
#define MINIMAP_NOOWNER -1
/*guiMiniMap********************************************************************************/

TguiMapPreview::TguiMapPreview(TguiMap *_guiMap, int _x, int _y, int _width, int _height)
				: TButton(AO_NORMAL, _x, _y, _width, _height)
{
	for (int i = 0; i < MINIMAP_BMP_COUNT; i++) {
		BmpData[i] = NULL;
		hwtidBmp[i] = 0;
	}

  myvisibility = NULL;
  myterrain = NULL;
	myelevation = NULL;
  myunit_occurrence = NULL;
  mybuildingowners = NULL;
  myunitowners = NULL;
  myhex_kingdoms = NULL;

	guiMap = _guiMap;

	ActualView = 0;
  refreshFlag = 0;
}

TguiMapPreview::~TguiMapPreview()
{
	int i;
	// odalokuj bitmapy
	for (i = 0; i < MINIMAP_BMP_COUNT; i++){
		if (BmpData[i] != NULL)
		{
			KMemFree(BmpData[i]);
      BmpData[i] = NULL;
		}
	}

  // odalokuj kopii sveta
  if (myvisibility != NULL) KMemFree(myvisibility);
  myvisibility = NULL;
  if (myterrain != NULL) KMemFree(myterrain);
  myterrain = NULL;
  if (myelevation != NULL) KMemFree(myelevation);
  myelevation = NULL;
  if (myunit_occurrence != NULL) KMemFree(myunit_occurrence);
  myunit_occurrence = NULL;
  if (mybuildingowners != NULL) KMemFree(mybuildingowners);
  mybuildingowners = NULL;
  if (myunitowners != NULL) KMemFree(myunitowners);
  myunitowners = NULL;
  if (myhex_kingdoms != NULL) KMemFree(myhex_kingdoms);
  myhex_kingdoms = NULL;

	// smaz textury z graf. karty
	for (i = 0; i < MINIMAP_COUNT; i++){
		if (hwtidBmp[i] != 0)
		{
			glDeleteTextures(1, &hwtidBmp[i]);
		}
	}

}

void TguiMapPreview::done()
{
	int i;
	// odalokuj bitmapy
	for (i = 0; i < MINIMAP_BMP_COUNT; i++){
		if (BmpData[i] != NULL)
		{
			KMemFree(BmpData[i]);
      BmpData[i] = NULL;
		}
	}

  // odalokuj kopii sveta
  if (myvisibility != NULL) KMemFree(myvisibility);
  myvisibility = NULL;
  if (myterrain != NULL) KMemFree(myterrain);
  myterrain = NULL;
  if (myelevation != NULL) KMemFree(myelevation);
  myelevation = NULL;
  if (myunit_occurrence != NULL) KMemFree(myunit_occurrence);
  myunit_occurrence = NULL;
  if (mybuildingowners != NULL) KMemFree(mybuildingowners);
  mybuildingowners = NULL;
  if (myunitowners != NULL) KMemFree(myunitowners);
  myunitowners = NULL;
  if (myhex_kingdoms != NULL) KMemFree(myhex_kingdoms);
  myhex_kingdoms = NULL;
}

// velke cislo
#define BIG_FLOAT	1000000.0f

// prevody mezi svetovymi souradnicemi a souradnicemi minimapy
#define WORLD2MINIX(x) basex+(int)((((x) - guiMap->maincam->xmin) * (width - 1) / rwidth)*xratio)
#define WORLD2MINIY(y) basey+(int)(((height - 1) - ((y) - guiMap->maincam->ymin) * (height - 1) / rheight)*yratio)
#define MINI2WORLDX(x) (( ((x) - basex) / xratio ) * rwidth / (width - 1) + guiMap->maincam->xmin)
#define MINI2WORLDY(y) (( (height - 2) - ((y) - basey) / yratio ) * rheight / (height - 1) + guiMap->maincam->ymin)

void TguiMapPreview::drawFrustum()
{
	float tg, tc;
	
	// nastaveni zobrazovaci matice
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(1, width - 1, height - 1, 1);
	glViewport(TaoKernel::aoKernel->parentpos.x + x + 1, SCREEN_HEIGHT - (TaoKernel::aoKernel->parentpos.y + y 
		+ height) + 1, width - 2, height - 2);

	// frustum hlavni kamery
	TFrustum fr;
	TRay ray;
	guiMap->maincam->getFrustum(&fr);

	// orezavaci rovina ve vzdalenosti 1/2*far_clip
	TPlane clipplane;
	P3F planevert;
	CopyV3(planevert.p, fr.dir.p);
	MulV3(planevert.p, (fr.far_clip/2));
	AddV3(planevert.p, fr.pos.p);
	clipplane.set(planevert, fr.dir);

	// rovina zeme
	TPlane groundplane;
	groundplane.set(0, 0, 1, -guiMap->maincam->centre.z);

	// levy horni roh kamery
	P3F leftup;
	ray.set(fr.pos, fr.dir, fr.near_clip, fr.far_clip);
	AddV3(ray.dir.p, fr.left.p);
	AddV3(ray.dir.p, fr.up.p);
	if (!groundplane.Collision(&ray, &tg)) tg = BIG_FLOAT;
	if (!clipplane.Collision(&ray, &tc)) tc = BIG_FLOAT;
	ray.getP3F(MIN(tg, tc), &leftup);
	
	// pravy horni roh kamery
	P3F rightup;
	ray.set(fr.pos, fr.dir, fr.near_clip, fr.far_clip);
	AddV3(ray.dir.p, fr.right.p);
	AddV3(ray.dir.p, fr.up.p);
	if (!groundplane.Collision(&ray, &tg)) tg = BIG_FLOAT;
	if (!clipplane.Collision(&ray, &tc)) tc = BIG_FLOAT;
	ray.getP3F(MIN(tg, tc), &rightup);

	// levy dolni roh kamery
	P3F leftdown;
	ray.set(fr.pos, fr.dir, fr.near_clip, fr.far_clip);
	AddV3(ray.dir.p, fr.left.p);
	AddV3(ray.dir.p, fr.down.p);
	if (!groundplane.Collision(&ray, &tg)) tg = BIG_FLOAT;
	if (!clipplane.Collision(&ray, &tc)) tc = BIG_FLOAT;
	ray.getP3F(MIN(tg, tc), &leftdown);

	// pravy dolni roh kamery
	P3F rightdown;
	ray.set(fr.pos, fr.dir, fr.near_clip, fr.far_clip);
	AddV3(ray.dir.p, fr.right.p);
	AddV3(ray.dir.p, fr.down.p);
	if (!groundplane.Collision(&ray, &tg)) tg = BIG_FLOAT;
	if (!clipplane.Collision(&ray, &tc)) tc = BIG_FLOAT;
	ray.getP3F(MIN(tg, tc), &rightdown);

	// vypoctene pruseciky spoji carama
	glBegin(GL_LINE_LOOP);
		glVertex2i(WORLD2MINIX(leftup.x), WORLD2MINIY(leftup.y) );
		glVertex2i(WORLD2MINIX(rightup.x), WORLD2MINIY(rightup.y) );
		glVertex2i(WORLD2MINIX(rightdown.x), WORLD2MINIY(rightdown.y) );
		glVertex2i(WORLD2MINIX(leftdown.x), WORLD2MINIY(leftdown.y) );
	glEnd();

	// obnoveni zobrazeni
	glViewport(0,0,SCREEN_WIDTH,SCREEN_HEIGHT);
	glPopMatrix();

}

void TguiMapPreview::drawObject()
{
	if (active == AO_PASIVE) return;

  //obnov obsah(Textura se da obnovit jen v hlavnim okne!!)
  if( refreshFlag ){
    refreshFlag = 0;
    for (int i = 0; i < MINIMAP_BMP_COUNT; i++) {
      TTexture *t;
      t = TTextureManager::tm->findTexture(BmpData[i]);
      TTextureManager::tm->reloadTextureNow(t);
    }
  }

	// okraj mapy
	glColor4fv(light_col);
	glRectUp(x,y,width,height);
	glRectDown(x,y,width,height);
	glColor3f(0, 0, 0);
	glRecti(x + 1, y + 1, x + width - 1, y + height - 1);

	// bitmapa mapy

	// protoze velikost textury je mocnina 2...
	float texright = MINIMAP_WIDTH / (float) MINIMAP_BMP_WIDTH;
	float texdown = MINIMAP_HEIGHT / (float) MINIMAP_BMP_HEIGHT;
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, hwtidBmp[ActualView]);
	glColor3f(1, 1, 1);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2i(x + 1, y + 1);
		
		glTexCoord2f(texright, 0.0f);
		glVertex2i(x + width - 1, y + 1);

		glTexCoord2f(texright, texdown);
		glVertex2i(x + width - 1, y + height - 1);

		glTexCoord2f(0.0f, texdown);
		glVertex2i(x + 1, y + height - 1);
	
	glEnd();
	glDisable(GL_TEXTURE_2D);


	// nakresli lichobeznik kamer
	if (ActualView == 0) glColor3f(0.5f, 0.5f, 0.5f);
  else glColor3f(1, 1, 1);
	drawFrustum();

}

int TguiMapPreview::workInput(INPUT *input)
{
	int ret = TButton::workInput(input);
	if PUSHED(this) {
		// zpetna vazba z minimapy do opravdove mapy
		guiMap->maincam->setCentre( MINI2WORLDX(input->mousepos.x-x),
			MINI2WORLDY(input->mousepos.y-y) );
		
	}
	return ret;
}

void TguiMapPreview::terrainColor(int hid, unsigned char *col)
{
	int iter = (int)myterrain[hid] - (int)TT_PLAIN;
	SETV3(col, ColorTerrain[iter][0], ColorTerrain[iter][1], ColorTerrain[iter][2] );
}

void TguiMapPreview::kingdomColor(int hid, unsigned char *col)
{
	int iter = myhex_kingdoms[hid];
  SETV3(col, ColorPlayer[iter][0], ColorPlayer[iter][1], ColorPlayer[iter][2] );
}


void TguiMapPreview::levelColor(int hid, unsigned char *col)
{
	unsigned char elv = (128 * myelevation[hid] ) / ELEVATION_TYPES_COUNT + 128;
	SETV3(col, elv, elv, elv);
		
}

void TguiMapPreview::raseColor(HEX *whex, unsigned char *col)
{
	SETV3(col, ColorRase[0][0], ColorRase[0][1], ColorRase[0][2]);
	// zmeni se na rasu
	SETV3(col, (unsigned char)(ColorRase[0][0] / ((float) whex->unit_occurrence.length+1)), 
		         (unsigned char)(ColorRase[0][1] / ((float) whex->unit_occurrence.length+1)), 
             (unsigned char)(ColorRase[0][2] / ((float) whex->unit_occurrence.length+1)));
	for (int i = 0; i < whex->unit_occurrence.length; i++)
	{
		col[0] += (unsigned char)(ColorRase[whex->unit_occurrence.data[i]][0] / ((float) whex->unit_occurrence.length+1));
		col[1] += (unsigned char)(ColorRase[whex->unit_occurrence.data[i]][1] / ((float) whex->unit_occurrence.length+1));
		col[2] += (unsigned char)(ColorRase[whex->unit_occurrence.data[i]][2] / ((float) whex->unit_occurrence.length+1));
	}
}

int TguiMapPreview::getHex(int x, int y)
{
	int hx, hy;
	float fx = MINI2WORLDX(x + 0.5f);
	float fy = MINI2WORLDY(y + 0.5f);
	
	if (guiMap->inWhichHex(fx, fy, &hx,&hy)){
		if ( (hx < MAP_BOUND_X) || (hx >= wwidth - MAP_BOUND_X) ||
			(hy < MAP_BOUND_Y) || (hy >= wheight - MAP_BOUND_Y) ) return -1;
		return hy * wwidth + hx;
	}
	return -1;

}

void TguiMapPreview::drawRect(int x1, int y1, int x2, int y2, unsigned char *col)
{
	int index;
  int k;
	if (x1 < 0) x1 = 0;
	if (y1 < 0) y1 = 0;
	if (x2 >= MINIMAP_BMP_WIDTH) x2 = MINIMAP_BMP_WIDTH - 1;
	if (y2 >= MINIMAP_BMP_HEIGHT) y2 = MINIMAP_BMP_HEIGHT - 1;
	if (col == NULL) {
		for (int y = y1; y <= y2; y++)
			for (int x = x1; x <= x2; x++){
				index = x + MINIMAP_BMP_WIDTH * y;
				CopyV3((BmpData[0] + MINIMAP_COLOR_CHANELS * index), (BmpData[3] + MINIMAP_COLOR_CHANELS * index));
			}

	} else {
		for (int y = y1; y <= y2; y++)
			for (int x = x1; x <= x2; x++){
				index = x + MINIMAP_BMP_WIDTH * y;
        for (k = 0; k < MINIMAP_COUNT; k++)
				SETV3((BmpData[k] + MINIMAP_COLOR_CHANELS * index), col[0], col[1], col[2]);
		}
	}
}

void TguiMapPreview::drawRect(int index, int radius, unsigned char *col)
{
	P3F p3f;
	guiMap->hexCentre(&p3f, index);
	int fx = WORLD2MINIX(p3f.x) - radius / 2;
	int fy = WORLD2MINIY(p3f.y) - radius / 2;

	if (col == NULL) {
		drawRect(fx - 1, fy - 1, fx + radius + 1, fy + radius + 1, NULL
			);
	} else {
		drawRect(fx, fy, fx + radius, fy + radius, col);
    if (radius > 4) {
		  drawRect(fx, fy - 1, fx + radius, fy - 1, col);
      drawRect(fx, fy + radius + 1, fx + radius, fy + radius + 1, col);
      drawRect(fx - 1, fy + 1, fx - 1, fy + radius - 1, col);
      drawRect(fx + radius + 1, fy + 1, fx +radius + 1, fy + radius - 1, col);
    } else if (radius == 2) {
		  drawRect(fx, fy - 1, fx + radius, fy - 1, ColorBlack);
      drawRect(fx, fy + radius + 1, fx + radius, fy + radius + 1, ColorBlack);
      drawRect(fx - 1, fy + 1, fx - 1, fy + radius - 1, ColorBlack);
      drawRect(fx + radius + 1, fy + 1, fx +radius + 1, fy + radius - 1, ColorBlack);
    } else {
		  drawRect(fx - 1, fy - 1, fx + radius + 1, fy - 1, ColorBlack);
		  drawRect(fx - 1, fy + radius + 1, fx + radius + 1, fy + radius + 1, ColorBlack);
		  drawRect(fx - 1, fy, fx - 1, fy + radius, ColorBlack);
		  drawRect(fx + radius + 1, fy, fx +radius + 1, fy + radius, ColorBlack);
    }
	}
}

void TguiMapPreview::placeBuildingNUnits(int playerid)
{
	int i, j;

  //data z worldu
	World::MAP* wmap=NULL;
	World::TLivingBuildingsContainer* lbuild = NULL;
	World::TLivingUnitsContainer *lunit = NULL;
	((TGame*)TGUI::gui_inst->game)->getWorldData(&wmap, &lunit, &lbuild);

  // viditelnost / historie
  World::TVisibilityMap *vmap;
  World::TVisibilityMapsContainer *visibility_maps = NULL;
  ((TGame*)TGUI::gui_inst->game)->getWorldVisibility(&visibility_maps);

  World::TVisibilityMapsContainer::iterator it = visibility_maps->begin();
  while ((it != visibility_maps->end()) && (it->first != playerid)) it++;
  if (it == visibility_maps->end()) return;
  vmap = it->second;

  //zamknout

  int indexHex;
	for (i = MAP_BOUND_Y; i < wmap->height - MAP_BOUND_Y; i++) {
		for (j = MAP_BOUND_X; j < wmap->width - MAP_BOUND_X; j++) {
      switch (vmap->getHexAt(j, i).getVisibility())
      {
        case VI_IN_SIGHT:
          indexHex = i * wmap->width + j;
          // jednotky + budovy
          updateHex(indexHex);
          break;
        case VI_SEEN:
          // history = &vmap->getHexAt(j, i).getHistory();  
          // history->data().building;
          break;
      
      }
		}
	}

  //odemknouot

}

World::LIVING_BUILDING* TguiMapPreview::getBuildingData(World::TLivingBuildingsContainer *lbuild, int lbuildid)
{
	return &(  (*lbuild)[lbuildid]->data() );
}

World::LIVING_UNIT* TguiMapPreview::getUnitData(World::TLivingUnitsContainer *lunit, int lunitid)
{
	return &(  (*lunit)[lunitid]->data() );
}


void TguiMapPreview::updateHex(int index)
{

  if (mybuildingowners[index] >= 0) drawRect(index, 4, ColorPlayer[mybuildingowners[index]]);
  else if (myunitowners[index] >= 0) drawRect(index, 2, ColorPlayer[myunitowners[index]]);
	
}

void TguiMapPreview::updateHW()
{
	
}

void TguiMapPreview::makeBitmap()
{
	
	rwidth = guiMap->maincam->xmax - guiMap->maincam->xmin;
	rheight = guiMap->maincam->ymax - guiMap->maincam->ymin;

	if (rwidth < rheight){
		basey = 0;	
		yratio = 1;
		
		xratio = rwidth / rheight;
		basex = (int)((1 - xratio) * (width - 2) / 2);
	} else {
		basex = 0;	
		xratio = 1;
		
		yratio = rheight / rwidth;
		basey = (int)((1 - yratio) * (height - 2) / 2);
	}


	int x, y, indexBmp, indexHex;
	
	for (int i = 0; i < MINIMAP_BMP_COUNT; i++) {
		memset(BmpData[i], 0, MINIMAP_COLOR_CHANELS * BmpHeight * BmpWidth);
		for (x = 0; x < BmpHeight * BmpWidth; x++)
			BmpData[i][x*4+3] = 255;
	}

  World::MAP *wmap = NULL;
  ((TGame*)TGUI::gui_inst->game)->getWorldData(&wmap, NULL, NULL);

	for (y = 0; y < BmpHeight; y++) {
		for (x = 0; x < BmpWidth; x++) {
			indexBmp = MINIMAP_COLOR_CHANELS * (BmpWidth * y + x);
			
			if ((indexHex = getHex(x, y)) > -1) {

				// 0...bitmapa s terenem
				terrainColor(indexHex, BmpData[0] + indexBmp);
				// 1...bitmapa s nadmorskymi vyskami
				levelColor(indexHex, BmpData[1] + indexBmp);
        // 2...bitmapa s kralovstvimi
        kingdomColor(indexHex, BmpData[2] + indexBmp);
				// 3...bitmapa s rasama
				raseColor(wmap->hexes[indexHex], BmpData[3] + indexBmp);

        // korektura barvy mesta
        if (wmap->hexes[indexHex]->terrain == TT_TOWN) {
          unsigned char *col;
          col = BmpData[0] + indexBmp;
          int pid = ((TGame*)TGUI::gui_inst->game)->guiMap->getTownHexOwner(indexHex);
          if ( ((x % 3) == 0) ^ ((y % 2) == 0) ) {
            col[0] = (ColorPlayer[pid][0]);
            col[1] = (ColorPlayer[pid][1]);
            col[2] = (ColorPlayer[pid][2]);
          }
/*          col[0] >>= 1;
          col[1] >>= 1;
          col[2] >>= 1;
          int pid = ((TGame*)TGUI::gui_inst->game)->guiMap->getTownHexOwner(indexHex);
          col[0] += (ColorPlayer[pid][0] >> 1);
          col[1] += (ColorPlayer[pid][1] >> 1);
          col[2] += (ColorPlayer[pid][2] >> 1);*/
        }


			}
		}
	}

}

void TguiMapPreview::applyVisibility(int playerid)
{
  int indexBmp, indexHex;

  World::TVisibilityMap *vmap;
  World::TVisibilityMapsContainer *visibility_maps = NULL;
	World::MAP* wmap = NULL;
	((TGame*)TGUI::gui_inst->game)->getWorldData(&wmap, NULL, NULL);
        ((TGame*)TGUI::gui_inst->game)->getWorldVisibility(&visibility_maps);

  World::TVisibilityMapsContainer::iterator it = visibility_maps->begin();
  while ((it != visibility_maps->end()) && (it->first != playerid)) it++;
  if (it == visibility_maps->end()) return;
  vmap = it->second;

	
	for (int y = 0; y < BmpHeight; y++) {
		for (int x = 0; x < BmpWidth; x++) {
			indexBmp = MINIMAP_COLOR_CHANELS * (BmpWidth * y + x) + 3;
			
			if ((indexHex = getHex(x, y)) > -1) {

        switch (myvisibility[indexHex])
        {
        case VI_NEVER_SEEN:
          SETV3( (BmpData[0] + indexBmp - 3), 0, 0, 0);
          SETV3( (BmpData[1] + indexBmp - 3), 0, 0, 0);
          SETV3( (BmpData[2] + indexBmp - 3), 0, 0, 0);
          SETV3( (BmpData[3] + indexBmp - 3), 0, 0, 0);
          break;
        case VI_SEEN:
          (BmpData[0] + indexBmp - 3)[0] >>= 1;
          (BmpData[0] + indexBmp - 3)[1] >>= 1;
          (BmpData[0] + indexBmp - 3)[2] >>= 1;
          break;
        }
			}

		}
	}
}


void TguiMapPreview::init()
{
	int i;
	
	// rozmery bitmapy
  BmpWidth = MINIMAP_BMP_WIDTH;
	BmpHeight = MINIMAP_BMP_HEIGHT;

	// alokuju pamet pro bitmapy minimapy
	for (i = 0; i < MINIMAP_BMP_COUNT; i++)
		BmpData[i] = (unsigned char *) KMemAlloc( sizeof(unsigned char) * BmpWidth * BmpHeight * MINIMAP_COLOR_CHANELS );

	// rozmery sveta
  World::MAP* wmap = NULL;
	((TGame*)TGUI::gui_inst->game)->getWorldData(&wmap, NULL, NULL);
  wwidth = wmap->width;
  wheight = wmap->height;

  // alokace poli pro kopii sveta
  myvisibility = (VISIBILITY *)KMemAlloc( sizeof(VISIBILITY) * wwidth * wheight);
  myterrain = (TERRAIN_TYPE *)KMemAlloc( sizeof(TERRAIN_TYPE) * wwidth * wheight);
  myelevation = (ELEVATION_TYPE *)KMemAlloc( sizeof(ELEVATION_TYPE) * wwidth * wheight);
  myunit_occurrence = (int *)KMemAlloc( sizeof(int) * wwidth * wheight);
  mybuildingowners = (int *)KMemAlloc( sizeof(int) * wwidth * wheight);
  myunitowners = (int *)KMemAlloc( sizeof(int) * wwidth * wheight);
  myhex_kingdoms = (int *)KMemAlloc( sizeof(int) * wwidth * wheight);

	//zadej texture manageru, pozdeji je nahraje do HW
	for (i = 0; i < MINIMAP_BMP_COUNT; i++)
          TTextureManager::tm->loadTextureQuery(BmpData[i], BmpWidth, BmpHeight, &(hwtidBmp[i]));

}

void TguiMapPreview::refresh(int playerid, TStep *stp)
{
  updateWorldCopy(playerid, stp);
  // sestavim bitmapu podle mapy sveta
	makeBitmap();
	// BmpData[0] = BmpData[3] + Budovy + Unity
	placeBuildingNUnits(playerid);
	// schova neviditelne hexy
	applyVisibility(playerid);
  //nastav flag pro obnoveni dat na hlavnim vlakne
  refreshFlag = 1;
}

void TguiMapPreview::updateVisibility(int playerid, TIntContainer *shown, TIntContainer *hidden)
{
//  updateWorldCopy(playerid, shown, hidden);
  updateWorldCopy(playerid, NULL);
  // sestavim bitmapu podle mapy sveta
	makeBitmap();
	// BmpData[0] = BmpData[3] + Budovy + Unity
	placeBuildingNUnits(playerid);
	// schova neviditelne hexy
	applyVisibility(playerid);
  //nastav flag pro obnoveni dat na hlavnim vlakne
  refreshFlag = 1;
}

void TguiMapPreview::refreshSpecial(int playerid)
{
  // sestavim bitmapu podle mapy sveta
	makeBitmap();
	// BmpData[0] = BmpData[3] + Budovy + Unity
	placeBuildingNUnits(playerid);
	// schova neviditelne hexy
	applyVisibility(playerid);
  //nastav flag pro obnoveni dat na hlavnim vlakne
  refreshFlag = 1;
}


void TguiMapPreview::updateWorldCopy(int playerid)
{
  int i;

  World::MAP *wmap = NULL;
  World::TLivingUnitsContainer *lu = NULL;
  World::TLivingBuildingsContainer *lb = NULL;
  World::TVisibilityMapsContainer *visibility_maps = NULL;
  World::TVisibilityMap *vmap = NULL;

  ((TGame*)TGUI::gui_inst->game)->getWorldData(&wmap, &lu, &lb);
  ((TGame*)TGUI::gui_inst->game)->getWorldVisibility(&visibility_maps);

  World::TVisibilityMapsContainer::iterator itm = visibility_maps->begin();
  while ((itm != visibility_maps->end()) && (itm->first != playerid)) itm++;
  if (itm == visibility_maps->end()) return;
  vmap = itm->second;
 
  for (i = 0; i < wwidth * wheight; i++)
  {
    mybuildingowners[i] = MINIMAP_NOOWNER;
    myunitowners[i] = MINIMAP_NOOWNER;
    myvisibility[i] = vmap->getHexById(i).getVisibility();
    myterrain[i] = wmap->hexes[i]->terrain;
    myelevation[i] = wmap->hexes[i]->elevation;
    myunit_occurrence[i] = 1; ///< doplnit smysluplnou hodnotu
    myhex_kingdoms[i] = 
      ((TGame*)TGUI::gui_inst->game)->guiMap->hexy[i]->owner;
    
    if (wmap->hexes[i]->unit > 0) myunitowners[i] = 
      getUnitData(lu, wmap->hexes[i]->unit)->player;
    if (wmap->hexes[i]->building > 0) mybuildingowners[i] = 
      getBuildingData(lb, wmap->hexes[i]->building)->player;

  }

}

void TguiMapPreview::updateWorldCopy(int playerid, TIntContainer *shown, TIntContainer *hidden)
{
  TIntContainer::iterator it;
  
  if (hidden) {
    it = hidden->begin();
    while ( it != hidden->end() ) {
      myvisibility[ (*it) ] = VI_SEEN;
      it++;
    }
  }

  if (shown) {
    it = shown->begin();
    while ( it != shown->end() ) {
      myvisibility[ (*it) ] = VI_IN_SIGHT;
      it++;
    }
  }

}

void TguiMapPreview::updateWorldCopy(int playerid, TStep *stp)
{
  TIntContainer::iterator it;
  if (stp == NULL) {
    updateWorldCopy(playerid);
    return;
  }
  
  it = stp->hidden.begin();
  while ( it != stp->hidden.end() ) {
    myvisibility[ (*it) ] = VI_SEEN;
    it++;
  }

  it = stp->shown.begin();
  while ( it != stp->shown.end() ) {
    myvisibility[ (*it) ] = VI_IN_SIGHT;
    it++;
  }

  myunitowners[stp->hex_id] = myunitowners[stp->left_hex_id];
  myunitowners[stp->left_hex_id] = -1;

}

void TguiMapPreview::moveUnit(int id_hex_from, int id_hex_to)
{
  // presun jednotky
  myunitowners[id_hex_to] = myunitowners[id_hex_from];
  myunitowners[id_hex_from] = -1;
  // sestavim bitmapu podle mapy sveta
	makeBitmap();
	// BmpData[0] = BmpData[3] + Budovy + Unity
	placeBuildingNUnits(((TGame*)TGUI::gui_inst->game)->iGUIOwner);
	// schova neviditelne hexy
	applyVisibility(((TGame*)TGUI::gui_inst->game)->iGUIOwner);
  //nastav flag pro obnoveni dat na hlavnim vlakne
  refreshFlag = 1;
}


/*TguiMiniMap*****************************************************************/

TguiMiniMap::TguiMiniMap(TguiMap *_guiMap, int _x, int _y, int _width, int _height)
: TGroupBar(AO_HIDDEN, _x, _y, _width, _height)
{
	wchar_t wtemp[LABEL_MAX_LENGTH];
	rm::TRM_txt_i *rmtxti = (rm::TRM_txt_i *)KSendGlobalMessage(MSG_GET_RM_TXT_I, MOD_GUI, MOD_RM, NULL);

  preview = new TguiMapPreview(_guiMap, MINIMAP_RADIO_HEIGHT + MINIMAP_BORDER, 0, _height, _height);
	TGroupBar::Add(preview);

  //nacti ikony
	rm::TRM_symbol_i* rms = (rm::TRM_symbol_i*) KSendGlobalMessage(MSG_GET_RM_SYMBOL_I, MOD_GUI, MOD_RM, NULL);
  rm::TSymbolData *sdUB,*sdUBS,*sdTT,*sdTTS, *sdK,*sdKS, *sdO,*sdOS;  
  rms->getSymbol(&sdUB,  SYMBOLS_BASIC_ICONS, 22);
  rms->getSymbol(&sdUBS, SYMBOLS_BASIC_ICONS, 25);
  rms->getSymbol(&sdTT,  SYMBOLS_BASIC_ICONS, 23);
  rms->getSymbol(&sdTTS, SYMBOLS_BASIC_ICONS, 26);
  rms->getSymbol(&sdK,   SYMBOLS_BASIC_ICONS, 50);
  rms->getSymbol(&sdKS,  SYMBOLS_BASIC_ICONS, 51);
  rms->getSymbol(&sdO,   SYMBOLS_BASIC_ICONS, 24);
  rms->getSymbol(&sdOS,  SYMBOLS_BASIC_ICONS, 27);
  delete rms; 

	int basex = 0;
	int basey = 0;
	radio[0] = new TRadioBox(AO_SELECTED, basex, basey + 0 * MINIMAP_RADIO_HEIGHT,
		MINIMAP_RADIO_HEIGHT - 2, MINIMAP_RADIO_HEIGHT - 2,
    sdUB,sdUBS,sdUB);
	Add(radio[0]);
	radio[1] = new TRadioBox(0, basex, basey + 1 * MINIMAP_RADIO_HEIGHT,
		MINIMAP_RADIO_HEIGHT - 2, MINIMAP_RADIO_HEIGHT - 2,
    sdTT,sdTTS,sdTT);
	Add(radio[1]);
	radio[2] = new TRadioBox(0, basex, basey + 2 * MINIMAP_RADIO_HEIGHT,
		MINIMAP_RADIO_HEIGHT - 2, MINIMAP_RADIO_HEIGHT - 2,
    sdK,sdKS,sdKS);
	Add(radio[2]);
	radio[3] = new TRadioBox(0, basex, basey + 3 * MINIMAP_RADIO_HEIGHT,
		MINIMAP_RADIO_HEIGHT - 2, MINIMAP_RADIO_HEIGHT - 2,
    sdO,sdOS,sdOS);
	Add(radio[3]);

  radio[0]->setHelp(rmtxti->getWChar(wtemp, LABEL_MAX_LENGTH, TEXT_MINIMAPUNITS));
  radio[1]->setHelp(rmtxti->getWChar(wtemp, LABEL_MAX_LENGTH, TEXT_MINIMAPHIGHTLEVELS));
  radio[2]->setHelp(rmtxti->getWChar(wtemp, LABEL_MAX_LENGTH, TEXT_MINIMAPKINGDOMS));
  radio[3]->setHelp(rmtxti->getWChar(wtemp, LABEL_MAX_LENGTH, TEXT_MINIMAPRASES));

  delete rmtxti;
}

void TguiMiniMap::init()
{
	preview->init();
}

void TguiMiniMap::done()
{
  preview->done();
}


void TguiMiniMap::refresh(int playerid, TStep *stp)
{
	preview->refresh(playerid, stp);
}

void TguiMiniMap::refreshSpecial(int playerid)
{
  preview->refreshSpecial(playerid);
}

int TguiMiniMap::workInput(INPUT *input)
{
	int ret = TGroupBar::workInput(input);
	for (int i = 0; i < MINIMAP_COUNT; i++)
		if (SELECTED(radio[i])) preview->ActualView = i;
	return ret;
}

void TguiMiniMap::updateVisibility(int playerid, TIntContainer *shown, TIntContainer *hidden)
{
  preview->updateVisibility(playerid, shown, hidden);
}

void TguiMiniMap::moveUnit(int id_hex_from, int id_hex_to)
{
  preview->moveUnit(id_hex_from, id_hex_to);
}


}//namespace
/*****************************************************************************/

