/**
*******************************************************************************

  @file /gui/common/minimap.h
  @brief mini-mapa
  

  @author Pavel
  @version 0.1
 
******************************************************************************/
#ifndef MINIMAP_H
#define MINIMAP_H

#include "gui/common/ActiveObjects.h"
#include "world/plan/plan.h"
#include "world/world.h"

namespace gui{

#define MINIMAP_COUNT			    4
#define MINIMAP_BMP_COUNT		  (MINIMAP_COUNT)
#define MINIMAP_BORDER			  6
#define MINIMAP_RADIO_HEIGHT	40


class TguiMap;

/** Obrazek s mini mapou. */
class TguiMapPreview : public TButton {
private:
	/// Graficka cast mapy.
	TguiMap *guiMap;

	/// Pomocne promenne pro urychleni vypoctu.
  //@{
  float xratio;
	float yratio;
	int basex;
	int basey;
	float rwidth;
	float rheight;
	int BmpWidth;
	int BmpHeight;
  //@}

  /// Kopie sveta.
  //@{
  /// Viditelnost.
  World::VISIBILITY *myvisibility;
  /// Teren.
  World::TERRAIN_TYPE *myterrain;
	/// Nadmorska vyska.
	World::ELEVATION_TYPE *myelevation;
  /// Vlastnici budov.
  int *mybuildingowners;
  /// Vlastnici jednotek.
  int *myunitowners;
  /// Jednotky ktere na hexu ziji.
  int *myunit_occurrence;
  /// Kralovstvi do ktereho hex spada.
  int *myhex_kingdoms;
  /// Sirka sveta.
  int wwidth;
  /// Vyska sveta.
  int wheight;
  //@}
  

	/// Rastery minimapy.
	unsigned char *BmpData[MINIMAP_BMP_COUNT];
	/// Hardwareove identifikatory pouzitich textur.
	HWTID hwtidBmp[MINIMAP_COUNT];
  /// Refreshovat pred dalsim vykreslenim.
  int refreshFlag;

	/// Nakresli do okynka s minimapou lichobeznik kamery.
	void drawFrustum();
	/// Vytvori rastery jednotlivych zobrazeni mapy.
	void makeBitmap();
	/// Do rasteru "nakresli" obdelnik [x1,y1]-[x2,y2] barvou col.
	void drawRect(int x1, int y1, int x2, int y2, unsigned char *col);
	/// Do prostredka hexu s indexem index nakresli ctverec o polomeru radius.
	void drawRect(int index, int radius, unsigned char *col);
	/// Na hexech, kde jsou budovy a panackove, nakresli ctverecky.
	void placeBuildingNUnits(int playerid);
	/// Do col ulozi barvu odpovidajici terenu na hexu int hid.
	inline void terrainColor(int hid, unsigned char *col);
	/// Do col ulozi barvu odpovidajici nadmorske vysce hexu int hid.
	inline void levelColor(int hid, unsigned char *col);
	/// Do col ulozi barvu odpovidajici rase, ktera na int hid zije.
	inline void raseColor(World::HEX *hex, unsigned char *col);
	/// Do col ulozi barvu odpovidajici kralovstvi, do ktereho patri hex hid.
	inline void kingdomColor(int hid, unsigned char *col);
	/// Vrati index hexu ktery zasahuje do pixelu [x,y] rasteru minimapy.
	int getHex(int x, int y);
	/// Podle buildid vrati budovu, ktera tomu odpovida nebo NULL.
	World::LIVING_BUILDING* getBuildingData(World::TLivingBuildingsContainer *lbuild, int lbuildid);
	/// Podle unitid vrati jednotku, ktera tomu odpovida nebo NULL.
	World::LIVING_UNIT* getUnitData(World::TLivingUnitsContainer *lunit, int lunitid);
  /// Zkopiruje si do vlastnich struktur svet, jak ho vidi hrac cislo playerid.
  void updateWorldCopy(int playerid);
  /// Upravi kopii sveta podle kroku stp.
  void updateWorldCopy(int playerid, TStep *stp);
  /// Upravi kopii sveta podle toho, co se objevilo a zmizelo.
  void updateWorldCopy(int playerid, TIntContainer *shown, TIntContainer *hidden);
public:	
	/// Cislo prave zobrazovane minimapy. Meni ho guiMiniMap podle vyberu.
	int ActualView;
	
  /// Graficky vystup a vstup od mysi.
  //@{
  virtual void drawObject();
	virtual int workInput(INPUT *input);
  //@}

	/// Konstruktor.
	TguiMapPreview(TguiMap *_guiMap, int _x, int _y, int _width, int _height);
	/// Destruktor: maze textury z hardware a odalokovava pamet pro rastery.
	~TguiMapPreview();
	/// Alokuje pamet a vytvari rastry.
	void init();
  /// Vrati pamet.
  void done();
	/// Smaze/nakresli na hexu ctverecek v zavislosti na tom zda tam je nebo neni budova/unita.
	inline void updateHex(int index);
	/// Schova hexy, ktere nejsou videt, melo by se volat po kazde zmene viditelnosti.
	void applyVisibility(int playerid);
	/// Vola se po skupine updateHex() nebo po applyVisibility(). Nahraje do HW zmenenou texturu.
	void updateHW();
	/** Vytvori nove bitmapy, stare textury z karty smaze, nahraje tam nove. */
  void refresh(int playerid, TStep *stp = NULL);
  /** Jako refresh ale shown a hidden zvlast. */
  void updateVisibility(int playerid, TIntContainer *shown, TIntContainer *hidden);
  /** Vytvori nove bitmapya a nahraje do karty, ale nevola updateWorldCopy. */
  void refreshSpecial(int playerid);
  /** Presune jednotku z hexu from na hex to. */
  void moveUnit(int id_hex_from, int id_hex_to);
};

/** Minimapa vcetne prepinacich buttonu */
class TguiMiniMap : public TGroupBar
{
private:
	TguiMapPreview *preview;
	TRadioBox *radio[MINIMAP_COUNT];
public:
	/// Pouze zakladni konstruktor, nic moc nealokuje ani nezavadi textury do karty.
	TguiMiniMap(TguiMap *_guiMap, int _x, int _y, int _width, int _height);
	/** Naalokuje pamet, vytvori bitmapy (tj potrebuje aby TGame znal svet), nahraje textury
		do karty. */
	void init();
  /** Vrati pamet. */
  void done();
	/** Ma smysl volat, pokud se zmenil svet (nebo jiny hrac sel na radu). Vytvori nove bitmapy,
		stare textury z karty smaze, nahraje tam nove. */
  void refresh(int playerid, TStep *stp = NULL);
  /** Jako refresh ale shown a hidden zvlast. */
  void updateVisibility(int playerid, TIntContainer *shown, TIntContainer *hidden);
  /** Vola se pokud je podezreni, ze jednotka prevzala nadvladu nad mestem. */
  void refreshSpecial(int playerid);
  /** Presune jednotku z hexu from na hex to. */
  void moveUnit(int id_hex_from, int id_hex_to);

  /// Vstup od mysi.
	virtual int workInput(INPUT *input);
};


} // namespace;




#endif
/*****************************************************************************/
