/**
*******************************************************************************

  @file /gui/common/mymath.cpp
  @brief Typy a funkce pro matematicke vypocty


  @author vajicek
  @version 0.1

******************************************************************************/
#include "gui/common/mymath.h"

namespace gui{

void Transform(float* m,float* x,float* y,float* z){

  P4F v,w;
  
  v.x = *x;
  v.y = *y;
  v.z = *z;
  v.u = 1;

  ApplyVectorMX4(v.p, m, w.p);

  *x = (float)(w.x/w.u);
  *y = (float)(w.y/w.u);
  *z = (float)(w.z/w.u);
}

void Transform(float* m,float* v){
  Transform(m, v , v+1, v+2);
}


void MulMatrixMX4(float *c, float *a, float *b){
	float d;
	for(int i = 0; i < 4; i++){
		for(int j = 0; j < 4; j++){
			d=0;
			for(int k = 0; k < 4; k++)
				d += a[ (i<<2) + k ] * b[ (k<<2) + j ];

			c[ (i<<2) + j ] = d;
		}
	}
}

void MultipleMulMatrixMX4(float *a, float* va[], int vac){ 
  MX4 b;
  int parity = 1;
  //
  CopyMX4(b, va[0]);
  //
  for(int i = 1; i < vac; i++){
    if(parity)
      MulMatrixMX4(a, b, va[i]);
    else
      MulMatrixMX4(b, a, va[i]);
    parity = !parity;
  }
  //
  if(!parity)
    CopyMX4(a, b);
}

void SubmatrixMX4( MX4 mr, MX3 mb, int i, int j ) {
	int di, dj, si, sj;
	// loop through 3x3 submatrix
	for( di = 0; di < 3; di ++ ) {
		for( dj = 0; dj < 3; dj ++ ) {
			// map 3x3 element (destination) to 4x4 element (source)
			si = di + ( ( di >= i ) ? 1 : 0 );
			sj = dj + ( ( dj >= j ) ? 1 : 0 );
			// copy element
			mb[di * 3 + dj] = mr[si * 4 + sj];
		}
	}
}

/*  
  Spocitase pomoci subdeterminantu radkovym rozkladem:
    detA = sum[0<=i<=4] (aij * subdet Ai)
*/
float DeterminantMX4( MX4 mr )
{
	float  det, result = 0, i = 1;
	MX3 msub3;
	int     n;
	for ( n = 0; n < 4; n++, i *= -1 )
	{
		SubmatrixMX4( mr, msub3, 0, n );
		det     = DeterminantMX3( msub3 );
		result += (float)mr[n] * det * i;
	}
	return( result );
}

/*
  Spocita se vyresenim soustavy 
  pomoci Cramerova pravidla (x = dx/dm)
*/
int InverseMX4( MX4 invers, MX4 orig )
{
	float  mdet = DeterminantMX4( orig );
	MX3 mtemp;
	int     i, j, sign;
	if ( fabs( mdet ) < 0.0005 ){
		SetIdentityMX4( invers );
		return( 0 );
	}
	for ( i = 0; i < 4; i++ )
		for ( j = 0; j < 4; j++ )
		{
			sign = 1 - ( (i +j) % 2 ) * 2;
			SubmatrixMX4( orig, mtemp, i, j );
			invers[i+j*4] = ( DeterminantMX3( mtemp ) * sign ) / mdet;
		}
	return( 1 );
}


//  Butterfly rule
float	DeterminantMX3( MX3 m )
{
	float	det;

	det =	  (float)m[0] * (m[4]*m[8] - m[7]*m[5] )
			  +	(float)m[3] * (m[7]*m[2] - m[1]*m[8] )
			  +	(float)m[6] * (m[1]*m[5] - m[4]*m[2] );
	return det;
}

void MulMatrixMX3(float *c,float *a,float *b){
	float d;
	for(int i = 0; i < 3; i++){
		for(int j = 0; j < 3; j++){
			d = 0;
			for(int k = 0; k < 3; k++)
				d += a[ i * 3 + k ] * b[ k * 3 + j ];

			c[ i * 3 + j] = d;
		}
	}
}

void ApplyVectorMX3(float* v,float* m,float* w){
	w[0]=v[0]*m[0]+v[1]*m[3]+v[2]*m[6];
	w[1]=v[0]*m[1]+v[1]*m[4]+v[2]*m[7];
	w[2]=v[0]*m[2]+v[1]*m[5]+v[2]*m[8];
}

//
void ApplyVectorMX4(float* v,float* m,float* w){
	w[0]=v[0]*m[0]+v[1]*m[4]+v[2]*m[8]+v[3]*m[12];
	w[1]=v[0]*m[1]+v[1]*m[5]+v[2]*m[9]+v[3]*m[13];
	w[2]=v[0]*m[2]+v[1]*m[6]+v[2]*m[10]+v[3]*m[14];
	w[3]=v[0]*m[3]+v[1]*m[7]+v[2]*m[11]+v[3]*m[15];
}

//( w1.w2 - v1.v2, w1.v2 + w2.v1 + v1 x v2 )
//http://skal.planet-d.net/demo/matrixfaq.htm#Q53
void QMUL(QUATER* Q1,QUATER* Q2,QUATER* Q3)
{
	Q1->w = Q2->w*Q3->w - Q2->x*Q3->x - Q2->y*Q3->y - Q2->z*Q3->z;
	Q1->x = Q2->w*Q3->x + Q3->w*Q2->x + Q2->y*Q3->z - Q2->z*Q3->y;
	Q1->y = Q2->w*Q3->y + Q3->w*Q2->y + Q2->z*Q3->x - Q2->x*Q3->z;
	Q1->z = Q2->w*Q3->z + Q3->w*Q2->z + Q2->x*Q3->y - Q2->y*Q3->x;
}

//http://skal.planet-d.net/demo/matrixfaq.htm#Q54
void QTOMX4(QUATER* q,MX4 mx4)
{
	float xx      = q->x * q->x;
  float xy      = q->x * q->y;
  float xz      = q->x * q->z;
  float xw      = q->x * q->w;
  float yy      = q->y * q->y;
  float yz      = q->y * q->z;
  float yw      = q->y * q->w;
  float zz      = q->z * q->z;
  float zw      = q->z * q->w;
  mx4[0]  = 1 - 2 * ( yy + zz );
  mx4[1]  =     2 * ( xy + zw );
  mx4[2]  =     2 * ( xz - yw );
  mx4[4]  =     2 * ( xy - zw );
  mx4[5]  = 1 - 2 * ( xx + zz );
  mx4[6]  =     2 * ( yz + xw );
  mx4[8]  =     2 * ( xz + yw );
  mx4[9]  =     2 * ( yz - xw );
  mx4[10] = 1 - 2 * ( xx + yy );
  mx4[12]  = mx4[13] = mx4[14] = mx4[3] = mx4[7] = mx4[11] = 0;
  mx4[15] = 1;
}


//http://skal.planet-d.net/demo/matrixfaq.htm
void QTOAA(QUATER* q,P3F* ax,float *ang)
{
	float d = (float)QMAG(*q);
	q->w/=d;
	q->x/=d;
	q->y/=d;
	q->z/=d;

  float cos_a = q->w;
  *ang = (float)acos( cos_a ) * 2;
  float sin_a = (float)sqrt( 1.0f - cos_a * cos_a );
  if ( fabs( sin_a ) < 0.0005f ) sin_a = 1;
  ax->x = q->x / sin_a;
  ax->y = q->y / sin_a;
  ax->z = q->z / sin_a;
}

//http://skal.planet-d.net/demo/matrixfaq.htm
void AATOQ(P3F* ax,float ang,QUATER* q)
{
	float d = (float)sqrt(ax->x*ax->x+ax->y*ax->y+ax->z*ax->z);

	float sin_a = (float)sin( ang / 2 );
  float cos_a = (float)cos( ang / 2 );

  if(d != 0){
	  q->x    = (ax->x/d) * sin_a;
    q->y    = (ax->y/d) * sin_a;
    q->z    = (ax->z/d) * sin_a;
  }
  else{
	  q->x    = 0;
    q->y    = 0;
    q->z    = 0;
  }
  q->w    = cos_a;
}


//http://www.gamasutra.com/features/19980703/quaternions_01.htm
void SLERP(const QUATER *q1, const QUATER *q2, float t, QUATER *qt)
{
  const float epsilon = 0.001f;

  float cosomega = ScalMulV4(q1->v4f, q2->v4f);

  float q2x,q2y,q2z,q2w;

  if(cosomega<0)
  {
    cosomega = -cosomega;
    q2x = -q2->x;
    q2y = -q2->y;
    q2z = -q2->z;
    q2w = -q2->w;
  }
  else
  {
    q2x = q2->x;
    q2y = q2->y;
    q2z = q2->z;
    q2w = q2->w;
  }

  if( (1 - cosomega) < epsilon ){
    qt->x = (q1->x +  (q2x - q1->x) * t);
    qt->y = (q1->y +  (q2y - q1->y) * t);
    qt->z = (q1->z +  (q2z - q1->z) * t);
    qt->w = (q1->w +  (q2w - q1->w) * t);   
  }
  else
  {
    float omega = (float)acos(cosomega);
    float sinomega = (float)fabs((float)sin(omega));
    float sin1 = (float)sin((1-t)*omega);
    float sin2 = (float)sin(t*omega);
    qt->x = (q1->x * sin1 + q2x * sin2)/sinomega;
    qt->y = (q1->y * sin1 + q2y * sin2)/sinomega;
    qt->z = (q1->z * sin1 + q2z * sin2)/sinomega;
    qt->w = (q1->w * sin1 + q2w * sin2)/sinomega;
  }
}

void MXTOQ(MX4 m, QUATER *q)
{
  float T = 1 + m[0] + m[5] + m[10];
  float S;

  if ( T > 0.00000001f ){
    S = (float)sqrt(T) * 2;
    q->x = ( m[6] - m[9] ) / S;
    q->y = ( m[8] - m[2] ) / S;
    q->z = ( m[1] - m[4] ) / S;
    q->w = 0.25f * S;
  }
  else {
    if ( (m[0] > m[5]) && (m[0] > m[10]) )  {// Column 0: 
      S  = (float)sqrt( 1.0f + m[0] - m[5] - m[10] ) * 2;
      q->x = 0.25f * S;
      q->y = (m[4] + m[1] ) / S;
      q->z = (m[2] + m[8] ) / S;
      q->w = (m[6] - m[9] ) / S;
    } else if ( m[5] > m[10] ) {// Column 1: 
      S  = (float)sqrt( 1.0f + m[5] - m[0] - m[10] ) * 2;
      q->x = (m[4] + m[1] ) / S;
      q->y = 0.25f * S;
      q->z = (m[9] + m[6] ) / S;
      q->w = (m[8] - m[2] ) / S;
    } else {// Column 2:
      S  = (float)sqrt( 1.0f + m[10] - m[0] - m[5] ) * 2;
      q->x = (m[2] + m[8] ) / S;
      q->y = (m[9] + m[6] ) / S;
      q->z = 0.25f * S;
      q->w = (m[1] - m[4] ) / S;
    }
  }
}

//Matrix to Axis Angle
void MXTOAA(MX4 m, P3F* ax, float* ang)
{
  float epsilon = 0.01f;
  if (fabs(m[1]-m[4])< epsilon && fabs(m[2]-m[8])< epsilon && fabs(m[9]-m[6])< epsilon) {
      // singularity found
      if (fabs(m[1]+m[4])<epsilon && fabs(m[2]+m[8])<epsilon && fabs(m[9]+m[6])<0.1) {
        // this singularity is identity matrix so angle = 0
        // note epsilon is greater in this case since we only have to distinguish between 0 and 180 degrees
        *ang = 0;
        ax->x = 1; // axis is arbitrary
        ax->y = 0;
        ax->z = 0;
        return;
      }
      // otherwise this singularity is angle = 180
      *ang = (float)M_PI;

      ax->x = (m[0]+1)/2;
      // can only take square root of positive number, always true for orthogonal matrix
      if (ax->x > 0) ax->x = (float)sqrt(ax->x);
      else ax->x = 0; // in case matrix has become de-orthogonalised

      ax->y = (m[5]+1)/2;
      // can only take square root of positive number, always true for orthogonal matrix
      if (ax->y > 0) ax->y = (float)sqrt(ax->y);
      else ax->y = 0; // in case matrix has become de-orthogonalised

      ax->z = (m[10]+1)/2;
      // can only take square root of positive number, always true for orthogonal matrix
      if (ax->z > 0)ax->z = (float)sqrt(ax->z);
      else ax->z = 0; // in case matrix has become de-orthogonalised

      int xZero = (fabs(ax->x)<epsilon);
      int yZero = (fabs(ax->y)<epsilon);
      int zZero = (fabs(ax->z)<epsilon);
      int xyPositive = (m[1] > 0);
      int xzPositive = (m[2] > 0);
      int yzPositive = (m[6] > 0);
      if (xZero && !yZero && !zZero) ax->y = -ax->y;
      else if (yZero && !zZero) ax->z = -ax->z;
      else if (zZero) ax->x = -ax->x;
      else if (xyPositive && xzPositive && yzPositive) return;
      else if (yzPositive) ax->x=-ax->x;
      else if (xzPositive) ax->y=-ax->y;
      else if (xyPositive) ax->z=-ax->z;
      return;
  }
  *ang = (float)acos ( ( m[0] + m[5] + m[10] - 1)/2);
  float s = (float)sqrt((m[9] - m[6])*(m[9] - m[6])+(m[2] - m[8])*(m[2] - m[8])+(m[4] - m[1])*(m[4] - m[1]));// used to normalise
  if (fabs(s) < 0.001f) s=1; 
  // prevent divide by zero, should not happen if matrix is orthogonal and should be
  // caught by singularity test above, but I've left it in just in case
  ax->x = (m[9] - m[6])/s;
  ax->y = (m[2] - m[8])/s;
  ax->z = (m[4] - m[1])/s;  
}

void MXTOTR_ROT(MX4 m,P3F* tr, QUATER *rot){
  if(tr){
    tr->x = m[12];
    tr->y = m[13];
    tr->z = m[14];
  }
  //
  if(rot){
    MX4 mm;
    CopyMX4(mm,m);
    MaskROT(mm);
    MXTOQ(mm, rot);
  }
}

//vytvor orientacni matici, jako LookAt
void ORTMX(MX4 m, P3F* P, P3F* T, P3F* N){
  P3F B;

  float sT = (float)SizeV3(T->p);
  MulV3(T->p,(1/sT));

  float sN = (float)SizeV3(N->p);
  MulV3(N->p,(1/sN));

  VectMulV3(B.p,N->p,T->p);  
  float sB = (float)SizeV3(B.p);
  MulV3(B.p,(1/sB));

  SetMX4(m, 
            B.x,  B.y,  B.z,  0,             
            N->x, N->y, N->z,  0,
            T->x, T->y, T->z,  0,
            P->x, P->y, P->z,  1);
}

void ORTMX2(MX4 m, P3F* P, P3F* T, P3F* UP){ 
  P3F N;
  P3F B;


  float sT = (float)SizeV3(T->p);
  MulV3(T->p,(1/sT));

  VectMulV3(B.p,UP->p,T->p);  
  float sB = (float)SizeV3(B.p);
  MulV3(B.p,(1/sB));

  VectMulV3(N.p,T->p,B.p);  
  float sN = (float)SizeV3(N.p);
  MulV3(N.p,(1/sN));

  SetMX4(m, 
            B.x,  B.y,  B.z,  0,             
            N.x, N.y, N.z,  0,
            T->x, T->y, T->z,  0,
            P->x, P->y, P->z,  1);
}

void AATOMX(MX4 m, float ang, float x, float y, float z){
  P3F ax;
  SETV3(ax.p, x, y, z);    
  QUATER q;
  AATOQ( &ax, ang, &q);
  QTOMX4(&q, m);
}

void QTOE(const QUATER *q1, float* heading, float* attitude, float* bank)
{
  float sqw = q1->w*q1->w;
  float sqx = q1->x*q1->x;
  float sqy = q1->y*q1->y;
  float sqz = q1->z*q1->z;
  // if normalised is one, otherwise is correction factor
  float unit = sqx + sqy + sqz + sqw; 
  float test = q1->x*q1->y + q1->z*q1->w;
  // singularity at north pole
  if (test > 0.499*unit) { 
    *heading = 2 * (float)atan2(q1->x,q1->w);
    *attitude = float(M_PI)/2;
    *bank = 0;
    return;
  }
  // singularity at south pole
  if (test < -0.499*unit) { 
    *heading = -2 * (float)atan2(q1->x,q1->w);
    *attitude = -float(M_PI)/2;
    *bank = 0;
    return;
  }

  *heading = (float)atan2(2*q1->y*q1->w-2*q1->x*q1->z , sqx - sqy - sqz + sqw);
  *attitude = (float)asin(2*test/unit);
  *bank = (float)atan2(2*q1->x*q1->w-2*q1->y*q1->z , -sqx + sqy - sqz + sqw);
}

void ROT(P3F* p, P3F* ax, float ang, P3F* q)
{
  QUATER q0, q1;
  //q0
  CopyV3(q0.p3f.p, ax->p);
  MulV3( q0.p3f.p, (float)sin(ang/2) );
  q0.w = (float)cos(ang/2);
  //q1
  CopyV4(q1.v4f, q0.v4f);
  QINV(q1);
  //
  QUATER pp;
  CopyV3(pp.p3f.p, p->p);
  pp.w = 1;
  QUATER qq1,qq2;
  QMUL(&qq1, &q0, &pp);
  QMUL(&qq2, &qq1, &q1);
  //
  CopyV4toV3(q->p, qq2.v4f);
}

int QUADR(float a,float b, float c, float* k1, float* k2)
{

	float d = b*b - 4*a*c;

  if(d<0)return 0;
  d = (float)sqrt(d);

  *k1 = (-b + d) / (2*a);
  *k2 = (-b - d) / (2*a);

	if(d==0)return 1;
  return 2;
}

int Linear3x3Solve(float* a,float* b, float* c, float* d, float* x){
  // 4*6 nasobeni 4*6 souctu + 3 deleni.. ale aspon je to prehledne
  float dm =  a[0]*b[1]*c[2] + a[3]*b[0]*c[1] + a[1]*b[2]*c[0] 
              - a[2]*b[1]*c[0] - a[0]*b[2]*c[1] - a[1]*b[0]*c[2];

  if ( fabs(dm) < 0.0000001)return 0;

  float dx =  d[0]*b[1]*c[2] + d[2]*b[0]*c[1] + d[1]*b[2]*c[0] 
              - d[2]*b[1]*c[0] - d[0]*b[2]*c[1] - d[1]*b[0]*c[2];

  float dy =  a[0]*d[1]*c[2] + a[2]*d[0]*c[1] + a[1]*d[2]*c[0] 
              - a[2]*d[1]*c[0] - a[0]*d[2]*c[1] - a[1]*d[0]*c[2];

  float dz =  a[0]*b[1]*d[2] + a[2]*b[0]*d[1] + a[1]*b[2]*d[0] 
              - a[2]*b[1]*d[0] - a[0]*b[2]*d[1] - a[1]*b[0]*d[2];
  //
  x[0] = dx/dm;
  x[1] = dy/dm;
  x[2] = dz/dm;
  return 1;
}

float Angle2D(P2F* A, P2F* B)
{
	P2F N;
	N.x = -A->y;
	N.y = A->x;
  //
  float vA = (float)SizeV2(A->p);
  float vB = (float)SizeV2(B->p);
  //
  float cosc = ScalMulV2(A->p, B->p)/(vA*vB);
	float alfa = (float)acos(cosc);
  //
  float cosn = ScalMulV2(B->p, N.p)/(vA*vB);
  if (cosn > 0) alfa=2*float(M_PI)-alfa;
  return alfa;
}

float Angle2Db(P2F* A, P2F* B){
  float vA = (float)SizeV2(A->p);
  float vB = (float)SizeV2(B->p);
  //
  float cosc = ScalMulV2(A->p, B->p)/(vA*vB);
	return (float)acos(cosc);
}

// spocita uhel mezi dvema vektory, 3D verze, 0-360
float Angle3D(P3F* A, P3F* B){
  //
  float cosc = ScalMulV3(A->p, B->p)/(SizeV3(A->p)*SizeV3(B->p));
	float ang = (float)acos(cosc);
  //
  if( (M_PI - ang) < 0.0001 ) return ang;
  //
  P3F C,D;
  //
  VectMulV3(C.p, A->p, B->p); //kolmice na rovinu AB
  VectMulV3(D.p, C.p, A->p);  //kolmice na robinu CA
  //uhel mezi B a D, lezi B pod/nad CA?
  float cosc2 = ScalMulV3(D.p, B->p)/(SizeV3(D.p)*SizeV3(B->p));
	float ang2 = (float)acos(cosc);
  if( ang2 > (float)(M_PI/2) )
    ang = (float)(2*M_PI - ang);
  return ang;
}

float Angle3Db(P3F* A, P3F* B){
  //
  float cosc = ScalMulV3(A->p, B->p)/(SizeV3(A->p)*SizeV3(B->p));
	float ang = (float)acos(cosc);
  //
  return ang;
}


}//namespace
/*****************************************************************************/

