/**
*******************************************************************************

  @file /gui/common/mymath.h
  @brief Typy a funkce pro matematicke vypocty


  @author vajicek
  @version 0.1

******************************************************************************/
#ifndef MYMATH_H
#define MYMATH_H

#ifdef WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#include <float.h>

namespace gui
{

///bod ve 2D - double
union P2D{
	struct{
		double x,y;
	};
	double p[2];
};
///bod ve 2D - float
union P2F{
	struct{
		float x,y;
	};
	float p[2];
};
///nastavi bod
#define SETP2(A,X,Y) {A.x=X;A.y=Y;}


///bod ve 3D - double, nebo bod ve 2D jako homogeni souradnice
union P3D{
	struct{
		double x,y,z;
	};
	double p[3];
  union P2D p2d;
};
///bod ve 3D - float
union P3F{
	struct{
		float x,y,z;
	};
	float p[3];
  union P2F p2f;
};
///nastavi bod
#define SETP3(A,X,Y,Z) {A.x=X;A.y=Y;A.z=Z;}


///bod ve 3D - double, jako homogeni souradnice
union P4D{
	struct{
		double x,y,z,u;
	};
	double p[4];
};
///bod ve 3D - float
union P4F{
	struct{
		float x,y,z,u;
	};
  P3F p3f;
	float p[4];
};
///nastavi bod
#define SETP4(A,X,Y,Z,U) {A.x=X;A.y=Y;A.z=Z;A.u=U;}


//vektory - treba se budou hodit
///vekror 4xdouble
typedef double V4D[4];
///vekror 4xfloat
typedef float V4F[4];
///vekror 3xdouble
typedef double V3D[3]; 
///vekror 3xint
typedef int V3I[3];
///vekror 2xdouble
typedef double V2D[2];
///vekror 2xint
typedef int V2I[2];
//
#define VX(a) (a)[0]
#define VY(a) (a)[1]
#define VZ(a) (a)[2]


/**nastavi V4*/
#define SETV4(A,X,Y,Z,U) {(A)[0]=X;(A)[1]=Y;(A)[2]=Z;(A)[3]=U;}
/**nastavi V3*/
#define SETV3(A,X,Y,Z) {(A)[0]=X;(A)[1]=Y;(A)[2]=Z;}
/**nastavi V3*/
#define SETV2(A,X,Y) {(A)[0]=X;(A)[1]=Y;}
/**kopiruje V4/P4 do V4/P4*/
#define CopyV4(a,b) {a[0]=b[0];a[1]=b[1];a[2]=b[2];a[3]=b[3];}
/**kopiruje V3/P3 do V3/P3*/
#define CopyV3(a,b) {a[0]=b[0];a[1]=b[1];a[2]=b[2];}
/**konvertuje V4D/P4D do P3/V3*/
#define CopyV4toV3(a,b) {a[0]=b[0]/b[3];a[1]=b[1]/b[3];a[2]=b[2]/b[3];}
/**kopiruje V2/P2 do V2/P2*/
#define CopyV2(a,b) {a[0]=b[0];a[1]=b[1];}
/**konvertuje V3D/P3D do P2/V2*/
#define CopyV3toV2(a,b) {a[0]=b[0]/b[2];a[1]=b[1]/b[2];}

/**scitani V4*/
#define AddV4(a,b) {a[0]+=b[0];a[1]+=b[1];a[2]+=b[2];a[3]+=b[3];}
/**scitani V3*/
#define AddV3(a,b) {a[0]+=b[0];a[1]+=b[1];a[2]+=b[2];}
/**scitani V2*/
#define AddV2(a,b) {a[0]+=b[0];a[1]+=b[1];}

/**rozdil V4/P4*/
#define SubV4(a,b) {a[0]-=b[0];a[1]-=b[1];a[2]-=b[2];a[3]-=b[3];}
/**rozdil V3/P3*/
#define SubV3(a,b) {a[0]-=b[0];a[1]-=b[1];a[2]-=b[2];}
/**rozdil V2/P2*/
#define SubV2(a,b) {a[0]-=b[0];a[1]-=b[1];}

/**skalarni soucin V3*/
#define ScalMulV3(a,b) (a[0]*b[0]+a[1]*b[1]+a[2]*b[2])
/**skalarni soucin V2*/
#define ScalMulV2(a,b) (a[0]*b[0]+a[1]*b[1])
/**skalarni soucin V4*/
#define ScalMulV4(a,b) (a[0]*b[0]+a[1]*b[1]+a[2]*b[2]+a[3]*b[3])

/**nasobeni V2 skalarem*/
#define MulV2(v,s) {v[0] = v[0]*(s);v[1] = v[1]*(s);}
/**nasobeni V3 skalarem*/
#define MulV3(v,s) {v[0] = v[0]*(s);v[1] = v[1]*(s);v[2] = v[2]*(s);}
/**nasobeni V4 skalarem*/
#define MulV4(v,s) {v[0] = v[0]*(s);v[1] = v[1]*(s);v[2] = v[2]*(s);v[3] = v[3]*(s);}

/**vektorovy soucin*/
#define VectMulV3(a,b,c) {a[0]=b[1]*c[2]-b[2]*c[1];a[1]=b[2]*c[0]-b[0]*c[2];a[2]=b[0]*c[1]-b[1]*c[0];}

/**normalize V2*/
#define NormalizeV2(a) {float __s=(float)sqrt(a[0]*a[0]+a[1]*a[1]);a[0]/=__s;a[1]/=__s;}

/**normalize V3*/
#define NormalizeV3(a) {float __s=(float)sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);a[0]/=__s;a[1]/=__s;a[2]/=__s;}

/**normalize V4*/
#define NormalizeV4(a) {float __s=(float)sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]+a[3]*a[3]);a[0]/=__s;a[1]/=__s;a[2]/=__s;a[3]/=__s;}

/**levoruka normala V2  
^
| ->
*/
#define NormalV2(a, b) {b[0] = -a[1]; b[1] = a[0];}

/**size V4*/
#define SizeV4(v) ((float)sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]+v[3]*v[3]))

/**size V3*/
#define SizeV3(v) ((float)sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]))

/**size V2*/
#define SizeV2(v) ((float)sqrt(v[0]*v[0]+v[1]*v[1]))

///matice 4 x 4
typedef float MX4[16];
///matice 3 x 3
typedef float MX3[9];


/**
Matice jsou chapany
0 1 2 3
4 5 6 7
8 9 1011
12131415
*/

///nastavi matice 4x4 - jeste porad je to lepsi?
#define		SetMX4(M,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16)\
			{M[0]=a1;M[1]=a2;M[2]=a3;M[3]=a4;M[4]=a5;M[5]=a6;M[6]=a7;M[7]=a8;\
			M[8]=a9;M[9]=a10;M[10]=a11;M[11]=a12;M[12]=a13;M[13]=a14;M[14]=a15;M[15]=a16;}

///kopiruje matici 4x4
#define		CopyMX4(M,N){for(int __i=0;__i<16;__i++)M[__i]=N[__i];}

///nastavi matice 3x3 - jeste porad je to lepsi?
#define		SetMX3(M,a1,a2,a3,a4,a5,a6,a7,a8,a9)\
			{M[0]=a1;M[1]=a2;M[2]=a3;M[3]=a4;M[4]=a5;M[5]=a6;M[6]=a7;M[7]=a8;M[8]=a9;};

///kopiruje matici 3x3
#define		CopyMX3(M,N){for(int __i=0;__i<9;__i++)M[__i]=N[__i];}

///vytvor matici rotace kolem X
#define		SetRotXMX4(M,ang) {	float __sinx=(float)sin(ang);float __cosx=(float)cos(ang);\
								M[0]=1;M[1]=0;M[2]=0;M[3]=0;\
								M[4]=0;M[5]=__cosx;M[6]=__sinx;M[7]=0;\
								M[8]=0;M[9]=-__sinx;M[10]=__cosx;M[11]=0;\
								M[12]=0;M[13]=0;M[14]=0;M[15]=1;}

///vytvor matici rotace kolem Y
#define		SetRotYMX4(M,ang) {	float __sinx=(float)sin(ang);float __cosx=(float)cos(ang);\
								M[0]=__cosx;M[1]=0;M[2]=__sinx;M[3]=0;\
								M[4]=0;M[5]=1;M[6]=0;M[7]=0;\
								M[8]=-__sinx;M[9]=0;M[10]=__cosx;M[11]=0;\
								M[12]=0;M[13]=0;M[14]=0;M[15]=1;}

///vytvor matici rotace kolem Z
#define		SetRotZMX4(M,ang) {	float __sinx=(float)sin(ang);float __cosx=(float)cos(ang);\
								M[0]=__cosx;M[1]=__sinx;M[2]=0;M[3]=0;\
								M[4]=-__sinx;M[5]=__cosx;M[6]=0;M[7]=0;\
								M[8]=0;M[9]=0;M[10]=1;M[11]=0;\
								M[12]=0;M[13]=0;M[14]=0;M[15]=1;}

///vytvor matici translace
#define		SetTransMX4(M,x,y,z){	M[0]=1;M[1]=0;M[2]=0;M[3]=0;\
									M[4]=0;M[5]=1;M[6]=0;M[7]=0;\
									M[8]=0;M[9]=0;M[10]=1;M[11]=0;\
									M[12]=x;M[13]=y;M[14]=z;M[15]=1;}


///vytvor matici zvetseni
#define		SetScaleMX4(M,x,y,z){	M[0]=x;M[1]=0;M[2]=0;M[3]=0;\
									M[4]=0;M[5]=y;M[6]=0;M[7]=0;\
									M[8]=0;M[9]=0;M[10]=z;M[11]=0;\
									M[12]=0;M[13]=0;M[14]=0;M[15]=1;}

///vytvor matici identity
#define		SetIdentityMX4(M){	(M)[0]=1;(M)[1]=0;(M)[2]=0;(M)[3]=0;\
								(M)[4]=0;(M)[5]=1;(M)[6]=0;(M)[7]=0;\
								(M)[8]=0;(M)[9]=0;(M)[10]=1;(M)[11]=0;\
								(M)[12]=0;(M)[13]=0;(M)[14]=0;(M)[15]=1;}
///zmena reprezentace matice radky <-> sloupecky
#define		ConvMX(M1,M2)	{M1[0]=M2[0];M1[1]=M2[4];M1[2]=M2[8];M1[3]=M2[12];\
							 M1[4]=M2[1];M1[5]=M2[5];M1[6]=M2[9];M1[7]=M2[13];\
							 M1[8]=M2[2];M1[9]=M2[6];M1[10]=M2[10];M1[11]=M2[14];\
							 M1[12]=M2[3];M1[13]=M2[7];M1[14]=M2[11];M1[15]=M2[15];}

/** maska */
#define MaskROT(M1) { M1[3]=0;\
                      M1[7]=0;\
                      M1[11]=0;\
                      M1[12]=0;M1[13]=0;M1[14]=0;}

/** maska */
#define MaskTRA(M1) { M1[0]=0;M1[1]=0;M1[2]=0;M1[3]=0;\
                      M1[4]=0;M1[5]=0;M1[6]=0;M1[7]=0;\
                      M1[8]=0;M1[9]=0;M1[10]=0;M1[11]=0;}

/** maska */
#define MaskSCA(M1) { M1[1]=0;M1[2]=0;M1[3]=0;\
                      M1[4]=0;M1[6]=0;M1[7]=0;\
                      M1[8]=0;M1[9]=0;M1[11]=0;\
                      M1[12]=0;M1[13]=0;M1[14]=0;}

/**nasobeni matic 4 x 4

	@param c vysledek operace a*b
	@param a prvni matice 4x4
	@param b druha matice 4x4
*/
extern void MulMatrixMX4(float *c,float *a,float *b);

/** nasob pole matic 
	@param a vysledek operace va[0]*va[1]*..*va[vac-1]
	@param va pole matic 4x4
	@param vac pocet matic
*/
extern void MultipleMulMatrixMX4(float *a, float* va[], int vac);

/**nasobeni matic 3 x 3

	@param c vysledek a*b
	@param a matice 3x3
	@param b matice 3x3
*/
extern void MulMatrixMX3(float *c,float *a,float *b);

/**nasobi vektor matici v*M

	@param v vektor delky 3
	@param m matice 3x3
	@param w vystupni vektor delky 3
*/
extern void ApplyVectorMX3(float* v,float* m,float* w);

/**nasobi vektor matici v*M

	@param v vektor delky 4
	@param m matice 4x4
	@param w vystupni vektor delky 4
*/
extern void ApplyVectorMX4(float* v,float* m,float* w);

/**makro nasobi vektor matici.*/
#define APPLYVECTORMX4(v, m, w) {\
	w[0]=v[0]*m[0]+v[1]*m[4]+v[2]*m[8]+v[3]*m[12];\
	w[1]=v[0]*m[1]+v[1]*m[5]+v[2]*m[9]+v[3]*m[13]; \
	w[2]=v[0]*m[2]+v[1]*m[6]+v[2]*m[10]+v[3]*m[14]; \
  w[3]=v[0]*m[3]+v[1]*m[7]+v[2]*m[11]+v[3]*m[15]; }

/**transformuje prostorove souradnice
	@param m transformacni matice
  @param[in,out] x x souradnice
  @param[in,out] y y souradnice
  @param[in,out] z z souradnice
*/
extern void Transform(float* m,float* x,float* y,float* z);

/** transformuje prostorove souradnice
	@param m transformacni matice
  @param[in,out] v vektor souradnic
*/
extern void Transform(float* m,float* v);

/** Prehod. */
#define SWAP(a,b,c) {c=a;a=b;b=c;}

/** Transponuje matici MX4 insitu.
*/
#define TransposeMX4(m,c) {	SWAP(m[1],m[4],c);SWAP(m[2],m[8],c);SWAP(m[3],m[12],c);\
							SWAP(m[6],m[9],c);SWAP(m[7],m[13],c);\
							SWAP(m[11],m[14],c);}
/** Transformuj submatici.
*/
#define TransposeSubmatrixMX4(m,c) {	SWAP(m[1],m[4],c);SWAP(m[2],m[8],c);\
							                        SWAP(m[6],m[9],c);}


/** Vybere podmatici 3x3 z MX4.
  @param[in] mr 
  @param[out] mb
  @param i Souradnice vynechaneho radku.
  @param j Souradnice vynechaneho sloupce.
*/
void SubmatrixMX4( MX4 mr, MX3 mb, int i, int j );

/** Determinant MX4.
  @param mr Matice.
  @return determinant.
*/
float DeterminantMX4( MX4 mr );

/** Inverze MX4.
  @param[out] invers Zinvertovana matice.
  @param[in] orig Matice.
*/
int InverseMX4( MX4 invers, MX4 orig );

/** Determinant MX3.
  @param m Matice.
  @return determinant.
*/
float	DeterminantMX3( MX3 m );


/** Quaterniony. */
union QUATER{
	struct{
		float x,y,z,w;
	};
	struct{
		P3F p3f;
   // float w;
	};
	P4F p4f;
  float v4f[4];
};

/**Osa+Uhel*/
struct AXISANG{
  P3F ax;
  float ang;
};
#define SETAA(AA,A,X,Y,Z) {(AA).ang=A;(AA).ax.x=X;(AA).ax.y=Y;(AA).ax.z=Z;}


/**quaternion magnitude*/
#define QMAG(Q) ((float)sqrt((Q).x*(Q).x+(Q).y*(Q).y+(Q).z*(Q).z+(Q).w*(Q).w))

/**inverse quaternion*/
#define QINV(Q) {float d=QMAG(Q);d=d*d;(Q).x=-(Q).x/d;(Q).y=-(Q).y/d;(Q).z=-(Q).z/d;(Q).w=(Q).w/d;}

/** Q1=Q2.Q3 
  @param[out] Q1 Soucin.
  @param Q2 Cininitel.
  @param Q3 Cininitel.
*/
extern void QMUL(QUATER* Q1,QUATER* Q2,QUATER* Q3);

/** Quaternion to MX4
  @param[in] q Quaternion.
  @param[out] mx4 Matice.
*/
extern void QTOMX4(QUATER* q,MX4 mx4);

/** MX4 to quaternion.
  @param[out] q Quaternion.
  @param[in] m Matice.
*/
extern void MXTOQ(MX4 m, QUATER *q);

/** MX4 to angle & axis.
  @param[in] m Matice.
  @param[out] ax Osa.
  @param[out] ang Uhel.
*/
extern void MXTOAA(MX4 m, P3F* ax, float* ang);

/** Quaternion to angle+axis.
*/
extern void QTOAA(QUATER* q,P3F* ax,float *ang);

/** Angle+axis to quaternion
  @param[in] ax Osa.
  @param[in] ang Uhel.
  @param[out] q Quaternion.
*/
extern void AATOQ(P3F* ax, float ang, QUATER* q);

/** Slerp.
  @param q1 Prvni kvaternion.
  @param q2 Druhy kvaternion.
  @param t  Interpolacni koeficient.
  @param[out] qt Vysledni kvaternion.
*/
extern void SLERP(const QUATER *q1, const QUATER *q2, float t, QUATER *qt);

/** Rozlozit matici na rotaci a posunuti.
  @param[out] m Matice.
  @param tr Posun.
  @param rot Rotace.
*/
extern void MXTOTR_ROT(MX4 m,P3F* tr, QUATER *rot);

/** Vyrobi orientacni matici z polohy, smeru a upvektoru.
  @param[out] m Matice. 
  @param P Poloha.
  @param T Tecna.
  @param N Normala.
*/
extern void ORTMX(MX4 m, P3F* P, P3F* T, P3F* N);

/** Vyrobi orientacni matici z polohy, smeru a praveho vektoru.
  @param[out] m Matice. 
  @param P Poloha.
  @param T Tecna.
  @param UP UpVektor.
*/
extern void ORTMX2(MX4 m, P3F* P, P3F* T, P3F* UP);

/** Vyrobi rotacni matici z angle+axis. 
  @param[out] m Matice.
  @param ang Uhel.
  @param x X souradnice osy.
  @param y Y souradnice osy.
  @param z Z souradnice osy.
*/
extern void AATOMX(MX4 m, float ang, float x, float y, float z);

/** Vyrobi eulerovi uhly z quaternionu
  @param[in] q1 Quaternion.
  @param[out] heading Heading..
  @param[out] attitude Attitude.
  @param[out] bank Bank.
*/
extern void QTOE(const QUATER *q1, float* heading, float* attitude, float* bank);

/** Rotace bodu kolem osy. 
  @param[in] p Vstupni bod.
  @param ax Osa.
  @param ang Uhel.
  @param[out] q Vystupni bod.
*/
extern void ROT(P3F* p, P3F* ax, float ang, P3F* q);

/** Zaokrouhlovani.

  Zaokrouhlovani je velkej problem a nejak na nej chybi funkce v CRT
  hezky <a href="http:\\ldesoras.free.fr/doc/articles/rounding_en.pdf">clanek</a>

  (int)x provadi truncate, -4.9 => -4 , 4.9 => 4

  tohle reseni je hloupy, casem snad na neco prijdem;
*/
#define ROUND(x) ((x)<0)?((int)((x)-0.5)):((int)((x)+0.5))

/** Zlomek. */
#define FRAC(x,y) (((float)(x))/((float)(y)));

/** Test na interval. */
#define INRANGE(x,a,b) (((x)>=(a))&&((x)<=(b)))

/** Vzdalenost. */
#define DIST(x1,y1,z1,x2,y2,z2) ( sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2) ))

/** Vzdalenost V3. */
#define DISTV3(v1,v2) ( sqrt( (v1[0]-v2[0])*(v1[0]-v2[0]) + (v1[1]-v2[1])*(v1[1]-v2[1]) + (v1[2]-v2[2])*(v1[2]-v2[2]) ))

/** Vzdalenost V2. */
#define DISTV2(v1,v2) ( sqrt( (v1[0]-v2[0])*(v1[0]-v2[0]) + (v1[1]-v2[1])*(v1[1]-v2[1]) ))

/** druha mocnina */
#define SQR(a) ((a)*(a))

/** minimum */
#ifndef MIN
	#define MIN(x,y) ((x)<(y)?(x):(y))
#endif
/** maximum */
#ifndef MAX
	#define MAX(x,y) ((x)>(y)?(x):(y))
#endif

/** Quadraticka rovnice.
  @param a Koeficient.
  @param b Koeficient.
  @param c Koeficient.
  @param k1 Koren 1.
  @param k2 Koren 2.
*/
int QUADR(float a,float b, float c, float* k1, float* k2);

/** Linearni soustava 3x3.
  @param a Prni radek matice
  @param b Druhy radek 
  @param c Treti
  @param d Prava strana.
  @param[out] x Reseni.
  @return 1
*/
int Linear3x3Solve(float* a,float* b, float* c, float* d, float* x);

/** Spocita uhel mezi dvema vektory, 2D verze, 0-360, zavisi na poradi.
  @param A Prvni vektor.
  @param B Druhy vektor.
  @return Uhel.
*/
float Angle2D(P2F* A, P2F* B);  

/** Spocita uhel mezi dvema vektory, 2D verze, 0-180.
  @param A Prvni vektor.
  @param B Druhy vektor.
  @return Uhel.
*/
float Angle2Db(P2F* A, P2F* B);  

/** Spocita uhel mezi dvema vektory, 3D verze, 0-360.
  @param A Prvni vektor.
  @param B Druhy vektor.
  @return Uhel.
*/
float Angle3D(P3F* A, P3F* B);  

/** Spocita uhel mezi dvema vektory, 3D verze, 0-180.
  @param A Prvni vektor.
  @param B Druhy vektor.
  @return Uhel.
*/
float Angle3Db(P3F* A, P3F* B);  

/** generuje nahodne cislo v intervalu <a,b>*/
#define RAND(a,b) ((a)+(((float)(b)-(a))*rand())/RAND_MAX)

/**negace V4*/
#define NegV4(a) {a[0]=-a[0];a[1]=-a[1];a[2]=-a[2];a[3]=-a[3];}
/**negace V3*/
#define NegV3(a) {a[0]=-a[0];a[1]=-a[1];a[2]=-a[2];}
/**negace V2*/
#define NegV2(a) {a[0]=-a[0];a[1]=-a[1];}

/** nejvetsi hodnota 64-bitoveho intu */
#define MAXINT	2147483647
/** odmocnina ze tri */
#define sqrt3	1.7320508076
/** proste M_PI*/
#ifndef M_PI
#define M_PI		3.1415926536
#endif
/** konstanta pro test na nulovost*/
#define ZERO_LIMIT 0.0001
/**test na nulovost vektoru*/
#define isZeroV3(a) ( (-ZERO_LIMIT<a[0])&&(a[0]<ZERO_LIMIT)&&(-ZERO_LIMIT<a[1])&&\
	(a[1]<ZERO_LIMIT)&&(-ZERO_LIMIT<a[2])&&(a[2]<ZERO_LIMIT) )
/**test na nulovost skalaru*/
#define isZero(a) ( (-ZERO_LIMIT < a) && (a < ZERO_LIMIT) )


/**prevod stupne -> radiany */
#define Deg2Rad(x) ((M_PI*(float)(x))/180)
/**prevod radiany -> stupne */
#define Rad2Deg(x) (((x)*180)/M_PI)

/** absolutni hodnota */
#define myabs(x) ( ((x) < 0) ? (-(x)) : (x) )
/** druha mocnina */
#define sqr(x)	((x)*(x))
/** test na lichost */
#define odd(x)	(myabs(x)%2)

}//namespace

#endif
/*****************************************************************************/
