/**
*******************************************************************************

  @file /gui/common/mymath_ext.h
  @brief Rozsireni mymath, hlavne objektove typy pro snadnejsi praci


  @author vajicek
  @version 0.1

******************************************************************************/
#ifndef MYMATH_EXT_H
#define MYMATH_EXT_H

#include "gui/common/mymath.h"

namespace gui
{

/** Objektova implementace vektoru3 */
class TVector{
public:
  /// x, y, z slozky
  float x,y,z;
  /** Konstruktor.
    Nastavi vse na 0.
  */
  TVector(){
    x=0;y=0;z=0;
  }
  /** Konstruktor.
  */
  TVector(float _x, float _y, float _z){
    x=_x;y=_y;z=_z;
  }
  /** Konstruktor.
  */
  TVector(P3F& p3f){
    x = p3f.x;
    y = p3f.y;
    z = p3f.z;
  }
  /** Konstruktor.
  */
  TVector(P2F& p2f){
    x = p2f.x;
    y = p2f.y;
    z = 0;
  }
  inline TVector& operator=(const P3F& a){
    x = a.x;
    y = a.y;
    z = a.z;
    return *this;
  }
  inline TVector& operator=(const P2F& a){
    x = a.x;
    y = a.y;
    return *this;
  }
  inline P2F toP2F(){
    P2F p2f;
    p2f.x = x;
    p2f.y = y;
    return p2f;
  }
  inline P3F toP3F(){
    P3F p3f;
    p3f.x = x;
    p3f.y = y;
    p3f.z = z;
    return p3f;
  }
  TVector& transform(MX4 m){
    Transform(m, &x, &y, &z);
    return *this;
  }
  TVector& rotate(float x, float y, float z){
    MX4 m;
    if(x){
      SetRotXMX4(m, x);
      transform(m);
    }
    if(y){
      SetRotXMX4(m, y);
      transform(m);
    }
    if(z){
      SetRotXMX4(m, z);
      transform(m);
    }
    return *this;
  }
  
  inline TVector& operator+=(const TVector &a){
    x+=a.x;
    y+=a.y;
    z+=a.z;
    return *this;
  }
  inline TVector& operator-=(const TVector &a){
    x-=a.x;
    y-=a.y;
    z-=a.z;
    return *this;
  }
  inline TVector& operator*=(const float alfa){
    x*=alfa;
    y*=alfa;
    z*=alfa;
    return *this;
  }
  inline TVector& operator/=(const float alfa){
    x/=alfa;
    y/=alfa;
    z/=alfa;
    return *this;
  }
  inline float len(){
    return sqrt(x*x + y*y + z*z);
  }
  inline TVector& norm(){
    float s = sqrt(x*x + y*y + z*z);
    x /= s;
    y /= s;
    z /= s;
    return *this;
  }

  friend TVector operator-(const TVector &a, const TVector &b);
  friend TVector operator+(const TVector &a, const TVector &b);
  friend TVector operator*(const TVector &a, const float alfa);
  friend TVector operator/(const TVector &a, const float alfa);
  friend TVector operator-(const TVector &a);
};

inline TVector operator-(const TVector &a){
  return a*(-1);
}
inline TVector operator/(const TVector &a, const float alfa){
  return TVector(a.x/alfa, a.y/alfa, a.z/alfa);
}
inline TVector operator*(const TVector &a, const float alfa){
  return TVector(a.x*alfa, a.y*alfa, a.z*alfa);
}
inline TVector operator-(const TVector &a, const TVector &b){
  return TVector(a.x - b.x, a.y - b.y, a.z - b.z);
}
inline TVector operator+(const TVector &a, const TVector &b){
  return TVector(a.x + b.x, a.y + b.y, a.z + b.z);
}


}//namespace

#endif
/*****************************************************************************/
