/**
******************************************************************************

  @file /net/locator.h
  @brief Trida hledajici okolni hry
  
  @author Vta
  @version 1.0

******************************************************************************/
#ifndef _LOCATOR_
#define _LOCATOR_

#include "net/comm.h"

namespace net
{

/** Funkce na hledani okolnich her */
int searchGame_thread(void * data);

/** Funkce na hledani okolnich LAN her */
int searchGameLAN_thread(void * data);

/// Informace o jedne okolni sitove hre
struct TGames
{
  char ip[16];        ///< IP adresa serveru
  char mapname[36];   ///< jmeno mapy
  char description[50];  ///< user defined description
  int players;        ///< pocet roli ve hre
};

/// Trida sitoveho klienta
class TLocator
{
  public:  
    /** Konstruktor s informaci, zda je klient lokalni nebo sitovy */
    TLocator();

    /** Destruktor. Ceka az skonci vsechna vlakna, ktere klient pustil */
    ~TLocator(); 

    /** Pozadavek na vyhledani okolnich her */
    void seachGames();

    /** Pozadavek na ukonceni vyhledavani okolnich her */
    void stopSearching();

    /** Pripravi sitove pripojeni */
    int start(); 

    /** Ukonci sitove pripojeni */
    int clean(); 

  private:
    /** Informace o okolnich internetovych hrach */
    DA<TGames*> * games;

    /** Informace o okolnich LAN hrach */
    DA<TGames*> * LANgames;

    /** Pomocny semafor pri cekani na vytvoreni vlakna */
    SDL_sem *sem;

    /** Zamkne mutex */
    void lock(void); 

    /** Odemke mutex */
    void unlock(void);

    /** Zamek pri pristupu k datum */
    SDL_mutex *mutex; 

    /** Id vlakna, ktere odesila zpravy */
    SDL_Thread * searchThread; 

    /** Id vlakna, ktere odesila zpravy */
    SDL_Thread * searchLANThread; 

    /** Informace, jestli se poustelo vlakno */
    int was_searched;

    /** Funkce na hledani okolnich her */
    friend int searchGame_thread(void * data);

    /** Funkce na hledani okolnich her */
    friend int searchGameLAN_thread(void * data);

    /** Informace, jestli byla jiz sit spustena */
    int net_running;

    /** Stav objektu */
    int status;

    /** Socket pouzivany pro pripojeni k webovemu serveru */
    mysocket fdweb;
};

}

#endif
