/**
******************************************************************************

  @file /net/mysocket.h
  @brief Definice vlastnich funkci pro komunikaci pres sit
 
  @author Vta
  @version 1.0

******************************************************************************/
#ifndef _MYSOCKET_
#define _MYSOCKET_
#ifdef WIN32
  #include <winsock.h>
#else
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <unistd.h>
  #include <netdb.h>
#endif

#ifdef WIN32
  typedef SOCKET mysocket;
  typedef int mysocklen_t;
#else
  typedef int mysocket;
  typedef socklen_t mysocklen_t;
#endif

namespace net
{

#define MY_INVALID_SOCKET -1 ///< chybovy kod pri vytvareni socketu

/// Vlastni funkce na odesilani dat pres sit
int MySend(mysocket sock,const char * msg,size_t len);

/// Vlastni funkce na odesilani diagramu pres sit
int MySendto (mysocket s, const char *buf, int len, int flags, const struct sockaddr *to, mysocklen_t tolen );

/// Vlastni funkce na prijimani dat pres sit
int MyRecv(mysocket sock,char * msg,size_t len);

/// Vlastni funkce na prijimani diagramu pres sit
int MyRecvfrom(mysocket s, char *buf, int len, int flags, struct sockaddr *from, mysocklen_t *fromlen ); 

/// Zahajeni sitoveho provozu
int MyStartNetwork();

/// Ukonceni sitoveho provozu
int MyStopNetwork();

/// Uzavreni MySocketu
int MyCloseSocket(mysocket sock);

/// Pripojeni k serveru
int MyConnect(mysocket s, const struct sockaddr *name, int namelen);

/// Naslouchani u serveru
int MyListen(mysocket sock,int backlog);

/// Alokovani MySocketu u serveru
int MyBind(mysocket sock, const struct sockaddr *addr, int namelen);

/// Vlastni funkce na select
int MySelect (int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);

/// Vytvoreni socketu
mysocket MySocket();

/// Vytovreni UDP socketu
mysocket MyUDPSocket();

}

#endif // define _MYSOCKET_
