/** @file /world/buildings/building.cpp

    Implementace hlavickoveho souboru /world/buildings/building.h
    
    @author Petr Wolf
*/

#include "world/typedefs.h"
#include "world/buildings/building.h"
#include "world/world.h"

using namespace World;
using namespace std;

World::TBuilding::TBuilding()
{

}

World::TBuilding::TBuilding(BUILDING * data, int id) : TStructHolder<BUILDING>(data, id)
{

}

void World::TBuilding::readFromTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char _path[MAX_STRLEN];

    interpreter.getVar(TCL_path(_path, varName, "hit_probability"), TVT_FLOAT, &_data->hit_probability);
    interpreter.getVar(TCL_path(_path, varName, "lives"), TVT_INT, &_data->lives);
    interpreter.getVar(TCL_path(_path, varName, "base_cost"), TVT_INT, &_data->base_cost);
    interpreter.getVar(TCL_path(_path, varName, "repair_cost"), TVT_INT, &_data->repair_cost);
    interpreter.getVar(TCL_path(_path, varName, "construction_duration"), TVT_INT, &_data->construction_duration);
    interpreter.getVar(TCL_path(_path, varName, "construction_position"), TVT_INT, &_data->construction_position);
    interpreter.getVar(TCL_path(_path, varName, "structure_type"), TVT_INT, &_data->structure_type);
    interpreter.getVar(TCL_path(_path, varName, "defense_bonus"), TVT_INT, &_data->defense_bonus);
    interpreter.getVar(TCL_path(_path, varName, "counterattack"), TVT_INT, &_data->counterattack);
    interpreter.getVar(TCL_path(_path, varName, "eyesight_distance"), TVT_INT, &_data->eyesight_distance);    
    interpreter.getArray(TCL_path(_path, varName, "terrain"), TVT_INT, TERRAIN_TYPES_COUNT + 1, &_data->terrain);
}

void World::TBuilding::writeToTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char _path[MAX_STRLEN];

    interpreter.setVar(TCL_path(_path, varName, "hit_probability"), TVT_FLOAT, &_data->hit_probability);
    interpreter.setVar(TCL_path(_path, varName, "lives"), TVT_INT, &_data->lives);
    interpreter.setVar(TCL_path(_path, varName, "base_cost"), TVT_INT, &_data->base_cost);
    interpreter.setVar(TCL_path(_path, varName, "repair_cost"), TVT_INT, &_data->repair_cost);
    interpreter.setVar(TCL_path(_path, varName, "construction_duration"), TVT_INT, &_data->construction_duration);
    interpreter.setVar(TCL_path(_path, varName, "construction_position"), TVT_INT, &_data->construction_position);
    interpreter.setVar(TCL_path(_path, varName, "structure_type"), TVT_INT, &_data->structure_type);
    interpreter.setVar(TCL_path(_path, varName, "defense_bonus"), TVT_INT, &_data->defense_bonus);
    interpreter.setVar(TCL_path(_path, varName, "counterattack"), TVT_INT, &_data->counterattack);
    interpreter.setVar(TCL_path(_path, varName, "eyesight_distance"), TVT_INT, &_data->eyesight_distance);
    interpreter.setArray(TCL_path(_path, varName, "terrain"), TVT_INT, TERRAIN_TYPES_COUNT + 1, &_data->terrain);
    interpreter.setArray(TCL_path(_path, varName, "unit_recruitment"), TVT_INT, _data->unit_recruitment.length, _data->unit_recruitment.data);
    interpreter.setArray(TCL_path(_path, varName, "bonuses"), TVT_INT, _data->bonuses.length, _data->bonuses.data);   
}

TLivingBuilding::TLivingBuilding()
{
    _hex = 0;
    _unit = 0;
}

TLivingBuilding::TLivingBuilding(LIVING_BUILDING * data, int id) : TStructHolder<LIVING_BUILDING>(data, id)
{
    _hex = 0;
    _unit = 0;
}

TLivingBuilding::TLivingBuilding(int id) : TStructHolder<LIVING_BUILDING>(id)
{
    _hex = 0;
    _unit = 0;
}

void TLivingBuilding::readFromXML(TPackage * package)
{
    if (_data == NULL) {
        alloc();
        LIVING_BUILDING_Init(_data);
    }

    _data->type                  = package->getInt("type",                  0, NULL);
    _data->player                = package->getInt("player",                0, NULL);
    package->getChar(_data->name , MAX_STRLEN, "name", 0, NULL);
    _data->lives                 = package->getInt("lives",                 0, NULL);
    _data->construction_duration = package->getInt("construction_duration", 0, NULL);
    _data->orientation           = package->getInt("orientation",           0, NULL);
}

void TLivingBuilding::writeToXML(TPackage * package)
{
    package->setInt(_data->type,                  "type",                  0, NULL);
    package->setInt(_data->player,                "player",                0, NULL);
    package->setChar(_data->name,                 "name",                  0, NULL);
    package->setInt(_data->lives,                 "lives",                 0, NULL);
    package->setInt(_data->construction_duration, "construction_duration", 0, NULL);
    package->setInt(_data->orientation,           "orientation",           0, NULL);
}

void TLivingBuilding::readFromTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char _path[MAX_STRLEN];

    interpreter.getVar(TCL_path(_path, varName, "type"), TVT_INT, &_data->type);
    interpreter.getVar(TCL_path(_path, varName, "player"), TVT_INT, &_data->player);
    interpreter.getVar(TCL_path(_path, varName, "lives"), TVT_INT, &_data->lives);
    interpreter.getVar(TCL_path(_path, varName, "construction_duration"), TVT_INT, &_data->construction_duration);
    interpreter.getVar(TCL_path(_path, varName, "orientation"), TVT_INT, &_data->orientation); 
}

void TLivingBuilding::writeToTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char _path[MAX_STRLEN];

    interpreter.setVar(TCL_path(_path, varName, "type"), TVT_INT, &_data->type);
    interpreter.setVar(TCL_path(_path, varName, "player"), TVT_INT, &_data->player);
    interpreter.setVar(TCL_path(_path, varName, "lives"), TVT_INT, &_data->lives);
    interpreter.setVar(TCL_path(_path, varName, "construction_duration"), TVT_INT, &_data->construction_duration);
    interpreter.setVar(TCL_path(_path, varName, "orientation"), TVT_INT, &_data->orientation);  

    interpreter.setVar(TCL_path(_path, varName, "hex"), TVT_INT, &_hex);
    interpreter.setVar(TCL_path(_path, varName, "unit"), TVT_INT, &_unit);
}

HEX_ID TLivingBuilding::hex()
{
    return _hex;
}

UNIT_ID TLivingBuilding::unit()
{
    return _unit;
}

