/** @file /world/plan/plan.cpp 
    
    Implementace hlavickoveho souboru plan.h
    
    @author PW
*/

#include <math.h>
#include "common/xml/package.h"
#include "common/xml/packsmall.h"
#include "common/mm.h"
#include "common/exc.h"
#include "world/plan/plan.h"
#include "world/world.h"

using namespace World;

bool World::HEX_Copy(HEX * hex, const HEX * src)
{
	hex->terrain        = src->terrain;
	hex->elevation      = src->elevation;
	hex->unit           = src->unit;
	hex->building       = src->building;
    
    if (hex->unit_occurrence.data)
        KMemFree(hex->unit_occurrence.data);
    if (hex->unit_occurrence.data = (UNIT_TYPE_ID *)KMemAlloc(src->unit_occurrence.length * sizeof(UNIT_TYPE_ID)))
        hex->unit_occurrence.length = src->unit_occurrence.length;

    for (int i=0; i < src->unit_occurrence.length; i++)
		hex->unit_occurrence.data[i] = src->unit_occurrence.data[i];

	return true;
}

TTerrain::TTerrain(TERRAIN * data, int id) : TStructHolder<TERRAIN>(data, id)
{

}

void TTerrain::readFromTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char path[MAX_STRLEN];
    
    interpreter.getVar(TCL_path(path, varName, "hit_probability_reduction"), TVT_FLOAT, &_data->hit_probability_reduction);
    interpreter.getVar(TCL_path(path, varName, "visibility_reduction"), TVT_INT, &_data->visibility_reduction);
    interpreter.getVar(TCL_path(path, varName, "healing_reduction"), TVT_FLOAT, &_data->healing_reduction);
}

void TTerrain::writeToTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char path[MAX_STRLEN];
    
    interpreter.setVar(TCL_path(path, varName, "hit_probability_reduction"), TVT_FLOAT, &_data->hit_probability_reduction);
    interpreter.setVar(TCL_path(path, varName, "visibility_reduction"), TVT_INT, &_data->visibility_reduction);
    interpreter.setVar(TCL_path(path, varName, "healing_reduction"), TVT_FLOAT, &_data->healing_reduction);
}

TKingdom::TKingdom(int id) : TStructHolder<KINGDOM>(id)
{

}

TKingdom::TKingdom(KINGDOM * data, int id) : TStructHolder<KINGDOM>(data, id)
{

}

void TKingdom::readFromXML(TPackage * package)
{
    if (_data == NULL) {
        alloc();
        KINGDOM_Init(_data);
    }

    _data->id      = package->getInt("id",      0, NULL);
    _data->roleid  = package->getInt("roleid",  0, NULL);
    _data->centerx = package->getInt("centerx", 0, NULL);
    _data->centery = package->getInt("centery", 0, NULL);
}

void TKingdom::writeToXML(TPackage * package)
{
    package->setInt(_data->id,      "id",      0, NULL);
    package->setInt(_data->roleid,  "roleid",  0, NULL);
    package->setInt(_data->centerx, "centerx", 0, NULL);
    package->setInt(_data->centery, "centery", 0, NULL);
}

void TKingdom::readFromTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char path[MAX_STRLEN];

    interpreter.getVar(TCL_path(path, varName, "id"),      TVT_INT, &_data->id);
    interpreter.getVar(TCL_path(path, varName, "roleid"),  TVT_INT, &_data->roleid);
    interpreter.getVar(TCL_path(path, varName, "centerx"), TVT_INT, &_data->centerx);
    interpreter.getVar(TCL_path(path, varName, "centery"), TVT_INT, &_data->centery);
}

void TKingdom::writeToTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char path[MAX_STRLEN];

    interpreter.setVar(TCL_path(path, varName, "id"),      TVT_INT, &_data->id);
    interpreter.setVar(TCL_path(path, varName, "roleid"),  TVT_INT, &_data->roleid);
    interpreter.setVar(TCL_path(path, varName, "centerx"), TVT_INT, &_data->centerx);
    interpreter.setVar(TCL_path(path, varName, "centery"), TVT_INT, &_data->centery);
}

TTown::TTown(int id) : TStructHolder<TOWN>(id)
{

}

TTown::TTown(TOWN * data, int id) : TStructHolder<TOWN>(data, id)
{

}

void TTown::readFromXML(TPackage * package)
{
    if (_data == NULL) {
        alloc();
        TOWN_Init(_data);
    }

//    package->getChar(_data->name, K8_MAX_NAME_LEN, "name", 0, NULL); 
    _data->owner    = package->getInt("owner",    0, NULL);
    _data->citysize = package->getInt("citysize", 0, NULL);
//    _data->payment  = package->getInt("payment",  0, NULL);
    INT_ARRAY_FROM_XML(package, _data->position, _data->citysize, "position");
}

void TTown::writeToXML(TPackage * package)
{
//    package->setChar(_data->name,    "name",     0, NULL);
    package->setInt(_data->owner,    "owner",    0, NULL);
    package->setInt(_data->citysize, "citysize", 0, NULL);
//    package->setInt(_data->payment,  "payment",  0, NULL);
    INT_ARRAY_TO_XML(package, _data->position, _data->citysize, "position");
}

void TTown::readFromTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char path[MAX_STRLEN];

    interpreter.getVar(TCL_path(path, varName, "owner"),    TVT_INT, &_data->owner);
    interpreter.getVar(TCL_path(path, varName, "citysize"), TVT_INT, &_data->citysize);
//    interpreter.getVar(TCL_path(path, varName, "payment"),  TVT_INT, &_data->payment);
    interpreter.getArray(TCL_path(path, varName, "position"), TVT_INT, _data->citysize, _data->position);
}

void TTown::writeToTCL(TTCL_Interpreter & interpreter, const char * varName)
{    
    char path[MAX_STRLEN];

    interpreter.setVar(TCL_path(path, varName, "owner"),    TVT_INT, &_data->owner);
    interpreter.setVar(TCL_path(path, varName, "citysize"), TVT_INT, &_data->citysize);
//    interpreter.setVar(TCL_path(path, varName, "payment"),  TVT_INT, &_data->payment);
    interpreter.setArray(TCL_path(path, varName, "position"), TVT_INT, _data->citysize, _data->position);
}

World::THex::THex(HEX * data, int id) : TStructHolder<HEX>(data, id)
{
    _town = 0;
}

World::THex::~THex()
{
    if (_data && _self_allocated) {
        KMemFree(_data->unit_occurrence.data);
    }
}

void World::THex::readFromXML(TPackage * package)
{
    _data->terrain   = (TERRAIN_TYPE)package->getInt("terrain", 0, NULL);
    _data->elevation = (ELEVATION_TYPE)package->getInt("elevation", 0, NULL);
    _data->kingdom   = (PLAYER_ID)package->getInt("kingdom", 0, NULL);
    _data->unit      = (UNIT_ID)package->getInt("unit", 0, NULL);
    _data->building  = (BUILDING_ID)package->getInt("building", 0, NULL);
    INT_ARRAY_FROM_XML(package, &_data->unit_occurrence.data, &_data->unit_occurrence.length, "unit_occurrence");
}

void World::THex::writeToXML(TPackage * package)
{
    package->setInt(_data->terrain,   "terrain",   0, NULL);
    package->setInt(_data->elevation, "elevation", 0, NULL);
    package->setInt(_data->unit, "unit", 0, NULL);
    package->setInt(_data->building, "building", 0, NULL);
    package->setInt(_data->kingdom, "kingdom", 0, NULL);
    INT_ARRAY_TO_XML(package, _data->unit_occurrence.data, _data->unit_occurrence.length, "unit_occurrence");
}

void World::THex::readFromTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char path[MAX_STRLEN];
    
    interpreter.getVar(TCL_path(path, varName, "terrain"),        TVT_INT, &_data->terrain);
    interpreter.getVar(TCL_path(path, varName, "elevation"),      TVT_INT, &_data->elevation);
    interpreter.getVar(TCL_path(path, varName, "unit"),           TVT_INT, &_data->unit);
    interpreter.getVar(TCL_path(path, varName, "building"),       TVT_INT, &_data->building);
    interpreter.getVar(TCL_path(path, varName, "kingdom"),        TVT_INT, &_data->kingdom);
    interpreter.getVar(TCL_path(path, varName, "town"),           TVT_INT, &_town);
}

void World::THex::writeToTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char path[MAX_STRLEN];
    
    interpreter.setVar(TCL_path(path, varName, "terrain"),        TVT_INT, &_data->terrain);
    interpreter.setVar(TCL_path(path, varName, "elevation"),      TVT_INT, &_data->elevation);
    interpreter.setVar(TCL_path(path, varName, "unit"),           TVT_INT, &_data->unit);
    interpreter.setVar(TCL_path(path, varName, "building"),       TVT_INT, &_data->building);
    interpreter.setVar(TCL_path(path, varName, "kingdom"),        TVT_INT, &_data->kingdom);
    interpreter.setVar(TCL_path(path, varName, "town"),           TVT_INT, &_town);

    // vyskyt jednotek
    interpreter.setVar(TCL_path(path, varName, "unit_occurrence"), TVT_INT_LIST, (std::vector<int> *)&unit_occurrence);
}

UNIT_ID World::THex::unit_id()
{
    return _data->unit;
}

BUILDING_ID World::THex::building_id()
{
    return _data->building;
}

int World::THex::town()
{
    return _town;
}

TMap::TMap()
{
    _hexes_self_allocated = false;
    _data = (MAP *)KMemAlloc(sizeof(MAP));
    _data->width  = 0;
    _data->height = 0;
    _data->hexes  = NULL;
    _thexes = NULL;
}

TMap::TMap(MAP * data, int id): TStructHolder<MAP>(data, id)
{
    _hexes_self_allocated = false;
    _thexes = (THex ***)KMemAlloc(_data->width * sizeof(THex **));
    
    for (int i = 0; i < _data->width ; i++) {
        _thexes[i] = (THex **)KMemAlloc(_data->height * sizeof(THex *));
        for (int j = 0; j < _data->height; j++) {
            _thexes[i][j] = new THex(_data->hexes[j*_data->width + i]);
        }
    }
}

void TMap::load(MAP * data)
{
    if (_data) {
        reset();
    }

    TStructHolder<MAP>::load(data);

    _thexes = (THex ***)KMemAlloc(_data->width * sizeof(THex **));
    
    for (int i = 0; i < _data->width ; i++) {
        _thexes[i] = (THex **)KMemAlloc(_data->height * sizeof(THex *));
        for (int j = 0; j < _data->height; j++) {
            _thexes[i][j] = new THex(_data->hexes[j*_data->width + i]);
        }
    }
}

void TMap::reload(HEX ** hexes)
{
    for (int i = 0; i < _data->width ; i++) {
        for (int j = 0; j < _data->height; j++) {
            _thexes[i][j]->load(hexes[j*_data->width + i]);
        }
    }

    _data->hexes = hexes;
}

HEX ** TMap::save(bool dealloc)
{
    return _data->hexes;
}

TMap::~TMap()
{
    reset();
}

void TMap::reset()
{
    // dealokuji vse, co jsem napachal
    int i, j;

    if (_thexes) {	
        // odmazu objekty t
        for (i = 0; i < _data->width; i++) {
            for (j = 0; j < _data->height; j++) {
                delete _thexes[i][j];
            }
            
            KMemFree(_thexes[i]);
        }

	    // pak radek zahlavi
	    KMemFree(_thexes);
    	
	    // nakonec nuluji pointer
	    _thexes = NULL;
    }

    if (_data && _self_allocated) {
        if (_data->hexes && _hexes_self_allocated)
            KMemFree(_data->hexes);
        KMemFree(_data);
        _data = NULL;
    }    
}

bool TMap::validCoords(const int x, const int y)
{
    return ((x >= 0) && (y >= 0) && (x < _data->width) && (y < _data->height));
}

bool TMap::validHexId(const int id)
{
    return ((id >= 0) && (id < _data->width * _data->height));
}

bool TMap::insideBorders(const int id)
{
    int x, y;

    if (idToCoords(id, &x, &y)) {
        // hex ma platne id 
        if ((x >= UNACCESSIBLE_BORDER_WIDTH) && (x < _data->width - UNACCESSIBLE_BORDER_WIDTH)) {
            // x-ova souradnice je v poradku
            if ((y >= UNACCESSIBLE_BORDER_HEIGHT) && (y < _data->height - UNACCESSIBLE_BORDER_HEIGHT)) {
                // y-ova souradnice je v poradku
                return true;
            }   
        }
    }

    return false;
}

bool TMap::idToCoords(const int id, int * x, int * y)
{
    if (validHexId(id)) {
        *x = id % _data->width;            
        *y = id / _data->width;
        return true;
	}
	else {
	    return false;
	}
}

bool TMap::coordsToId(const int x, const int y, int * id)
{
    if (validCoords(x, y)) {
        *id = y * _data->width + x;
        
        return true;
    }
    else
        return false;
}

THex * TMap::getHexAt(const int x, const int y)
{
	if ((x < _data->width) && (y < _data->height))
		return _thexes[x][y];
	else
		THROW(E_8K_World, "Invalid hex coords");
}

THex * TMap::getHexById(const int id)
{
	int x;
	int y;

	if (idToCoords(id, &x, &y))
		return _thexes[x][y];
	else
		THROW(E_8K_World, "Invalid hex id");
}

TIntContainer TMap::getNeighboursOrientation(const int hex_id, const int orientation)
{
    int x, y, neighbour;
    TIntContainer result;

    // prevod na skutecnou orientaci modelu
    int real_orientation = (orientation + 3) % 12;

    if (idToCoords(hex_id, &x, &y)) {
        if (y%2) {
            // lichy sloupec
            switch (real_orientation) {
                case 0:
                    // hex "vpravo nahore"
                    if (coordsToId(x, y + 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "vpravo dole"
                    if (coordsToId(x, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 1:
                    // hex "vpravo dole"
                    if (coordsToId(x, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 2:
                    // hex "vpravo dole"
                    if (coordsToId(x, y - 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "dole"
                    if (coordsToId(x, y - 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 3:
                    // hex "dole"
                    if (coordsToId(x, y - 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 4:
                    // hex "dole"
                    if (coordsToId(x, y - 2, &neighbour))
                        result.push_back(neighbour);
                    // hex "vlevo dole"
                    if (coordsToId(x - 1, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 5:
                    // hex "vlevo dole"
                    if (coordsToId(x - 1, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 6:
                    // hex "vlevo dole"
                    if (coordsToId(x - 1, y - 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "vlevo nahore"
                    if (coordsToId(x - 1, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 7:
                    // hex "vlevo nahore"
                    if (coordsToId(x - 1, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 8:
                    // hex "vlevo nahore"
                    if (coordsToId(x - 1, y + 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "nad"
                    if (coordsToId(x, y + 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 9:
                    // hex "nad"
                    if (coordsToId(x, y + 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 10:
                    // hex "nad"
                    if (coordsToId(x, y + 2, &neighbour))
                        result.push_back(neighbour);
                    // hex "vpravo nahore"
                    if (coordsToId(x, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 11:
                    // hex "vpravo nahore"
                    if (coordsToId(x, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
            }
        }
        else {
            // sudy sloupec
            switch (real_orientation) {
                case 0:
                    // hex "vpravo nahore"
                    if (coordsToId(x + 1, y + 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "vpravo dole"
                    if (coordsToId(x + 1, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 1:
                    // hex "vpravo dole"
                    if (coordsToId(x + 1, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 2:
                    // hex "vpravo dole"
                    if (coordsToId(x + 1, y - 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "dole"
                    if (coordsToId(x, y - 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 3:
                    // hex "dole"
                    if (coordsToId(x, y - 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 4:
                    // hex "dole"
                    if (coordsToId(x, y - 2, &neighbour))
                        result.push_back(neighbour);
                    // hex "vlevo dole"
                    if (coordsToId(x, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 5:
                    // hex "vlevo dole"
                    if (coordsToId(x, y - 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 6:
                    // hex "vlevo dole"
                    if (coordsToId(x, y - 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "vlevo nahore"
                    if (coordsToId(x, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 7:
                    // hex "vlevo nahore"
                    if (coordsToId(x, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 8:
                    // hex "vlevo nahore"
                    if (coordsToId(x, y + 1, &neighbour))
                        result.push_back(neighbour);
                    // hex "nad"
                    if (coordsToId(x, y + 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 9:
                    // hex "nad"
                    if (coordsToId(x, y + 2, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 10:
                    // hex "nad"
                    if (coordsToId(x, y + 2, &neighbour))
                        result.push_back(neighbour);
                    // hex "vpravo nahore"
                    if (coordsToId(x + 1, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
                case 11:
                    // hex "vpravo nahore"
                    if (coordsToId(x + 1, y + 1, &neighbour))
                        result.push_back(neighbour);
                    break;
            }
        }
    }

    return result;
}

TIntContainer * TMap::getNeighbours(const int x, const int y)
{
    if (validCoords(x, y)) {
        int _x = x;
        int _y = y;    
        COORDS coords[6];
        TIntContainer * result = new TIntContainer();

        if (y%2) {
            // lichy sloupec
            
            // hex "nad"
            coords[0].x = x;
            coords[0].y = y + 2;

            // hex "vpravo nahore"
            coords[1].x = x;
            coords[1].y = y + 1;

            // hex "vpravo dole"
            coords[2].x = x;
            coords[2].y = y - 1;

            // hex "dole"
            coords[3].x = x;
            coords[3].y = y - 2;

            // hex "vlevo dole"
            coords[4].x = x - 1;
            coords[4].y = y - 1;

            // hex "vlevo nahore"
            coords[5].x = x - 1;
            coords[5].y = y + 1;
        }
        else {
            // sudy sloupec

            // hex "nad"
            coords[0].x = x;
            coords[0].y = y + 2;

            // hex "vpravo nahore"
            coords[1].x = x + 1;
            coords[1].y = y + 1;

            // hex "vpravo dole"
            coords[2].x = x + 1;
            coords[2].y = y - 1;

            // hex "dole"
            coords[3].x = x;
            coords[3].y = y - 2;

            // hex "vlevo dole"
            coords[4].x = x;
            coords[4].y = y - 1;

            // hex "vlevo nahore"
            coords[5].x = x;
            coords[5].y = y + 1;
        }

        // pruchod souradnich
        int temp;
        for (int i = 0; i < 6; i++) {
            if (coordsToId(coords[i].x, coords[i].y, &temp)) 
                result->push_back(temp);            
        }

        return result;
    }
	else
		THROW(E_8K_World, "Invalid hex coords");
}

TIntContainer * TMap::getNeighbours(const int id)
{
    int x, y;
    idToCoords(id, &x, &y);

    return getNeighbours(x, y);
}

TIntContainer TMap::getCircle(HEX_ID center, int perimeter)
{
    TIntContainer range;
    TIntContainer * waiting = new TIntContainer(); // hexy cekajici na expanzi
    TIntContainer * next, *temp;
    TIntContainer * neighbours;                    // okoli   
    TIntIterator jt;
    int i, hex;

    if (center) {
        // inicializace
        range.push_back(center);
        next = getNeighbours(center);

        // v cyklu pres dosah utoku naleznu vsechny hexy, na ktere dosahnu
        for (i = 0; i < perimeter; i++) {
            // vyndani vsech uzlu z waiting, jejich prozkoumani 
            // a pridani jejich sousedu do waiting
            
            // prohozeni pointeru
            temp = waiting;
            waiting = next;
            next = temp;
            // smazani kontejneru pro expanzi v pristi iteraci
            next->clear();

            // expanze uzlu, priprava dalsi iterace
            while (waiting->size() > 0) {
                hex = waiting->back();
                waiting->pop_back();

                // pridani hexu do "hotovych" hexu v okoli
                range.push_back(hex);
                
                // pripadni kandidati do dalsi iterace - dosud nepridani sousedi "hotoveho" hexu
                neighbours = getNeighbours(hex);
                for (jt = neighbours->begin(); jt != neighbours->end(); jt++) {
                    if ((next->findIterator(*jt) == next->end()) && (range.findIterator(*jt) == range.end()))
                        next->push_back(*jt);
                }
            }
        }
        delete next;
        delete waiting;
        return range;
    }
    else {
        THROW(E_8K, "Chyba v konzistenci dat.");
    }
}

int TMap::width()
{
    return _data->width;
}

int TMap::height()
{
    return _data->height;
}

int TMap::maxId()
{
  int m = _data->width * _data->height - 1;
  return (m > 0) ? m : 0;
}

void TMap::writeToXML(TPackage * package)
{
    // ulozeni rozmeru mapy
    package->setInt(_data->width,  "width",  0, NULL);
    package->setInt(_data->height, "height", 0, NULL);
    
    // ulozeni hexu
    package->setEmpty("hexes", 0, NULL);
    TPackSmall * xml_hex;
    for (int i = 0; i < _data->width * _data->height; i++) {
        package->setEmpty("hexes", 0, "hex", i, NULL);
        xml_hex = new TPackSmall(package->getSubtag("hexes", 0, "hex", i, NULL));
        getHexById(i)->writeToXML(xml_hex);
        delete xml_hex;
    }
}

void TMap::readFromXML(TPackage * package)
{
    if (_data) {
        KMemFree(_data);
    }

    MAP * map = (MAP *)KMemAlloc(sizeof(MAP));
    TPackSmall * xml_hex;
    int i;

    // nacteni rozmeru mapy
    map->width = package->getInt("width", 0, NULL);
    map->height = package->getInt("height", 0, NULL);

    // alokace hexu
	map->hexes = (HEX **)KMemAlloc(map->width * map->height * sizeof(HEX *));
    for (i = 0; i < map->width * map->height; i++) {
		map->hexes[i] = (HEX *)KMemAlloc(sizeof(HEX));
        HEX_Init(map->hexes[i]);
    }

    // alokace THexu
    load(map);
    _self_allocated = true;
    _hexes_self_allocated = true;

    // nacteni dat v hexech a predani hexu k dealokaci do objektu THex
    for (i = 0; i < map->width * map->height; i++) {
        xml_hex = new TPackSmall(package->getSubtag("hexes", 0, "hex", i, NULL));
        getHexById(i)->readFromXML(xml_hex);   
        getHexById(i)->self_allocated(true);
        delete xml_hex;
    }
}

THistoryHex::THistoryHex(HISTORY_HEX * data, int id) : TStructHolder<HISTORY_HEX>(data, id)
{

}

TVisibilityHex::TVisibilityHex(THex * actual)
{
    _visibility = VI_NEVER_SEEN;
    _actual = actual;
    _history = NULL;
    _world = NULL;
}

TVisibilityHex::~TVisibilityHex()
{
    if (_history && _history->_self_allocated) {
        delete _history;
    }
}

void TVisibilityHex::setHistory(HISTORY_HEX * history)
{
    _history = new THistoryHex(history);
    _visibility = VI_SEEN;
}

bool TVisibilityHex::setActual(int unit_id, int building_id, int town_id)
{
    // zapamatuji si dosavadni vyhled na hex
    bool result = !(_visibility == VI_IN_SIGHT);

    // pripadne odstranim historicky zaznam
    if (_history && _history->_self_allocated) {
        delete _history;
        _history = NULL;
    }

    // nastavim aktualni viditelnost
    _visibility = VI_IN_SIGHT;
    
    // aktualizuji seznam entit, ktere na hex vidi
    if (unit_id && (_units.findIterator(unit_id) == _units.end()))
        _units.push_back(unit_id);
    if (building_id && (_buildings.findIterator(building_id) == _buildings.end()))
        _buildings.push_back(building_id);
    if (town_id && (_towns.findIterator(town_id) == _towns.end()))
        _towns.push_back(town_id);

    return result;
}

bool TVisibilityHex::removeActual(int unit_id, int building_id, int town_id)
{
    // Odstranim jednotku, resp. budovu z dohledu na svet. 
    // Byla-li poslendi, vratim true

    TIntContainer::iterator iter;

    if (unit_id && ((iter = _units.findIterator(unit_id)) != _units.end()))
        _units.erase(iter);
    if (building_id && ((iter = _buildings.findIterator(building_id)) != _buildings.end()))
        _buildings.erase(iter);
    if (town_id && ((iter = _towns.findIterator(town_id)) != _towns.end()))
        _towns.erase(iter);

    bool result = ((_units.size() == 0) && (_buildings.size() == 0) && (_towns.size() == 0));
    if (result) {
        // hex se dostava mimo vyhled - ulozim, co jsem videl
        HISTORY_HEX * history_hex = (HISTORY_HEX *)KMemAlloc(sizeof(HISTORY_HEX));
        history_hex->building = _actual->data().building;
        history_hex->turn_id = _world->turn_id;
        setHistory(history_hex);
        _history->_self_allocated = true;
    }

    return result;
}

VISIBILITY TVisibilityHex::getVisibility()
{
    return _visibility;
}

THex & TVisibilityHex::getActual()
{
    if ((_visibility == VI_IN_SIGHT) || (_visibility == VI_SEEN))
        return * _actual;
    else
        THROW(E_8K_INVALID_REQUEST, "Hex is not in actual sight");
}

THistoryHex & TVisibilityHex::getHistory()
{
    if (_visibility == VI_SEEN)
        return * _history;
    else
        THROW(E_8K_INVALID_REQUEST, "Hex is in actual sight or has not been seen yet");
}

void TVisibilityHex::readFromXML(TPackage * package)
{
    switch (_visibility = (VISIBILITY)package->getInt("visibility", 0, NULL))
    {
    case VI_IN_SIGHT:
        // aktualne v dohledu - zadna data
        setActual(0, 0);
        break;
    case VI_NEVER_SEEN:
        // mimo dohled - nedelam nic
        break;
    case VI_SEEN:
        // kdysi viden - zde nacitam xml
        HISTORY_HEX * history;
        history = (HISTORY_HEX *)KMemAlloc(sizeof(HISTORY_HEX));
        history->building = package->getInt("history", 0, "building", 0, NULL);
        history->turn_id = package->getInt("history", 0, "turn_id", 0, NULL);
        setHistory(history);
        break;       
    default:
        THROW(E_8K_WORLD_INVALID_DATA, "unknown value in 'visibility' tag");
    }    
}

void TVisibilityHex::writeToXML(TPackage * package)
{
    package->setInt(_visibility, "visibility", 0, NULL);
    switch (_visibility) {
        case VI_IN_SIGHT:
            // aktualne v dohledu - zadna data
            break;
        case VI_NEVER_SEEN:
            // mimo dohled - zadna data
            break;
        case VI_SEEN:
            // kdysi viden - zde ukladam XML
            package->setInt(_history->data().building, "history", 0, "building", 0, NULL);
            package->setInt(_history->data().turn_id, "history", 0, "turn_id", 0, NULL);
            break;
        default:
            THROW(E_8K_WORLD_INVALID_DATA, "unknown value in '_visibility' variable");
    }
}

TVisibilityMap::TVisibilityMap(TMap & whole_map) : _map(whole_map)
{
    int i, j;

    _map = whole_map;
    _hexes = (TVisibilityHex ***)KMemAlloc(_map._data->width * sizeof(TVisibilityHex **));
    for (i = 0; i < _map._data->width; i++) {
        _hexes[i] = (TVisibilityHex **)KMemAlloc(_map._data->height * sizeof(TVisibilityHex *));
        for (j = 0; j < _map._data->height; j++)
            _hexes[i][j] = new TVisibilityHex(_map.getHexAt(i,j));
    }
}

TVisibilityMap::~TVisibilityMap()
{
    reset();
}

void TVisibilityMap::reset()
{
    int i, j;

    if (_hexes) {
        for (i = 0; i < _map._data->width; i++) {
            for (j = 0; j < _map._data->height; j++) {
                delete _hexes[i][j];
            }
            KMemFree(_hexes[i]);
        }
            
        KMemFree(_hexes);
    }
    _hexes = NULL;
}

TVisibilityHex & TVisibilityMap::getHexAt(const int x, const int y)
{
    if ((x < _map._data->width) && (y < _map._data->height))
		return * _hexes[x][y];
	else
		THROW(E_8K_World, "Invalid hex coords");
}

TVisibilityHex & TVisibilityMap::getHexById(const int id)
{
	int x;
	int y;

	if (_map.idToCoords(id, &x, &y))
		return * _hexes[x][y];
	else
		THROW(E_8K_World, "Invalid hex id");
}

void TVisibilityMap::writeToXML(TPackage * package)
{
    int i;
    TPackSmall * xml_hex;

    // ulozeni hexu
    for (i = 0; i < _map._data->width * _map._data->height; i++) {
        package->setEmpty("hex", i, NULL);
        xml_hex = new TPackSmall(package->getSubtag("hex", i, NULL));
        getHexById(i).writeToXML(xml_hex);
        delete xml_hex;
    }
}

void TVisibilityMap::readFromXML(TPackage * package)
{
    int i;
    TPackSmall * xml_hex;

    // nacteni dat v hexech
    for (i = 0; i < _map._data->width * _map._data->height; i++) {
        xml_hex = new TPackSmall(package->getSubtag("hex", i, NULL));
        getHexById(i).readFromXML(xml_hex);       
        delete xml_hex;
    }
}

void TVisibilityMapsContainer::load(HISTORY_HEX *** hm, TMap & map)
{
    int i, j, k;

    // projdu pole masek viditelnosti
    for (i = 1; i < MAX_PLAYERS + 1; i++) {
        if (hm[i] != NULL) {
            // pro daneho hrace mam masku viditelnosti
            (*this)[i] = new TVisibilityMap(map);
            // projdu vsechny hexy a tam, kde mam nejaka data, je ulozim do pameti
            for (j = 0; j < map.data().width; j++) {
                for (k = 0; k < map.data().height; k++) {
                    if (hm[i][j * map.data().height + k] != NULL) {
                        (*this)[i]->getHexAt(j,k).setHistory(hm[i][j * map.data().height + k]);
                    }
                }
            }
            KMemFree(hm[i]);
        }
    }

    KMemFree(hm);
}

HISTORY_HEX *** TVisibilityMapsContainer::save(bool dealloc)
{
    HISTORY_HEX *** result = (HISTORY_HEX ***)KMemAlloc((MAX_PLAYERS+1) * sizeof(HISTORY_HEX **)); 
    TVisibilityMapsContainer::iterator it;
    TVisibilityHex * visibility_hex;
    int i, j;

    // inicializace map
    for (i = 0; i <= MAX_PLAYERS; i++) 
        result[i] = NULL;

    // naplneni map daty a nasledne uvolneni objektu TVisibilityMap
    for (it = begin(); it != end(); it++) {
        result[it->first] = (HISTORY_HEX **)KMemAlloc(it->second->_map.data().width * it->second->_map.data().height * sizeof(HISTORY_HEX *));

        // naplneni hexu daty
        for (i = 0; i < it->second->_map.data().width; i++) {
            for (j = 0; j < it->second->_map.data().height; j++) {
                visibility_hex = &it->second->getHexAt(i, j);
                if (visibility_hex->getVisibility() == VI_SEEN) {
                    result[it->first][i * it->second->_map.data().height + j] = &(visibility_hex->getHistory().data());
                    visibility_hex->getHistory().self_allocated(false);
                }
                else {
                    result[it->first][i * it->second->_map.data().height + j] = NULL; 
                }
            }
        }

        if (dealloc) {
            delete it->second;
            it->second = NULL;
        }
    }

    // celkove uvolneni
    if (dealloc) {
        clear();
    }

    // navratove hodnoty
    return result;
}

void TVisibilityMapsContainer::reset()
{
    TVisibilityMapsContainer::iterator it;
    for (it = begin(); it != end(); it++)
        delete it->second;

    clear();
}

void TVisibilityMapsContainer::writeToXML(TPackage * package)
{
    TVisibilityMapsContainer::iterator it;
    TPackSmall * small;

    for (it = begin(); it != end(); it++) {
        package->setEmpty("visibility", it->first, NULL);
        small = new TPackSmall(package->getSubtag("visibility", it->first, NULL));
        it->second->writeToXML(small);
        delete small;
    }
}

void TVisibilityMapsContainer::readFromXML(TPackage * package)
{
    TVisibilityMapsContainer::iterator it;
    TPackSmall * small;

    for (it = begin(); it != end(); it++) {
        small = new TPackSmall(package->getSubtag("visibility", it->first, NULL));
        it->second->readFromXML(small);
        delete small;
    }
}
