/** @file /world/players/player.cpp

    Implementace hlavickoveho souboru /world/players/player.j

    @author Petr Wolf
*/

#include "common/rm/rmmap.h"
#include "world/players/player.h"
#include "world/useful.h"

using namespace World;
using namespace rm;

TPlayer::TPlayer()
{
    _relations_self_allocated = false;
}

TPlayer::TPlayer(PLAYER * data, int id) : TStructHolder<PLAYER>(data, id)
{
    _relations_self_allocated = false;    
}

TPlayer::TPlayer(int id) : TStructHolder<PLAYER>(id)
{
    _relations_self_allocated = false;
}

TPlayer::~TPlayer()
{
    if (_relations_self_allocated) {
        for (int i = 1; i <= MAX_PLAYERS; i++) {
            if (_data->game_data.relations[i] != NULL) {
                KMemFree(_data->game_data.relations[i]);
                _data->game_data.relations[i] = NULL;
            }
        }    
    }
}

void TPlayer::writeToXML(TPackage * package)
{
    // client data
    package->setChar(_data->client_data.name, "client_name", 0, NULL);
    package->setInt(_data->client_data.client_id, "client_id", 0, NULL);
    package->setInt(_data->client_data.type, "type", 0, NULL);
    package->setInt(_data->client_data.acommands, "acommands", 0, NULL);
    package->setInt(_data->client_data.connected, "connected", 0, NULL);

    // game data
    package->setChar(_data->game_data.name, "game_name", 0, NULL);
    SET_INT(package, "money", _data->game_data.money);
    SET_INT(package, "state", _data->game_data.state);

    // statistics
    SET_INT(package, "units_killed", _data->statistics.units_killed);
    SET_INT(package, "units_lost", _data->statistics.units_lost);
    SET_INT(package, "units_created", _data->statistics.units_created);
    SET_INT(package, "buildings_built", _data->statistics.buildings_built);
    SET_INT(package, "buildings_captured", _data->statistics.buildings_captured);
    SET_INT(package, "buildings_lost", _data->statistics.buildings_lost);
    SET_INT(package, "buildings_destroyed", _data->statistics.buildings_destroyed);
    SET_INT(package, "towns_captured", _data->statistics.towns_captured);
    SET_INT(package, "towns_lost", _data->statistics.towns_lost);
    SET_INT(package, "kingdoms_captured", _data->statistics.kingdoms_captured);
    SET_INT(package, "kingdoms_lost", _data->statistics.kingdoms_lost);
    SET_INT(package, "money_gained", _data->statistics.money_gained);

    // relations
    for (int i = 1; i <= MAX_PLAYERS; i++) {
        if (_data->game_data.relations[i] != NULL) {
            package->setInt(_data->game_data.relations[i]->rs, "relations", 0, "relationship", i, "rs", 0, NULL);
            package->setInt(_data->game_data.relations[i]->offeredrs, "relations", 0, "relationship", i, "offeredrs", 0, NULL);
            // this was missing!!
            package->setInt(_data->game_data.relations[i]->relationship_properties.belief, "relations", 0, "relationship", i, "belief", 0, NULL);
            package->setInt(_data->game_data.relations[i]->relationship_properties.offer_seen_by_enemy, "relations", 0, "relationship", i, "offer_seen_by_enemy", 0, NULL);
            package->setInt(_data->game_data.relations[i]->relationship_properties.guess_belief, "relations", 0, "relationship", i, "guess_belief", 0, NULL);
        }
    }
}

void TPlayer::readFromXML(TPackage * package)
{
    if (_data == NULL) {
        alloc();
        PLAYER_Init(_data);
    }

    // client data
    package->getChar(_data->client_data.name, MAX_STRLEN, "client_name", 0, NULL);
    _data->client_data.client_id = package->getInt("client_id", 0, NULL);
    _data->client_data.type      = (PLAYER_TYPE)package->getInt("type", 0, NULL);
    _data->client_data.acommands      = (PLAYER_ALLOWED_COMMANDS)package->getInt("acommands", 0, NULL);
    _data->client_data.connected = (package->getInt("connected", 0, NULL) != 0);

    // game data
    package->getChar(_data->game_data.name, MAX_STRLEN, "game_name", 0, NULL);
    GET_INT(package, "money", _data->game_data.money);
    GET_TINT(package, "state", _data->game_data.state, PLAYER_STATE);

    // statistics
    GET_INT(package, "units_killed", _data->statistics.units_killed);
    GET_INT(package, "units_lost", _data->statistics.units_lost);
    GET_INT(package, "units_created", _data->statistics.units_created);
    GET_INT(package, "buildings_built", _data->statistics.buildings_built);
    GET_INT(package, "buildings_captured", _data->statistics.buildings_captured);
    GET_INT(package, "buildings_lost", _data->statistics.buildings_lost);
    GET_INT(package, "buildings_destroyed", _data->statistics.buildings_destroyed);
    GET_INT(package, "towns_captured", _data->statistics.towns_captured);
    GET_INT(package, "towns_lost", _data->statistics.towns_lost);
    GET_INT(package, "kingdoms_captured", _data->statistics.kingdoms_captured);
    GET_INT(package, "kingdoms_lost", _data->statistics.kingdoms_lost);
    GET_INT(package, "money_gained", _data->statistics.money_gained);

    // relations
    TPackSmall * small = new TPackSmall(package->getSubtag("relations", 0, NULL));
    TXMLdata * data = small->getXML();
    DA<int> ids;
    data->getListOfIds(&ids, NULL, 0, NULL);
    for (int i = 1; i <= MAX_PLAYERS; i++) {
        if (ids[i] == 1) {
            _data->game_data.relations[i] = (ai_ns::diplomacy_ns::TDipRelation *)KMemAlloc(sizeof(ai_ns::diplomacy_ns::TDipRelation));
            _data->game_data.relations[i]->rs = (ai_ns::diplomacy_ns::TRelationship)package->getInt("relations", 0, "relationship", i, "rs", 0, NULL);
            _data->game_data.relations[i]->offeredrs = (ai_ns::diplomacy_ns::TRelationship)package->getInt("relations", 0, "relationship", i, "offeredrs", 0, NULL);
            // this was missing!!
            _data->game_data.relations[i]->relationship_properties.belief=(ai_ns::diplomacy_ns::TRelationship)package->getInt("relations", 0, "relationship", i, "belief", 0, NULL);
            _data->game_data.relations[i]->relationship_properties.guess_belief=(ai_ns::diplomacy_ns::TRelationship)package->getInt("relations", 0, "relationship", i, "guess_belief", 0, NULL);
            _data->game_data.relations[i]->relationship_properties.offer_seen_by_enemy=(ai_ns::diplomacy_ns::TRelationship)package->getInt("relations", 0, "relationship", i, "offer_seen_by_enemy", 0, NULL);

        }
        else {
            _data->game_data.relations[i] = NULL;
        }
    }
    delete small;
    _relations_self_allocated = true;
}

void TPlayer::readFromTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char _path[MAX_STRLEN];

    interpreter.getVar(TCL_path(_path, varName, "name"), TVT_STRING, _data->game_data.name);
    interpreter.getVar(TCL_path(_path, varName, "money"), TVT_INT, &_data->game_data.money);
    interpreter.getVar(TCL_path(_path, varName, "state"), TVT_INT, &_data->game_data.state);
    interpreter.getVar(TCL_path(_path, varName, "type"), TVT_INT, &_data->client_data.type);
    interpreter.getVar(TCL_path(_path, varName, "acommands"), TVT_INT, &_data->client_data.acommands);
}

void TPlayer::writeToTCL(TTCL_Interpreter & interpreter, const char * varName)
{
    char _path[MAX_STRLEN];

    interpreter.setVar(TCL_path(_path, varName, "name"), TVT_STRING, _data->game_data.name);
    interpreter.setVar(TCL_path(_path, varName, "money"), TVT_INT, &_data->game_data.money);
    interpreter.setVar(TCL_path(_path, varName, "state"), TVT_INT, &_data->game_data.state);
    interpreter.setVar(TCL_path(_path, varName, "type"), TVT_INT, &_data->client_data.type);
    interpreter.setVar(TCL_path(_path, varName, "acommands"), TVT_INT, &_data->client_data.acommands);

    // relations
    int relations[MAX_PLAYERS + 1];
    relations[0] = 0;
    for (int i = 1; i <= MAX_PLAYERS; i++) {
        if (_data->game_data.relations[i] != NULL) 
            relations[i] = _data->game_data.relations[i]->rs;
        else
            relations[i] = 0;
    }
    interpreter.setArray(TCL_path(_path, varName, "relations"), TVT_INT, MAX_PLAYERS + 1, relations);
}

TPlayerContainer::TPlayerContainer()
{
    player_on_turn = 0;
    ai_data = NULL;
    _dummy = NULL;
}

TPlayerContainer::~TPlayerContainer()
{
    reset();
}

TPlayer * & TPlayerContainer::operator[](const PLAYER_ID _Key)
{
    if (_Key == 0)
        return _dummy;
    else
        return TDAHolder<PLAYER, PLAYER_ID, TPlayer>::operator[](_Key);
}

void TPlayerContainer::reset()
{
    if (ai_data != NULL) 
        for (int i = 1; i < MAX_PLAYERS; i++)
            if (ai_data->plan[i] != NULL) {
                KMemFree(ai_data->plan[i]);
                ai_data->plan[i] = NULL;
            }

    TDAHolder<PLAYER, PLAYER_ID, TPlayer>::reset();
}

void TPlayerContainer::save(TRoleInfo ** info)
{
    int i = 0;
    TPlayerIterator  it;
        
    for (it = begin(); it != end(); it++) {
        info[it->first]->money = it->second->data().game_data.money;
        info[it->first]->stats = it->second->data().statistics;
        for (i = 1; i <= MAX_PLAYERS; i++) {
            if (it->second->data().game_data.relations[i]) {
              *(info[it->first]->relations[i]) = *(it->second->data().game_data.relations[i]);
            }
            else {
                info[it->first]->relations[i] = NULL;
            }
        }        
    }
}

TPlayer * TPlayerContainer::playerOnTurn()
{
    return operator[](player_on_turn);
}

PLAYER_ID TPlayerContainer::playerIdOnTurn()
{
    return player_on_turn;
}

PLAYER_ID TPlayerContainer::setPlayerOnTurn(PLAYER_ID player_id)
{
    return (player_on_turn = player_id);
}

PLAYER_ID TPlayerContainer::nearestHumanPlayerId()
{
    // nalezeni aktivniho hrace
    TPlayerIterator i = find(player_on_turn);

    if (i == end()) {
        // hrace s danym id jsem nenasel -> skocim na zacatek
        i = begin();
    }   

    while ((i->second->data().game_data.state != PS_ACTIVE) || (i->second->data().client_data.type != PT_HUMAN)) {        
        i++;
        if (i == end())
            i = begin();

        if (i->first == player_on_turn) {
            // dokoncil jsem kolecko a hrace jsem nenasel
            return 0;
        }
    }

    return i->first; 
}

int TPlayerContainer::numberOfHumanPlayers(PLAYER_ID ignore)
{
    int count = 0;
    for (TPlayerContainer::iterator it = begin(); it != end(); it++) {
        if ((it->second->data().game_data.state == PS_ACTIVE) && (it->second->data().client_data.type == PT_HUMAN))
            if (ignore != it->first)
                count++;
    }

    return count;
}

int TPlayerContainer::next()
{
    // nalezeni aktivniho hrace
    TPlayerIterator i = find(player_on_turn);
    
    // posun na dalsiho hrace
    if (i == end()) {
        // hrace s danym id jsem nenasel (zacatek hry)
        i = begin();
        player_on_turn = i->first;
        return 0;
    }
    else {
        // hrac nalezen, predavam dal
        int next_turn = 0;
        bool player_defeated = true;

        while (player_defeated) {
            i++;
            if (i == end()) {
                // doslo k preklopeni do noveho kola
                i = begin();   
                next_turn = 1;
            }

            if (i->second->data().game_data.state == PS_ACTIVE)
                player_defeated = false;

            player_on_turn = i->first;
        }

        return next_turn;
    }
}

void TPlayerContainer::unitKilled(PLAYER_ID owner, PLAYER_ID killer)
{
    if (owner)
        (*this)[owner]->data().statistics.units_lost++;
    if (killer)
        (*this)[killer]->data().statistics.units_killed++;
}

void TPlayerContainer::unitCreated(PLAYER_ID owner)
{
    if (owner)
        (*this)[owner]->data().statistics.units_created++;
}

void TPlayerContainer::buildingBuilt(PLAYER_ID owner)
{
    if (owner)
        (*this)[owner]->data().statistics.buildings_built++;
}

void TPlayerContainer::buildingCaptured(PLAYER_ID owner, PLAYER_ID previous_owner)
{
    if (owner)
        (*this)[owner]->data().statistics.buildings_captured++;
    if (previous_owner)
        (*this)[previous_owner]->data().statistics.buildings_lost++;
}

void TPlayerContainer::buildingDestroyed(PLAYER_ID owner, PLAYER_ID destroyer)
{
    if (owner)
        (*this)[owner]->data().statistics.buildings_lost++;
    if (destroyer)
        (*this)[destroyer]->data().statistics.buildings_destroyed++;
}

void TPlayerContainer::townCaptured(PLAYER_ID owner, PLAYER_ID previous_owner)
{
    if (owner)
        (*this)[owner]->data().statistics.towns_captured++;
    if (previous_owner)
        (*this)[previous_owner]->data().statistics.towns_lost++;
}

void TPlayerContainer::kingdomCaptured(PLAYER_ID owner, PLAYER_ID previous_owner)
{
    if (owner)
        (*this)[owner]->data().statistics.kingdoms_captured++;
    if (previous_owner)
        (*this)[previous_owner]->data().statistics.kingdoms_lost++;
}

void TPlayerContainer::moneyGained(PLAYER_ID owner, MONEY amount)
{
    if (owner)
        (*this)[owner]->data().statistics.money_gained += amount;
}
