/** @file /world/struct_holder.h

    Obecna sablona pro objektovou nadstavbu obycejne struktury

    @author Petr Wolf
*/

#ifndef WORLD_STRUCT_HOLDER_H
#define WORLD_STRUCT_HOLDER_H

#include "common/mm.h"

namespace World
{
    /// Sablona pro objektovou nadstavbu C-ckovych datovych struktur zajistujici vlastni (de)alokaci a spravu
    template<class T>
    class TStructHolder
    {
    protected:
        T * _data;
        int _id;
        bool _self_allocated;
    public:
        TStructHolder();
        TStructHolder(T * data, int id = 0);
        TStructHolder(int id);
        ~TStructHolder();

        void load(T * data);
        void reset();
        void alloc();
        void self_allocated(bool allocated);
        T & data();
        int id();
    };

    template<class T>
    TStructHolder<T>::TStructHolder()
    {
        _data = NULL;
        _id   = 0;
        _self_allocated = false;
    }

    template<class T>
    TStructHolder<T>::TStructHolder(T * data, int id)
    {
        _data = data;
        _id = id;
        _self_allocated = false;
    }

    template<class T>
    TStructHolder<T>::TStructHolder(int id)
    {
        _data = NULL;
        _id = id;
        _self_allocated = false;
    }

    template<class T>
    TStructHolder<T>::~TStructHolder()
    {
        reset();
    }

    template<class T>
    void TStructHolder<T>::reset()
    {
        if (_data && _self_allocated) {
            KMemFree(_data);            
        }
        _data = NULL;
    }

    template<class T>
    void TStructHolder<T>::load(T * data)
    {
        _data = data;
    }

    template<class T>
    void TStructHolder<T>::alloc()
    {
        _data = (T *)KMemAlloc(sizeof(T));
        int size = sizeof(T);
        _self_allocated = true;
    }

    template<class T>
    void TStructHolder<T>::self_allocated(bool allocated)
    {
        _self_allocated = allocated;
    }

    template<class T>
    T & TStructHolder<T>::data()
    {
       if (_data == NULL) 
            _data = (T *)KMemAlloc(sizeof(T));

        return * _data;
    }

    template<class T>
    int TStructHolder<T>::id()
    {
        return _id;
    }
}

#endif 
/*****************************************************************************/
