/** @file /world/typedefs.cpp

    Implementace hlavickoveho souboru /world/typedefs.h

    @author Petr Wolf
*/

#include <vector>
#include "common/mm.h"
#include "useful.h"
#include "typedefs.h"


void INT_ARRAY_TO_XML(TPackage * package, int * data, int length, char * tag) 
{
    package->setEmpty(tag, 0, NULL); 
    TPackSmall small(package->getSubtag(tag, 0, NULL));
    
    for (int i = 0; i < length; i++) {
        small.setInt(data[i], "item", i+1, NULL);
    }
}

void INT_ARRAY_FROM_XML(TPackage * package, int ** data, int * length, char * tag)
{
    TPackSmall small(package->getSubtag(tag, 0, NULL));
    TXMLdata * xml_data = small.getXML();
    DA<int> ids;
    xml_data->getListOfIds(&ids, NULL, 0, NULL);
    std::vector<int> result;
    int i;
    size_t j;

    for (i = ids.getNext(-1); i != -1; i = ids.getNext(i)) {
        if (ids[i] == 1) {  
            result.push_back(small.getInt("item", i, NULL));
        }
    }

    // dealokace puvodnich dat
    if (*data != NULL) 
        KMemFree(*data);
    
    // alokace novych
    *data = (int *)KMemAlloc(result.size() * sizeof(int));

    // nastaveni delky
    *length = (int)result.size();

    // naplneni daty
    for (j = 0; j < result.size(); j++) 
        (*data)[j] = result[j]; 

}

void INT_ARRAY_FROM_XML(TPackage * package, int data[], int length, char * tag)
{
    TPackSmall small(package->getSubtag(tag, 0, NULL));
    TXMLdata * xml_data = small.getXML();
    DA<int> ids;
    xml_data->getListOfIds(&ids, NULL, 0, NULL);
    std::vector<int> result;
    int i;
    size_t j;

    for (i = ids.getNext(-1); i != -1; i = ids.getNext(i)) {
        if (ids[i] == 1) {  
            result.push_back(small.getInt("item", i, NULL));
        }
    }

    // naplneni daty
    for (j = 0; (j < result.size()) && (j < (size_t)length); j++)
        data[j] = result[j];
}

int TIntContainer::find(const int item)
{
    for (size_t i = 0; i < size(); i++) {
        if (at(i) == item)
            return (int)i;
    }

    return -1;
}

TIntIterator TIntContainer::findIterator(const int item)
{
    TIntIterator i;
    for (i = begin(); i != end(); i++) {
        if (*i == item)
            return i;
    }

    return end();
}

void TIntContainer::readFromXML(TPackage * package)
{   
    // vymazani starych prvku
    clear();

    // nacteni novych
    TXMLdata * xml_data = package->getXML();
    DA<int> ids;
    xml_data->getListOfIds(&ids, NULL, 0, NULL);

    for (int i = ids.getNext(-1); i != -1; i = ids.getNext(i)) {
        if (ids[i] == 1) {       
            push_back(package->getInt("item", i, NULL));
        }
    }
}
    
void TIntContainer::writeToXML(TPackage * package)
{
    for (size_t i = 0; i < size(); i++)
        package->setInt(at(i), "item", (int)i+1, NULL);
}

TIntContainer TIntContainer::difference(TIntContainer & a, TIntContainer & b)
{
    TIntContainer result;
    std::map<int, bool> numbers;
    std::map<int, bool>::iterator i;
    TIntContainer::iterator it;

    for (it = a.begin(); it != a.end(); it++) 
        numbers[*it] = true;
    for (it = b.begin(); it != b.end(); it++)
        numbers[*it] = false;

    for (i = numbers.begin(); i != numbers.end(); i++)
        if (i->second)
            result.push_back(i->first);

    return result;
}

void TIntArray::readFromXML(TPackage * package)
{
    // vymazani starych prvku
    clear();

    // nacteni novych
    TXMLdata * xml_data = package->getXML();
    DA<int> ids;
    xml_data->getListOfIds(&ids, NULL, 0, NULL);

    for (int i = ids.getNext(-1); i != -1; i = ids.getNext(i)) {
        if (ids[i] == 1) {   
            (*this)[i] = package->getInt("item", i, NULL);
        }
    }
}
    
void TIntArray::writeToXML(TPackage * package)
{
    for (TIntArray::iterator i = begin(); i != end(); i++) {
        package->setInt(i->second, "item", i->first, NULL);
    }
}

void TStep::readFromXML(TPackage * package)
{
    GET_INT(package, "hex_id", hex_id);
    GET_INT(package, "left_hex_id", left_hex_id);
    GET_INT(package, "cost", cost);
    READ_FROM_XML(package, &shown, "shown");
    READ_FROM_XML(package, &hidden, "hidden");
}

void TStep::writeToXML(TPackage * package)
{
    SET_INT(package, "hex_id", hex_id);
    SET_INT(package, "left_hex_id", left_hex_id);
    SET_INT(package, "cost", cost);
    WRITE_TO_XML(package, &shown, "shown");
    WRITE_TO_XML(package, &hidden, "hidden");
}

void TPath::readFromXML(TPackage * package)
{
    TPackSmall * xml;
    TXMLdata * xml_data = package->getXML();
    DA<int> ids;
    xml_data->getListOfIds(&ids, NULL, 0, NULL);
    TStep step;

    for (int i = ids.getNext(-1); i != -1; i = ids.getNext(i)) {
        if (ids[i] == 1) {  
            xml = new TPackSmall(package->getSubtag("step", i, NULL));
            step.readFromXML(xml);
            push_back(step);
            delete xml;
        }
    }
}

void TPath::writeToXML(TPackage * package)
{
    TPackSmall * xml;
    int i = 1;

    for (TPathIterator iter = begin(); iter != end(); iter++) {
        package->setEmpty("step", i, NULL);
        xml = new TPackSmall(package->getSubtag("step", i, NULL));
        iter->writeToXML(xml);
        delete xml;
        i++;
    }
}

/*
template<class T> void K8_ARRAY_free(K8_ARRAY<T> * array)
{
    KMemFree(array->data);
    array->length = 0;
}

template<class T> bool K8_ARRAY_alloc(K8_ARRAY<T> * array, const int length)
{
    if (array->data = (T *)KMemAlloc(length * sizeof(T))) {
        array->length = length;
        return true;
    }
    else
        return false;
}
*/
