/** @file /world/typedefs.h

    Souhrnny header obsahujici spolecne definice a includy vsech
    dalsich definicnich headeru modulu world.

	@author Petr Wolf
*/

#ifndef WORLD_TYPEDEFS
#define WORLD_TYPEDEFS

#include <vector>
#include <map>
#include "common/compatibility.h"
#include "common/mm.h"
#include "common/utils.h"
#include "common/types.h"
#include "common/xml/xml_struct.h"

/// jednoznacne ID typu herni jednotky
typedef int UNIT_TYPE_ID;

/// jednoznacne ID herni jednotky
typedef int UNIT_ID;

/// orientace herni jednotky
typedef int UNIT_ORIENTATION;

/// jednoznacne ID typu budovy
typedef int BUILDING_TYPE_ID;

/// jednoznacne ID herni budovy
typedef int BUILDING_ID;

/// orientace herni budovy
typedef int BUILDING_ORIENTATION;

/// jednoznacne ID hrace
typedef int PLAYER_ID;

/// jednoznacne ID bonusu (koupeneho, nebo ziskaneho postupem na vyssi uroven)
typedef int UNIT_PROPERTY_BONUS_ID;

/// ID hexu
typedef int HEX_ID;

/// ID kralovstvi
typedef int KINGDOM_ID;

/// jednoznacne ID TCL skriptu
typedef int TCL_SCRIPT_ID;

/// ID uskutecnenych hernich akci
typedef unsigned long int ACTION_ID;

/// ID (od 1 do n) herniho kola
typedef int ROUND_ID;

/// pocet (maximalni) hracu ve hre
#define MAX_PLAYERS 8

/// pocet (maximalni) kralovstvi ve hre
#define MAX_KINGDOMS 8

/// Maximalni pocet reakci, ktere maji zvuk
#define MAX_ACTION_SOUNDS 10

/// Rezim hry
enum GAME_MODE
{
    // rezim nebyl dosud zvolen
    GM_NA

    // simulace v editoru 
   ,GM_EDITOR

    // hra na jednom pocitaci 
   ,GM_LOCAL

    // sitova hra zalozena z tohoto pocitace
   ,GM_NET_SERVER
    
    // pripojeni se k sitove hre zalozene na jinem pocitaci
   ,GM_NET_CLIENT
};

/// Podminky ukonceni hry
enum END_GAME_CONDITION
{
    /// vyhra obsazenim vetsiny mest nebo kralovstvi (default)
    EGC_CONQUER

    /// vitezstvi znicenim vsech protivniku
   ,EGC_DEATHMATCH
};

/// obecna sablona pole pevne delky pro predavani dat ve strukturach
template<class T> struct K8_ARRAY
{
    int length;
    T * data;
};

/** Zapis pole typu K8_ARRAY<int> do XML */
void INT_ARRAY_TO_XML(TPackage * package, int * data, int length, char * tag); 

/** Nacteni pole typu K8_ARRAY<int> z XML */
void INT_ARRAY_FROM_XML(TPackage * package, int ** data, int * length, char * tag);

void INT_ARRAY_FROM_XML(TPackage * package, int data[], int length, char * tag);


/// prvek kontejnery K8_CONTAINER
template<class T_ID, class T_DATA> struct K8_CONTAINER_ELEMENT
{
    T_ID id;
    T_DATA data;
};

/// obecny kontejner nesouci data svazana s konkretnim ID
template<class T_ID, class T_DATA> struct K8_CONTAINER
{
    int length;
    K8_CONTAINER_ELEMENT<T_ID, T_DATA> * data;
};

/// Obecny kontejner pro predavani spojaku celociselnych hodnot (typicky id)
class TIntContainer : public std::vector<int>, public TXML_Struct
{
public:
    /// Nalezeni indexu prvku s hodnotou \c item
    int find(const int item);

    /// Nalezeni iteratoru pro prvek s hodnorou \c item
    std::vector<int>::iterator findIterator(const int item);
    
    /// Nacteni z XML.
    /// @param package XML zprava
    void readFromXML(TPackage * package);
    
    /// Zapis do XML.
    /// @param package XML zprava
    void writeToXML(TPackage * package);

    static TIntContainer difference(TIntContainer & a, TIntContainer & b);
};

/// Iterator pro TIntContainer
typedef std::vector<int>::iterator TIntIterator;

/// Obecne ridke pole int-int dvojic
class TIntArray : public std::map<int, int>, public TXML_Struct
{
public:
    /// Nacteni z XML.
    /// @param package XML zprava
    void readFromXML(TPackage * package);
    
    /// Zapis do XML.
    /// @param package XML zprava
    void writeToXML(TPackage * package);
};

/// dilci krok cesty jednotky
class TStep : public TXML_Struct
{
public:
    /// hex, ze ktereho jednotka odesla
    HEX_ID left_hex_id; 

    /// hex, na ktery jednotka vstoupila
    HEX_ID hex_id;

    /// cena (v bodech pohybu), kterou za to musi jednotka vynalozit
    int cost;

    /// seznam hexu, ktere se pohybem odkryly do sfery aktualniho vyhledu
    TIntContainer shown;

    /// seznam hexu, ktere se pohybem ztratily z aktivniho vyhledu
    TIntContainer hidden;

    /// Nacteni z XML. 
    /// @param package XML zprava
    void readFromXML(TPackage * package);
    
    /// Zapis do XML.
    /// @param package XML zprava
    void writeToXML(TPackage * package);
};

/// Cesta po mape
class TPath : public std::vector<TStep>, public TXML_Struct
{
public:
    /// Nacteni z XML. 
    /// @param package XML zprava
    void readFromXML(TPackage * package);
    
    /// Zapis do XML.
    /// @param package XML zprava
    void writeToXML(TPackage * package);
};

typedef TPath::iterator TPathIterator;

#include "common/TCL/tcl_script.h"
#include "world/weather.h"
#include "world/plan/plan.h"
#include "world/players/player.h"
#include "world/buildings/building.h"
#include "world/units/unit.h"
#include "world/init.h"

#endif
/*****************************************************************************/
