/** @file world/weather.h

    Pocasi ve svete

    @author Petr Wolf
*/

#ifndef WORLD_WEATHER_H
#define WORLD_WEATHER_H

#include "common/mutex.h"
#include "world/typedefs.h"
#include "world/struct_holder.h"
#include "world/tcl_xml.h"

namespace World
{
    /// stav pocasi
    enum WEATHER_STATE
    {
        /// slunecno
        WS_SUNNY
        
        // dest
       ,WS_RAIN
        
        // snih
       ,WS_SNOW
    };

    /// reprezentace pocasi - stav a doba trvani
    struct WEATHER
    {
        /// aktualni stav pocasi
        WEATHER_STATE state;

        /// doba trvani
        int duration;
    };

    /// Objektova nadstavba reprezentujici pocasi
    class TWeather: public TStructHolder<WEATHER>, public TTCL_XML_Struct, public TMutex
    {
    public:
        /// konstruktor
        TWeather();

        /// konstruktor prejimajici popis pocasi
        /// @param data aktualni pocasi
        TWeather(WEATHER * data);

        /// zapis do XML
        void writeToXML(TPackage * package);

        /// nacten z XML
        void readFromXML(TPackage * package);
        
        /// zapis do TCL
        void readFromTCL(TTCL_Interpreter & interpreter, const char * varName);
        
        /// nacteni z TCL
        void writeToTCL(TTCL_Interpreter & interpreter, const char * varName);
    };
};

#endif

