/** @file /world/world_client.h

	Zakladni tridy, konstanty a definice pro reprezentaci herniho sveta z pohledu
	klienta.
 
	@author Petr Wolf
*/

#ifndef WORLD_CLIENT_H
#define WORLD_CLIENT_H

#include <stdarg.h>
#include "common/Msg.h"
#include "common/mm.h"
#include "world/world.h"
#include "world/world_engine.h"
#include "world/history.h"

namespace World {

/// Handler pro zpravy modulu WorldClient
RVAL WORLD_CLIENT_MSG_HANDLER(MESSAGE_ID id, SENDER s, PARAM p);

/// Interni stavy modulu WORLD_CLIENT
enum WORLD_CLIENT_STATE {
    /// pocatecni stav
    WCS_EMPTY

    /// inicializace probehla
   ,WCS_WORLD_INITIALIZED

    /// nactena herni data
   ,WCS_WORLD_LOADED

    /// hra zahajena
   ,WCS_GAME_STARTED

    /// hra ukoncena
   ,WCS_GAME_ENDED
};

/** Hlavni trida klienstkeho modulu World. */
class TWorldClient
{
private:
    /// reprezentace sveta
    World::TWorld * world;

    /// vlastnictvi sveta
    bool world_ownership;

    /// historie hernich akci
    World::TClientHistory * history;

    /// rutiny sveta
    World::TWorldEngine * engine;

    /// id hrace, ktereho modul predstavuje
    PLAYER_ID player_id;

    /// interni stav
    WORLD_CLIENT_STATE state;

    /// pristup do struktur z msg handleru WORLD_CLIENT
	friend RVAL WORLD_CLIENT_MSG_HANDLER(MESSAGE_ID id, SENDER s, PARAM p);

    /// pristup do struktur z msg handleru WORLD_HANDLER
    friend RVAL WORLD_HANDLER_MSG_HANDLER(MESSAGE_ID mid, SENDER s, PARAM p);
public:
    /// Implicitni konstruktor modulu
    TWorldClient();
    
    /** Konstruktor specifikujici navic vlastnictvi mapy
        @param world_owner Nastavit na \c true, pokud bude klient na pocitaci sam nebo
                           "majitelem" sdilene reprezentace sveta. Nastavit na \c false,
                           pokud bude klient sdilet cizi reprezentaci sveta.
    */
    TWorldClient(bool world_owner);
    
    /// Destruktor modulu
	~TWorldClient(); 

    /// Inicializace modulu
    void init(PLAYER_ID id, World::TWorld * external_world, World::TWorldEngine * external_engine, World::TClientHistory * external_history);    

    /// Nahrani mapy ze sitove zpravy
    void loadFromNet(TPackage * package);

    /// Ulozeni mapy do souboru
    void saveMap(char * filename);

    /// Transceiver pro moduly WorldClient */
    static TMessageTransceiver world_client_transceiver;

    /// Globalni funkce po inicializovani konkretniho modulu a jeho message handleru
    static void initWorldClient(PLAYER_ID player_id, int client_id, World::TWorld * world = NULL, World::TWorldEngine * engine = NULL, World::TClientHistory * history = NULL);

    /// Globalni funkce pro destrukci konkretniho modulu WORLD_CLIENT 
    static void destroyWorldClient(PLAYER_ID id);

    /// Globalni funkce pro prvotni inicializaci modulu WORLD_CLIENT
    static void initWorldClients();

    /// Globalni funkce pro destrukci vsech modulu WORLD_CLIENT
    static void destroyWorldClients();

    void reload() { world->rules->reload(); }
};

//// Globalni promenna, pro primy pristup ke klientskemu modulu World. Obsahuje pole pointeru na klientske moduly. 
extern TWorldClient * world_client[MAX_PLAYERS + 1];

/// Mutex pro zamykani modulu world_client 
extern TMutex world_client_mutex;

/// Funkce pro zasilani zprav z TCL do WORLD_CLIENT
int WorldClient_SendMessage(ClientData Data, Tcl_Interp * pInterp, int argc, char * argv[]);

}//namespace
#endif
/*****************************************************************************/
